import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.TreeSelectionModel;
import java.net.URL;
import java.io.IOException;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JPanel;
import java.awt.*;
import java.awt.event.*;

public class OnlineHelp extends JPanel {
    private JEditorPane htmlPane;
    private static boolean DEBUG = false;
    private URL helpURL;

    //Optionally play with line styles.  Possible values are
    //"Angled", "Horizontal", and "None" (the default).
    private boolean playWithLineStyle = false;
    private String lineStyle = "Angled"; 

    public OnlineHelp() {
        

        //Create the nodes.
        DefaultMutableTreeNode top = new DefaultMutableTreeNode("PCN help");
        createNodes(top);

        //Create a tree that allows one selection at a time.
        final JTree tree = new JTree(top);
        tree.getSelectionModel().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);

        //Listen for when the selection changes.
        tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                   tree.getLastSelectedPathComponent();

                if (node == null) return;

                Object nodeInfo = node.getUserObject();
                if (node.isLeaf()) {
                    BookInfo book = (BookInfo)nodeInfo;
                    displayURL(book.bookURL);
                    if (DEBUG) {
                        System.out.print(book.bookURL + ":  \n    ");
                    }
                } else {
                    displayURL(helpURL); 
                }
                if (DEBUG) {
                    System.out.println(nodeInfo.toString());
                }
            }
        });

        if (playWithLineStyle) {
            tree.putClientProperty("JTree.lineStyle", lineStyle);
        }

        //Create the scroll pane and add the tree to it. 
        JScrollPane treeView = new JScrollPane(tree);

        //Create the HTML viewing pane.
        htmlPane = new JEditorPane();
        htmlPane.setEditable(false);
        initHelp();
        JScrollPane htmlView = new JScrollPane(htmlPane);

        //Add the scroll panes to a split pane.
        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        splitPane.setTopComponent(treeView);
        splitPane.setBottomComponent(htmlView);

        Dimension minimumSize = new Dimension(100, 50);
        htmlView.setMinimumSize(minimumSize);
        treeView.setMinimumSize(minimumSize);
        splitPane.setDividerLocation(150); //XXX: ignored in some releases
                                           //of Swing. bug 4101306
        //workaround for bug 4101306:
        //treeView.setPreferredSize(new Dimension(100, 100)); 

        splitPane.setPreferredSize(new Dimension(600, 450));

        //Add the split pane to this frame.
        this.add(splitPane, BorderLayout.CENTER);
    }

    private class BookInfo {
        public String bookName;
        public URL bookURL;
        public String prefix = "file:" 
                               + System.getProperty("user.dir")
                               + System.getProperty("file.separator");
                              
        public BookInfo(String book, String filename) {
            bookName = book;
            try {
                bookURL = new URL(prefix + filename);
            } catch (java.net.MalformedURLException exc) {
                System.err.println("Attempted to create a BookInfo "
                                   + "with a bad URL: " + bookURL);
                bookURL = null;
            }
        }

        public String toString() {
            return bookName;
        }
    }

    private void initHelp() {
        String s = null;
        try {
            s = "file:" 
                + System.getProperty("user.dir")
                + System.getProperty("file.separator")
                
                +"onlineHelp\\Introduction.htm";
            if (DEBUG) {
                System.out.println("Help URL is " + s);
            }
            helpURL = new URL(s);
            displayURL(helpURL);
        } catch (Exception e) {
            System.err.println("Couldn't create help URL: " + s);
        }
    }

    private void displayURL(URL url) {
        try {
            htmlPane.setPage(url);
        } catch (IOException e) {
            System.err.println("Attempted to read a bad URL: " + url);
        }
    }

    private void createNodes(DefaultMutableTreeNode top) {
        DefaultMutableTreeNode category = null;
        DefaultMutableTreeNode book = null;

        category = new DefaultMutableTreeNode("Introduction");
        top.add(category);

        
        book = new DefaultMutableTreeNode(new BookInfo
            ("Introduction",
            "introduction.htm"));
        category.add(book);
        
        book = new DefaultMutableTreeNode(new BookInfo
            ("System requirements",
            "systemReq.htm"));
        category.add(book);

        
        
        category = new DefaultMutableTreeNode("Contents");
        top.add(category);

        //VM
        book = new DefaultMutableTreeNode(new BookInfo
            ("Contents",
             "Contents.htm"));
        category.add(book);

        category = new DefaultMutableTreeNode("Network Creation");
        top.add(category);
        book = new DefaultMutableTreeNode(new BookInfo
            ("Network Creation",
             "create.htm"));
        category.add(book);
        category = new DefaultMutableTreeNode("Training");
        top.add(category);
        book = new DefaultMutableTreeNode(new BookInfo
            ("Training",
             "train.htm"));
        category.add(book);
        category = new DefaultMutableTreeNode("Recognition");
        top.add(category);
        book = new DefaultMutableTreeNode(new BookInfo
            ("Recognition",
             "recog.htm"));
        category.add(book);
        
        category = new DefaultMutableTreeNode("Network Results");
        top.add(category);
        book = new DefaultMutableTreeNode(new BookInfo
            ("Network Results",
             "results.htm"));
        category.add(book);
        
        category = new DefaultMutableTreeNode("Loading / saving Networks");
        top.add(category);
        book = new DefaultMutableTreeNode(new BookInfo
            ("Loading / saving Networks",
             "loadsave.htm"));
        category.add(book);
        category = new DefaultMutableTreeNode("File Formats");
        top.add(category);
        book = new DefaultMutableTreeNode(new BookInfo
            ("File Formats",
             "fileformat.htm"));
        category.add(book);
    }

    //public static void main(String[] args) {
    //    JPanel frame = new OnlineHelp();
    //}
}
