/* Copyright (c) 2001 C. Grigorescu S.E. Grigorescu */

#ifndef BYTE_IMAGE_H
#define BYTE_IMAGE_H

#include "Image.h"

#define ON  255
#define OFF 0

/**  Implements data structures and functions for manipulating binary images.
 */

class ByteImage : public Image< byte > 
{
 public:
  /*! Creates an image with the specified \a name without allocating space for the image data. */
  ByteImage(char *name = "") : Image< byte >(name) {}

  /*! Creates an image of size \a wxh with the specified \a name and allocates space for storing the image data. */
  ByteImage(int w, int h, char *name = "") : Image< byte > (w,h,name) {}

  /*! \ingroup point_alg
    Performs pixelwise logical AND between the current image and another image. The operation is performed only if the two images have the same size. 
*/
  ByteImage &operator&=(const ByteImage & img);

  /*! \ingroup point_alg
    Performs pixelwise logical OR between the current image and another image. The operation is performed only if the two images have the same size. 
*/
  ByteImage &operator|=(const ByteImage & img);  

  /*! \ingroup point_alg
    Performs pixelwise logical XOR between the current image and another image. The operation is performed only if the two images have the same size. 
*/
  ByteImage &operator^=(const ByteImage & img);  

  /*! \ingroup point_alg
    Performs pixelwise logical negation of the current image. */
  ByteImage &operator!(void);  

  /*! \ingroup conv
    Copy operator that allows conversion between \link Image Image< byte >\endlink and \link #ByteImage ByteImage\endlink.
*/
  ByteImage &operator=(Image< byte > &img); 
};

#endif
