/* Copyright (c) 2001 C. Grigorescu */

#ifndef _CONNECTED_COMPONENTS_H_
#define _CONNECTED_COMPONENTS_H_

#include "misc.h"

/** \ingroup morph_alg 
*/                                                                
int ConnectedComponents(IntImage &img, CONN_TYPE connectivity, IntImage &out);
/*! \fn ConnectedComponents(IntImage &img, CONN_TYPE connectivity, IntImage &out);

  Labels the image \a img based on its connected components and returns the number of these connected components. The labeled image is stored in \a out. The parameter \a connectivity will determine the type of neighborhood that will be used during the labeling. This can be:
  - FOUR: four-connectivity, in which case only top, bottom, left and right pixels are considered as neighbors;
  - EIGHT: eight-connectivity, in which case also the diagonal pixels are considered as neighbors.
 */

/** \ingroup morph_alg 
*/                                                                
int ConnectedComponents(ByteImage &img, CONN_TYPE connectivity, IntImage &out);
/*! \fn ConnectedComponents(ByteImage &img, CONN_TYPE connectivity, IntImage &out);

  Labels the image \a img based on its connected components and returns the number of these connected components. The labeled image is stored in \a out. The parameter \a connectivity will determine the type of neighborhood that will be used during the labeling. This can be:
  - FOUR: four-connectivity, in which case only top, bottom, left and right pixels are considered as neighbors;
  - EIGHT: eight-connectivity, in which case also the diagonal pixels are considered as neighbors.
 */

/** \ingroup morph_alg 
*/                                                                
int ConnectedComponents(IntImage &img, CONN_TYPE connectivity = FOUR);
/*! \fn ConnectedComponents(IntImage &img, CONN_TYPE connectivity = FOUR);

  Labels the image \a img based on its connected components and returns the number of these connected components. The labeled image is stored in \a img. The parameter \a connectivity will determine the type of neighborhood that will be used during the labeling. This can be:
  - FOUR: four-connectivity, in which case only top, bottom, left and right pixels are considered as neighbors;
  - EIGHT: eight-connectivity, in which case also the diagonal pixels are considered as neighbors.
*/

int Root(int r, int *lab, int size_x, int size_y);
int FindRoot(int r, int *lab);
void Union(int x, int y, int *lab);

/** \example test_conn_comp.cpp */

#endif
