/* Copyright (c) 2001 S.E. Grigorescu */

#ifndef CONTRAST_H
#define CONTRAST_H

#include "Image.h"

/** \ingroup point_alg 
*/                                                                
template<class T> void Contrast(Image< T >& input, float min, float max, Image< T >& output);
/*! \fn template< class T > void Contrast(Image< T >& input, float min, float max, Image< T >& output);

  Performs the contrast stretching of the input image and stores the result in the output image. The contrast stretching is done according to the following formula:

\f$ g_{new}(i, j) = (max - min) \displaystyle \frac{g_{old}(i, j) - minimum}{maximum - minimum} + min \f$ 
where 
\f$ g_{old}(i, j) \f$ and \f$ g_{new}(i, j) \f$ are the gray levels in a given point in the input and output image, respectively, \f$ minimum \f$ and \f$ maximum \f$ are the minimum and maximum gray values in the input image.
*/

/** \ingroup point_alg 
*/                                                                
template<class T> void Contrast(Image< T >& input, float min, float max);
/*! \fn template< class T > void Contrast(Image< T >& input, float min, float max);

  Performs the contrast stretching of the input image in place. The contrast stretching is done according to the following formula:

\f$ g_{new}(i, j) = (max - min) \displaystyle \frac{g_{old}(i, j) - minimum}{maximum - minimum} + min \f$ 
where 
\f$ g_{old}(i, j) \f$ and \f$ g_{new}(i, j) \f$ are the gray levels in a given point in the input and output image, respectively, \f$ minimum \f$ and \f$ maximum \f$ are the minimum and maximum gray values in the input image.
*/

/** \ingroup point_alg 
*/                                                                
template<class T> void Contrast(Image< T >& input, float min, float max, float lo, float hi, Image< T >& output);
/*! \fn template< class T > void Contrast(Image< T >& input, float min, float max, float lo, float hi, Image< T >& output);

  Performs the contrast stretching of the input image and stores the result in the output image. The contrast stretching is done according to the following formulas:

\f$ g_{new}(i, j) = min \f$ if \f$ g_{old}(i, j) \leq lo \f$,

\f$ g_{new}(i, j) = max \displaystyle \frac{g_{old}(i, j) - lo}{hi - lo} + min \f$ if \f$ lo < g_{old}(i, j) < hi \f$, and

\f$ g_{new}(i, j) = max \f$ if \f$ g_{old}(i, j) \geq hi \f$.

where \f$ g_{old}(i, j) \f$ and \f$ g_{new}(i, j) \f$ are the gray levels in a given point in the input and output image, respectively.
*/

/** \ingroup point_alg 
*/                                                                
template<class T> void Contrast(Image< T >& input, float min, float max, float lo, float hi);
/*! \fn template< class T > void Contrast(Image< T >& input, float min, float max, float lo, float hi);

  Performs the contrast stretching of the input image in place. The contrast stretching is done according to the following formulas:

\f$ g_{new}(i, j) = min \f$ if \f$ g_{old}(i, j) \leq lo \f$,

\f$ g_{new}(i, j) = max \displaystyle \frac{g_{old}(i, j) - lo}{hi - lo} + min \f$ if \f$ lo < g_{old}(i, j) < hi \f$, and

\f$ g_{new}(i, j) = max \f$ if \f$ g_{old}(i, j) \geq hi \f$.

where \f$ g_{old}(i, j) \f$ and \f$ g_{new}(i, j) \f$ are the gray levels in a given point in the input and output image, respectively.
*/

/**  \example test_contrast.cpp */
  
#endif

