/* Copyright (c) 2001 S.E. Grigorescu */

#ifndef CONVERSION_H
#define CONVERSION_H

#include <math.h>
#include "Image.h"
#include "ByteImage.h"
#include "IntImage.h"
#include "FloatImage.h"

/** \ingroup conv 
*/                                                                
void byte2int(Image< byte >& input, Image< int >& output);
/*! \fn void byte2int(Image< byte >& input, Image< int >& output);

  Performs the conversion of a binary image to a gray level image. In the gray level image the gray levels are stored as integers. */

/** \ingroup conv 
*/
void byte2int(ByteImage& input, IntImage& output);
/*! \fn void byte2int(ByteImage& input, IntImage& output);

  Performs the conversion of a binary image to a gray level image. In the gray level image the gray levels are stored as integers.
*/

/** \ingroup conv 
*/                                                                
void byte2float(Image< byte >& input, Image< float >& output);
/*! \fn void byte2float(Image< byte >& input, Image< float >& output);

  Performs the conversion of a binary image to a gray level image. In the gray level image the gray levels are stored as floats.
*/

/** \ingroup conv 
*/                                                                
void byte2float(ByteImage& input, FloatImage& output);
/*! \fn void byte2float(ByteImage& input, FloatImage& output);

  Performs the conversion of a binary image to a gray level image. In the gray level image the gray levels are stored as floats.
*/

/** \ingroup conv 
*/                                                                
void int2byte(Image< int >& input, Image< byte >& output);
/*! \fn void int2byte(Image< int >& input, Image< byte >& output);

  Performs the conversion of a gray level image to a binary image.
*/

/** \ingroup conv 
*/                                                                
void int2byte(IntImage& input, ByteImage& output);
/*! \fn void int2byte(IntImage& input, ByteImage& output);

  Performs the conversion of a gray level image to a binary image.
*/

/** \ingroup conv 
*/                                                                
void int2float(Image< int >& input, Image< float >& output);
/*! \fn void int2float(Image< int >& input, Image< float >& output);

  Performs the conversion of a gray level image whose gray levels are stored as integers to a gray level image whose gray levels are stored as floats.
*/

/** \ingroup conv 
*/                                                                
void int2float(IntImage& input, FloatImage& output);
/*! \fn void int2float(IntImage& input, FloatImage& output);

  Performs the conversion of a gray level image whose gray levels are stored as integers to a gray level image whose gray levels are stored as floats.
*/

/** \ingroup conv 
*/                                                                
void float2byte(Image< float >& input, Image< byte >& output);
/*! \fn void float2byte(Image< float >& input, Image< byte >& output);

  Performs the conversion of a gray level image to a binary image.
*/

/** \ingroup conv 
*/                                                                
void float2byte(FloatImage& input, ByteImage& output);
/*! \fn void float2byte(FloatImage& input, ByteImage& output);

  Performs the conversion of a gray level image to a binary image.
*/

/** \ingroup conv 
*/                                                                
void float2int(Image< float >& input, Image< int >& output);
/*! \fn void float2int(Image< float >& input, Image< int >& output);

  Performs the conversion of a gray level image whose gray levels are stored as floats to a gray level image whose gray levels are stored as integers.
*/

/** \ingroup conv 
*/                                                                
void float2int(FloatImage& input, IntImage& output);
/*! \fn void float2int(FloatImage& input, IntImage& output);

  Performs the conversion of a gray level image whose gray levels are stored as floats to a gray level image whose gray levels are stored as integers.
*/

/**  \example test_conversion.cpp */

#endif
