/* Copyright (c) 2001 C. Grigorescu */

#ifndef _DISTANCE_TRANSFORM_
#define _DISTANCE_TRANSFORM_

#include "ByteImage.h"
#include "IntImage.h"

enum DM_TYPE { EUCLID=0, EUCLID_1=1, CITY_BLOCK=2, CHESSBOARD=3};
const int MAX_INT = 65535;


/*! \ingroup morph_alg 
*/
void DistanceTransform(ByteImage &input, DM_TYPE dm_type, IntImage &output);
/*! \fn void DistanceTransform(ByteImage &input, DM_TYPE dm_type, IntImage &output);

  Computes the distance transform of the \a input image using the distance map type given by \a dm_type and stores the result in the \a output image. Currently defined distance map types are:
  - EUCLID: the distance function is the Euclidian distance.
  The implementation uses chamfering with the \f$ 5,7,11 \f$ chamfer mask. 
  - EUCLID_1: the distance function is the Euclidian distance.
  The implementation uses chamfering with the \f$ 3,4 \f$ chamfer mask. 
  - CITY_BLOCK: the distance function is the city-block distance (only the horizontal and vertical pixels
are considered as neighbors, i.e. 4-connectivity).
  - CHESSBOARD: the distance function is of chessboard type (the horizontal, vertical and diagonal pixels are considered as neighbors, i.e. 8-connectivity).

The implementation is based on the algorithm described in "Distance transformation in digital images" by  G. Borgefors, Comput. Vision, Graph. %Image Process., no. 34, pp. 344-371, 1986.
 */
/** \example test_dist_trans.cpp */

#endif
