/* Copyright (c) 2001 C. Grigorescu S.E. Grigorescu */

/** \defgroup io Input/output operations
    This group contains methods for image input/output operations.
*/

/** \defgroup conv Conversion operations
    This group contains conversion functions for the available image formats.
*/

/** \defgroup img_st Image statistics
    This group contains methods for computing the minimum, the maximum, the average, and the standard deviation of the gray level distribution in an image.
*/

/** \defgroup point_alg Point operations 
    This group contains methods and stand-alone functions implementing point operations.
*/

/** \defgroup neigh_alg Neighborhood operations
    This group contains stand-alone functions implementing neighborhood operations.
*/

/** \defgroup geom_alg Geometrical operations
    This group contains methods as well as stand-alone functions implementing geometrical transformations of an image. 
*/
  
/** \defgroup morph_alg Morphological operations
    This group contains stand-alone functions implementing morphological operations.
*/

/** \defgroup hist_alg Histogram related operations
    This group contains methods implementing histogram computations and other histogram related operations.
*/

/** \defgroup sig_alg Other signal-processing related operations
    This group contains stand-alone functions implementing convolution, downsampling, upsampling, and noise addition. 
*/

/** \defgroup img_trans Image transforms
    This group contains classes and stand-alone functions implementing the Fast Fourier Transform and the Fast Wavelet Transform. 
*/

/** \defgroup edge_alg Edge detection 
    This group contains stand-alone functions implementing the following edge detection algorithms: Prewitt, Sobel, Kirsch, Laplace. 
*/

/** \defgroup misc Miscellaneous 
    This group contains general-purpose classes and functions (timing class and random number generators).  */

/**  \mainpage  Tools for  %Image  Processing  (TiP) 

\subsection sub1 Features

This version of TiP offers the following capabilities:

- \ref io: The package offers facilities for reading and writing images in different formats, provided that Imlib package is installed. All formats accepted by Imlib can be used for reading and writing (PPM,
   PGM, TIFF, GIF, PNG, XPM, JPEG, EPS, EIM, etc.);

- Storage of image gray levels in different formats (\link ByteImage unsigned byte\endlink, \link IntImage int\endlink, \link FloatImage float\endlink) suitable for different types of processing. Moreover, the package offers \link conv conversion facilities\endlink between different formats.

- %Image processing algorithms: 
   - \ref img_st - the computation of the minimum, the maximum, the average, and the standard deviation of the gray level distribution in an image; 
   - \ref point_alg  - arithmetical operations between images or between an image and a scalar, logical operations between images or between an image and a scalar, thresholding, and contrast stretching;
   - \ref neigh_alg  -  the computation of mean,   median,  standard  deviation, variance, minimum, and maximum on a rectangular neighborhood of a pixel; 
   - \ref geom_alg - translation, rotation, crop,  zoom,  and pad; 
   - \ref morph_alg - erosion, dilation, opening,  closing, connected components, and skeletons;
   - \ref sig_alg - upsampling and  downsampling; convolution with  various  types  of convolution  kernels;  
   - \link hist_alg Histogram related operations\endlink;  
   - \ref edge_alg algorithms -  Prewitt,  Sobel, Kirsch;  
   - \link img_trans Image transforms\endlink - Fast Fourier Transform (direct and inverse); Fast Wavelet Transform (direct and inverse);  
   - Noise generation and addition; 

- \ref misc functions: 
   - Timing; 
   - Random number generators.

*/

