/* Copyright (c) 2001 C. Grigorescu S.E. Grigorescu */

#ifndef INT_IMAGE_H
#define INT_IMAGE_H

#include "Image.h"

/** 
  Implements data structures and functions for manipulating images whose gray levels are represented as integer values.
 */

class IntImage : public Image< int > 
{
 public:  
  /*! Creates an image with the specified \a name without allocating space for the image data. */
  IntImage(char *name= "") : Image< int >(name) {}

  /*! Creates an image of size \a wxh with the specified \a name and allocates space for storing the image data. */
  IntImage(int w, int h, char *name = "") : Image< int >(w, h, name) {}

  /*! \ingroup point_alg
    Performs pixelwise logical AND between the current image and another image. The operation is performed only if the two images have the same size. */
  IntImage &operator&=(const IntImage & img);

  /*! \ingroup point_alg
    Performs pixelwise logical OR between the current image and another image. The operation is performed only if the two images have the same size. */
  IntImage &operator|=(const IntImage & img);  

  /*! \ingroup conv
    Copy operator that allows conversion between \link Image Image< int >\endlink and \link #IntImage IntImage\endlink. */
  IntImage &operator=(Image< int > &img); 
};

#endif
