/* Copyright (c) 2001 C. Grigorescu */

#ifndef INT_KERNEL_H
#define INT_KERNEL_H

#include "Kernel.h"
/**
    Implements data structures and functions for manipulating convolution kernels */

class IntKernel : public Kernel < int >
{
 public:
  /*! Creates a kernel with the specified \a name without allocating space for the kernel data. */
  IntKernel(char *name="") : Kernel< int >(name) {}

  /*! Creates a kernel of size \a wxh with the specified \a name and type (\a kernel_type) and allocates space for storing the kernel data.
    Currently supported types of kernels are:
    - LAPLACIAN_1 \verbatim 
       0 -1  0 
      -1  4 -1  
       0 -1  0 \endverbatim
    - LAPLACIAN_2 \verbatim 
      -1 -1 -1 
      -1  8 -1 
      -1 -1 -1 \endverbatim
    - SOBEL_H     \verbatim 
      -1 -2 -1  
       0  0  0  
       1  2  1 \endverbatim
    - SOBEL_V     \verbatim 
      -1  0  1 
      -2  0  2 
      -1  0  1 \endverbatim
    - PREWITT_H   \verbatim 
      -1 -1 -1  
       0  0  0  
       1  1  1 \endverbatim
    - PREWITT_V   \verbatim  
       1  0 -1  
       1  0 -1  
       1  0 -1 \endverbatim
    - KIRSCH_1    \verbatim  
       5  5  5 
      -3  0 -3 
      -3 -3 -3 \endverbatim
    - KIRSCH_2    \verbatim 
      -3  5  5 
      -3  0  5 
      -3 -3 -3 \endverbatim
    - KIRSCH_3    \verbatim 
      -3 -3  5 
      -3  0  5 
      -3 -3  5 \endverbatim
    - KIRSCH_4    \verbatim 
      -3 -3 -3 
      -3  0  5 
      -3  5  5 \endverbatim
    - KIRSCH_5    \verbatim 
      -3 -3 -3 
      -3  0 -3  
       5  5  5 \endverbatim
    - KIRSCH_6    \verbatim 
      -3 -3 -3  
       5  0 -3  
       5  5 -3 \endverbatim
    - KIRSCH_7    \verbatim  
       5 -3 -3  
       5  0 -3  
       5 -3 -3 \endverbatim
    - KIRSCH_8    \verbatim  
       5  5 -3  
       5  0 -3 
      -3 -3 -3 \endverbatim
    - KERNEL_INT_UNKNOWN (unspecified or supplied from a file).
  */
  IntKernel(INT_KERNEL_TYPE _kernel_type=LAPLACIAN_1, char *name="");

  /*! Invert (change the sign of) the kernel coefficients.
   */
  IntKernel &operator^(IntKernel &kernel);
};

#endif

