/* Copyright (c) 2001 S.E. Grigorescu */

#ifndef LOCALOP_H
#define LOCALOP_H

#include <math.h>
#include <string>
#include <vector>
#include <algorithm>
#include "Image.h"

/** \ingroup neigh_alg 
*/
template<class T> void LocalOp(Image< T >& input, string op_type, int w_x, int w_y, Image< T >& output);
/*! \fn template< class T > void LocalOp(Image< T >& input, string op_type, int w_x, int w_y, Image< T >& output);

  Computes the value of each pixel in the output image as statistics of the pixel values inside a window of size \f$ w_x \times w_y\f$ centered in the corresponding pixel in the input image. The type of statistics is determined by the parameter \a op_type. Six types of statistics are allowed:
  - the mean, in which case the parameter \a op_type should be "mean";
  - the median, in which case the parameter \a op_type should be "median";
  - the standard deviation, in which case the parameter \a op_type should be "stdev";
  - the variance, in which case the parameter \a op_type should be "var";
  - the min, in which case the parameter \a op_type should be "min";
  - the max, in which case the parameter \a op_type should be "max".
*/

/** \ingroup neigh_alg 
*/
template<class T> void LocalOp(Image< T >& input, string op_type, int w, Image< T >& output);
/*! \fn template< class T > void LocalOp(Image< T >& input, string op_type, int w, Image< T >& output);

  Computes the value of each pixel in the output image as statistics of the pixel values inside a window of size \f$ w \times w\f$ centered in the corresponding pixel in the input image. The type of statistics is determined by the parameter \a op_type. Six types of statistics are allowed:
  - the mean, in which case the parameter \a op_type should be "mean";
  - the median, in which case the parameter \a op_type should be "median";
  - the standard deviation, in which case the parameter \a op_type should be "stdev";
  - the variance, in which case the parameter \a op_type should be "var";
  - the min, in which case the parameter \a op_type should be "min";
  - the max, in which case the parameter \a op_type should be "max".
*/

/** \ingroup neigh_alg 
*/
template< class T > void LocalOp(Image< T >& input, string op_type, int w_x, int w_y);
/*! \fn template< class T > void LocalOp(Image< T >& input, string op_type, int w_x, int w_y);

  Computes the value of each pixel in the output image as statistics of the pixel values inside a window of size \f$ w_x \times w_y\f$ centered in the corresponding pixel in the input image. The operation is done in place. The type of statistics is determined by the parameter \a op_type. Six types of statistics are allowed:
  - the mean, in which case the parameter \a op_type should be "mean";
  - the median, in which case the parameter \a op_type should be "median";
  - the standard deviation, in which case the parameter \a op_type should be "stdev";
  - the variance, in which case the parameter \a op_type should be "var";
  - the min, in which case the parameter \a op_type should be "min";
  - the max, in which case the parameter \a op_type should be "max".
*/

/** \ingroup neigh_alg 
*/
template<class T> void LocalOp(Image< T >& input, string op_type, int w);
/*! \fn template< class T > void LocalOp(Image< T >& input, string op_type, int w);

  Computes the value of each pixel in the output image as statistics of the pixel values inside a window of size \f$ w \times w\f$ centered in the corresponding pixel in the input image. The operation is done in place. The type of statistics is determined by the parameter \a op_type. Six types of statistics are allowed:
  - the mean, in which case the parameter \a op_type should be "mean";
  - the median, in which case the parameter \a op_type should be "median";
  - the standard deviation, in which case the parameter \a op_type should be "stdev";
  - the variance, in which case the parameter \a op_type should be "var";
  - the min, in which case the parameter \a op_type should be "min";
  - the max, in which case the parameter \a op_type should be "max".
*/

template<class T> T compute_mean(Image< T >&, int, int, int, int);
template<class T> T compute_median(Image< T >&, int, int, int, int);
template<class T> T compute_stdev(Image< T >&, int, int, int, int);
template<class T> T compute_var(Image< T >&, int, int, int, int);
template<class T> T compute_min(Image< T >&, int, int, int, int);
template<class T> T compute_max(Image< T >&, int, int, int, int);

/** \example test_localop.cpp */

#endif
