/* Copyright (c) 2001 S.E. Grigorescu */

#ifndef NOISE_H
#define NOISE_H

#include <string>
#include <math.h>
#include "Image.h"
#include "Rnd_no_gen.h"

/** \ingroup sig_alg 
*/                                                                
template<class T> void Noise(Image< T >& input, string noise_type, float a, float b, int seed = 1);
/*! \fn template< class T > void Noise(Image< T >& input, string noise_type, float a, float b, int seed = 1);

  Adds noise of type \a noise_type to the input image. Three types of noise are allowed:  
 - Gaussian noise with mean "a" and standard deviation "b". For this purpose the parameter \a noise_type should be "normal".
 - Noise with uniform distribution in the interval [a, b]. For this purpose the parameter \a noise_type should be "uniform".
 - Salt and pepper noise. For this purpose the parameter \a noise_type should be "salt&pepper". The parameter \a a indicates what percentage of the image pixels are affected by the noise. The parameter \a b is ignored. 
 
 The parameter \a seed is used to initialize the random generator. Its default value is 1.
*/

/** \ingroup sig_alg 
*/                                                                
template<class T> void Noise(Image< T >& input, int seed = 1);
/*! \fn template< class T > void Noise(Image< T >& input, int seed = 1);

  Adds Gaussian noise with mean 0 and standard deviation 1 to the input image.  

  The parameter \a seed is used to initialize the random generator. Its default value is 1.
*/

/** \ingroup sig_alg 
*/                                                                
template<class T> void Noise(Image< T >& input, float a, float b, int seed = 1);
/*! \fn template< class T > void Noise(Image< T >& input, float, float, int seed = 1);

  Adds Gaussian noise with mean \a a and standard deviation \a b to the input image.
  
  The parameter \a seed is used to initialize the random generator. Its default value is 1.
*/

/** \ingroup sig_alg 
*/                                                                
template<class T> void Noise(Image< T >& input, string noise_type, int seed = 1);
/*! \fn template< class T > void Noise(Image< T >& input, string, int seed = 1);

  Adds noise of type \a noise_type to the input image. Three types of noise are allowed:  
 - Gaussian noise with mean \a 0 and standard deviation \a 1. For this purpose the parameter \a noise_type should be "normal".
 - Noise with uniform distribution in the interval \a [0, \a 1]. For this purpose the parameter \a noise_type should be "uniform".
 - Salt and pepper noise. For this purpose the parameter \a noise_type should be "salt&pepper". The percentage of the image pixels that are affected by the noise is randomly chosen.  

 The "seed" parameter is used to initialize the random generator. Its default value is 1.
*/

/** \example test_noise.cpp */

#endif





