/* Copyright (c) 2001 C. Grigorescu */
#ifndef _TIMER_H_
#define _TIMER_H_

#include <sys/time.h>
#include <unistd.h>

/** \ingroup misc 
    Implements timing mechanisms. 
*/
class Timer
{
 private:
  int   running;
  int   seconds;
  int   microseconds;
  timeval start;
  timeval stop;

 public:
  /*! Constructor */
  Timer::Timer() {
    running = 0;
    seconds = 0;
    microseconds = 0;
  }                 

  /*! Starts the timer. */
  void  Start();

  /*! Stops the timer. */
  void  Stop();

  /*! Gets elapsed time, in seconds (user time), since the timer was started. */
  float getTime();

  /*! Prints the total elapsed time (user time) between the Start() and the last getTime() or Stop() calls. */
  void  Print();

  /*! Prints elapsed time between Start() and current point (equivalent
    to getTime() followed by Print()). */
  void  Tick();
};

/** \example test_timer.cpp */

#endif
