/* Copyright (c) 2001 S.E. Grigorescu */

#ifndef ZOOMING_H
#define ZOOMING_H

#include <math.h>
#include "Image.h"

/** \ingroup geom_alg 
*/                                                                
template<class T> void Zoom(Image< T >& input, float xf, float yf, Image< T >& output);
/*! \fn template< class T > void Zoom(Image< T >& input, float xf, float yf, Image< T >& output);

  Performs the zooming of the input image using a factor \a xf along the x-axis and a factor \a yf along the y-axis; stores the result in the output image. 
*/

/** \ingroup geom_alg 
*/                                                                
template<class T> void Zoom(Image< T >& input, float xyf, Image< T >& output);
/*! \fn template< class T > void Zoom(Image< T >& input, float xyf, Image< T >& output);

  Performs the zooming of the input image using the same factor \a xyf along both axes and stores the result in the output image. 
*/

/** \ingroup geom_alg 
*/                                                                
template<class T> void Zoom(Image< T >& input, float xf, float yf);
/*! \fn template< class T > void Zoom(Image< T >& input, float xf, float yf);

  Performs the zooming of the input image in place using a factor \a xf along the x-axis and a factor \a yf along the y-axis.
*/

/** \ingroup geom_alg 
*/                                                                
template<class T> void Zoom(Image< T >& input, float xyf);
/*! \fn template< class T > void Zoom(Image< T >& input, float xyf);

  Performs the zooming of the input image in place using the same factor \a xyf along both axes.
*/

/** \example test_zoom.cpp */
#endif

