/* Copyright (c) 2001 S.E. Grigorescu */

#include "Contrast.h"

template void Contrast< int >(Image< int >&, float, float, Image< int >&);
template void Contrast< byte >(Image< byte >&, float, float, Image< byte >&);
template void Contrast< float >(Image< float >&, float, float, Image< float >&);
template void Contrast< int >(Image< int >&, float, float, float, float, Image< int >&);
template void Contrast< byte >(Image< byte >&, float, float, float, float, Image< byte >&);
template void Contrast< float >(Image< float >&, float, float, float, float, Image< float >&);
template void Contrast< int >(Image< int >&, float, float);
template void Contrast< byte >(Image< byte >&, float, float);
template void Contrast< float >(Image< float >&, float, float);
template void Contrast< int >(Image< int >&, float, float, float, float);
template void Contrast< byte >(Image< byte >&, float, float, float, float);
template void Contrast< float >(Image< float >&, float, float, float, float);
 
template< class T > void Contrast(Image< T >& input, float min, float max, Image< T >& output)
{
  double m, M;
  Image< T > res;
  res = input;
  m = input.min();
  M = input.max();

  for (int i = 0; i < input.getHeight(); i++)
    for (int j = 0; j < input.getWidth(); j++)
      res[i][j] = (T)((max-min)*(input[i][j] - m)/(M - m) + min);
  output = res;
}

template< class T > void Contrast(Image< T >& input, float min, float max)
{
  Contrast(input, min, max, input);
}
 
template< class T > void Contrast(Image< T >& input, float min, float max, float lo, float hi, Image< T >& output)
{
  double m, M;
  Image< T > res;
  res = input;
  m = input.min();
  M = input.max();

  for (int i = 0; i < input.getHeight(); i++)
    for (int j = 0; j < input.getWidth(); j++) {
      if (input[i][j] >= lo && input[i][j] <= hi) res[i][j] = (T)((max-min)*(input[i][j] - m)/(M - m) + min);
      if (input[i][j] < lo && input[i][j] <= hi) res[i][j] = (T)min;
      if (input[i][j] > hi) res[i][j] = (T)max;
    }
  output = res;
}
 
template< class T > void Contrast(Image< T >& input, float min, float max, float lo, float hi)
{
  Contrast(input, min, max, lo, hi, input);
}






