/* Copyright (c) 2001 C. Grigorescu */

#include "config.h"
#include "Convolution.h"
 
#ifdef HAVE_LIBFFTW
#include "fftwlib.h"
#endif                                                                                       

#define RANGE(i, j, sy, sx) ( (i>=0) && (j>=0) && (i<sy) && (j<sx) )

template void Convolution< int >(Image< int > &input1, Image< int > &input2, Image< int > &output);
template void Convolution< byte >(Image< byte > &input1, Image< byte > &input2, Image< byte > &output);
template void Convolution< float >(Image< float > &input1, Image< float > &input2, Image< float > &output);

template< class T >
void Convolution(Image< T > &input1, Image< T > &input2, Image< T > &output) 
{
#ifdef HAVE_LIBFFTW
  if (typeid(T) == typeid(float)) {
    T *data = NULL;
    try {
      data = (T *)calloc(input1.getSize(), sizeof(T));
      Assert<Fatal_error>(data);         
    }
    catch(Fatal_error e) { e.print("Unable to allocate memory"); }
    if ((input1.getWidth()!= input2.getWidth()) && 
	(input1.getHeight()!= input2.getHeight())) {
      ERROR("Cannot convolve float images with different size");
    }
    
    _convolve((float *)(input1.getPixels()),(float *)(input2.getPixels()), 
	      input1.getWidth(), input1.getHeight(), (float *)(data));
    output.setPixels(input1.getWidth(), input1.getHeight(), data);
    free(data);
  }
  else
#endif
  {
    int wi = input1.getWidth(), hi = input1.getHeight(),
        wk = input2.getWidth(), hk = input2.getHeight();
    int wk2 = wk>>1, hk2 = hk>>1, kk, mm;
    T *data = NULL;
    try {
     data = (T *)calloc(input1.getSize(), sizeof(T));
     Assert<Fatal_error>(data);         
    }
    catch(Fatal_error e) { e.print("Unable to allocate memory"); }
    
    for(int i=0; i<hi;i++)
      for(int j=0;j<wi;j++) {
	for(int k=-hk2;k<hk2;k++)
	  for(int m=-wk2;m<wk2;m++) {
	    //if(RANGE(i+k, j+m, hi, wi))
	      kk = i + k; mm = j + m;	          		
	      if(kk<0) kk = i - k;
	      else if(kk>=hi) kk -= hi;
	      if(mm<0) mm = j - m; 	
	      else if(mm>=wi) mm -= wi;
	      data[i*wi+j] += (input2)[k+hk2][m+wk2] *
		              (input1)[kk][mm];
	  }
      }
    output.setPixels(wi, hi, data);
    free(data);
  }
}


    
