/* Copyright (c) 2001 C. Grigorescu */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "misc.h"
#include "DistanceTransform.h"

int min(int x, int y) {  return(((x)<(y)) ? (x):(y)); }                                       

void DistanceTransform(ByteImage &input, DM_TYPE dm_type, IntImage &output)
{
  int weights[3]; 
  switch(dm_type) {
  case EUCLID:
    weights[0] = 5;
    weights[1] = 7;
    weights[2] = 11;
    break;
  case EUCLID_1:
    weights[0] = 3;
    weights[1] = 4;
    weights[2] = MAX_INT;
    break;
  case CITY_BLOCK:
    weights[0] = 1;
    weights[1] = MAX_INT;
    weights[2] = MAX_INT;
    break;
  case CHESSBOARD:
    weights[0] = 1;
    weights[1] = 1;
    weights[2] = MAX_INT;
    break;
  default: 
    cout << "Unknown distance map type " << endl;
  }

  int w = input.getWidth();
  int h = input.getHeight();
  output.makeImage(w,h);
  IntImage temp1, temp2;
  temp1.makeImage(w,h);
  for (int i=0;i<h;i++)
    for (int j=0;j<w;j++) 
      if (input[i][j] != ON) {
      temp1[i][j] = 0;
      output[i][j] = 0;
      }
      else {
      temp1[i][j] = MAX_INT;
      output[i][j] = MAX_INT;
      }

  //  forward scan
  int min1;
  for (int i=2;i<h;i++)
    for (int j=2;j<w-2;j++) {
      min1 = temp1[i][j];
      min1 = min(min1, (temp1[i][j-1]+weights[0]));
      min1 = min(min1, (temp1[i-1][j]+weights[0]));
      min1 = min(min1, (temp1[i-1][j-1]+weights[1]));
      min1 = min(min1, (temp1[i-1][j+1]+weights[1]));
      min1 = min(min1, (temp1[i-1][j-2]+weights[2]));
      min1 = min(min1, (temp1[i-2][j-1]+weights[2]));
      min1 = min(min1, (temp1[i-2][j+1]+weights[2]));
      min1 =  min(min1, (temp1[i-1][j+2]+weights[2]));
      temp1[i][j] = min1;
    }
  // backward scan
  for (int i=h-3;i>=0;i--)
    for (int j=w-3;j>=2;j--) {
      min1 = output[i][j];
      min1 = min(min1, (output[i][j+1]+weights[0]));
      min1 = min(min1, (output[i+1][j]+weights[0]));
      min1 = min(min1, (output[i+1][j+1]+weights[1]));
      min1 = min(min1, (output[i+1][j-1]+weights[1]));
      min1 = min(min1, (output[i+1][j+2]+weights[2]));
      min1 = min(min1, (output[i+2][j+1]+weights[2]));
      min1 = min(min1, (output[i+2][j-1]+weights[2]));
      min1 = min(min1, (output[i+1][j-2]+weights[2]));
      output[i][j] = min1;
    }
  for (int i=2;i<h-2;i++)
    for (int j=2;j<w-2;j++)
      output[i][j] = min(temp1[i][j],output[i][j]);
}
