/* Copyright (c) 2001 S.E. Grigorescu */

#include "Histogram.h"
#include <math.h>
#include <strstream.h> 

template class Histogram< int >;
template class Histogram< float >;

template< class T > Histogram< T >::Histogram(Image< T > &input, int no_bins = 100)
{
  bins = no_bins;
  hist = new double[bins];
  for (int i = 0; i < bins; hist[i++] = 0.0) ;
  img = input;
  min_x = (double)img.min();
  max_x = (double)img.max();
  compute_histogram();

#ifdef HAVE_LIBPLOTTER
  grapher_disp = NULL;
  grapher_wrt = NULL;
  Width = Height = 600;
  bg = "White"; 
  top_label = "";
  x_label = "";
  y_label = "";
  title_font_name = "Helvetica";
  xy_font_name = "Helvetica";
  title_font_size = 0.07;
  xy_font_size = 0.0525;
  final_min_x = (double)img.min();
  final_max_x = (double)img.max();
  final_min_y = 0.0;
  final_max_y = (double)img.getSize();
  final_spacing_x = final_spacing_y = 00.;
  final_log_axis = 0;
  final_transpose_axes = false;
  fill_fraction = 1.0;
  x_range_spec = false;
  y_range_spec = false;
#endif
}

template< class T > Histogram< T >::Histogram(Image< T > &input, int no_bins, T min, T max)
{
  bins = no_bins;
  hist = new double[bins];
  for (int i = 0; i < bins; hist[i++] = 0.0) ;
  img = input;
  min_x = (double)min;
  max_x = (double)max;
  compute_histogram();

#ifdef HAVE_LIBPLOTTER
  grapher_disp = NULL;
  grapher_wrt = NULL;
  Width = Height = 600;
  bg = "White"; 
  top_label = "";
  x_label = "";
  y_label = "";
  title_font_name = "Helvetica";
  xy_font_name = "Helvetica";
  title_font_size = 0.07;
  xy_font_size = 0.0525;
  final_min_x = (double)img.min();
  final_max_x = (double)img.max();
  final_min_y = 0.0; 
  final_max_y = (double)img.getSize(); 
  final_spacing_x = final_spacing_y = 0.0;  
  final_log_axis = 0;
  final_transpose_axes = false;
  fill_fraction = 1.0;
  x_range_spec = false;
  y_range_spec = false;
#endif
}

template< class T > Histogram< T >::~Histogram()
{
  delete[] hist;

#ifdef HAVE_LIBPLOTTER
  closeWindow();
#endif
}

template< class T > void Histogram< T >::compute_histogram()
{
  for (int i = 0; i < bins; hist[i++] = 0.0) ;
  for (int i = 0; i < img.getSize(); i++)
    if (img.getPixels()[i] >= min_x && img.getPixels()[i] <= max_x)
      hist[(int)floor((bins-1)*(img.getPixels()[i]-min_x)/(max_x-min_x))]++;
}

template< class T > void Histogram< T >::equalize(Image< T > &output)
{
  output = img;
  double *c_hist = new double[bins];
  c_hist[0] = hist[0];
  for (int i = 1; i < bins; i++)
    c_hist[i] = c_hist[i-1] + hist[i];
  
  for (int i = 0; i < img.getHeight(); i++)
    for (int j = 0; j < img.getWidth(); j++) 
      if (img[i][j] >= min_x && img[i][j] <= max_x) {
	double tmp = c_hist[(int)floor((bins-1)*(img[i][j]-min_x)/(max_x-min_x))];
	output[i][j] = (T)((max_x-min_x)*tmp/c_hist[bins-1] + min_x);
    }
  delete[] c_hist;
}

#ifdef HAVE_LIBPLOTTER
template< class T > void Histogram< T >::showHistogram()
{
  Point *p = NULL;
  p = new Point[4 * bins];

  for (int i = 0; i < 4 * bins; i++)
    set_point_attr(p[i]);

  int counter = 0; 
  for (int i = 0; i < bins; i++)
    if (add_point(hist[i], min_x+i*(max_x-min_x)/bins, 
		  min_x+(i+1)*(max_x-min_x)/bins, 4*counter, p))
      counter++;
  
  array_bounds (p, 4 * counter, final_transpose_axes, 1,
		&final_min_x, &final_min_y, &final_max_x, &final_max_y,
		x_range_spec, y_range_spec, x_range_spec, y_range_spec);
  
  ostrstream dim;
  dim << Width << "x" << Height << ends;
  if (grapher_disp == NULL) grapher_disp = new_grapher ("X", NULL, bg, dim.str(),
							"yes", "500", "yes",
							"A4", "0", false);
  
  begin_graph (grapher_disp, 1.0, 0.0, 0.0);  
  set_graph_parameters(grapher_disp, 0.005, "black", top_label, title_font_name, 
		       title_font_size, 0.01, AXES_AND_BOX, 
		       final_min_x, final_max_x, final_spacing_x, 
		       final_min_y, final_max_y, final_spacing_y,
		       false, false, 0.7, 0.7, 0.2, 0.2, 
		       xy_font_name, xy_font_size, x_label, 
		       xy_font_name, xy_font_size, y_label, 
		       false, final_log_axis, 0, 0, 0, 1, 1.3, final_transpose_axes);
  draw_frame_of_graph (grapher_disp, false);
  plot_point_array(grapher_disp, p, 4*counter);
  delete[] p;
  end_polyline_and_flush (grapher_disp);
  end_graph(grapher_disp);
}

template< class T > void Histogram< T >::set_point_attr(Point &point)
{
  point.x = 0.0;
  point.y = 0.0;
  point.symbol = 1;
  point.symbol_size = 0.05;
  point.symbol_font_name = "ZapfDingbats";
  point.linemode = 1;
  point.line_width = 0.001;
  point.fill_fraction = fill_fraction;
  point.use_color = false;
  point.have_x_errorbar = false;
  point.have_y_errorbar = false;
  point.pendown = true;
}

template< class T > bool Histogram< T >::add_point(double y, double x1, double x2, 
						   int pos, Point *p)
{
  if (x_range_spec && (x1 < final_min_x || x1 > final_max_x))
    return false;
  else {
    double y1, y2;
    y1 = y_range_spec ? final_min_y : 0.0;
    y2 = (y_range_spec && y > final_max_y) ? final_max_y : y;

    p[pos  ].x = x1; p[pos  ].y = y1;
    p[pos+1].x = x1; p[pos+1].y = y2;
    p[pos+2].x = (x_range_spec && x2 > final_max_x) ? final_max_x : x2; p[pos+2].y = y2;
    p[pos+3].x = (x_range_spec && x2 > final_max_x) ? final_max_x : x2; p[pos+3].y = y1;
    return true;
  }
}

template< class T > void Histogram< T >::writeHistogram(char *type, char *file_name)
{
  if (strcmp(type, "Gif") * strcmp(type, "PS") *
      strcmp(type, "Fig") * strcmp(type, "Bmp") == 0) {
    if (grapher_wrt != NULL) {
      delete_grapher(grapher_wrt);
      grapher_wrt = NULL;
    }
    FILE* fo = fopen(file_name, "w");

    Point * p = (Point *)xmalloc (4 * bins * sizeof (Point));
    for (int i = 0; i < 4 * bins; i++)
      set_point_attr(p[i]);
   
    int counter = 0;
    for (int i = 0; i < bins; i++)
      if (add_point(hist[i], min_x+i*(max_x-min_x)/bins, 
		  min_x+(i+1)*(max_x-min_x)/bins, 4*counter, p))
	counter++;
    array_bounds (p, 4 * counter, final_transpose_axes, 1,
		  &final_min_x, &final_min_y, &final_max_x, &final_max_y,
		  x_range_spec, y_range_spec, x_range_spec, y_range_spec);
    ostrstream dim; 
    dim << Width << "x" << Height << ends;
    if (grapher_wrt == NULL) grapher_wrt = new_grapher (type, fo, bg, dim.str(), 
							"yes", "500", "yes", 
							"A4", "0", false);
    begin_graph (grapher_wrt, 1, 0, 0);  
    set_graph_parameters (grapher_wrt, .005, "black", top_label, title_font_name, 
			title_font_size, .01, AXES_AND_BOX, final_min_x, final_max_x, 
			final_spacing_x, final_min_y, final_max_y, final_spacing_y, 
			false, false, .7, .7, .2, .2, xy_font_name, xy_font_size, x_label, 
			xy_font_name, xy_font_size, y_label, false, final_log_axis, 
			0, 0, 0, 1, 1.3, final_transpose_axes);
    draw_frame_of_graph (grapher_wrt, true); 
    plot_point_array (grapher_wrt, p, 4*counter);
    free(p);
    end_polyline_and_flush (grapher_wrt);
    end_graph(grapher_wrt);
    delete_grapher (grapher_wrt);
    grapher_wrt = NULL;
    fclose(fo); 
  }
  else 
    cout << "Format unsupported! Supported formats are: \"Gif\", \"PS\", \"Fig\", and \"Bmp\"." << endl; 
}

template< class T > void Histogram< T >::closeWindow()
{
  if (grapher_disp != NULL) {
    delete_grapher(grapher_disp);
    grapher_disp = NULL;
  }
}
#endif


