/* Copyright (c) 2001 S.E. Grigorescu */

#include "LocalOp.h"

template void LocalOp< int >(Image< int >&, string, int, int, Image< int >&);
template void LocalOp< byte >(Image< byte >&, string, int, int, Image< byte >&);
template void LocalOp< float >(Image< float >&, string, int, int, Image< float >&);
template void LocalOp< int >(Image< int >&, string, int, Image< int >&);
template void LocalOp< byte >(Image< byte >&, string, int, Image< byte >&);
template void LocalOp< float >(Image< float >&, string, int, Image< float >&);
template void LocalOp< int >(Image< int >&, string, int, int);
template void LocalOp< byte >(Image< byte >&, string, int, int);
template void LocalOp< float >(Image< float >&, string, int, int);
template void LocalOp< int >(Image< int >&, string, int);
template void LocalOp< byte >(Image< byte >&, string, int);
template void LocalOp< float >(Image< float >&, string, int);

template int   compute_mean< int >  (Image< int >&,   int, int, int, int);
template byte  compute_mean< byte > (Image< byte >&,  int, int, int, int);
template float compute_mean< float >(Image< float >&, int, int, int, int);
template int   compute_median< int >  (Image< int >&,   int, int, int, int);
template byte  compute_median< byte > (Image< byte >&,  int, int, int, int);
template float compute_median< float >(Image< float >&, int, int, int, int);
template int   compute_stdev< int >  (Image< int >&,   int, int, int, int);
template byte  compute_stdev< byte > (Image< byte >&,  int, int, int, int);
template float compute_stdev< float >(Image< float >&, int, int, int, int);
template int   compute_var< int >  (Image< int >&,   int, int, int, int);
template byte  compute_var< byte > (Image< byte >&,  int, int, int, int);
template float compute_var< float >(Image< float >&, int, int, int, int);
template int   compute_min< int >  (Image< int >&,   int, int, int, int);
template byte  compute_min< byte > (Image< byte >&,  int, int, int, int);
template float compute_min< float >(Image< float >&, int, int, int, int);
template int   compute_max< int >  (Image< int >&,   int, int, int, int);
template byte  compute_max< byte > (Image< byte >&,  int, int, int, int);
template float compute_max< float >(Image< float >&, int, int, int, int);

template< class T > void LocalOp(Image< T >& input, string op_type, int w_x, int w_y, Image< T >& output)
{
  int i, j, i_min, i_max, j_min, j_max, wx, wy;
  Image< T > res;
  res = input;
  wx = (int)floor(w_x/2.0);
  wy = (int)floor(w_y/2.0);
  for (i = 0; i < input.getHeight(); i++) {
    i_min = (i - wy) > 0 ? i - wy : 0;
    i_max = (i_min+w_y-1) < input.getHeight() ? i_min+w_y-1 : input.getHeight()-1;
    for (j = 0; j < input.getWidth(); j++){
      j_min = (j - wx) > 0 ? j - wx : 0;
      j_max = (j_min+w_x-1) < input.getWidth() ? j_min+w_x-1 : input.getWidth()-1;
      if (op_type == "mean")   res[i][j] = compute_mean(input, i_min, i_max, j_min, j_max);
      if (op_type == "median") res[i][j] = compute_median(input, i_min, i_max, j_min, j_max);
      if (op_type == "stdev")  res[i][j] = compute_stdev(input, i_min, i_max, j_min, j_max);
      if (op_type == "var")    res[i][j] = compute_var(input, i_min, i_max, j_min, j_max);
      if (op_type == "min")    res[i][j] = compute_min(input, i_min, i_max, j_min, j_max);
      if (op_type == "max")    res[i][j] = compute_max(input, i_min, i_max, j_min, j_max);
    }
  }
  output = res;
}

template< class T > void LocalOp(Image< T >& input, string op_type, int w, Image< T >& output)
{
  LocalOp(input, op_type, w, w, output);
}

template< class T > void LocalOp(Image< T >& input, string op_type, int w_x, int w_y)
{
  LocalOp(input, op_type, w_x, w_y, input);
}

template< class T > void LocalOp(Image< T >& input, string op_type, int w)
{
  LocalOp(input, op_type, w, w, input);
}

template<class T> T compute_mean(Image< T >& input, int y_min, int y_max, int x_min, int x_max)
{
  double res;
  res = 0;
  for (int i = y_min; i <= y_max; i++)
    for (int j = x_min; j <= x_max; j++)
      res += input[i][j];
  res /= ((y_max-y_min+1)*(x_max-x_min+1));
  return (T)res;
}

template<class T> T compute_median(Image< T >& input, int y_min, int y_max, int x_min, int x_max)
{
  vector<T> pixels;
  for (int i = y_min; i <= y_max; i++)
    for (int j = x_min; j <= x_max; j++)
      pixels.push_back(input[i][j]);
  sort(pixels.begin(), pixels.end());
  if (pixels.size()% 2 == 1) return pixels[(pixels.size()-1)/2];
  else return (T)((pixels[pixels.size()/2]+pixels[pixels.size()/2-1])/2.0);
}

template<class T> T compute_stdev(Image< T >& input, int y_min, int y_max, int x_min, int x_max)
{
  double res, mu;
  int N;
  N = (y_max-y_min+1)*(x_max-x_min+1);
  res = 0;
  mu = 0;
  for (int i = y_min; i <= y_max; i++)
    for (int j = x_min; j <= x_max; j++) {
      res += input[i][j]*input[i][j];
      mu += input[i][j];
    }
  return (T)sqrt((res-mu)/N);
}

template<class T> T compute_var(Image< T >& input, int y_min, int y_max, int x_min, int x_max)
{
  double res, mu;
  int N;
  N = (y_max-y_min+1)*(x_max-x_min+1);
  res = 0;
  mu = 0;
  for (int i = y_min; i <= y_max; i++)
    for (int j = x_min; j <= x_max; j++) {
      res += input[i][j]*input[i][j];
      mu += input[i][j];
    }
  return (T)((res-mu)/N);
}

template<class T> T compute_min(Image< T >& input, int y_min, int y_max, int x_min, int x_max)
{
  T res;
  res = input[y_min][x_min];
  for (int i = y_min; i <= y_max; i++)
    for (int j = x_min; j <= x_max; j++)
      res = res > input[i][j] ? input[i][j] : res;
  return res;
}

template<class T> T compute_max(Image< T >& input, int y_min, int y_max, int x_min, int x_max)
{
  T res;
  res = input[y_min][x_min];
  for (int i = y_min; i <= y_max; i++)
    for (int j = x_min; j <= x_max; j++)
      res = res < input[i][j] ? input[i][j] : res;
  return res;
}








