/* Copyright (c) 2001 A. Jalba */

#include "Morphop.h"

template 
void Erosion3x3< int >(Image< int > &input, int iter, CONN_TYPE connect, Image< int > &output);
template 
void Erosion3x3< byte >(Image< byte > &input, int iter, CONN_TYPE connect, Image< byte > &output);
template 
void Dilation3x3< int >(Image< int > &input, int iter, CONN_TYPE connect, Image< int > &output);
template 
void Dilation3x3< byte >(Image< byte > &input, int iter, CONN_TYPE connect, Image< byte > &output);
template
void Opening3x3< int >(Image< int > &input, int iter, CONN_TYPE connect, Image< int > &output);
template
void Opening3x3< byte >(Image< byte > &input, int iter, CONN_TYPE connect, Image< byte > &output);
template
void Closing3x3< int >(Image< int > &input, int iter, CONN_TYPE connect, Image< int > &output);
template
void Closing3x3< byte >(Image< byte > &input, int iter, CONN_TYPE connect, Image< byte > &output);

template
void FlatErosion< int >(Image< int > &input, ByteKernel &kern, Image< int > &output);
template
void FlatErosion< byte >(Image< byte > &input, ByteKernel &kern, Image< byte > &output);
template
void FlatDilation< int >(Image< int > &input, ByteKernel &kern, Image< int > &output);
template
void FlatDilation< byte >(Image< byte > &input, ByteKernel &kern, Image< byte > &output);
template
void FlatOpening< int >(Image< int > &input, ByteKernel &kern, Image< int > &output);
template
void FlatOpening< byte >(Image< byte > &input, ByteKernel &kern, Image< byte > &output);
template
void FlatClosing< int >(Image< int > &input, ByteKernel &kern, Image< int > &output);
template
void FlatClosing< byte >(Image< byte > &input, ByteKernel &kern, Image< byte > &output);

template
void Dilation< int >(Image< int > &input, ByteKernel &kern, Image< int > &output);
template
void Dilation< byte >(Image< byte > &input, ByteKernel &kern, Image< byte > &output);
template
void Erosion< int >(Image< int > &input, ByteKernel &kern, Image< int > &output);
template
void Erosion< byte >(Image< byte > &input, ByteKernel &kern, Image< byte > &output);
template
void Opening< int >(Image< int > &input, ByteKernel &kern, Image< int > &output);
template
void Opening< byte >(Image< byte > &input, ByteKernel &kern, Image< byte > &output);
template
void Closing< int >(Image< int > &input, ByteKernel &kern, Image< int > &output);
template
void Closing< byte >(Image< byte > &input, ByteKernel &kern, Image< byte > &output);

template
void TopHat< int >(Image< int > &input, ByteKernel &kern, Image< int > &output);
template
void TopHat< byte >(Image< byte > &input, ByteKernel &kern, Image< byte > &output);
template
void BottomHat< int >(Image< int > &input, ByteKernel &kern, Image< int > &output);
template
void BottomHat< byte >(Image< byte > &input, ByteKernel &kern, Image< byte > &output);

template
void HitOrMiss< int >(Image< int > &input, ByteKernel &kern1, 
		      ByteKernel &kern2, Image< int > &output);
template
void HitOrMiss< byte >(Image< byte > &input, ByteKernel &kern1, 
		      ByteKernel &kern2, Image< byte > &output);
template
void TemplateMatch< int >(Image< int > &input, ByteKernel &kern, Image< int > &output);
template
void TemplateMatch< byte >(Image< byte > &input, ByteKernel &kern, Image< byte > &output);

template< class T > static 
void Erosion3x3(Image< T > &input, CONN_TYPE connect, Image< T > &output);
template < class T > static 
void Dilation3x3(Image< T > &input, CONN_TYPE connect, Image< T > &output);


template< class T > static 
void Erosion3x3(Image< T > &input, CONN_TYPE connect, Image< T > &output)
{
  int h=input.getHeight(), w=input.getWidth();
  
  for(int i=0;i<h;i++)
    for(int j=0;j<w;j++) {
      int val=OFF;
      if(input[i][j]!=OFF) {
	val=ON;
	for(int k=-1;k<=1;k++) 
	  for(int m=-1;m<=1;m++) 
	    if(RANGE(i+k, j+m, h, w)) 
	      if(((connect==FOUR) && (((k==0) && (m!=0)) || 
		  ((k!=0) && (m==0)))) || (connect==EIGHT)) { 
		if(input[i+k][j+m]==OFF) { 
		  val=OFF; k=1; break;
		}
	      }
      }
      output[i][j]=val;
    }
}

template< class T >
void Erosion3x3(Image< T > &input, int iter, CONN_TYPE connect, Image< T > &output)
{
  Image< T >tmp(input);

  output = tmp;

  Image< T > *pr=&tmp, *pw=&output, *tmpp;

  for(int i=0;i<iter;i++) {
    Erosion3x3(*pr, connect, *pw);

    tmpp=pr;
    pr=pw;
    pw=tmpp;
  }

  output=tmp;
}

template< class T > static 
void Dilation3x3(Image< T >&input, CONN_TYPE connect, Image< T >&output)
{
  int h=input.getHeight(), w=input.getWidth();
  
  for(int i=0;i<h;i++)
    for(int j=0;j<w;j++) {
      int val=ON;
      if(input[i][j]==OFF) {
	val=OFF;
	for(int k=-1;k<=1;k++) 
	  for(int m=-1;m<=1;m++) 
	    if(RANGE(i+k, j+m, h, w)) 
	      if(((connect==FOUR) && (((k==0) && (m!=0)) || 
		  ((k!=0) && (m==0)))) || (connect==EIGHT)) { 
		if(input[i+k][j+m]!=OFF) { 
		  val=ON; k=1; break;
		}
	      }
      }
      output[i][j]=val;
    }
}

template< class T >
void Dilation3x3(Image< T > &input, int iter, CONN_TYPE connect, Image< T > &output)
{
  Image< T >tmp(input);

  output = tmp;

  Image< T >*pr=&tmp, *pw=&output, *tmpp;

  for(int i=0;i<iter;i++) {
    Dilation3x3(*pr, connect, *pw);
 
    tmpp=pr;
    pr=pw;
    pw=tmpp;
  }

  output=tmp;
}

template< class T >
void Opening3x3(Image< T > &input, int iter, CONN_TYPE connect, Image< T > &output)
{
  Erosion3x3< T >(input, iter, connect, output);
  Dilation3x3< T >(output, iter, connect, output);
}

template< class T >
void Closing3x3(Image< T > &input, int iter, CONN_TYPE connect, Image< T > &output)
{
  Dilation3x3< T >(input, iter, connect, output);
  Erosion3x3< T >(output, iter, connect, output);
}

template< class T >
void FlatErosion(Image< T > &input, ByteKernel &kern, Image< T > &output)
{
  int h=input.getHeight(), w=input.getWidth();
  int hk=kern.getHeight(), wk=kern.getWidth();
  int hk2=hk>>1, wk2=wk>>1;

  if(!((hk & 1) && (wk & 1))) {
    ERROR("Structuring element must have odd size !")
    return;
  }  
  
  Image< T > tmp(input);

  for(int i=0;i<h;i++)
    for(int j=0;j<w;j++) {
      int min=ON;
      for(int k=0;k<hk;k++)
	for(int m=0;m<wk;m++) {
	  int posx=i+k-hk2;
	  int posy=j+m-wk2;
	  if(RANGE(posx, posy, h, w) && (kern[k][m]>OFF))
	    if(input[posx][posy]<min) min=input[posx][posy];
	}
      tmp[i][j]=min;
    }
  output=tmp;
}

template< class T > 
void FlatDilation(Image< T > &input, ByteKernel &kern, Image< T > &output)
{
  int h=input.getHeight(), w=input.getWidth();
  int hk=kern.getHeight(), wk=kern.getWidth();
  int hk2=hk>>1, wk2=wk>>1;

  if(!((hk & 1) && (wk & 1))) {
    ERROR("Structuring element must have odd size !")
    return;
  }  

  Image< T > tmp(input);

  for(int i=0;i<h;i++)
    for(int j=0;j<w;j++) {
      int max=OFF;
      for(int k=0;k<hk;k++)
	for(int m=0;m<wk;m++) {
	  int posx=i+k-hk2;
	  int posy=j+m-wk2;
	  if(RANGE(posx, posy, h, w) && (kern[k][m]>OFF))
	    if(input[posx][posy]>max) max=input[posx][posy];
	}
      tmp[i][j]=max;
    }
  output=tmp;
}

template< class T >
void FlatOpening(Image< T > &input, ByteKernel &kern, Image< T > &output)
{
  FlatErosion< T >(input, kern, output);
  FlatDilation< T >(output, kern, output);
}

template< class T >
void FlatClosing(Image< T > &input, ByteKernel &kern, Image< T > &output)
{
  FlatDilation< T >(input, kern, output);
  FlatErosion< T >(output, kern, output);
}

template< class T > 
void Erosion(Image< T > &input, ByteKernel &kern, Image< T > &output)
{
  int h=input.getHeight(), w=input.getWidth();
  int hk=kern.getHeight(), wk=kern.getWidth();
  int hk2=hk>>1, wk2=wk>>1;

  if(!((hk & 1) && (wk & 1))) {
    ERROR("Structuring element must have odd size !")
    return;
  }  
  
  Image< T > tmp(input);

  for(int i=0;i<h;i++)
    for(int j=0;j<w;j++) {
      int min=ON;
      for(int k=0;k<hk;k++)
	for(int m=0;m<wk;m++) {
	  int posx=i+k-hk2;
	  int posy=j+m-wk2;
	  if(RANGE(posx, posy, h, w)) {
	    int val=input[posx][posy]-kern[k][m];
	    if(val<min) min=val;
	  }
	}
      tmp[i][j]=min;
    }
  output=tmp;
}

template< class T > 
void Dilation(Image< T > &input, ByteKernel &kern, Image< T > &output)
{
  int h=input.getHeight(), w=input.getWidth();
  int hk=kern.getHeight(), wk=kern.getWidth();
  int hk2=hk>>1, wk2=wk>>1;

  if(!((hk & 1) && (wk & 1))) {
    ERROR("Structuring element must have odd size !")
    return;
  }  

  Image< T > tmp(input);

  for(int i=0;i<h;i++)
    for(int j=0;j<w;j++) {
      int max=OFF;
      for(int k=0;k<hk;k++)
	for(int m=0;m<wk;m++) {
	  int posx=i+k-hk2;
	  int posy=j+m-wk2;
	  if(RANGE(posx, posy, h, w)) {
	    int val=input[posx][posy]+kern[k][m];
	    if(val>max) max=val;
	  }
	}
      tmp[i][j]=max;
    }
  output=tmp;
}

template< class T >
void Opening(Image< T > &input, ByteKernel &kern, Image< T > &output)
{
  Erosion< T >(input, kern, output);
  Dilation< T >(output, kern, output);
}

template< class T >
void Closing(Image< T > &input, ByteKernel &kern, Image< T > &output)
{
  Dilation< T >(input, kern, output);
  Erosion< T >(output, kern, output);
}

template< class T >
void TopHat(Image< T > &input, ByteKernel &kern, Image< T > &output)
{
  Image< T > tmp(input);

  Opening< T >(input, kern, output);

  tmp-=output;

  output = tmp; 
}

template< class T >
void BottomHat(Image< T > &input, ByteKernel &kern, Image< T > &output)
{
  Image< T > tmp(input);

  Closing< T >(input, kern, output);

  tmp-=output;

  output = tmp; 
}

template< class T >
void HitOrMiss(Image< T > &input, ByteKernel &kern1, ByteKernel &kern2, Image< T > &output)
{
  Image< T > compl_in(input);

  compl_in=input.max();
  compl_in-=input;

  FlatErosion(input, kern1, output);
  FlatErosion(compl_in, kern2, compl_in);

  //output&=compl_in;
  for(int i=0;i<output.getSize();i++) 
    output[0][i] &= compl_in[0][i];
}

template< class T >
void TemplateMatch(Image< T > &input, ByteKernel &kern, Image< T > &output)
{
  ByteKernel compl_k(kern);
  
  compl_k.clearImage(kern.max());
  compl_k-=kern;

  HitOrMiss(input, kern, compl_k, output);
}





