/* Copyright (c) 2001 A. Jalba */

#include <queue>
#include <stdlib.h>

#include "Watershed.h"

static const int MAXGREYVAL = 256;

static void sortPixels(ByteImage &im, int *sorted_pixels, int *hist)
{
  int i, s, *index[MAXGREYVAL];
  byte *tmp;

  for (i=0, tmp=im[0]; i<im.getSize(); i++, tmp++) 
    hist[*tmp]++;

  s=0;
  for (i=0;i<MAXGREYVAL; i++)
    { 
       index[i] = sorted_pixels+s; 
       s+=hist[i];
    }

  for (i=0, tmp=im[0]; i<im.getSize(); i++, tmp++)
    *(index[*tmp]++) = i;
}

#define MASK -2
#define WSHED 0
#define INIT -1
#define RANGE(i, j, h, w) ( (i>=0) && (j>=0) && (i<h) && (j<w) )

void Watershed(ByteImage &im, CONN_TYPE connect, IntImage &out)
{
  int h, x, y, gval=0, pixel, cur_dist, cur_label=0, imsize = im.getSize(), *dist_im=0, *SortPixels=0, *hist=0;
  int *current, *out_im, width=im.getWidth(), height=im.getHeight();
  queue<int> q;

  try
    {
      dist_im = (int *)calloc(imsize, sizeof(int));
      Assert<Fatal_error>(dist_im);
      SortPixels = (int *)malloc(imsize * sizeof(int));
      Assert<Fatal_error>(SortPixels);
      hist = (int *)calloc(MAXGREYVAL, sizeof(int));
      Assert<Fatal_error>(hist);
    }
  catch(Fatal_error e) { e.print("Unable to allocate memory"); }

  out.makeImage(width, height);
  out.clearImage(INIT);
  out_im = out[0];

  sortPixels(im, SortPixels, hist);
	
  for(h=0;h<MAXGREYVAL;h++) {
    for(current=SortPixels+gval; current<SortPixels+gval+hist[h]; current++) {
      pixel = *current;
      out_im[pixel]=MASK;      
      x=pixel % width;
      y=pixel / width;      
      for(int k=-1;k<=1;k++)
	for(int m=-1;m<=1;m++) {
	  if(RANGE(y+k, x+m, height, width)) 
	    if(((connect==FOUR) && (((k==0) && (m!=0)) || ((k!=0) && (m==0)))) || 
	       ((connect==EIGHT) && ((k!=0) || (m!=0)) )) {
	      if((out[y+k][x+m]>0) || (out[y+k][x+m]==WSHED)) {
		q.push(pixel);
		dist_im[pixel]=1;
		k=1; break;
	      }
	    }
	}
    }      

    cur_dist=1;
    q.push(-1);

#define POP { pixel=q.front(); q.pop(); }
      
    for(;;) {
      POP
      if(pixel==-1) {
	if(q.empty()) break;
	else {
	  q.push(-1);
	  cur_dist++;
	  POP
	}
      }
      x=pixel % width;
      y=pixel / width;
      for(int k=-1;k<=1;k++)
	for(int m=-1;m<=1;m++) {
	  if(RANGE(y+k, x+m, height, width)) 
	    if(((connect==FOUR) && (((k==0) && (m!=0)) || ((k!=0) && (m==0)))) || 
	       ((connect==EIGHT) && ((k!=0) || (m!=0)) )) {
	      int neigh = (y+k)*width + (x+m);
	      if((dist_im[neigh]<cur_dist) && ((out_im[neigh]>0) || (out_im[neigh]==WSHED)) ) { 
		if(out_im[neigh]>0) {
		  if((out_im[pixel]==MASK) || (out_im[pixel]==WSHED))
		    out_im[pixel]=out_im[neigh];
		  else if(out_im[pixel]!=out_im[neigh])
		    out_im[pixel]=WSHED;
		}
		else if(out_im[pixel]==MASK) out_im[pixel]=WSHED;
	      }
	      else if((out_im[neigh]==MASK) && (dist_im[neigh]==0)) {
		dist_im[neigh]=cur_dist+1;
		q.push(neigh);
	      }
	    }
	  }
    }  // endfor(;;)	    	       

    for(current=SortPixels+gval; current<SortPixels+gval+hist[h]; current++) {
      pixel = *current;
      dist_im[pixel]=0;	
      if(out_im[pixel]==MASK) {
	cur_label++;
	q.push(pixel);
	out_im[pixel]=cur_label;
	while(!q.empty()) {
	  POP
	  x=pixel % width;
	  y=pixel / width;
	  for(int k=-1;k<=1;k++)
	    for(int m=-1;m<=1;m++) {
	      if(RANGE(y+k, x+m, height, width)) 
		if(((connect==FOUR) && (((k==0) && (m!=0)) || ((k!=0) && (m==0)))) || 
		   ((connect==EIGHT) && ((k!=0) || (m!=0)) )) {
		  int neigh = (y+k)*width + (x+m);
		  if(out_im[neigh]==MASK) {
		    q.push(neigh);
		    out_im[neigh]=cur_label;
		  }
		}
	    }
	}
      }
    }
    gval+=hist[h];
  }  //endfor h    	    	

  for(y=0;y<height;y++)
    for(x=0;x<width;x++) {
      if(out[y][x]>0) {
	for(int k=-1;k<=1;k++)
	  for(int m=-1;m<=1;m++) {
	    if(RANGE(y+k, x+m, height, width)) 
	      if(((connect==FOUR) && (((k==0) && (m!=0)) || ((k!=0) && (m==0)))) || 
		   ((connect==EIGHT) && ((k!=0) || (m!=0)) )) {
		if((out[y][x]>out[y+k][x+m]) && (out[y+k][x+m]>0)) {
		  out[y][x]=WSHED;
		  k=1; break;
		}
	      }
	  }
      }
    }

  free(dist_im);
  free(SortPixels);
  free(hist);
}		





