/* Copyright (c) 2001 S.E. Grigorescu */

#include "Zooming.h"

template void Zoom< int >(Image< int >&, float, float, Image< int >&);
template void Zoom< byte >(Image< byte >&, float, float, Image< byte >&);
template void Zoom< float >(Image< float >&, float, float, Image< float >&);
template void Zoom< int >(Image< int >&, float, Image< int >&);
template void Zoom< byte >(Image< byte >&, float, Image< byte >&);
template void Zoom< float >(Image< float >&, float, Image< float >&);
template void Zoom< int >(Image< int >&, float, float);
template void Zoom< byte >(Image< byte >&, float, float);
template void Zoom< float >(Image< float >&, float, float);
template void Zoom< int >(Image< int >&, float);
template void Zoom< byte >(Image< byte >&, float);
template void Zoom< float >(Image< float >&, float);
 
template< class T > void Zoom(Image< T >& input, float xf, float yf, Image< T >& output)
{
  Image< T > res((int)rint(xf*input.getWidth()), (int)rint(yf*input.getHeight()), 
		 output.getName());
  double alfa, beta;
  double xo, yo;
  int ir, jr;

  for (int i = 0; i < res.getHeight(); i++)
    for (int j = 0; j < res.getWidth(); j++){
      xo = ((double)j)/xf; yo = ((double)i)/yf;
      alfa = xo - floor(xo);
      beta = yo - floor(yo);
      jr = (int)floor(xo); ir = (int)floor(yo);
      if (jr >= 0 && jr < input.getWidth()-1 && ir >= 0 && ir < input.getHeight()-1)
	res[i][j] = (T)((1-alfa)*(1-beta)*input[ir][jr] + alfa*(1-beta)*input[ir][jr+1] + 
	  (1-alfa)*beta*input[ir+1][jr] + alfa*beta*input[ir+1][jr+1]);      
      if (jr == -1 && jr < input.getWidth()-1 && ir >= 0 && ir < input.getHeight()-1)
	res[i][j] = (T)((1-beta)*input[ir][jr+1] + beta*input[ir+1][jr+1]);      
      if (jr >= 0 && jr == input.getWidth()-1 && ir >= 0 && ir < input.getHeight()-1)
	res[i][j] = (T)((1-beta)*input[ir][jr] + beta*input[ir+1][jr]);      
      if (jr >= 0 && jr < input.getWidth()-1 && ir == -1 && ir < input.getHeight()-1)
	res[i][j] = (T)((1-alfa)*input[ir+1][jr] + alfa*input[ir+1][jr+1]);      
      if (jr >= 0 && jr < input.getWidth()-1 && ir >= 0 && ir == input.getHeight()-1)
	res[i][j] = (T)((1-alfa)*input[ir][jr] + alfa*input[ir][jr+1]);
    }
  output = res;
}

template< class T > void Zoom(Image< T >& input, float xyf, Image< T >& output)
{
  Zoom(input, xyf, xyf, output);
}

template< class T > void Zoom(Image< T >& input, float xf, float yf)
{
  Zoom(input, xf, yf, input);
}

template< class T > void Zoom(Image< T >& input, float xyf)
{
  Zoom(input, xyf, xyf, input);
}





