/* Copyright (c) 2001 C. Grigorescu */

#include <stdio.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#include <stdlib.h>
#include <iostream.h>
#include "Timer.h"
#include "misc.h"
 
void Timer::Start()
{
  if (running) return; 
  int e = gettimeofday(&start,NULL);
  if (e == 0) 
    running = 1;
  else 
    ERROR("Cannot retrieve system time");
}

void Timer::Stop()
{
  if (!running) return;
  int e = gettimeofday(&stop, NULL);
  if (e == 0) {
    running = 0;

    if (start.tv_usec > stop.tv_usec) {
      stop.tv_usec += 1000000;
      stop.tv_sec--;
    }
    seconds      = stop.tv_sec  - start.tv_sec;
    microseconds = stop.tv_usec - start.tv_usec;
  }
  else 
    ERROR("Cannot retrieve system time");
}

void Timer::Tick()
{
  if (!running)
    return;

  int e = gettimeofday(&stop, NULL);
  if (e == 0) {
    if (start.tv_usec > stop.tv_usec) {
      stop.tv_usec += 1000000;
      stop.tv_sec--;
    }
    seconds      = stop.tv_sec  - start.tv_sec;
    microseconds = stop.tv_usec - start.tv_usec;
    
    cout.fill('0');
    cout << "Elapsed time : \t" << seconds << ".";
    cout.width(3);
    cout << microseconds/1000 << " sec. " << endl;
  }
  else 
    ERROR("Cannot retrieve system time");
}

void Timer::Print()
{  
  cout.fill('0');
  cout << "Elapsed time : \t" << seconds << ".";
  cout.width(3);
  cout << microseconds/1000 << " sec. " << endl;
}

float Timer::getTime()
{
  if (!running)
    return 0;

  int e = gettimeofday(&stop, NULL);

  if (e == 0) {
    if (start.tv_usec > stop.tv_usec) {
      stop.tv_usec += 1000000;
      stop.tv_sec--;
    }
    seconds      = stop.tv_sec  - start.tv_sec;
    microseconds = stop.tv_usec - start.tv_usec;
  
    return (seconds + (float)(microseconds)/1000000.0);
  }
  else
    return 0;
}

void MemoryStatistics()
{
  rusage res_usage;
  int e = getrusage(RUSAGE_SELF, &res_usage);
  if (e == 0) {
    long total = res_usage.ru_ixrss +  res_usage.ru_idrss +  res_usage.ru_isrss ;
    cout << "Memory used: " << total << "\t from which: " << endl;
    cout << "\t " << res_usage.ru_ixrss << " - shared " << endl;
    cout << "\t " << res_usage.ru_idrss << " - unshared " << endl;
    cout << "\t " << res_usage.ru_isrss << " - stack " << endl;
    cout << "\t " << res_usage.ru_maxrss << " -  " << endl;
  }
  else
    ERROR("Cannot retrieve memory statistics");
}
