/* Copyright (c) 2001 C. Grigorescu S.E. Grigorescu */

#include "ByteImage.h"

ByteImage &ByteImage::operator&=(const ByteImage &img)
{
  if((width!=img.width) || (height!=img.height)) { 
    ERROR("The images have different sizes"); 
  }
    
  try { 
    Assert<Fatal_error>(img.pixels); 
  }
  catch(Fatal_error e) {e.print("Invalid data");}

  for(int i=0;i<getSize();i++)
    if ((pixels[0][i] == ON) && (img.pixels[0][i] == ON))
      pixels[0][i] = ON;
    else
      pixels[0][i] = OFF;

  return *this;
}

ByteImage &ByteImage::operator|=(const ByteImage &img)
{
  if((width!=img.width) || (height!=img.height)) { 
    ERROR("The images have different sizes"); 
  }
    
  try { 
    Assert<Fatal_error>(img.pixels); 
  }
  catch(Fatal_error e) {e.print("Invalid data");}

  for(int i=0;i<getSize();i++)
    if ((pixels[0][i] == OFF) && (img.pixels[0][i] == OFF))
      pixels[0][i] = OFF;
    else
      pixels[0][i] = ON;

  return *this;
}

ByteImage &ByteImage::operator^=(const ByteImage &img)
{
  if((width!=img.width) || (height!=img.height)) { 
    ERROR("The images have different sizes"); 
  }
    
  try { 
    Assert<Fatal_error>(img.pixels); 
  }
  catch(Fatal_error e) {e.print("Invalid data");}

  for(int i=0;i<getSize();i++)
    if (((pixels[0][i] == OFF) && (img.pixels[0][i] == OFF)) ||
	((pixels[0][i] == ON) && (img.pixels[0][i] == ON)))
      pixels[0][i] = OFF;
    else
      pixels[0][i] = ON;
  
  return *this;
}


ByteImage &ByteImage::operator!(void)
{
  for(int i=0;i<getSize();i++)
    if (pixels[0][i] == OFF)
      pixels[0][i] = ON;
    else
      pixels[0][i] = OFF;
  return (*this);
}

ByteImage &ByteImage::operator=(Image< byte >& img)
{
  width = img.getWidth();
  height = img.getHeight();
  setName(img.getName());
  if (img.getWidth()*img.getHeight() != 0)
    setPixels(img.getWidth(), img.getHeight(), img.getPixels());
  else
    pixels = NULL;
  return *this;
}

// Commented because of memory leaks (Stroustroup p.282)
//
//  ByteImage ByteImage::operator&(const ByteImage &img)
//  {
//    if((width!=img.width) || (height!=img.height)) { 
//      ERROR("The images have different sizes"); 
//    }
    
//    try { 
//      Assert<Fatal_error>(img.pixels); 
//    }
//    catch(Fatal_error e) {e.print("Invalid data");}

//    ByteImage rez("rez");
//    rez.makeImage(width, height);

//    for(int i=0;i<getSize();i++)
//      rez.pixels[0][i] = pixels[0][i] & img.pixels[0][i];

//    return rez;
//  }

//  ByteImage ByteImage::operator|(const ByteImage &img)
//  {
//    if(*this!=img) { ERROR("The images have different sizes"); }
    
//    try { Assert<Fatal_error>(img.pixels); }
//    catch(Fatal_error e) {e.print();}

//    ByteImage rez("rez");
//    rez.makeImage(width, height);

//    for(int i=0;i<getSize();i++)
//      rez.pixels[0][i] = pixels[0][i] | img.pixels[0][i];

//    return rez;
//  }




