/* Copyright (c) 2001 C. Grigorescu */

#include <math.h>
#include <fstream.h>
#include "imlib_io.h"
#include "FloatKernel.h"

void FloatKernel::makeRectangularPulse(float max_amplitude, int support)
{
  int i, j, pos_x, pos_y, x_coord, y_coord, index; 
  int env_x, env_y;

  kernel_type = RECTANGULAR_PULSE;
  for (j=0; j<height; j++)
    for (i=0; i<width; i++)
      pixels[0][j*width+i] = 0.0;

  pos_x = (int)(width/2+width%2);
  pos_y = (int)(height/2+height%2);

  if (support > width/2) support = width/2;
  if (support > height/2) support = height/2;

  env_x = (int)(support/2 + support%2);    
  env_y = (int)(support/2 + support%2);    
  for (j=-env_y;j<env_y;j++)
    for (i=-env_x;i<env_x;i++) {
      x_coord = pos_x + i;
      y_coord = pos_y + j;
      if (y_coord >= height)
	y_coord =  pos_y + j - height;
      if (y_coord < 0)
	y_coord = pos_y + j + height;
      index = (int)(y_coord*width + x_coord);
      if ((index >=0) && (index < width*height))
	if ((i == -env_x) || (i == env_x-1) || (j == -env_y) || (j == env_y-1))
	  pixels[0][index] = max_amplitude/2;
	else
	  pixels[0][index] = max_amplitude;
    }
}

void FloatKernel::makeTriangularPulse(float max_amplitude, int support)
{
  int i, j, pos_x, pos_y, x_coord, y_coord, index; 
  int env_x, env_y;
  float val_x, val_y;

  kernel_type = TRIANGULAR_PULSE;
  for (j=0; j<height; j++)
    for (i=0; i<width; i++)
      pixels[0][j*width+i] = 0.0;

  pos_x = (int)(width/2+width%2);
  pos_y = (int)(height/2+height%2);

  if (support > width/2) support = width/2;
  if (support > height/2) support = height/2;

  env_x = (int)(support/2 + support%2);    
  env_y = (int)(support/2 + support%2);    
  for (j=-env_y;j<=0;j++)
    for (i=-env_x;i<=0;i++) {
      x_coord = pos_x + i - 1;
      y_coord = pos_y + j - 1;
      val_x = (float)(env_x+i)/(float)(env_x);
      val_y = (float)(env_y+j)/(float)(env_y);
      if (y_coord >= height)
	y_coord =  pos_y + j - height;
      if (y_coord < 0)
	y_coord = pos_y + j + height;
      index = (int)(y_coord*width + x_coord);
      if ((index >=0) && (index < width*height)) {
  	pixels[0][index] = max_amplitude * MIN(val_x,val_y);
      }      
    }
  for (j=0;j<env_y;j++)
    for (i=-env_x;i<=0;i++) {
      x_coord = pos_x + i - 1;
      y_coord = pos_y + j;
      val_x = (float)(env_x+i)/(float)(env_x);
      val_y = (float)(-env_y+j)/(float)(-env_y);
      if (y_coord >= height)
	y_coord =  pos_y + j - height;
      if (y_coord < 0)
	y_coord = pos_y + j + height;
      index = (int)(y_coord*width + x_coord);
      if ((index >=0) && (index < width*height)) {
  	pixels[0][index] = max_amplitude * MIN(val_x,val_y);
      }      
    }
  for (j=-env_y;j<=0;j++)
    for (i=0;i<env_x;i++) {
      x_coord = pos_x + i;
      y_coord = pos_y + j - 1;
      val_x = (float)(-env_x+i)/(float)(-env_x);
      val_y = (float)(env_y+j)/(float)(env_y);
      if (y_coord >= height)
	y_coord =  pos_y + j - height;
      if (y_coord < 0)
	y_coord = pos_y + j + height;
      index = (int)(y_coord*width + x_coord);
      if ((index >=0) && (index < width*height)) {
  	pixels[0][index] = max_amplitude * MIN(val_x,val_y);
      }      
    }
  for (j=0;j<env_y;j++)
    for (i=0;i<env_x;i++) {
      x_coord = pos_x + i;
      y_coord = pos_y + j;
      val_x = (float)(-env_x+i)/(float)(-env_x);
      val_y = (float)(-env_y+j)/(float)(-env_y);
      if (y_coord >= height)
	y_coord =  pos_y + j - height;
      if (y_coord < 0)
	y_coord = pos_y + j + height;
      index = (int)(y_coord*width + x_coord);
      if ((index >=0) && (index < width*height)) {
  	pixels[0][index] = max_amplitude * MIN(val_x,val_y);
      }      
    }
}

void FloatKernel::makeGauss(float sigma)
{
  kernel_type = GAUSS;
  makeGauss(sigma,1.0,0.0);
}

void FloatKernel::makeGauss(float sigma, float aspect, float orientation)
{
  kernel_type = GAUSS;
  int i, j, index, x_coord, y_coord;
  int pos_x, pos_y, env_x, env_y;
  float co;
  float cth, sth, x1, y1;
  float gauss;
 
  find_limits(sigma, aspect, orientation);
 
   /* initialization */
  for (j=0; j<height; j++)
    for (i=0; i<width; i++)
      pixels[0][j*width+i] = 0.0;

  env_x = (int)(h_m_x);
  env_y = (int)(h_m_y);
  pos_x = (int)(width/2+width%2);
  pos_y = (int)(height/2);
  
  cth = cos(PI*orientation/180.0);
  sth = sin(PI*orientation/180.0);
  co = 1/(sqrt(2*PI)*sigma);
  
  for (j=0; j<height; j++)
    for (i=0; i<width; i++)
      pixels[0][j*width+i] = 0.0;
  
  for (j=-env_y;j<env_y;j++)
    for (i=-env_x;i<env_x;i++) {
      x1 = i*cth - j*sth;
      y1 = j*cth + i*sth;
      gauss = co*exp((-(x1*x1)-(aspect*y1)*(aspect*y1))/(2*sigma*sigma));
      x_coord = pos_x + i;
      y_coord = pos_y + j;
      if (y_coord >= height)
	y_coord =  pos_y + j - height;
      if (y_coord < 0)
	y_coord = pos_y + j + height;
      index = (int)(y_coord*width + x_coord);
      if ((index >=0) && (index < width*height))
	pixels[0][index] = gauss;
    }                                                            
}

void FloatKernel::makeDoG(float sigma, float ratio)
{
  kernel_type = DOG;
  makeDoG(sigma, 1.0, 0.0, ratio);
}

void FloatKernel::makeDoG(float sigma, float aspect, float orientation, float ratio)

{
  int i, j, env_x, env_y, x_coord, y_coord, index;
  int pos_x, pos_y;
  float l1_norm, l2_norm, gauss1, gauss2, gauss;
  float co1, co2, cth, sth, x1, y1;
 
  kernel_type = DOG;
  find_limits(sigma, aspect, orientation);
 
  env_x = (int)(h_m_x);
  env_y = (int)(h_m_y);
  pos_x = (int)(width/2+width%2);
  pos_y = (int)(height/2);
 
  for (j=0; j<height; j++)
    for (i=0; i<width; i++)
      pixels[0][j*width+i] = 0.0;
 
  l1_norm = 0;
  l2_norm = 0;
 
  cth = cos(PI*orientation/180.0);
  sth = sin(PI*orientation/180.0);
  co1 = 1/(sqrt(2*PI)*sigma);
  co2 = 1/(sqrt(2*PI)*ratio*sigma);
 
  for (j=-env_y;j<env_y;j++)
    for (i=-env_x;i<env_x;i++) {
      x1 = i*cth - j*sth;
      y1 = j*cth + i*sth;
      gauss1 = co1 * exp((-(x1*x1)-(aspect*y1)*(aspect*y1))/(2*sigma*sigma));
      gauss2 = co2 * exp((-(x1*x1)-(aspect*y1)*(aspect*y1))/
			 (2*ratio*ratio*sigma*sigma));
      gauss = gauss1 - gauss2;
      x_coord = pos_x + i;
      y_coord = pos_y + j;
      if (y_coord >= height)
	y_coord = pos_y + j - height;
      if (y_coord < 0)
	y_coord = pos_y + j + height;
      index = (int)(y_coord*width + x_coord);
      if ((index >=0) && (index < width*height)) {
	pixels[0][index] = gauss;
	l1_norm += gauss;
	l2_norm += gauss*gauss;
      }
    }
  l1_norm = l1_norm/(width*height);
  l2_norm = sqrt(l2_norm);

  for (j=-env_y;j<env_y;j++)
    for (i=-env_x;i<env_x;i++) {
      index = (int)((pos_y+j)*width + pos_x+i);
      if ((index >=0) && (index < width*height)) {
	pixels[0][index] = pixels[0][index]-l1_norm;
	pixels[0][index] = pixels[0][index]/l2_norm;
      }
    }
}   

void FloatKernel::makeGabor(float sigma, float aspect, float orientation,
			      float lambda, float phase)
{
  int i, j, env_x, env_y, index, x_coord, y_coord;
  float cth, sth, ph, x1, y1;
  int pos_x, pos_y;
  float gauss;
  float l1_norm, l2_norm, co;
  
  kernel_type = GABOR;
  find_limits(sigma, aspect, orientation);
  
  env_x = (int)(h_m_x);
  env_y = (int)(h_m_y);
  pos_x = (int)(width/2+width%2);
  pos_y = (int)(height/2);
 
  l1_norm = 0.0;
  l2_norm = 0.0;
  
  cth = cos(PI*orientation/180.0);
  sth = sin(PI*orientation/180.0);
  co = 1/(sqrt(2*PI)*sigma);
 
   /* initialization */
  for (j=0; j<height; j++)
    for (i=0; i<width; i++)
      pixels[0][j*width+i] = 0.0;
  
  /* transform the phase in radians */
  ph = PI*phase/180.0;                                                                   

  for (j=-env_y;j<env_y;j++)
    for (i=-env_x;i<env_x;i++) {
      x1 = i*cth - j*sth;
      y1 = j*cth + i*sth;
      gauss = co*exp((-(x1*x1)-(aspect*y1)*(aspect*y1))/(2*sigma*sigma));
      x_coord = pos_x + i;
      y_coord = pos_y + j;
      if (y_coord >= height)
	y_coord =  pos_y + j - height;
      if (y_coord < 0)
	y_coord = pos_y + j + height;
      index = (int)(y_coord*width + x_coord);
      if ((index >=0) && (index < width*height)) {
	pixels[0][index] = gauss * cos(2*PI*x1/lambda + ph);
	l1_norm = l1_norm + pixels[0][index];
	l2_norm = l2_norm + pixels[0][index]*pixels[0][index];
      }
    }
  l1_norm = l1_norm/(width*height);
  l2_norm = sqrt(l2_norm);
  
  for (j=-env_y;j<env_y;j++)
    for (i=-env_x;i<env_x;i++) {
      index = (int)((pos_y+j)*width + pos_x+i);
      if ((index >=0) && (index < width*height)) {
	pixels[0][index] = pixels[0][index]-l1_norm;
	pixels[0][index] = pixels[0][index]/l2_norm;
      }
    }
}

void FloatKernel::makeLoG(float sigma)
{
  kernel_type = LOG;
  makeLoG(sigma,1.0,0.0);
}

void FloatKernel::makeLoG(float sigma, float aspect, float orientation)
{
  int i, j, env_x, env_y, index, x_coord, y_coord;
  float cth, sth, x1, y1;
  int pos_x, pos_y;
  float gauss;
  float l1_norm, l2_norm, co;

  kernel_type = LOG;
  find_limits(sigma, aspect, orientation);
  
  env_x = (int)(h_m_x);
  env_y = (int)(h_m_y);
  pos_x = (int)(width/2+width%2);
  pos_y = (int)(height/2);
  cth = cos(PI*orientation/180.0);
  sth = sin(PI*orientation/180.0);

  l1_norm = 0.0;
  l2_norm = 0.0;

  for (j=-env_y;j<env_y;j++)
    for (i=-env_x;i<env_x;i++) {
      x1 = i*cth - j*sth;
      y1 = j*cth + i*sth;
      co = 1-(x1*x1+y1*y1)/(2*sigma*sigma);
      gauss = co*exp((-(x1*x1)-(aspect*y1)*(aspect*y1))/(2*sigma*sigma));
      x_coord = pos_x + i;
      y_coord = pos_y + j;
      if (y_coord >= height)
	y_coord =  pos_y + j - height;
      if (y_coord < 0)
	y_coord = pos_y + j + height;
      index = (int)(y_coord*width + x_coord);
      if ((index >=0) && (index < width*height)) {
	pixels[0][index] = gauss;
	l1_norm = l1_norm + pixels[0][index];
	l2_norm = l2_norm + pixels[0][index]*pixels[0][index];
      }
    }
  l1_norm = l1_norm/(width*height);
  l2_norm = sqrt(l2_norm);
  
  for (j=-env_y;j<env_y;j++)
    for (i=-env_x;i<env_x;i++) {
      index = (int)((pos_y+j)*width + pos_x+i);
      if ((index >=0) && (index < width*height)) {
	pixels[0][index] = pixels[0][index]-l1_norm;
	pixels[0][index] = pixels[0][index]/l2_norm;
      }
    }
}

void FloatKernel::find_limits(float sigma, float aspect, float orientation)
{
  float proj_1_x, proj_2_x, proj_1_y, proj_2_y;
 
  h_m_x = 7 * sigma;
  h_m_y = 7 * (h_m_x)/aspect;
 
  if (orientation <= 90)
    {
    proj_1_x = h_m_x * cos(PI*orientation/180.0);
    proj_2_y = h_m_y * cos(PI*orientation/180.0);
    }
  else
    {
    proj_1_x = h_m_x * -cos(PI*orientation/180.0);
    proj_2_y = h_m_y * -cos(PI*orientation/180.0);
    }
 
  proj_2_x = h_m_x * sin(PI*orientation/180.0);
  proj_1_y = h_m_y * sin(PI*orientation/180.0);
 
  if (proj_1_x > proj_1_y)
    h_m_x = proj_1_x;
  else
    h_m_x = proj_1_y;
  if (proj_2_x > proj_2_y)
    h_m_y = proj_2_x;
  else
    h_m_y = proj_2_y;
 
  if (h_m_x >= width/2)
    h_m_x = width/2+width%2;
  if (h_m_y >= height/2)
    h_m_y = height/2+height%2;
}
