/* Copyright (c) 2001 C. Grigorescu */

#include <fstream.h>
#include <string.h>
#include "Kernel.h"

template class Kernel< byte >;
template class Kernel< int >;
template class Kernel< float >;

template< class T >
void Kernel< T >::readKernelCoeffs(char *name)
{
  if (typeid(T) == typeid(byte)) kernel_type = KERNEL_BYTE_UNKNOWN;
  if (typeid(T) == typeid(int)) kernel_type = KERNEL_INT_UNKNOWN;
  if (typeid(T) == typeid(float)) kernel_type = KERNEL_FLOAT_UNKNOWN;

  int w, h;
  ifstream in;
  char *comment[1024];
  in.open(name,ios::in);
  if (in == NULL) {
    cout << "\nError: Cannot read kernel coefficients from file: \"" << name 
	 <<"\"" << endl; 
    assert(0);
    return;
  }
  char c;
  in.get(c);

  while (c == '#') {
    in.unget();
    in.gets(comment);
    c = in.get();
  }
  in.unget();

  in >> h;                                                                        
  in >> w;
  makeImage(w,h);
  
  for (int j=0;j<height;j++)
    for (int i=0;i<width;i++)
      in >> pixels[j][i] ;
  
  in.close();                                                                       
}

template< class T >
void Kernel< T >::writeKernelCoeffs(char *name, char *comment)
{
  if (!pixels)
    throw Fatal_error ("Empty buffer to write to file!");

  ofstream out;
  out.open(name,ios::out);
  if (out == NULL)
    cout << "\nError: Cannot write kernel coefficients to file: \"" << name 
	 <<"\"" << endl;
  
  out << "# " << comment << endl;
  out << height << " ";
  out << width << endl;
  
  for (int j=0;j<height;j++) {
    for (int i=0;i<width;i++) 
      out << pixels[j][i] << " ";
    out << endl;
  }
  out.close(); 
}




