/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        benchmark.cpp
 * Description: Demo application recording a single object.
 * Created:     2001-09-02
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: benchmark.cpp,v $
 * Revision 1.3  2001/12/05 11:21:49  slange
 * - bug removed
 *
 * Revision 1.2  2001/12/05 11:17:39  slange
 * - resolving conflicts
 *
 * Revision 1.1  2001/09/02 18:30:27  slange
 * added benchmark routine
 *
 * --------------------------------------------------------------------
 */

#include "cvtkfacade.h"
#include "defaultfactory.h"
#include <iostream>
#include <fstream>
#include <unistd.h>
#include "cvtkexceptions.h"
#include "fileis.h"


using cvtk::CSingleImage;
using cvtk::CImageSource;
using cvtk::CPPM;
using cvtk::CObjectList;
using cvtk::CObject;
using cvtk::CSingleImage;

using cvtk::CCVTKFacadeFactory;
using cvtk::CCVTKFacade;
using cvtk::CDefaultFactory;


void printUsage(char* str)
{
  cout << "Usage: " << str << " lookuptable.lut templates.tmpl"
       << " image.ppm" << endl;
}

int main(int argc, char* argv[]) {
  if (argc!=4) {
    printUsage(argv[0]);
    exit(1);
  }

  CCVTKFacadeFactory* factory = new CDefaultFactory(argv[1], argv[2]); 
  CCVTKFacade* cvtk = factory->getCVTKFacade();
 
  CImageSource* imgSrc = new CSingleImage(argv[3]);

  CPPM in(imgSrc->getWidth(), imgSrc->getHeight());
  CObjectList output;
 
  in = imgSrc->getImage(in);
  cerr << "Start" << endl;
   
  for (int wdh=0; wdh < 1000; wdh++) {


    cvtk->analyse(&in, &output);
    if (wdh%100==0)
      cerr << wdh << endl;
    output.clear();
  }
  cerr << "Ready" << endl;
}







