#ifndef _calibratecontrolleruv_h_
#define _calibratecontrolleruv_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        calibratecontroller.h
 * Description: Qt Controller of the calibration tool. 
 * Created:     2001-09-04
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * ------------------------- Revision History -------------------------
 * $Log: calibratecontrolleruv.h,v $
 * Revision 1.1  2001/09/04 20:00:35  slange
 * - initial checkin of uvhistogram calibration classes
 *
 * --------------------------------------------------------------------
 */

#include <qapplication.h>
#include <fstream.h>
#include "imagesource.h"
#include "cvtk.h"
#include "kmsegmentation.h"
#include <qtimer.h>
#include "uvhistogram.h"

/* collects a vector of training examples x and a corresponding vector xC with
 * the class information of every x as needed by reinforced learning-/
 * segmentation-algorithms
 */
namespace cvtk {
#define SHIFTLOOKUP 2
  class CController : public QObject
  {
  Q_OBJECT
  public:
    explicit CController(CImageSource* imageSource, 
			 CRGBMap& mask, 
			 CReinforcedLearning<2>* learner);
    ~CController();
  public slots:
    void newObject();
    void addFrame();
    void stopExec();
    void segment();
    void makeLookup();
    void changeSpecFG(int);
    void changeSpecBG(int);
    void exec();                  // Thread execution function
  signals:
    void message(const char* str, int ms);
    void frameCountChanged(int n);
    void objectCountChanged(int n);
    void imageChanged(const CRGBMap& ppm);
  private:
    void insertHistInX();
    CImageSource* imageSource;
    CPPM ppm;
    CRGBMap mask;
    int nObj;
    int nFr;
    int specFG;
    int specBG;
    QTimer* timer;
    CController(const CController&);// no copies allowed
    CController& operator= (const CController&);

    vector<int> xC;              // class identification of every x
    vector<Vektor2Dim> x;        // collected examples x

    CReinforcedLearning<2>* learner;
    CUVHistogram* hist;
  };
}
  
#endif






