#ifndef _calibratewidget_h_
#define _calibratewidget_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        calibratewidget.h
 * Description: Qt widget of the calibration tool. 
 * Created:     2001-03-28
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://www.cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 *
 * -------------------------- Revision History -------------------------
 * $Log: calibratewidget.h,v $
 * Revision 1.6  2001/09/04 15:31:27  slange
 * - corrected bugs in:
 *   - templatecontroller.cpp: use QString to emit messages
 *   - monitor.cpp: used x instead of y to check against height
 * - labeled the sliders in calibratewidget
 *
 * Revision 1.5  2001/08/30 13:24:30  slange
 * - added sliders to effect the specificity of the kmsegmentation
 *
 * Revision 1.4  2001/04/29 00:55:49  slange
 * - finished the first complete draft of the calibration tool.
 *   - it now generates a lookup-table for values 1..64 (>> 2)
 *   - writes the entire table to stdout.
 * - fixes in k-means segmentation
 * - added button to generate lookup table in the calibrator-widget
 *
 * Revision 1.3  2001/04/28 18:05:45  slange
 * - removed patterngenerato.[h/cpp]
 * - added abstract interface CReinforcedLearning
 * - added CKMSegmentation
 *   - class definition and part of implementation
 * - added "train" button to calibratorwidget
 * - changed calibratecontroller to work with the new classes
 *
 * Revision 1.2  2001/04/23 18:42:52  slange
 * added copyright notice to all source files
 *
 * ---------------------------------------------------------------------
 */

#include "imagewidget.h"
#include <qapplication.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qslider.h>
#include <qstatusbar.h>

namespace cvtk {

  class CCalibrateWidget : public QWidget
  {
  Q_OBJECT
  public:
    CCalibrateWidget(const CRGBMap& image);
    CCalibrateWidget(int width, int height);
    ~CCalibrateWidget();
  public slots:
    void setImage(const CRGBMap& image);
    void message(const char* message, int ms=0);
    void setObjectCount(int n);
    void setFrameCount(int n);
  signals:
    void newCommand();
    void addCommand();
    void trainCommand();
    void lookupCommand();
    void specFGChanged(int);
    void specBGChanged(int);
  private slots:
    void newButtonClicked();
    void addButtonClicked();
    void trainButtonClicked();
    void lookupButtonClicked();
    void specFGSliderChanged(int);
    void specBGSliderChanged(int);
  private:
    QPushButton* newButton;
    QPushButton* addButton;
    QPushButton* trainButton;
    QPushButton* lookupButton;
    QStatusBar*  statusBar;
    QLabel*      objectLabel;
    QLabel*      frameLabel;
    QSlider*     specFGSlider;
    QSlider*     specBGSlider;
    QLabel*      bgLabel;
    QLabel*      fgLabel;
    void initWidget();
    void doResize(int w, int h);
    CImageWidget imageWidget;
  };
}
    
#endif
    









