#ifndef _colorsegmentation_h_
#define _colorsegmentation_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        colorsegmentation.h
 * Description: Algorithms used for the color-based segmentation. 
 * Created:     2001-03-28
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://www.cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * ------------------------- Revision History -------------------------
 * $Log: colorsegmentation.h,v $
 * Revision 1.7  2001/08/10 14:11:57  slange
 * working on the next build
 *
 * Revision 1.6  2001/08/09 23:15:08  slange
 * - reorganised chaincoding.[h/cpp] (former regions.h, regionfinder.cpp)
 * - added a destructor to color segmentation
 * - added a constructor to region list
 *
 * Revision 1.5  2001/08/09 21:47:20  slange
 * - working on the chaincoding algorithm
 * - changed regionlist from a simple typedef to a class and
 *   moved it into regionlist.h (former region.h)
 *
 * Revision 1.4  2001/08/09 17:00:54  slange
 * - reorganized colorsegmentation and the lookuptable
 * - some other corrections
 *
 * Revision 1.3  2001/08/04 16:43:07  slange
 * - added some mehtods needed by template calibration
 * - added mouseClick event receiver
 *
 * Revision 1.2  2001/04/23 18:42:52  slange
 * added copyright notice to all source files
 *
 * --------------------------------------------------------------------
 */

#include "lookuptable.h"
#include "cvtk.h"

namespace cvtk {

  /* A segmentation strategy using pixel based algorithms. Configured
   * by a decision strategy for a single pixel.
   */
  class CColorSegmentation : public CSegmentationStrategy {
  public:
    CColorSegmentation(const CPixelDecisionStrategy* lut, 
		       int width=0, int height=0);
    virtual ~CColorSegmentation();
    
    virtual CRegionMap* segment(const CRGBMap* image);   
    virtual CRegionMap* segment(const CRGBMap* image, 
				CRegionMap* regions);

  private:
    const CPixelDecisionStrategy* lut;
    int width, height;
  };

}

#endif
