#ifndef _cvtk_h_
#define _cvtk_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        cvtk.h
 * Description: Abstract interfaces of the layered architecture
 * Created:     2001-08-06
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: cvtk.h,v $
 * Revision 1.8  2001/08/10 14:11:57  slange
 * working on the next build
 *
 * Revision 1.7  2001/08/10 01:23:45  slange
 * finished first pass over
 *
 * Revision 1.6  2001/08/10 00:57:05  slange
 * - moved search into namespace cvtk
 * - restructured historymatching and objecthistory
 *
 * Revision 1.5  2001/08/09 23:51:08  slange
 * reorganised templatematching.[h/cpp]
 *
 * Revision 1.4  2001/08/09 23:15:08  slange
 * - reorganised chaincoding.[h/cpp] (former regions.h, regionfinder.cpp)
 * - added a destructor to color segmentation
 * - added a constructor to region list
 *
 * Revision 1.3  2001/08/09 21:47:20  slange
 * - working on the chaincoding algorithm
 * - changed regionlist from a simple typedef to a class and
 *   moved it into regionlist.h (former region.h)
 *
 * Revision 1.2  2001/08/07 12:40:14  slange
 * - still refactoring
 * - initial checkin of defaultfactory.cpp, objectlist.h region.h
 *
 * Revision 1.1  2001/08/06 15:59:47  slange
 * - started refactoring of the library
 * - first commit of cvtk.h cvtkfacade.h defaultfacade.cpp
 *   defaultfacade.h defaultfactory.h
 *
 * --------------------------------------------------------------------
 */

/* include extern declarations of not abstract datastructures
 */
#include "mapwrapper.h"
#include "rgbmap.h"
#include "classifiedlist.h"
#include "regionlist.h"
#include "objectlist.h"
#include "objecthistory.h"

/* include extern declaration of the image source
 */
#include "imagesource.h"

namespace cvtk {

  /************************ data structures **************************/

  typedef CMapWrapper<int> CRegionMap;
  typedef CClassifiedList<CObjectHistory, list<CObjectHistory> >
    CObjectHistoryList;
  
  /******************* interfaces to the layers **********************/

  /* interface to the segmenation layer
   */
  class CSegmentationStrategy {
  public:                                
    virtual ~CSegmentationStrategy() {};
    virtual CRegionMap* segment(const CRGBMap*) =0;
    virtual CRegionMap* segment(const CRGBMap*, CRegionMap*) =0;
  };

  /* interface to the chaincoding layer
   */
  class CChainCodingStrategy {
  public:
    virtual ~CChainCodingStrategy() {};
    virtual CRegionList* codeRegions(const CRegionMap*) =0;
    virtual CRegionList* codeRegions(const CRegionMap*, 
				     CRegionList*) =0;
  };

  /* interface to the object recognition layer
   */
  class CObjectRecognitionStrategy {
  public:
    virtual ~CObjectRecognitionStrategy() {};
    virtual CObjectList* findObjects(CRegionList*) =0;
    virtual CObjectList* findObjects(CRegionList*, CObjectList*) =0;
  };

  /* interface to the object history matching layer
   */
  class CHistoryMatchingStrategy {
  public:
    virtual ~CHistoryMatchingStrategy() {};
    virtual CObjectHistoryList* 
      matchHistory(CObjectList*, CObjectHistoryList*) =0;
  };

  /* interface to the output generating layer
   */
  class COutput {
  public:
    virtual ~COutput() {};
    virtual CObjectHistoryList* 
      setTimeStamp(CObjectHistoryList*, int) =0; 
    virtual CObjectHistoryList* 
      calcMovement(CObjectHistoryList*) =0;
    virtual CObjectList* getOutput(const CObjectHistoryList*) =0;
    virtual CObjectList* getOutput(const CObjectHistoryList*,
				   CObjectList*) =0;
  };

}

#endif      /* _cvtk_h_ */





