#ifndef _cvtkfacade_h_
#define _cvtkfacade_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        cvtkfacade.h
 * Description: abstract interface of the facade a whole image analysis
 *              pipe 
 * Created:     2001-08-06
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: cvtkfacade.h,v $
 * Revision 1.4  2001/09/01 13:28:41  slange
 * - working on monitor functions
 *
 * Revision 1.3  2001/08/10 14:11:57  slange
 * working on the next build
 *
 * Revision 1.2  2001/08/07 12:40:14  slange
 * - still refactoring
 * - initial checkin of defaultfactory.cpp, objectlist.h region.h
 *
 * Revision 1.1  2001/08/06 15:59:47  slange
 * - started refactoring of the library
 * - first commit of cvtk.h cvtkfacade.h defaultfacade.cpp
 *   defaultfacade.h defaultfactory.h
 *
 * --------------------------------------------------------------------
 */

#include "rgbmap.h"
#include "objectlist.h"

/* A client, who's wants to use a whole implementation of the image
 * processing pipe should use the system through this Facade. Every
 * implementation of the architecture should provide an Abstract
 * Factory, that returns an initialized pipe.
 *
 * The client should include only this and the  factory's header.
 * Necessary initialization arguments should be passed to the factories
 * constructor.
 *
 * This facade and the factory have to be declared abstract, to cleanly
 * seperate the client from the libraries header.
 */

namespace cvtk {  

  /* interface to the facade
   */
  class CCVTKFacade {
  public:
    virtual CObjectList* analyse(const CRGBMap*) =0;
    virtual CObjectList* analyse(const CRGBMap*, CObjectList*) =0;
    virtual void visualizeAnalysis(CRGBMap& visual) =0;
    virtual ~CCVTKFacade() {};
  };

  /* abstract interface to a facade factory
   */
  class CCVTKFacadeFactory {
  public:
    virtual CCVTKFacade* getCVTKFacade() =0;
    virtual ~CCVTKFacadeFactory() {};
  };

}

#endif      /* _cvtkfacade_h_ */
