#ifndef _defaultfacade_h_
#define _defaultfacade_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        defaultfacade.h
 * Description: declaration of a facade that uses all layers and hides
 *              a whole pipe   
 * Created:     2001-08-06
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: defaultfacade.h,v $
 * Revision 1.4  2001/09/02 17:39:07  slange
 * cvs error!
 *
 * Revision 1.3  2001/09/01 13:28:41  slange
 * - working on monitor functions
 *
 * Revision 1.2  2001/08/07 12:40:14  slange
 * - still refactoring
 * - initial checkin of defaultfactory.cpp, objectlist.h region.h
 *
 * Revision 1.1  2001/08/06 15:59:47  slange
 * - started refactoring of the library
 * - first commit of cvtk.h cvtkfacade.h defaultfacade.cpp
 *   defaultfacade.h defaultfactory.h
 *
 * --------------------------------------------------------------------
 */

#include "cvtkfacade.h"
#include "cvtk.h"

namespace cvtk {

  /* The default facade is configured with one object of each layer
   * and pipes them.
   */
  class CDefaultFacade : public CCVTKFacade {
  public:
    CDefaultFacade(CSegmentationStrategy*,
		   CChainCodingStrategy*,
		   CObjectRecognitionStrategy*,
		   CHistoryMatchingStrategy*,
		   COutput*,
		   bool copyOutput=true);
    virtual CObjectList* analyse(const CRGBMap*);
    virtual CObjectList* analyse(const CRGBMap*, CObjectList*);
    virtual void visualizeAnalysis(CRGBMap& visual);
    virtual ~CDefaultFacade();

  protected:
    CSegmentationStrategy* segmentation;
    CChainCodingStrategy* chainCoding;
    CObjectRecognitionStrategy* objectRecognition;
    CHistoryMatchingStrategy* historyMatching;
    COutput* output;

    CRegionMap regionMap;             // buffers to minimize object
    CRegionList regionList;           // construction and destruction
    CObjectList objectList;
    CObjectHistoryList historyList;
    CObjectList oList;

    int timeStamp;
    bool copyOutput;
  };
      
}

#endif       /* _defaultfacade_h_ */
