/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        histogram.cpp
 * Description: Implementation of one- and two-dimensional histograms.
 * Created:     2001-04-27
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * ------------------------- Revision History -------------------------
 * $Log: histogram.cpp,v $
 * Revision 1.4  2001/08/30 13:24:30  slange
 * - added sliders to effect the specificity of the kmsegmentation
 *
 * Revision 1.3  2001/05/03 01:19:18  slange
 * - Initial revision of CRGBHistogram
 * - added template class CHistogram3Dim
 *
 * Revision 1.2  2001/04/27 20:51:16  slange
 * - finished implementation of CHistogram2Dim
 * - bug fixes
 *
 * Revision 1.1  2001/04/27 16:57:07  slange
 * - added 1- and 2-dimensional histograms
 * - changed typedef CRGBMap to a class definition
 * - added histogram functions to CRGBMap
 *
 * --------------------------------------------------------------------
 */

#ifndef EXPORT_NOT_IMPLEMENTED
#include "histogram.h"
#endif

#include "string.h"
#include "assert.h"

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int maxValue>
cvtk::CHistogram<maxValue>::CHistogram()
{
  memset(histogram, 0, (sizeof histogram));
}

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int maxValue>
cvtk::CHistogram2Dim<maxValue>::CHistogram2Dim()
{
  assert(sizeof histogram == maxValue*maxValue*sizeof(int)); 
        // should not fail
  memset(histogram, 0, (sizeof histogram));
}  

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int maxValue>
cvtk::CHistogram3Dim<maxValue>::CHistogram3Dim()
{
  assert(sizeof histogram == maxValue*maxValue*maxValue*sizeof(int)); 
        // should not fail
  memset(histogram, 0, (sizeof histogram));
}  

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int maxValue> vector<Vektor2Dim> 
cvtk::CHistogram2Dim<maxValue>::getAsList(int threshold, int mod)
{
  vector<Vektor2Dim> list;
  for (int x=0; x < maxValue; x++)
    for (int y=0; y < maxValue; y++) {
      assert(histogram[x][y] == (*this)[x][y] 
	     && histogram[x][y] == magnitude(x,y));
      if (histogram[x][y] > threshold)
	switch(mod&LIST_PROPORTIONAL) {
	case LIST_PROPORTIONAL:
	  for (int i=0; i < histogram[x][y]; i+=threshold+1)
	    list.push_back(Vektor2Dim((double)x,(double)y));
	  break;
	default:
	  list.push_back(Vektor2Dim((double)x,(double)y));
	  break;
	}
    }
  return list;
}

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int maxValue> vector<Vektor3Dim>
cvtk::CHistogram3Dim<maxValue>::getAsList(int threshold, int mod)
{
  vector<Vektor3Dim> list;
  for (int x=0; x < maxValue; x++)
    for (int y=0; y < maxValue; y++) 
     for (int z=0; z < maxValue; z++) {
       if (histogram[x][y][z] > threshold)
	 switch(mod&LIST_PROPORTIONAL) {
	 case LIST_PROPORTIONAL:
	   for (int i=0; i < histogram[x][y][z]; i+=threshold+1)
	     list.push_back(Vektor3Dim((double)x,(double)y, (double)z));
	  break;
	default:
	  list.push_back(Vektor3Dim((double)x,(double)y, (double)z));
	  break;
	 }
     }
  return list;
}

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int maxValue>
ostream& operator<< (ostream& out, const cvtk::CHistogram<maxValue>& h)
{
  for (int i=0; i < maxValue; i++)
    out << i << " " << h[i] << endl;
  return out;
}

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int maxValue>
ostream& operator<< (ostream& out, 
		     const cvtk::CHistogram2Dim<maxValue>& h)
{
  for (int x=0; x < maxValue; x++)
    for (int y=0; y < maxValue; y++)
      out << x << " " << y << " " <<  h[x][y] << endl;
  return out;
}







