#ifndef _historymatching_h_
#define _historymatching_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        historymatching.h
 * Description: Deklaration of an object's history
 * Created:     2001-08-10
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: historymatching.h,v $
 * Revision 1.4  2001/10/30 11:25:29  slange
 * added a linear movement model to historymatchin.[h/cpp] to
 * model position of undetected objects
 *
 * Revision 1.3  2001/09/02 17:39:07  slange
 * cvs error!
 *
 * Revision 1.2  2001/08/10 14:11:57  slange
 * working on the next build
 *
 * Revision 1.1  2001/08/10 00:57:05  slange
 * - moved search into namespace cvtk
 * - restructured historymatching and objecthistory
 *
 * --------------------------------------------------------------------
 */

#include "cvtk.h"
#include "search.h"

namespace cvtk {

  class CMovementModel {
  public:
    virtual CObject calcNextState(const CObjectHistory&) = 0;
  };

  class CLinearMovementModel : public CMovementModel {
  public:
    virtual CObject calcNextState(const CObjectHistory&);
  };

  class CHistoryMatching : public CHistoryMatchingStrategy {
  public:
    CHistoryMatching(unsigned int sizeOfHistory=3, 
		     CProblemSolvingStrategy* matchStrat=0,
		     int deleteAfterNTicks=4);
    
    CObjectHistoryList* matchHistory(CObjectList* objs, 
				     CObjectHistoryList* hist);

  private:
    unsigned int sizeOfHistory;
    CProblemSolvingStrategy* matchStrat;
    static unsigned int uid;
    int deleteAfterNTicks;
  };

}

#endif      /* _historymatching_h_ */
