#ifndef _imagewidget_h_
#define _imagewidget_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        imagewidget.h
 * Description: Qt widget, that holds and displays an image.
 * Created:     2001-03-22
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://www.cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 *
 * ------------------------- Revision History -------------------------
 * $Log: imagewidget.h,v $
 * Revision 1.4  2001/08/10 14:11:57  slange
 * working on the next build
 *
 * Revision 1.3  2001/08/04 16:58:30  slange
 * added mousePressed signal to imagewidget
 *
 * Revision 1.2  2001/04/23 18:42:52  slange
 * added copyright notice to all source files
 *
 * --------------------------------------------------------------------
 */

#include <qapplication.h>
#include <qimage.h>
#include "rgbmap.h"
#include "mutex.h"

namespace cvtk {
  class CImageWidget : public QWidget
  {
  Q_OBJECT

  public:
    CImageWidget(const CRGBMap& image, QWidget* parent = 0, 
		 WFlags f = WType_TopLevel);
    CImageWidget(int width, int height, QWidget* parent=0, 
		 WFlags f = WType_TopLevel);
    ~CImageWidget();

  signals:
    void mousePressed(QMouseEvent*);

  public slots:
    void setImage(const CRGBMap& image);

  protected:
    void paintEvent(QPaintEvent *ev);
    virtual void mousePressEvent(QMouseEvent*);

  private:
    CMutex dataMutex;                    // data member access synchro.
    CMutex paintMutex;                   // paintEvent synchronization
    int w, h;
    unsigned int* buf;
    QImage image;
    QPixmap pixmap;
  };
}

#endif

