/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        lookuptable.cpp
 * Description: Implementation of the CLookUpTable template class.
 * Created:     2001-04-19
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 *
 *-------------------------- Revision History -------------------------
 * $Log: lookuptable.cpp,v $
 * Revision 1.7  2001/08/28 13:16:48  slange
 * - traced down a bug: historylist to small!
 *
 * Revision 1.6  2001/08/10 14:11:57  slange
 * working on the next build
 *
 * Revision 1.5  2001/08/09 17:00:54  slange
 * - reorganized colorsegmentation and the lookuptable
 * - some other corrections
 *
 * Revision 1.4  2001/08/04 19:41:17  slange
 * corrected getNumberOfClasses in lookuptable.h
 *
 * Revision 1.3  2001/04/29 00:55:49  slange
 * - finished the first complete draft of the calibration tool.
 *   - it now generates a lookup-table for values 1..64 (>> 2)
 *   - writes the entire table to stdout.
 * - fixes in k-means segmentation
 * - added button to generate lookup table in the calibrator-widget
 *
 * Revision 1.2  2001/04/23 18:42:52  slange
 * added copyright notice to all source files
 *
 * --------------------------------------------------------------------
 */

#ifndef EXPORT_NOT_IMPLEMENTED
#include "lookuptable.h"
#endif

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int SHIFT>
cvtk::CLookUpTable<SHIFT>::CLookUpTable() 
  : CPixelDecisionStrategy(), nClass(0)
{
  table_size = 256>>SHIFT;
  memset(table, 0, 
	 table_size*table_size*table_size * sizeof(table[0][0][0]));
}

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int SHIFT>
int cvtk::CLookUpTable<SHIFT>::getNumberOfClasses() const 
{ 
  return nClass; 
};

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int SHIFT>
void cvtk::CLookUpTable<SHIFT>::setNumberOfClasses(int numC)  
{ 
  nClass=numC; 
};

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int SHIFT>
unsigned char& cvtk::CLookUpTable<SHIFT>::lookup(int r, int g, int b)
{
  return table[r>>SHIFT][g>>SHIFT][b>>SHIFT];
}

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int SHIFT>
const unsigned char&
cvtk::CLookUpTable<SHIFT>::lookup(const RGBValue& rgb) const
{
  return lookup(rgb.r, rgb.g, rgb.b);
}

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int SHIFT>
unsigned char& cvtk::CLookUpTable<SHIFT>::lookup(const RGBValue& rgb)
{
  return lookup(rgb.r, rgb.g, rgb.b);
}

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int SHIFT>
const unsigned char&
cvtk::CLookUpTable<SHIFT>::lookup(int r, int g, int b) const
{
  return table[r>>SHIFT][g>>SHIFT][b>>SHIFT];
}

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int SHIFT>
ostream& operator<< (ostream& out, 
		     const cvtk::CLookUpTable<SHIFT>& table)
{
  /*  cout << "#rgbArray" << endl;
      cout << "2 2" << endl;   */                 
  // !!!!!!!!!! SHIFT; NCLASSES !!!!!!!!!!!! //
  for (unsigned int r=0; r < table.getSize(); r++)
    for (unsigned int g=0; g < table.getSize(); g++)
      for (unsigned int b=0; b < table.getSize(); b++) {
	if (table.lookup(r<<SHIFT,g<<SHIFT,b<<SHIFT) > 0)
	  out << r << " " << g << " " << b << " " 
	      << static_cast<unsigned int>(table.lookup(r<<SHIFT,
							g<<SHIFT,
							b<<SHIFT)) 
	      << endl;
      }
  return out;
}

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<int SHIFT>
istream& operator>> (istream& in, cvtk::CLookUpTable<SHIFT>& table)
{
  int r,g,b, c;
  while (in) {
    in >> r >> g >> b >> c;
    table.lookup(r<<SHIFT,g<<SHIFT,b<<SHIFT)=c;
    if (c>table.getNumberOfClasses()) table.setNumberOfClasses(c);
  }
  return in;
}







