/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        main.cpp
 * Description: Demo application using the segmentation and the
 *              regionfinder.
 * Created:     2001-04-22
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: main.cpp,v $
 * Revision 1.30  2001/12/05 11:17:39  slange
 * - resolving conflicts
 *
 * Revision 1.29  2001/09/04 15:31:27  slange
 * - corrected bugs in:
 *   - templatecontroller.cpp: use QString to emit messages
 *   - monitor.cpp: used x instead of y to check against height
 * - labeled the sliders in calibratewidget
 *
 * Revision 1.28  2001/09/01 13:28:41  slange
 * - working on monitor functions
 *
 * Revision 1.27  2001/08/30 20:10:10  slange
 * - robot demo is working!!!
 *
 * Revision 1.26  2001/08/30 18:48:50  slange
 * - moving the robot to the ball
 *
 * Revision 1.25  2001/08/30 17:49:43  slange
 * - detect first robot
 *
 * Revision 1.24  2001/08/30 17:43:55  slange
 * - added cvtk image analysis to robot demo
 *
 * Revision 1.23  2001/08/30 17:25:32  slange
 * - added ft_interface to drive a fischertechnik robot
 * - working on demo application
 *
 * Revision 1.22  2001/08/10 14:11:57  slange
 * working on the next build
 *
 * Revision 1.21  2001/07/28 21:51:15  slange
 * - finished search algorithm
 *
 * Revision 1.20  2001/07/16 20:02:34  slange
 * - working on the history matching
 *
 * Revision 1.19  2001/07/15 22:19:13  slange
 * - added pendel.cpp, file to demonstrate library.
 *
 * Revision 1.18  2001/07/11 17:51:55  slange
 * - found and corrected bug in the region algorithm
 *
 * Revision 1.17  2001/07/07 21:46:18  slange
 * - working on template matching: complete!
 * - todo: implement delete and stop after first match!
 *
 * Revision 1.16  2001/07/06 02:04:18  slange
 * - working on the template matching: completed center matching
 *
 * Revision 1.15  2001/07/04 23:00:03  uid51551
 * - working on the templatematching
 * - calculation of the region center
 *
 * Revision 1.14  2001/06/21 23:53:42  slange
 * - initial checkin of templatematching.[h/cpp]
 * - added method clear() to CRegionList
 *
 * Revision 1.13  2001/06/21 15:57:46  slange
 * initial checkin of objecttemplate.[h/cpp ]
 *
 * Revision 1.12  2001/06/21 00:11:45  slange
 * - use single image in main for testing and profiling
 *
 * Revision 1.11  2001/06/20 11:07:53  slange
 * added cleanup function to main
 *
 * Revision 1.10  2001/06/19 18:09:24  slange
 * - use QTimer instead of posix threads to read new images from the webcam
 * - implemented a cleanup routine to delete the webcam even in case of a call
 *   to exit(state)
 * BUG: ctrl-c causes the whole system to hang on smp - machines.
 * REASON: filedescriptor of the videodevice is not closed!
 *
 * Revision 1.9  2001/05/06 17:06:56  slange
 * - adapted regonlist
 *
 * Revision 1.8  2001/05/03 15:04:09  slange
 * - added lookup table data for orange/yellow/blue on green/black/white
 *   background to rsc
 * - fixes
 *
 * Revision 1.7  2001/05/03 00:26:29  slange
 * - first completly working draft of calibration, colorsegmentation and region
 *   finding/representation.
 * - bug fixes
 *
 * Revision 1.6  2001/04/29 00:55:49  slange
 * - finished the first complete draft of the calibration tool.
 *   - it now generates a lookup-table for values 1..64 (>> 2)
 *   - writes the entire table to stdout.
 * - fixes in k-means segmentation
 * - added button to generate lookup table in the calibrator-widget
 *
 * Revision 1.5  2001/04/27 20:51:16  slange
 * - finished implementation of CHistogram2Dim
 * - bug fixes
 *
 * Revision 1.4  2001/04/27 16:57:07  slange
 * - added 1- and 2-dimensional histograms
 * - changed typedef CRGBMap to a class definition
 * - added histogram functions to CRGBMap
 *
 * Revision 1.3  2001/04/24 19:37:14  slange
 * initialise augmented states to track orange/yellow/blue on green/white/black background
 *
 * Revision 1.2  2001/04/23 18:42:52  slange
 * added copyright notice to all source files
 *
 * --------------------------------------------------------------------
 */

#include <iostream>
#include "ft_interface.h"
#include "cvtkfacade.h"
#include "defaultfactory.h"
#include <qapplication.h>
#include "webcam.h"
#include "imagewidget.h"
#include "vektor.h"

void printUsage(char* str)
{
  cout << "Usage: " << str << " lookuptable.lut templates.tmpl" 
       << endl;
}


/*************************** robot commands **************************/
void fore(ftHandle ft)
{
  setMotor(ft, 3, LEFT, VERYFAST);
  setMotor(ft, 1, RIGHT, VERYFAST);
}
void back(ftHandle ft)
{
  setMotor(ft, 1, LEFT, VERYFAST);
  setMotor(ft, 3, RIGHT, VERYFAST);
}
void left(ftHandle ft)
{
  setMotor(ft, 3, RIGHT, FAST);
  setMotor(ft, 1, RIGHT, FAST);
}
void right(ftHandle ft)
{
  setMotor(ft, 3, LEFT, FAST);
  setMotor(ft, 1, LEFT, FAST);
}
void stop(ftHandle ft)
{
  setMotor(ft, 3, STOP, VERYFAST);
  setMotor(ft, 1, STOP, VERYFAST);
}

/***************************** controller ****************************/
using cvtk::CWebCam;
using cvtk::CPPM;
using cvtk::CImageWidget;
using cvtk::CObjectList;
using cvtk::CObject;

using cvtk::CCVTKFacadeFactory;
using cvtk::CCVTKFacade;
using cvtk::CDefaultFactory;

CWebCam* globalCam;

void cleanup()
{
#ifdef DEBUG
  //  cerr << "in cleanup" << endl;
#endif
  delete globalCam;
}

int main(int argc, char* argv[]) {
  if (argc!=3) {
    printUsage(argv[0]);
    exit(1);
  }

  ftHandle ft;
  ft=openInterface("/dev/ttyS1");

  QApplication app(argc, argv);

  CCVTKFacadeFactory* factory = new CDefaultFactory(argv[1], argv[2]); 
  CCVTKFacade* cvtk = factory->getCVTKFacade();
 
  globalCam = new CWebCam();
  if (atexit(cleanup)!=0) {
    cerr << "Can't register cleanup function" << endl;
    delete globalCam;
    exit(0);
  }
  CWebCam& imgSrc = *globalCam;

  CPPM in(imgSrc.getWidth(), imgSrc.getHeight());
  CObjectList output;
  CImageWidget imageW(imgSrc.getWidth(), imgSrc.getHeight());
 
  app.setMainWidget(&imageW);
  imageW.show();  
  
  while (!isDigiOn(ft, 8)) {
    in = imgSrc.getImage(in);

    cvtk->analyse(&in, &output);
    cvtk->visualizeAnalysis(in);

    imageW.setImage(in);
    app.processEvents();

    if (output[0].size() == 0) {
      cerr << "No robot found." << endl;
      stop(ft);
    }
    else if (output[0].size() != 1) {
      cerr << "More than one robot found!" << endl;
      stop(ft);
    }
    else {                        // found a single robot
      int robWinkel = (output[0][0].orientation+90) % 360;

      cerr << "Robot Position:\n"
	   << "x: " << output[0][0].center[0] 
	   << " y: " << output[0][0].center[1]
	   << " alpha: " << robWinkel
	   << " v: " << output[0][0].velocity
	   << " d: " << output[0][0].direction << endl;
      if (output.size() < 2) {
	cerr << "Never found a ball" << endl;
	stop(ft);
      }
      else if (output[1].size()==0) {
	cerr << "No ball found" << endl;
	stop(ft);
      }
      else {
	cerr << "Ball Position:\n"
	     << "x: " << output[1][0].center[0] 
	     << " y: " << output[1][0].center[1]
	     << " alpha: " << output[1][0].orientation
	     << " v: " << output[1][0].velocity
	     << " d: " << output[1][0].direction << endl;
	
	Vektor<int, 2> rb = output[1][0].center 
	  - output[0][0].center;

	cerr << "Ball Vector:\n"
	     << "x:" <<rb[0] << " y: " << rb[1]
	     << "alpha: " << rb.angle()
	     <<endl;
	
	cerr << endl;

	CObject* o = &output[0][0];

	cout << o->uid << " " << o->objectClass << " " 
	     << o->timeStamp << " "
	     << o->center << " " 
	     << o->orientation << " " 
	     << o->direction << " " 
	     << o->velocity << endl;

	// move to direction:
	
	if (abs(rb.angle()-robWinkel) > 10) {
	  if ((rb.angle()-robWinkel < 0 && 
	       rb.angle()-robWinkel > -180) ||
	      (rb.angle()-robWinkel > 180))
	    left(ft);
	  else 
	    right(ft);
	}
	else { // winkel ok! fahren??
	  if (rb.distance()>35)
	    fore(ft);
	  else
	    stop(ft);
	}
      }

    }
    output.clear();
  }
  stop(ft);
}
