/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        mapwrapper.cpp
 * Description: Implementation of the CMapWrapper template class.
 * Created:     2001-03-27
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 *
 * ------------------------- Revision History -------------------------
 * $Log: mapwrapper.cpp,v $
 * Revision 1.6  2001/08/13 15:36:40  slange
 * - build successfully
 * - initial checkin of objectlist.cpp
 * - removed graphics.h (now rgbmap.h) and ppm.cpp
 *
 * Revision 1.5  2001/08/09 17:00:54  slange
 * - reorganized colorsegmentation and the lookuptable
 * - some other corrections
 *
 * Revision 1.4  2001/08/07 13:42:40  slange
 * - restructured CMapWrapper's header and implementation
 * - added a constructor with a buffer as argument
 *
 * Revision 1.3  2001/04/28 21:11:02  slange
 * - first complete implementation of kmsegmentation
 *
 * Revision 1.2  2001/04/23 18:42:52  slange
 * added copyright notice to all source files
 *
 * --------------------------------------------------------------------
 */

#include "mapwrapper.h"
#include <stdlib.h>

/****************************+* CBuffer ******************************/

#ifndef EXPORT_NOT_IMPLEMENTED
export 
#endif
template<class T>
CMapWrapper<T>::Buffer::~Buffer()
{ 
  delete[] map;
  if(buf!=0) free(buf);
#ifdef DEBUG 
  cerr << "~CMapWrapper: Delete Buffer\n";
#endif
}

#ifndef EXPORT_NOT_IMPLEMENTED
export 
#endif
template<class T>
CMapWrapper<T>::Buffer::Buffer(int width, int height) 
  : CLockable(), width(width), height(height), refCount(1)
{
  buf = static_cast<T*>(calloc(width*height, sizeof (T)));

  map = new T*[height];
  T* pos=buf;
  for(int i=0; i < height; i++, pos+=width)  // setup rowpointers
    map[i] = pos;
#ifdef DEBUG
  cerr << "CMapWrapper: New Buffer\n";
#endif
}   

#ifndef EXPORT_NOT_IMPLEMENTED
export 
#endif
template<class T>
CMapWrapper<T>::Buffer::Buffer(int width, int height, T* buffer) 
  : CLockable(), width(width), height(height), refCount(1), 
    buf(buffer)
{
  map = new T*[height];
  T* pos=buf;
  for(int i=0; i < height; i++, pos+=width)  // setup rowpointers
    map[i] = pos;
#ifdef DEBUG
  cerr << "CMapWrapper: New Buffer\n";
#endif
}  

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<class T>
void CMapWrapper<T>::Buffer::reSize(int width, int height)   
{
  if (this->width != width || this->height != height) {
    this->width = width;
    this->height= height;
    buf = static_cast<T*>(realloc(buf, width*height*sizeof (T)));
    delete[] map;
    map = new T*[height];
    T* pos=buf;
    for(int i=0; i < height; i++, pos+=width)
      map[i] = pos;
  } 
} 

#ifndef EXPORT_NOT_IMPLEMENTED
export 
#endif
template<class T>
void CMapWrapper<T>::Buffer::clear() 
{
  memset(buf, 0, sizeof(T) * width * height);
};

/***************************** CMapWrapper ***************************/

#ifndef EXPORT_NOT_IMPLEMENTED
export 
#endif
template<class T>
CMapWrapper<T>::CMapWrapper()
{ 
  buf = new Buffer(0,0); 
}

#ifndef EXPORT_NOT_IMPLEMENTED
export 
#endif
template<class T>
CMapWrapper<T>::CMapWrapper(int width, int height)
{
  buf = new Buffer(width, height);
}

#ifndef EXPORT_NOT_IMPLEMENTED
export 
#endif
template<class T>
CMapWrapper<T>::CMapWrapper(int width, int height, T* buffer)
{
  buf = new Buffer(width, height, buffer);
}

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<class T>
CMapWrapper<T>::CMapWrapper(const CMapWrapper<T>& mw)
{
  mw.buf->refCount++;
  buf = mw.buf;
}

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<class T>
CMapWrapper<T>::~CMapWrapper()
{
  if (--buf->refCount == 0) delete buf;
}

template<class T>
ostream& operator<< (ostream& out, CMapWrapper<T>& mapwrapper)
{
  out << mapwrapper.getWidth() << " ";
  out << mapwrapper.getHeight() << " ";
  for (int y=0; y < mapwrapper.getHeight(); y++) {
    out << endl;
    for (int x=0; x < mapwrapper.getWidth(); x++)
      out << mapwrapper[y][x] << "  ";
  }
  return out;
}

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<class T>
CMapWrapper<T>& CMapWrapper<T>::operator= (const CMapWrapper<T>& mw)
{
  if (this==&mw)               // do nothing, if assigned to itself
    return *this;
  mw.buf->refCount++;          // add reference to assigned buffer
  if (--buf->refCount==0) delete buf; // remove reference from old buf.
  buf=mw.buf;
  return *this;
}
  
#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<class T>
void CMapWrapper<T>::resize (int width, int height)
{
  buf->reSize(width, height);
}

#ifndef EXPORT_NOT_IMPLEMENTED
export
#endif
template<class T>
void CMapWrapper<T>::clear ()
{
  buf->clear();
}









