/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        mutex.cpp
 * Description: Implementation of a very simple mutex object.
 * Created:     2001-04-22
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://www.cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * ------------------------------- Revision History --------------------------
 * $Log: mutex.cpp,v $
 * Revision 1.2  2001/04/23 18:42:52  slange
 * added copyright notice to all source files
 *
 * ---------------------------------------------------------------------------
 */


#include "mutex.h"
#include <unistd.h>

CMutex::CMutex() : key(Key()), requests(queue<Key>())
{}

CMutex::Key CMutex::hold()
{
  Key newKey;
  requests.push(newKey);
  while(requests.front() != newKey) usleep(20);   // wait to be the next
  return key=newKey;
}
bool CMutex::isLocked() const
{ return !requests.empty(); }

bool CMutex::release(const Key& k)
{
  if (!requests.empty() && requests.front()==k) {
    requests.pop();
    return true;
  }
  else return requests.empty();
}

int CMutex::Key::lastId = 0;
    
/*  
int main() {
  CMutex m1;
  CMutex m2;

  CMutex::Key k1;
  CMutex::Key k2;
  CMutex::Key k3;

  k1 = m1.hold();
  cout << "hold m1\n";
  k2 = m2.hold();
  cout << "hold m2\n";

  cout << m1.release(k1) << endl;
  k3 = m1.hold();
  cout << "hold m1 again \n";
  
  cout << m1.release(k1) << " Wrong key\n";
  // m1.hold(); would block now!
  cout << m1.release(k3) << endl;

  cout << m2.release(k2) << endl;

  }*/








