#ifndef _mutex_h_
#define _mutex_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        mutex.h
 * Description: Very simple Mutex.
 * Created:     2001-03-22
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://www.cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * ------------------------------- Revision History --------------------------
 * $Log: mutex.h,v $
 * Revision 1.2  2001/04/23 18:42:52  slange
 * added copyright notice to all source files
 *
 * ---------------------------------------------------------------------------
 */

#include <queue>

class CMutex
{
public:
  class Key {
  public:
    Key() { id = ++lastId; };
    Key(const Key& k) : id(k.id) {};
    bool operator==(const Key& k) const { return this->id == k.id; }
    Key& operator=(const Key& k) { this->id = k.id; return *this; }
  private:
    int id;
    static int lastId;
  };    
  CMutex();
  Key hold();
  bool release(const Key&);
  bool isLocked() const;
 
private:
  CMutex(CMutex&);
  CMutex& operator= (const CMutex&);
  Key key; 
  queue<Key> requests;
};

#endif








