#ifndef _objecthistory_h_
#define _objecthistory_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        objecthistory.h
 * Description: Deklaration of an object's history
 * Created:     2001-08-07
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: objecthistory.h,v $
 * Revision 1.10  2001/09/02 17:39:07  slange
 * cvs error!
 *
 * Revision 1.9  2001/08/10 01:50:06  slange
 * trying to build again
 *
 * Revision 1.8  2001/08/10 00:57:05  slange
 * - moved search into namespace cvtk
 * - restructured historymatching and objecthistory
 *
 * Revision 1.7  2001/08/09 17:00:54  slange
 * - reorganized colorsegmentation and the lookuptable
 * - some other corrections
 *
 * Revision 1.6  2001/08/07 12:40:14  slange
 * - still refactoring
 * - initial checkin of defaultfactory.cpp, objectlist.h region.h
 *
 * --------------------------------------------------------------------
 */

#include "objectlist.h"

namespace cvtk {

  class CObjectHistory {
  public:
    CObjectHistory(int objectClass, 
		   int uid, 
		   unsigned int sizeOfHistory);
    int objectClass;
    int uid;
    list<CObject> history;
    void insert(CObject& obj);
    bool operator== (const CObjectHistory&);
    int nTicksNotSeen;               // hasn't been detected for n ticks
  private:

    unsigned int sizeOfHistory;
  };

}

/************************* inline functions **************************/

inline 
bool cvtk::CObjectHistory::operator== (const CObjectHistory& right)
{
  return objectClass == right.objectClass 
    && uid == right.uid;
}

#endif      /* _history_h_ */
