#ifndef _objectlist_h_
#define _objectlist_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        objectlist.h
 * Description: Deklaration of the objects
 * Created:     2001-08-07
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: objectlist.h,v $
 * Revision 1.2  2001/08/09 17:00:54  slange
 * - reorganized colorsegmentation and the lookuptable
 * - some other corrections
 *
 * Revision 1.1  2001/08/07 12:40:14  slange
 * - still refactoring
 * - initial checkin of defaultfactory.cpp, objectlist.h region.h
 *
 * --------------------------------------------------------------------
 */

#include "classifiedlist.h"
#include "vektor.h"

namespace cvtk {

  /* data structure describing an object. the uid should identify an
   * object through image series.
   */
  class CObject {
  public:
    CObject(Vektor<int, 2> center,
	    int orientation,
	    int objectClass);

    Vektor<int, 2> center;
    int orientation;                 // orientation of the object
    int objectClass;                 // type of object
    int timeStamp;
    unsigned int uid;                // unique identifier of the object
    double direction;                // direction of movement
    double velocity;                 // velocity of movement
  };

  typedef CClassifiedList<CObject> CObjectList;

}

#endif      /* _objectlist_h_ */
