#ifndef _objecttemplate_h_
#define _objecttemplate_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        objecttemplate.h
 * Description: Definition of the object-template datastructure. 
 * Created:     2001-06-21
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: objecttemplate.h,v $
 * Revision 1.6  2001/08/04 16:43:07  slange
 * - added some mehtods needed by template calibration
 * - added mouseClick event receiver
 *
 * Revision 1.5  2001/08/04 14:50:29  slange
 * added stream input functions to object templates
 *
 * Revision 1.4  2001/07/06 02:04:18  slange
 * - working on the template matching: completed center matching
 *
 * Revision 1.3  2001/07/05 16:19:50  slange
 * - added tolerance to object tempplate definition
 *
 * Revision 1.2  2001/07/04 23:00:04  uid51551
 * - working on the templatematching
 * - calculation of the region center
 *
 * Revision 1.1  2001/06/21 15:57:46  slange
 * initial checkin of objecttemplate.[h/cpp ]
 *
 * --------------------------------------------------------------------
 */

/*
 * Definition of template specification syntax:
 * OBJECT
 * [OBJECT..]
 *
 * OBJECT = 
 * object
 * ID
 * CENTER
 * [AXIS
 *  [MARK..]]
 * end
 *
 * CENTER =
 * center COLCLASS [AREA TOLERANCE]
 *
 * AXIS =
 * axis COLCLASS DISTANCETOCENTER TOLERANCE [AREA TOLERANCE]
 *
 * MARK =
 * mark COLCLASS DISTANCETOCENTER TOLERANCE DIRECTION TOLERANCE 
 *      [AREA TOLERANCE]
 */

#include <iostream>
#include <vector>

class CCenter {
public:
  CCenter() : colClass(0), area(0), areaTol(0) {};
  int colClass;
  int area;
  int areaTol;
  friend ostream& operator<< (ostream&, const CCenter&);
  friend istream& operator>> (istream&, CCenter&);
};

class CAxis : public CCenter { 
public: 
  CAxis() : CCenter(), distanceToCenter(0), distanceTol(0) {};
  int distanceToCenter;
  int distanceTol;
  friend istream& operator>> (istream&, CAxis&);
  friend ostream& operator<< (ostream&, const CAxis&);
};

class CMark : public CAxis {
public:
  CMark() : CAxis(), direction(0), directionTol(0) {};
  int direction;
  int directionTol;
  friend istream& operator>> (istream&, CMark&);
  friend ostream& operator<< (ostream&, const CMark&);
};

class CObjectTemplate {
public:
  CObjectTemplate() : id(0) {};
  int id;
  CCenter center;
  CAxis axis;
  vector<CMark> marks;
  friend istream& operator>> (istream&, CObjectTemplate&);
  friend ostream& operator<< (ostream&, const CObjectTemplate&);
};

typedef vector<CObjectTemplate> CObjectTemplateList;

istream& operator>> (istream&, CObjectTemplateList&);
ostream& operator<< (ostream&, const CObjectTemplateList&);

#endif







