/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        output.cpp
 * Description: basic implementation of the output layer
 * Created:     2001-08-10
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: output.cpp,v $
 * Revision 1.4  2001/10/30 11:07:29  slange
 * introduced time resolution to output.h, to calculate the velocity
 * and direction
 *
 * Revision 1.3  2001/08/10 01:23:45  slange
 * finished first pass over
 *
 * --------------------------------------------------------------------
 */

#include "output.h"

cvtk::CObjectHistoryList*
cvtk::CDefaultOutput::setTimeStamp(CObjectHistoryList* h, 
				   int timeStamp)
{
  if (!h)
    return 0;
  for (CObjectHistoryList::iterator c=h->begin(); c != h->end(); c++)
    for (CObjectHistoryList::innerIterator i=c->begin(); 
	 i != c->end(); i++)
      if (i->history.size()>0)
	i->history.front().timeStamp = timeStamp;
  return h;
}

cvtk::CObjectHistoryList*
cvtk::CDefaultOutput::calcMovement(CObjectHistoryList* h)
{
  if (!h)
    return 0;
  Vektor<int, 2> move;
  int timeDiff;
  CObject* t0;                            // actual object
  CObject* t1;                            // object, one time step ago
  list<CObject>::iterator t;
  double dist=0;
  int resolution;
  for (CObjectHistoryList::iterator c=h->begin(); c != h->end(); c++)
    for (CObjectHistoryList::innerIterator i=c->begin(); 
	 i != c->end(); i++) {
      if (i->history.size()<tRes+1) 
	resolution=i->history.size()-1;
      else
	resolution=tRes;
      t0 = &(*(i->history.begin()));
      for (t=i->history.begin(); resolution > 0; resolution--, t++);
      t1=&(*t);
      move = t0->center - t1->center;
      dist = move.distance();
      timeDiff = t0->timeStamp - t1->timeStamp;
      if (dist>0) 
	i->history.front().direction = move.angle();
      if (timeDiff != 0)
	i->history.front().velocity = (move.distance()*timeUnit)
	  /timeDiff;
    }
  return h;
}

cvtk::CDefaultOutput::CDefaultOutput(int timeUnit, 
				     unsigned int tRes)
  : COutput(), timeUnit(timeUnit), tRes(tRes)
{
  if (!timeUnit) {
    cerr << "COuput: timeUnit==0, stopped to prevent division "
	 << "through 0" << endl;
    exit(1);
  }
};

cvtk::CObjectList* 
cvtk::CDefaultOutput::getOutput(const CObjectHistoryList* h)
{
  if (!h)
    return 0;
  return getOutput(h, new CObjectList(h->size()));
}
	
cvtk::CObjectList* 
cvtk::CDefaultOutput::getOutput(const CObjectHistoryList* h,
				CObjectList* o)
{
  if (!h || !o)
    return 0;
  CObjectHistoryList* hist = const_cast<CObjectHistoryList*>(h);
  if (h->size() > o->size())
    o->resize(h->size());
  for (CObjectHistoryList::iterator c=hist->begin();
       c != hist->end(); c++)
    for (CObjectHistoryList::innerIterator i=c->begin();
	 i != c->end(); i++)
      if (i->history.size()>0) 
	o->push_back(i->history.front().objectClass, 
		     i->history.front());
  return o;
}


