/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        pendel.cpp
 * Description: Demo application recording a single object.
 * Created:     2001-07-15
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: pendel.cpp,v $
 * Revision 1.13  2001/08/28 13:16:48  slange
 * - traced down a bug: historylist to small!
 *
 * Revision 1.12  2001/08/14 08:28:38  slange
 * found a bug in templatemantching.cpp
 *
 * Revision 1.11  2001/08/10 14:11:57  slange
 * working on the next build
 *
 * Revision 1.10  2001/08/04 20:55:48  slange
 * - removed clock ticks, process time is not a good messure :)
 *
 * Revision 1.9  2001/08/04 20:01:29  slange
 * corrected bug: region list was to small
 *
 * Revision 1.8  2001/08/02 18:46:32  slange
 * - initial checkin of templatewidget.[h/cpp]
 * - working on velocity
 *
 * Revision 1.7  2001/08/02 17:19:56  slange
 * - first checkin of output.[h/cpp]
 *
 * Revision 1.6  2001/07/29 10:27:30  slange
 * - finished object history matching!
 *
 * Revision 1.5  2001/07/28 22:22:43  slange
 * - found and traced down a bug
 *
 * Revision 1.4  2001/07/28 21:51:15  slange
 * - finished search algorithm
 *
 * Revision 1.3  2001/07/16 20:02:34  slange
 * - working on the history matching
 *
 * Revision 1.2  2001/07/16 14:35:26  slange
 * - initial check in of objecthistory.[h/cpp]
 * - working on pendular demonstration and history
 *
 * Revision 1.1  2001/07/15 22:19:13  slange
 * - added pendel.cpp, file to demonstrate library.
 *
 * --------------------------------------------------------------------
 */

#include "cvtkfacade.h"
#include "defaultfactory.h"
#include "webcam.h"
#include <iostream>
#include <fstream>
#include <unistd.h>
#include "imagewidget.h"
#include <qapplication.h>
#include "cvtkexceptions.h"
#include "fileis.h"


using cvtk::CWebCam;
using cvtk::CImageSource;
using cvtk::CPPM;
using cvtk::CImageWidget;
using cvtk::CObjectList;
using cvtk::CObject;
using cvtk::CSingleImage;

using cvtk::CCVTKFacadeFactory;
using cvtk::CCVTKFacade;
using cvtk::CDefaultFactory;


void printUsage(char* str)
{
  cout << "Usage: " << str << " lookuptable.lut templates.tmpl" 
       << endl;
}


CWebCam* globalCam;

void cleanup()
{
#ifdef DEBUG
  //  cerr << "in cleanup" << endl;
#endif
  delete globalCam;
}


int main(int argc, char* argv[]) {
  if (argc!=3) {
    printUsage(argv[0]);
    exit(1);
  }

  QApplication app(argc, argv);

  CCVTKFacadeFactory* factory = new CDefaultFactory(argv[1], argv[2]); 
  CCVTKFacade* cvtk = factory->getCVTKFacade();
 
  globalCam = new CWebCam();
  if (atexit(cleanup)!=0) {
    cerr << "Can't register cleanup function" << endl;
    delete globalCam;
    exit(0);
  }
  CWebCam& imgSrc = *globalCam;

  CPPM in(imgSrc.getWidth(), imgSrc.getHeight());
  CImageWidget imageW(imgSrc.getWidth(), imgSrc.getHeight());
 
  app.setMainWidget(&imageW);
  imageW.show();
  int c=0;
    
  CObjectList output;

  for (int wdh=0; wdh < 300; wdh++) {
    in = imgSrc.getImage(in);
    //timeStamp=clock();

    cvtk->analyse(&in, &output);

    for (CObjectList::iterator cl=output.begin(); 
	 cl != output.end(); cl++)
      for (CObjectList::innerIterator o=cl->begin();
	   o != cl->end(); o++)
      cout << o->uid << " " << o->objectClass << " " 
	   << o->timeStamp << " "
	   << o->center << " " << o->direction << " " 
	   << o->velocity << endl;
    
    imageW.setImage(in);
    app.processEvents();
    c++;
    output.clear();
  }
}







