/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        regionlist.cpp
 * Description: Implementation of the region list
 * Created:     2001-08-09
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: regionlist.cpp,v $
 * Revision 1.2  2001/08/09 23:15:08  slange
 * - reorganised chaincoding.[h/cpp] (former regions.h, regionfinder.cpp)
 * - added a destructor to color segmentation
 * - added a constructor to region list
 *
 * Revision 1.1  2001/08/09 21:47:20  slange
 * - working on the chaincoding algorithm
 * - changed regionlist from a simple typedef to a class and
 *   moved it into regionlist.h (former region.h)
 *
 * --------------------------------------------------------------------
 */

#include "regionlist.h"

cvtk::CRegionList::CRegionList(int nClasses) 
  : CClassifiedList<CRegion>(nClasses)
{};

const cvtk::CRegion& 
cvtk::CRegionList::findNearest(const Vektor<int, 2>& v) const
{
  CRegionList* me = const_cast<CRegionList*>(this);
  double distance = 999999999999999.;
  CRegion* reg = 0;
  for (iterator c=me->begin(); c!=me->end(); c++)
    for (innerIterator r = c->begin(); r!=c->end(); r++)
      if ((r->getCenter()-v).distance()
	  <= distance) {
	distance=(r->getCenter()-v).distance();
	reg= &(*r);
      }
  if (reg==0) {
    cerr << "CRegion::findNearest(): region list is empty!" << endl;
    exit(1);
  }
  return *reg;             
}




