#ifndef _rgbhistogram_h_
#define _rgbhistogram_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        rgbhistogram.h
 * Description: histogram in a rgb color-space.
 * Created:     2001-05-03
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 *
 * ------------------------- Revision History -------------------------
 * $Log: rgbhistogram.h,v $
 * Revision 1.4  2001/09/02 17:39:07  slange
 * cvs error!
 *
 * Revision 1.3  2001/08/30 13:24:30  slange
 * - added sliders to effect the specificity of the kmsegmentation
 *
 * Revision 1.2  2001/05/03 15:04:09  slange
 * - added lookup table data for orange/yellow/blue on green/black/white
 *   background to rsc
 * - fixes
 *
 * Revision 1.1  2001/05/03 01:19:18  slange
 * - Initial revision of CRGBHistogram
 * - added template class CHistogram3Dim
 *
 * --------------------------------------------------------------------
 */

#include "histogram.h"
#include "rgbmap.h"

namespace cvtk {

  template<int maxValue>
  class CRGBHistogram : public CHistogram3Dim<maxValue>
  {
  public:
    CRGBHistogram();
    
    void add(const RGBValue& rgbVal);
    int magnitude(const RGBValue& rgbVal) const;

    int& operator[] (const RGBValue& rgbVal);
    const int& operator[] (const RGBValue& rgbVal) const;  
  };

}

/////////////////////// Inlined Implementation ////////////////////////

template<int maxValue>
inline void cvtk::CRGBHistogram<maxValue>::add(const RGBValue& rgbVal)
{ (*this)[rgbVal]++; };

template<int maxValue> inline int 
cvtk::CRGBHistogram<maxValue>::magnitude(const RGBValue& rgbVal) const
{ return (*this)[rgbVal]; };

template<int maxValue> inline int& 
cvtk::CRGBHistogram<maxValue>::operator[] (const RGBValue& rgbVal)
{
  return histogram[rgbVal.r][rgbVal.g][rgbVal.b];
}

template<int maxValue> inline const int& 
cvtk::CRGBHistogram<maxValue>::operator[] (const RGBValue& rgbVal)const
{
  return histogram[rgbVal.r][rgbVal.g][rgbVal.b];
}

#ifdef EXPORT_NOT_IMPLEMENTED
#include "rgbhistogram.cpp"
#endif

#endif




