#ifndef _rgbmap_h_
#define _rgbmap_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        rgbmap.h
 * Description: Deklaration of this library's rgb image encapsulation
 * Created:     2001-08-07
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 * 
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: rgbmap.h,v $
 * Revision 1.3  2001/08/09 23:16:26  slange
 * - initial checkin of new rgbmap.h
 *
 * --------------------------------------------------------------------
 */

#include "mapwrapper.h"

namespace cvtk {

  /*************************** rgb values ****************************/
  
  /* simple rgb value
   */
  typedef struct { unsigned char r, g, b; } RGBValue; // 24bpp 
  ostream& operator<< (ostream&, const RGBValue&); // input and output 
  istream& operator>> (istream&, RGBValue&);       // for RGBValues
  
  /************************* image classes ***************************/

  /* A CRGBMap is instatiation of CMapWrapper wrapping rgb values
   */ 
  class CRGBMap : public CMapWrapper<RGBValue> 
  {
  public: 
    CRGBMap(int width=0, int height=0);
    CRGBMap(const CRGBMap& rgbMap);       // copy constructor
  };  

  /* A CPPM is an image in ppm format with in- and output functions. 
   */
  class CPPM : public CRGBMap {            
  public:
    enum PPMFormat { ASCII=3, BINARY=6};  // possible formats

    CPPM(PPMFormat type=ASCII);
    CPPM(int width, int height, PPMFormat type=ASCII);
    CPPM(const CPPM&);
    CPPM(const CRGBMap&, PPMFormat type=ASCII);

    CPPM& operator= (const CPPM&);        // assignment operator

    PPMFormat getFormat() const;          // member access functions
    void setFormat(PPMFormat format); 
    int getMaxColorValue() const { return MaxColorValue; };

    friend ostream& operator<< (ostream&, const CPPM&);
    friend istream& operator>> (istream&, CPPM&);      

  private:
    PPMFormat type;
    static const int MaxColorValue=255;   // fixed to 255!! 
  };

}

  /************************ inline functions *************************/

inline ostream& cvtk::operator<< (ostream& out, 
				  const RGBValue& val)
{
  out << static_cast<int>(val.r) << " " 
      << static_cast<int>(val.g) << " "
      << static_cast<int>(val.b);
  return out;
}
inline istream& cvtk::operator>> (istream& in, RGBValue& val)
{
  int num[3];
  for(int i=0; i < 3; i++) {
    in >> num[i];
    if (num[i]<0 || num[i]>255) {
      cerr << "RGBValue:operator>>: wrong input value" << endl;
      exit(1);  
    }
  }
  val.r = static_cast<unsigned char>(num[0]);
  val.g = static_cast<unsigned char>(num[1]);
  val.b = static_cast<unsigned char>(num[2]);
  return in;
}

#endif      /* _rgbmap_h_ */



