#ifndef _searchmatch_h_
#define _searchmatch_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        searchmatch.h
 * Description: Deklaration of the problem specific search 
 *              implementation
 * Created:     2001-07-27
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: searchmatch.h,v $
 * Revision 1.3  2001/08/10 00:57:05  slange
 * - moved search into namespace cvtk
 * - restructured historymatching and objecthistory
 *
 * Revision 1.2  2001/07/28 21:51:15  slange
 * - finished search algorithm
 *
 * Revision 1.1  2001/07/27 20:35:34  slange
 * - initial checkin of searchmatch.[h/cpp]
 *
 * --------------------------------------------------------------------
 */

#include "search.h"
#include "cvtk.h"

namespace cvtk {

  class CMatch {
  public:
    CMatch(CObject*, CObjectHistory*);
    CObject* obj;
    CObjectHistory* hist;
  };
  
  class CMatchState : public CEvaluatedProblemState {
  public:
    CMatchState(CObjectList::iterator, CObjectHistoryList::iterator);
    CMatchState(const CMatchState&);
    CMatchState& operator= (const CMatchState&);
    
    CObjectList::iterator objs;
    CObjectHistoryList::iterator hists;
    vector<CObject*> objToMatch;
    vector<CObject*> objUnMatched;
    list<CObjectHistory*> histToMatch;
    vector<CObjectHistory*> histUnMatched;
    vector<CMatch> matches;
  };

  class CMatchGoal : public CGoalTestStrategy {
  public:
    CMatchGoal() {};
    virtual bool isGoal(const CProblemState*) const;
  };

  class CMatchCost : public CEvaluationStrategy {
  public:
    CMatchCost () {};
    virtual double evaluate(const CProblemState*) const;
  };

  class CMatchHeuristic : public CEvaluationStrategy {
  public:
    CMatchHeuristic () {};
    virtual double evaluate(const CProblemState*) const;
  };

  class CMatchExpand : public CExpandStrategy {
  public:
    CMatchExpand() {};
    virtual void expand(CProblemState*, vector<CProblemState*>*) const;
  };

}

/***************************** INLINES *******************************/

inline cvtk::CMatch::CMatch(CObject* obj, CObjectHistory* hist)
  : obj(obj), hist(hist)
{};

#endif













