#ifndef _templatecontroller_h_
#define _templatecontroller_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        templatecontroller.h
 * Description: Qt Controller of the calibration tool. 
 * Created:     2001-08-04
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA  02111-1307, USA.
 *
 * ------------------------- Revision History -------------------------
 * $Log: templatecontroller.h,v $
 * Revision 1.5  2001/09/01 17:40:09  slange
 * - working on monitor
 * - moved template calibration into namespace "cvtk"
 *
 * Revision 1.4  2001/08/04 19:25:49  slange
 * - finished template controller
 *
 * Revision 1.3  2001/08/04 16:43:07  slange
 * - added some mehtods needed by template calibration
 * - added mouseClick event receiver
 *
 * Revision 1.2  2001/08/04 14:11:57  slange
 * working on the template calibration's functionallity
 *
 * Revision 1.1  2001/08/04 12:03:09  slange
 * - initial checkin of templatecontroller.[h/cpp] and template.cpp
 * - finished GUI of the template calibration tool
 *
 * --------------------------------------------------------------------
 */

#include <qapplication.h>
#include <fstream.h>
#include "imagesource.h"
#include "cvtk.h"
#include <qtimer.h>
#include "objecttemplate.h"

namespace cvtk {

  class CController : public QObject
  {
  Q_OBJECT
  public:
    explicit CController(CImageSource* imageSource,
			 CSegmentationStrategy* semgentation,
			 CChainCodingStrategy* regionFinder);
    ~CController();
  public slots:
    void newTemplate();
    void grabFrame();
    void mouseClick(QMouseEvent*);
    void saveTemplates();
    void stopExec();
    void exec();                       // Thread execution function
  signals:
    void message(const char* str, int ms);
    void imageChanged(const CRGBMap& ppm);
  private:
    CImageSource* imageSource;
    CSegmentationStrategy* segmentation;
    CChainCodingStrategy* regionFinder;
    CRegionMap segments;
    CRegionList regionList;
    CObjectTemplateList templateList;
    CObjectTemplate templ;
    int templateID;
    int markType;
    Vektor<int, 2> center;
    int orientation;

    bool grabbing;
    CPPM ppm;
    QTimer* timer;
    CController(const CController&);   // no copies allowed
    CController& operator= (const CController&);
  };

}  
#endif






