#ifndef _templatematching_h_
#define _templatematching_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        templatematching.h
 * Description: Object template matching algorithm 
 * Created:     2001-06-21
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------ Revision History --------------------------
 * $Log: templatematching.h,v $
 * Revision 1.8  2001/08/10 14:11:57  slange
 * working on the next build
 *
 * Revision 1.7  2001/08/09 23:51:08  slange
 * reorganised templatematching.[h/cpp]
 *
 * Revision 1.6  2001/08/02 17:19:56  slange
 * - first checkin of output.[h/cpp]
 *
 * Revision 1.5  2001/07/16 20:02:34  slange
 * - working on the history matching
 *
 * Revision 1.4  2001/07/16 14:35:26  slange
 * - initial check in of objecthistory.[h/cpp]
 * - working on pendular demonstration and history
 *
 * Revision 1.3  2001/07/15 22:19:13  slange
 * - added pendel.cpp, file to demonstrate library.
 *
 * Revision 1.2  2001/07/06 02:04:18  slange
 * - working on the template matching: completed center matching
 *
 * Revision 1.1  2001/06/21 23:53:42  slange
 * - initial checkin of templatematching.[h/cpp]
 * - added method clear() to CRegionList
 *
 * --------------------------------------------------------------------
 */

/*
 * Takes a list of object templates and tries to find all matches
 * in a segment list.
 */

#include "cvtk.h"
#include "objecttemplate.h"

namespace cvtk {

  class CTemplateMatching : public CObjectRecognitionStrategy {
  public:
    CTemplateMatching(CObjectTemplateList* templates,
		      int flags=0);
    virtual ~CTemplateMatching();

    virtual CObjectList* findObjects(CRegionList* rlist);
    virtual CObjectList* findObjects(CRegionList* rlist, 
				     CObjectList* objList);

    enum Flags {DELETE_AFTER_MATCH=1, STOP_AFTER_MATCH=2};
  private:
    CObjectTemplateList* templates;
    bool deleteAfterMatch;
    bool stopAfterMatch;
    
    bool matchCenter(const  CObjectTemplate&,
		     CRegionList*,
		     CObjectList*,
		     vector<int>&);
    bool matchAxis(const CObjectTemplate&,
		   CRegionList*,
		   CObjectList*,
		   vector<int>&);
    bool matchMarks(const CObjectTemplate&, 
		    CRegionList*,
		    CObjectList*,
		    vector<int>&,
		    unsigned int);
  };

}

#endif    /* _templatematching_h_ */



