/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        calibratewidget.cpp
 * Description: Implementation of the widget for the calibration tool.
 * Created:     2001-03-22
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * ------------------------- Revision History -------------------------
 * $Log: templatewidget.cpp,v $
 * Revision 1.5  2001/09/01 17:40:09  slange
 * - working on monitor
 * - moved template calibration into namespace "cvtk"
 *
 * Revision 1.4  2001/08/04 16:58:30  slange
 * added mousePressed signal to imagewidget
 *
 * Revision 1.3  2001/08/04 16:43:07  slange
 * - added some mehtods needed by template calibration
 * - added mouseClick event receiver
 *
 * Revision 1.2  2001/08/04 12:03:09  slange
 * - initial checkin of templatecontroller.[h/cpp] and template.cpp
 * - finished GUI of the template calibration tool
 *
 * Revision 1.1  2001/08/02 18:46:32  slange
 * - initial checkin of templatewidget.[h/cpp]
 * - working on velocity
 *
 * --------------------------------------------------------------------
 */

#include "templatewidget.h"

cvtk::CTemplateWidget::CTemplateWidget(const CRGBMap& image) 
  : QWidget(0, "CVTK Calibrate Templates", WType_TopLevel), 
    imageWidget(image, this, 0)
{
  initWidget();
  doResize(imageWidget.width(), imageWidget.height());
}
cvtk::CTemplateWidget::CTemplateWidget(int width, int height) 
  : QWidget(0, "CVTK Calibrate Templates", WType_TopLevel), 
    imageWidget(width, height, this, 0)
{
  initWidget();
  doResize(imageWidget.width(), imageWidget.height());
}
cvtk::CTemplateWidget::~CTemplateWidget()
{
  delete newButton;
  delete grabButton;
  delete saveButton;
  delete statusBar;
}
void cvtk::CTemplateWidget::setImage(const CRGBMap& image)
{
  int oldWidth=imageWidget.width();
  int oldHeight=imageWidget.height();
  imageWidget.setImage(image);
  if (oldWidth != imageWidget.width() 
      || oldHeight != imageWidget.height())
    doResize(imageWidget.width(), imageWidget.height());
}
void cvtk::CTemplateWidget::doResize(int w, int h)
{
  w = (w<220) ? 220 : w;
  h = (h<190) ? 190 : h;
  setFixedSize(w,h+130);
  imageWidget.move((w-imageWidget.width())/2, 
		   (h-imageWidget.height())/2);
  newButton->setGeometry(10,h+10, (w-30)/2, 40);
  grabButton->setGeometry(w/2+5, h+10, (w-30)/2, 40);
  saveButton->setGeometry(10,h+60, (w-30)/2, 40);
  statusBar->setGeometry(0,h+110,w,20);
}
void cvtk::CTemplateWidget::initWidget()
{
  // Setup status bar 
  statusBar=new QStatusBar(this);
  statusBar->message("CVTK Calibrate Templates", 3000);

  // add buttons
  newButton=  new QPushButton("Add Template", this);
  grabButton= new QPushButton("Grabbing On/Offf", this);
  saveButton= new QPushButton("Save", this);
  
  // connect internal messages
  connect(newButton, SIGNAL(clicked()), 
	  this, SLOT(newButtonClicked()));
  connect(grabButton, SIGNAL(clicked()), 
	  this, SLOT(grabButtonClicked()));
  connect(saveButton, SIGNAL(clicked()), 
	  this, SLOT(saveButtonClicked()));
  connect(&imageWidget, SIGNAL(mousePressed(QMouseEvent*)),
	  this, SLOT(mousePressed(QMouseEvent*)));

  message("Let's get to work.", 3000);
}

/* basic slot implementations */

void cvtk::CTemplateWidget::message(const char* message, int ms)
{
  if (ms==0)
    statusBar->message(message);
  else
    statusBar->message(message, ms);
}
void cvtk::CTemplateWidget::newButtonClicked()
{
  emit newCommand();
}
void cvtk::CTemplateWidget::grabButtonClicked()
{
  emit grabCommand();
}
void cvtk::CTemplateWidget::saveButtonClicked()
{
  emit saveCommand();
}
void cvtk::CTemplateWidget::mousePressed(QMouseEvent* event)
{
  emit mouseClick(event);
}
#include "templatewidget.moc"














