#ifndef _templatewidget_h_
#define _templatewidget_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        calibratewidget.h
 * Description: Qt widget of the calibration tool. 
 * Created:     2001-03-28
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://www.cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * ------------------------- Revision History -------------------------
 * $Log: templatewidget.h,v $
 * Revision 1.5  2001/09/01 17:40:09  slange
 * - working on monitor
 * - moved template calibration into namespace "cvtk"
 *
 * Revision 1.4  2001/08/04 16:58:30  slange
 * added mousePressed signal to imagewidget
 *
 * Revision 1.3  2001/08/04 16:43:07  slange
 * - added some mehtods needed by template calibration
 * - added mouseClick event receiver
 *
 * Revision 1.2  2001/08/04 12:03:09  slange
 * - initial checkin of templatecontroller.[h/cpp] and template.cpp
 * - finished GUI of the template calibration tool
 *
 * Revision 1.1  2001/08/02 18:46:32  slange
 * - initial checkin of templatewidget.[h/cpp]
 * - working on velocity
 *
 * --------------------------------------------------------------------
 */

#include "imagewidget.h"
#include <qapplication.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qstatusbar.h>

namespace cvtk {

  class CTemplateWidget : public QWidget
  {
  Q_OBJECT
  public:
    CTemplateWidget(const CRGBMap& image);
    CTemplateWidget(int width, int height);
    ~CTemplateWidget();
  public slots:
    void setImage(const CRGBMap& image);
    void message(const char* message, int ms=0);
  signals:
    void newCommand();
    void grabCommand();
    void saveCommand();
    void mouseClick(QMouseEvent*);
  private slots:
    void newButtonClicked();
    void grabButtonClicked();
    void saveButtonClicked();
    void mousePressed(QMouseEvent*);
  private:
    QPushButton* newButton;
    QPushButton* grabButton;
    QPushButton* saveButton;
    QStatusBar*  statusBar;
    void initWidget();
    void doResize(int w, int h);
    CImageWidget imageWidget;
  };

}    
#endif
    









