#ifndef _uvhistogram_h_
#define _uvhistogram_h_

/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        uvhistogram.h
 * Description: histogram in an (H)UV color-space approximation working
 *              with RGBs.
 * Created:     2001-04-28
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 *
 * ------------------------- Revision History -------------------------
 * $Log: uvhistogram.h,v $
 * Revision 1.5  2001/09/04 20:59:13  slange
 * - ready
 *
 * Revision 1.4  2001/09/02 17:39:07  slange
 * cvs error!
 *
 * Revision 1.3  2001/08/30 13:24:30  slange
 * - added sliders to effect the specificity of the kmsegmentation
 *
 * Revision 1.2  2001/05/03 01:19:18  slange
 * - Initial revision of CRGBHistogram
 * - added template class CHistogram3Dim
 *
 * Revision 1.1  2001/04/27 23:15:42  slange
 * - added an UV - Histogram (histogram in an approximation of the HUV space)
 * - changed the Calibrate::CController
 *   - to work with histograms
 *   - to keep a vector of examples in memory
 *   - output file dropped
 *   - to tread the background as a (special) object
 * - fixes
 *
 * --------------------------------------------------------------------
 */

#include "histogram.h"
#include "rgbmap.h"

namespace cvtk {

  /* Histogram of UV values (in an approximation of the HUV color space)
   * u = (r-b) + 256
   * v = (2*g-r-b)/2 + 256
   * u,v e [0,512[ with r,g,b e [0,256[
   */
  class CUVHistogram : public CHistogram2Dim<512>
  {
  public:
    CUVHistogram();
    
    void add(const RGBValue& rgbVal);
    int magnitude(const RGBValue& rgbVal) const;

    int& operator[] (const RGBValue& rgbVal);
    const int& operator[] (const RGBValue& rgbVal) const;  
  };

}

//////////////////// Inlined Implementation ///////////////////////////

inline void cvtk::CUVHistogram::add(const RGBValue& rgbVal)
{ (*this)[rgbVal]++; };
inline int cvtk::CUVHistogram::magnitude(const RGBValue& rgbVal) const
{ return (*this)[rgbVal]; };
inline int& cvtk::CUVHistogram::operator[] (const RGBValue& rgbVal)
{
  return histogram[(rgbVal.r-rgbVal.b) + 256]                  // u
                  [(2*rgbVal.g-rgbVal.r-rgbVal.b)/2 + 256];  // v
}
inline const int& cvtk::CUVHistogram::operator[] (const RGBValue& rgbVal)const
{
  return histogram[(rgbVal.r-rgbVal.b) + 256]                  // u
                  [(2*rgbVal.g-rgbVal.r-rgbVal.b)/2 + 256];  // v
}

#endif




