/*
 * Package:     cvtk - Computer Vision Tool Kit
 * File:        webcam.cpp
 * Description: Implementation of the video4linux image-source. 
 *              Initialisation of the webcam "inspired" by grabber.c .
 * Created:     2001-03-28
 * Author:      Sascha Lange
 * Mail:        Sascha.Lange@Uni-Osnabrueck.De
 *
 * Web:         http://www.cvtk.sourceforge.net
 *
 * Copyright (C) 2001 Sascha Lange
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 *
 * ------------------------- Revision History -------------------------
 * $Log webcam.cpp$
 * --------------------------------------------------------------------
 */

#include "webcam.h"
#include <fcntl.h>
#include <unistd.h>
#include <cstdio>
#include <sys/ioctl.h>

#include <linux/videodev.h>

cvtk::CWebCam::CWebCam(int width, int height, string videoDev)
  : CImageSource(), width(width), height(height)
{  
  fileDesc = open(videoDev.c_str(), O_RDWR);
  if (fileDesc < 0) {
    perror(videoDev.c_str());
    exit(1);
  }

  struct video_capability cap;
  struct video_window win;
  struct video_picture vpic;

  if (ioctl(fileDesc, VIDIOCGCAP, &cap) < 0) {  // get capabilities
      perror("VIDIOGCAP");
      close(fileDesc);
      exit(1);
  }
  if (ioctl(fileDesc, VIDIOCGWIN, &win) < 0) {
    perror("VIDIOCGWIN");
    close(fileDesc);
    exit(1);
  }

  if (width!=0 && height !=0) {
    win.width = width; win.height = height;
    ioctl(fileDesc, VIDIOCSWIN, &win);            // set frame size
    
    if (ioctl(fileDesc, VIDIOCGWIN, &win) < 0) {  // get frame size again to check
      perror("VIDIOCGWIN");
      close(fileDesc);
    }
    if (win.width != static_cast<unsigned>(width) || 
	win.height != static_cast<unsigned>(height)) {
      perror("SIZE DOESNT MATCH");
      close(fileDesc);
      exit(1);
    }
  }
  else {
    this->width = win.width;
    this->height = win.height;
  }
  if (ioctl(fileDesc, VIDIOCGPICT, &vpic) < 0) {
    perror("VIDIOCGPICT");
    close(fileDesc);
    exit(1);
  }
  vpic.depth=24;                            // set depth and palette
  vpic.palette=VIDEO_PALETTE_RGB24;
  if(ioctl(fileDesc, VIDIOCSPICT, &vpic) < 0) {
    perror("VIDIOCGPICT");
    close(fileDesc);
    exit(1);;
  }
}

cvtk::CWebCam::~CWebCam()
{
  close(fileDesc);
}

cvtk::CRGBMap cvtk::CWebCam::getImage()
{ 
  CRGBMap rgbMap(width, height);
  getImage(rgbMap);
  return rgbMap;
}

cvtk::CRGBMap& cvtk::CWebCam::getImage(CRGBMap& rgbMap)
{
  if (getWidth()!=rgbMap.getWidth() || 
      getHeight()!=rgbMap.getHeight()) exit(1);   // exception !!!
  
  rgbMap.lock();                      // rgbMap LOCKED
  /* Attention! "hard" cast of RGBValue* to char* using 
   * reinterpret_cast! 
   */
  read(fileDesc, reinterpret_cast<char*>(rgbMap.getBuffer()), 
       getWidth()*getHeight()*sizeof(char)*3);
  rgbMap.unlock();                    // rgbMap UNLOCKED
  return rgbMap;
}
