/*
 *  Program: ramp
 *
 *  Generates a test image of a specified size, with a linear intensity ramp.
 */

/* From the Vista library: */
#include <vista/Vlib.h>
#include <vista/mu.h>
#include <vista/option.h>
#include <vista/VImage.h>

/* From the standard C library: */
#include <stdio.h>

int main (argc, argv)
    int argc;
    char *argv[];
{
    /* Command line options: */
    static VShort size = 256;
    static VOptionDescRec options[] = {
	{ "size", VShortRepn, 1, (VPointer) & size,
	      VOptionalOpt, NULL, "Number of image rows and columns" }
    };

    FILE *out_file;
    VImage image;
    int row, column, value;

    /* Parse command line arguments and identify the output file: */
    VParseFilterCmd (VNumber (options), options, argc, argv, NULL, & out_file);
    if (size <= 0)
	VError ("Size must be positive; %d isn't", (int) size);

    /* Create the image: */
    image = VCreateImage (1, (int) size, (int) size, VUByteRepn);
    if (! image)
	exit (1);

    /* Fill the image with a ramp: */
    for (row = 0; row < size; row++) {
	value = (float) row / (size - 1) * 255;
	for (column = 0; column < size; column++)
	    VPixel (image, 0, row, column, VUByte) = value;
    }

    /* Write out the image: */
    if (! VWriteImages (out_file, NULL, 1, & image))
	exit (1);

    fprintf (stderr, "%s: Created a %d x %d ramp image.\n",
	     argv[0], (int) size, (int) size);
    return 0;
}
