/***************************************************************************
* Filename: adaptive-contrast-enhancement.c
* Copyright (C) 2000-2001 Daniel X. Pape. 
* 
* This operator enhances local contrast, paying special (adaptive)
* attention to any edges it comes across.
* 
***************************************************************************
* Revision History: 
* Friday, July 24, 1998 - Original 
***************************************************************************
* This program is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2 of the License, or 
* (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software 
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
***************************************************************************/


#include "ipxPluginInterface.h"

#include "libgimp/gimp.h"


#include "localOperator.h"
#include "ipxUtils.h"

#include <math.h>

#define IPX_PLUGIN_DATE "v0.3.2 - July 09, 2001"


static gdouble r=0.5;


GimpPlugInInfo PLUG_IN_INFO =
{
  NULL,    /* init_proc */
  NULL,    /* quit_proc */
  query,   /* query_proc */
  run,     /* run_proc */
};

void
query ()
{
  static GimpParamDef args[] =
  {
    { GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive" },
    { GIMP_PDB_IMAGE, "image", "Input image (unused)" },
    { GIMP_PDB_DRAWABLE, "drawable", "Input drawable" },
  };

  static GimpParamDef *return_vals = NULL;
  static int nargs = sizeof (args) / sizeof (args[0]);
  static int nreturn_vals = 0;

  gimp_install_procedure ("plug_in_ipx-adaptive-contrast-enhancement",
			  "Edge Detection with Adaptive-Contrast-Enhancement Operator",
			  "This plugin calculates ...",
			  "Daniel X. Pape",
			  "Daniel X. Pape",
              IPX_PLUGIN_DATE,
			  "<Image>/Filters/IPX/Adaptive-Contrast-Enhancement-Edge-Detect",
			  "RGB*, GRAY*",
			  GIMP_PLUGIN,
			  nargs, nreturn_vals,
			  args, return_vals);
}



void
run (gchar *name, gint nparams, GimpParam *param, gint *nreturn_vals,
GimpParam **return_vals)
{
    static GimpParam values[1];
    GimpDrawable *drawable;
    GimpRunModeType run_mode;
    GimpPDBStatusType status = GIMP_PDB_SUCCESS;
    
    run_mode = param[0].data.d_int32;
    
    *nreturn_vals = 1;
    *return_vals = values;
    
    values[0].type = GIMP_PDB_STATUS;
    values[0].data.d_status = status;
    
    switch (run_mode)
    {
        case GIMP_RUN_INTERACTIVE:
         /*  Possibly retrieve data  */
    /* gimp_get_data ("plug_in_ipx-adaptive-contrast-enhancement", &bvals); */
            
                /*  First acquire information with a dialog  */
/*             if (! adaptive-contrast-enhancement_dialog ()) */
/*                 return; */
            break;
            
        case GIMP_RUN_NONINTERACTIVE:
                /*  Make sure all the arguments are there!  */
            if (nparams != 3)
                status = GIMP_PDB_CALLING_ERROR;
            if (status == GIMP_PDB_SUCCESS)
            {
            }
            break;
            
        case GIMP_RUN_WITH_LAST_VALS:
                /*  Possibly retrieve data  */
/*             gimp_get_data ("plug_in_ipx-adaptive-contrast-enhancement", &bvals); */
            break;
            
        default:
            break;
    }
    
    
        /*  Get the specified drawable  */
    drawable = gimp_drawable_get (param[2].data.d_drawable);
    
        /*  Make sure that the drawable is gray or RGB color  */
    if (gimp_drawable_is_rgb (drawable->id) ||
        gimp_drawable_is_gray (drawable->id))
    {
        gimp_tile_cache_ntiles(2*(drawable->width / gimp_tile_width() + 1));

        doWindowProcessing(drawable, 1);
        
        if (run_mode != GIMP_RUN_NONINTERACTIVE)
            gimp_displays_flush ();
        
        
            /*  Store data  */
/*         if (run_mode == GIMP_RUN_INTERACTIVE) */
/*             gimp_set_data ("plug_in_ipx-adaptive-contrast-enhancement", &bvals, sizeof (bvals));     */
    }
    else
    {
        status = GIMP_PDB_EXECUTION_ERROR;
    }
    
    gimp_drawable_detach (drawable);
    
    values[0].data.d_status = status;   
}


void initOperator(void) {
    setWindowSize(3);
}


guchar processWindow(guchar **window, gint currentPassNumber)
{    
    gdouble S, Q, d, E, CPrime, w;
    gint i,j;
    gdouble centerPixel;
    
    S = Q = 0.0;
    
    for(i=1; i<windowSize-1; i++) {
        for(j=1; j<windowSize-1; j++) {
            gint tmp1, tmp2;
            tmp1 = abs(window[i-1][j+1] + window[i][j+1] + window[i+1][j+1] - 
                       window[i-1][j-1] - window[i][j-1] - window[i+1][j-1]);
            tmp2 = abs(window[i-1][j+1] + window[i-1][j] + window[i-1][j-1] - 
                       window[i+1][j+1] - window[i+1][j] - window[i+1][j-1]);
            d = MAX(tmp1,tmp2);
            S += d * window[i][j];
            Q += d;
        }		
    }
    
    E = S/Q;
    
    centerPixel = window[halfWindowSize+1][halfWindowSize+1];
    CPrime = pow( abs(centerPixel - E) / (centerPixel + E), r );
	
    if(centerPixel <= E)	
        w = (E*(1-CPrime)) / ((1+CPrime));
    else
        w = (E*(1+CPrime)) / ((1-CPrime));
    
    
    return adjust(w);   
}


/* ******************************** unused ******************************** */

void cleanupOperator(void) {}

void preProcessWindowPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
void postProcessWindowPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}

void preProcessPointPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
guchar processPoint(guchar point, gint currentPassNumber) {return point;}
void postProcessPointPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
