/***************************************************************************
* Filename: agglomerative-region-growing.c 
* Copyright (C) 2000-2001 Daniel X. Pape. 
* 
* This operator agglomerates gray values and can be used for
* region-growing and image segmentation.
* 
***************************************************************************
* Revision History: 
* Saturday, July 18, 1998 - Original 
***************************************************************************
* This program is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2 of the License, or 
* (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software 
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
***************************************************************************/


#include "ipxPluginInterface.h"

#include "libgimp/gimp.h"


#include "localOperator.h"
#include "ipxUtils.h"

#include <math.h>

#define IPX_PLUGIN_DATE "v0.3.2 - July 09, 2001"


GimpPlugInInfo PLUG_IN_INFO =
{
  NULL,    /* init_proc */
  NULL,    /* quit_proc */
  query,   /* query_proc */
  run,     /* run_proc */
};

void
query ()
{
  static GimpParamDef args[] =
  {
    { GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive" },
    { GIMP_PDB_IMAGE, "image", "Input image (unused)" },
    { GIMP_PDB_DRAWABLE, "drawable", "Input drawable" },
  };

  static GimpParamDef *return_vals = NULL;
  static int nargs = sizeof (args) / sizeof (args[0]);
  static int nreturn_vals = 0;

  gimp_install_procedure ("plug_in_ipx-agglomerative-region-growing",
			  "Agglomerative Region Growing",
			  "This plugin ...",
			  "Daniel X. Pape",
			  "Daniel X. Pape",
              IPX_PLUGIN_DATE,
			  "<Image>/Filters/IPX/Agglomerative-Region-Growing",
			  "RGB*, GRAY*",
			  GIMP_PLUGIN,
			  nargs, nreturn_vals,
			  args, return_vals);
}



void
run (gchar *name, gint nparams, GimpParam *param, gint *nreturn_vals,
GimpParam **return_vals)
{
    static GimpParam values[1];
    GimpDrawable *drawable;
    GimpRunModeType run_mode;
    GimpPDBStatusType status = GIMP_PDB_SUCCESS;
    
    run_mode = param[0].data.d_int32;
    
    *nreturn_vals = 1;
    *return_vals = values;
    
    values[0].type = GIMP_PDB_STATUS;
    values[0].data.d_status = status;
    
    switch (run_mode)
    {
        case GIMP_RUN_INTERACTIVE:
                /*  Possibly retrieve data  */
/*             gimp_get_data ("plug_in_ipx-agglomerative-region-growing", &bvals); */
            
                /*  First acquire information with a dialog  */
/*             if (! agglomerative-region-growing_dialog ()) */
/*                 return; */
            break;
            
        case GIMP_RUN_NONINTERACTIVE:
                /*  Make sure all the arguments are there!  */
            if (nparams != 3)
                status = GIMP_PDB_CALLING_ERROR;
            if (status == GIMP_PDB_SUCCESS)
            {
            }
            break;
            
        case GIMP_RUN_WITH_LAST_VALS:
                /*  Possibly retrieve data  */
/*             gimp_get_data ("plug_in_ipx-agglomerative-region-growing", &bvals); */
            break;
            
        default:
            break;
    }
    
    
        /*  Get the specified drawable  */
    drawable = gimp_drawable_get (param[2].data.d_drawable);
    
        /*  Make sure that the drawable is gray or RGB color  */
    if (gimp_drawable_is_rgb (drawable->id) ||
        gimp_drawable_is_gray (drawable->id))
    {
        gimp_tile_cache_ntiles(2*(drawable->width / gimp_tile_width() + 1));

        doWindowProcessing(drawable, 1);
        
        if (run_mode != GIMP_RUN_NONINTERACTIVE)
            gimp_displays_flush ();
        
        
            /*  Store data  */
/*         if (run_mode == GIMP_RUN_INTERACTIVE) */
/*             gimp_set_data ("plug_in_ipx-agglomerative-region-growing", &bvals, sizeof (bvals));     */
    }
    else
    {
        status = GIMP_PDB_EXECUTION_ERROR;
    }
    
    gimp_drawable_detach (drawable);
    
    values[0].data.d_status = status;
}


void initOperator(void)
{
    setWindowSize(3);
}


guchar processWindow(guchar **window, gint currentPassNumber)
{
    int max, min, ber, u, u0, u1, u2, d, w;
    int flag, nunt, nobe, z, v;
    gint result[2];
    guchar f[9];

    f[0] = window[1][2];
    f[1] = window[0][2];
    f[2] = window[0][1];
    f[3] = window[0][0];
    f[4] = window[1][0];
    f[5] = window[2][0];
    f[6] = window[2][1];
    f[7] = window[2][2];
    f[8] = window[1][1];


    v = u = flag = 0;
    w = 2;
            
    minmax_char(f, 9, result);
    min = result[0];
    max = result[1];

    ber = max - min;

    
    while(flag == 0)
    {    
        u0 = (min + ber)*(u/w);
        u1 = (min + ber)*(u+1)/w;
        u2 = (min + ber)*(u+2)/w;
        nunt = nobe = 0;
        
        for(z=0; z<9; z++)
        {
            if( (f[z] >= u0) && (f[z] <= u1) )
                nunt++;
            if( (f[z] > u1) && (f[z] <= u2) )
                nobe++;                         
        }
        
        if((u0 == u1) && (u1 == u2))
            nunt = nobe = 0;
        
        if(nunt == nobe)
        {
            flag = 1;
            min = 255;
            for(z=0; z<9; z++) {
                d = abs(u1 - f[z]);
                if(d < min) {
                    min = d;
                    v = f[z];
                }
            }
        }
        else
        {
            w = 2*w;
            if(nunt > nobe)
                u = 2*u;
            else
                u = 2*u + 1;
        }
        
    }
    
    return v;
    
}


/* ******************************** unused ******************************** */

void cleanupOperator(void) {}

void preProcessWindowPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
void postProcessWindowPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}

void preProcessPointPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
guchar processPoint(guchar point, gint currentPassNumber) {return point;}
void postProcessPointPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
