/***************************************************************************
* Filename: closing.c 
* Copyright (C) 2000-2001 Daniel X. Pape. 
* 
* A morphological operator, implemented in terms of "dilating," then
* "eroding" the image.
* 
***************************************************************************
* Revision History: 
* Sunday, January 16, 2000 - Original 
***************************************************************************
* This program is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2 of the License, or 
* (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software 
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
***************************************************************************/


#include "ipxPluginInterface.h"

#include "libgimp/gimp.h"


#include "localOperator.h"
#include "localOperatorP.h"
#include "ipxUtils.h"

#include <stdlib.h>
#include <math.h>
#include <values.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <stdio.h>
#include <assert.h>

#define IPX_PLUGIN_DATE "v0.3.2 - July 09, 2001"

#define IS_EVEN(x)   (((x) % 2 == 0) ? 1 : 0)


/* globals that localOperator uses */

extern gint gWidth, gHeight;
extern gint gBytesPerPixel;

extern gint gX1, gY1, gX2, gY2;
extern gint gMaskWidth,gMaskHeight; 

        /* local stuff */
static gint gDilationIters, gErosionIters;
static gint gNGEO=1, gN8=1;
static gboolean gDoMaximum;

guchar processWindow(guchar **w, gint currentPassNumber)
{ return 0;} /* not defined */

void doWindowProcessingLocal(GimpDrawable *drawable);
guchar processWindowLocal(guchar **window, gint currentCol);


GimpPlugInInfo PLUG_IN_INFO =
{
  NULL,    /* init_proc */
  NULL,    /* quit_proc */
  query,   /* query_proc */
  run,     /* run_proc */
};

void
query ()
{
  static GimpParamDef args[] =
  {
    { GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive" },
    { GIMP_PDB_IMAGE, "image", "Input image (unused)" },
    { GIMP_PDB_DRAWABLE, "drawable", "Input drawable" },
  };

  static GimpParamDef *return_vals = NULL;
  static int nargs = sizeof (args) / sizeof (args[0]);
  static int nreturn_vals = 0;

  gimp_install_procedure ("plug_in_ipx-closing",
			  "Closing Operator",
			  "This plugin calculates ...",
			  "Daniel X. Pape",
			  "Daniel X. Pape",
              IPX_PLUGIN_DATE,
			  "<Image>/Filters/IPX/Closing",
			  "RGB*, GRAY*",
			  GIMP_PLUGIN,
			  nargs, nreturn_vals,
			  args, return_vals);
}



void
run (gchar *name, gint nparams, GimpParam *param, gint *nreturn_vals,
GimpParam **return_vals)
{
    
    static GimpParam values[1];
    GimpDrawable *drawable;
    GimpRunModeType run_mode;
    GimpPDBStatusType status = GIMP_PDB_SUCCESS;
    
    run_mode = param[0].data.d_int32;
    
    *nreturn_vals = 1;
    *return_vals = values;
    
    values[0].type = GIMP_PDB_STATUS;
    values[0].data.d_status = status;
    
    switch (run_mode)
    {
        case GIMP_RUN_INTERACTIVE:
                /*  Possibly retrieve data  */
/*             gimp_get_data ("plug_in_ipx-dilation", &bvals); */
            
                /*  First acquire information with a dialog  */
/*             if (! dilation_dialog ()) */
/*                 return; */
            break;
            
        case GIMP_RUN_NONINTERACTIVE:
                /*  Make sure all the arguments are there!  */
            if (nparams != 3)
                status = GIMP_PDB_CALLING_ERROR;
            if (status == GIMP_PDB_SUCCESS)
            {
            }
            break;
            
        case GIMP_RUN_WITH_LAST_VALS:
                /*  Possibly retrieve data  */
/*             gimp_get_data ("plug_in_ipx-dilation", &bvals); */
            break;
            
        default:
            break;
    }
    
    
        /*  Get the specified drawable  */
    drawable = gimp_drawable_get (param[2].data.d_drawable);
    
        /*  Make sure that the drawable is gray or RGB color  */
    if (gimp_drawable_is_rgb (drawable->id) ||
        gimp_drawable_is_gray (drawable->id))
    {
        gimp_tile_cache_ntiles(2*(drawable->width / gimp_tile_width() + 1));

        doWindowProcessingLocal(drawable);
        
        if (run_mode != GIMP_RUN_NONINTERACTIVE)
            gimp_displays_flush ();
        
        
            /*  Store data  */
/*         if (run_mode == GIMP_RUN_INTERACTIVE) */
/*             gimp_set_data ("plug_in_ipx-dilation", &bvals, sizeof (bvals));     */
    }
    else
    {
        status = GIMP_PDB_EXECUTION_ERROR;
    }
    
    gimp_drawable_detach (drawable);
    
    values[0].data.d_status = status;    
}


void initOperator(void)
{
    setWindowSize(3);
}


void doWindowProcessingLocal(GimpDrawable *drawable)
{
    GimpPixelRgn srcPR, tmpPR, dstPR;
    gint new_pixel_value;

    guchar *dstrow, *d;

    guchar **rows;
    guchar **window;
                
    gint i,row, col, last_row_retrieved;
    gint currentPass, totalPasses;

    GimpPixelRgn *currentSrcPR;
    GimpPixelRgn *currentDstPR;

    
#if 0
    printf("Waiting... (pid %d)\n", getpid());
    kill(getpid(), SIGSTOP);
#endif

    gDilationIters = 1; gErosionIters = 1;
    gDoMaximum = FALSE; /* minimum first, then maximum */
    totalPasses = gDilationIters + gErosionIters;
    
        /* Get the input area. This is the bounding box of the selection in
         *  the image (or the entire image if there is no selection).
         */
    
    gimp_drawable_mask_bounds (drawable->id, &gX1, &gY1, &gX2, &gY2);
    gMaskWidth = gX2-gX1;
    gMaskHeight = gY2-gY1;

    assert(gMaskWidth > 0 && gMaskHeight > 0);
        
    gimp_progress_init ("Processing...");
    
    gWidth = drawable->width;
    gHeight = drawable->height;
    gBytesPerPixel = drawable->bpp;

    
    initOperator();

    
        /*  allocate row buffers and window array */
    rows = initRowBuffers(); 
    window = initWindowBuffer();
    dstrow = (guchar *) malloc (gMaskWidth * sizeof(guchar) * gBytesPerPixel);

        /*  initialize the pixel regions  */
    initPixelRegions(drawable, &srcPR, &tmpPR, &dstPR, totalPasses);
        

        /************************************************************/
        /************************************************************/
        /* do processing, first pass */

    if(totalPasses == 1)
    {
        currentSrcPR = &srcPR;
        currentDstPR = &dstPR;      
    }
    else
    {
        currentSrcPR = &srcPR;
        currentDstPR = &tmpPR;      
    }
    
    if(IS_EVEN(gNGEO))
        gN8=0;

    
        /* get the first "windowSize" rows into the row buffers */
    last_row_retrieved = fillRowBuffers(rows, currentSrcPR);
    
    
        /*  loop through the rows  */
    for (row = gY1; row < gY2; row++)
    {
        d = dstrow;

            /* loop over pixels in this row */
        for (col = 0; col < gMaskWidth * gBytesPerPixel; col++)
        {
            getWindowFromRows(col, window, rows);
            new_pixel_value = processWindowLocal(window, col);
            *d++ = new_pixel_value;
        }
        
            /*  store the dst row */
        gimp_pixel_rgn_set_row (currentDstPR, dstrow, gX1, row, gMaskWidth);
        
        last_row_retrieved =
            updateRowBuffers(rows, currentSrcPR, last_row_retrieved);

        if ((row % 5) == 0)
            gimp_progress_update(
                (double) row / ((double) gMaskHeight * totalPasses));
    }
    
    

        /************************************************************/
        /************************************************************/
        /* do processing, intermediate and final passes */
 
    for(currentPass=1; currentPass < totalPasses; currentPass++)
    {

        if(currentPass == totalPasses - 1)
        {
            currentSrcPR = &tmpPR;
            currentDstPR = &dstPR;      
        }
        else
        {
            currentSrcPR = &tmpPR;
            currentDstPR = &tmpPR;      
        }


        gNGEO++;
        gN8 = 1;
        
        if(IS_EVEN(gNGEO))
            gN8=0;
        
            /* get the first "windowSize" rows into the row buffers */
        last_row_retrieved = fillRowBuffers(rows, currentSrcPR);
        
            /*  loop through the rows  */
        for (row = gY1; row < gY2; row++)
        {
            d = dstrow;
            
                /* loop over pixels in this row */
            for (col = 0; col < gMaskWidth * gBytesPerPixel; col++)
            {
                getWindowFromRows(col, window, rows);
                new_pixel_value = processWindowLocal(window, col);
                *d++ = new_pixel_value;
            }
            
                /*  store the dst row */
            gimp_pixel_rgn_set_row (currentDstPR, dstrow, gX1, row, gMaskWidth);
            
            last_row_retrieved =
                updateRowBuffers(rows, currentSrcPR, last_row_retrieved);
            
            if ((row % 5) == 0)
                gimp_progress_update (
                    ((double) row + (currentPass * gMaskHeight)) /
                    ((double) gMaskHeight * totalPasses));
        }
    }


    
        /*  update the region on the screen */
    gimp_drawable_flush (drawable);
    gimp_drawable_merge_shadow (drawable->id, TRUE);
    gimp_drawable_update (drawable->id, gX1, gY1, gMaskWidth, gMaskHeight);



    for(i=0; i<windowSize; i++)
        free(rows[i]);
    free(rows);

    for(i=0; i<windowSize; i++)
        free(window[i]);
    free(window);

    free(dstrow);
    
}



guchar processWindowLocal(guchar **window, gint currentCol)
{
    
    guchar f[9];
    static guchar M1, M2, M3;
    guchar v;

    f[0] = window[1][2];
    f[1] = window[0][2];
    f[2] = window[0][1];
    f[3] = window[0][0];
    f[4] = window[1][0];
    f[5] = window[2][0];
    f[6] = window[2][1];
    f[7] = window[2][2];
    f[8] = window[1][1];


    if(currentCol == 0)
    { 		
        if(gDoMaximum)
        {			
            if(gN8 == 1)
            {
                M1 = MAX(MAX(f[3], f[4]), f[5]);
                M2 = MAX(MAX(f[2], f[6]), f[8]);
                M3 = MAX(MAX(f[0], f[1]), f[7]);
					
                v = MAX(MAX(M1, M2), M3);
            }
            else
            {
                v = MAX(MAX(MAX(MAX(f[0], f[2]), f[4]), f[6]), f[8]);
            }				
        }
        else
        {			
            if(gN8 == 1)
            {
                M1 = MIN(MIN(f[3], f[4]), f[5]);
                M2 = MIN(MIN(f[2], f[6]), f[8]);
                M3 = MIN(MIN(f[0], f[1]), f[7]);
					
                v = MIN(MIN(M1, M2), M3);
            }
            else
            {
                v = MIN(MIN(MIN(MIN(f[0], f[2]), f[4]), f[6]), f[8]);
            }				
        }		
    }
    else { 		
        if(gN8 == 1)
        {
            M1 = M2; M2 = M3;
            if(gDoMaximum)
            {
                M3 = MAX(MAX(f[0], f[1]), f[7]);
                v = MAX(MAX(M1, M2), M3);
            }
            else
            {
                M3 = MIN(MIN(f[0], f[1]), f[7]);
                v = MIN(MIN(M1, M2), M3);
            }
        }
        else
        {
            if(gDoMaximum)
            {
                v = MAX(MAX(MAX(MAX(f[0], f[2]), f[4]), f[6]), f[8]);
            }
            else
            {
                v = MIN(MIN(MIN(MIN(f[0], f[2]), f[4]), f[6]), f[8]);
            }
        }		
    }

    return v;	
}


/* ******************************** unused ******************************** */

void cleanupOperator(void) {}

void preProcessWindowPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
void postProcessWindowPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}

void preProcessPointPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
guchar processPoint(guchar point, gint currentPassNumber) {return point;}
void postProcessPointPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
