/***************************************************************************
* Filename: ipxUtils.c 
* Copyright (C) 2000-2001 Daniel X. Pape. 
* 
* This file contains some simple utilities for the plugins to use, such
* as min/max calculation and sorting routines.
* 
***************************************************************************
* Revision History: 
* Thursday, July 23, 1998 - Original 
***************************************************************************
* This program is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2 of the License, or 
* (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software 
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
***************************************************************************/


#include "ipxUtils.h"

#include <values.h>
#include <assert.h>


void minmax_char(guchar *a, gint len, gint *result) {

    guchar i, min, max;

    min = 255;
    max = 0; 

    
        /* compute a min and max of a range of numbers. I got this */
        /* out of CLR...  - it only does 3n/2 comparisions. */
    
    if((len % 2) == 0) {
        for(i=0; i<len; i+=2){
            if(a[i] < a[i+1]) {
                if(a[i] < min)
                    min = a[i];
                else if(a[i+1] > max)
                    max = a[i+1];
            }
            else {
                if(a[i+1] < min)
                    min = a[i+1];
                else if(a[i] > max)
                    max = a[i];
            }
        }
    }
    else {
        
        for(i=0; i<len-1; i+=2){
            if(a[i] < a[i+1]) {
                if(a[i] < min)
                    min = a[i];
                else if(a[i+1] > max)
                    max = a[i+1];
            }
            else {
                if(a[i+1] < min)
                    min = a[i+1];
                else if(a[i] > max)
                    max = a[i];
            }
        }
        
        if(a[len-1] > max)
            max = a[len-1];
        if(a[len-1] < min)
            min = a[len-1];
    }       
    
    
    result[0] = min;
    result[1] = max;
}



void minmax_int(gint *a, gint len, gint *result) {

    gint i, min, max;

    min = INT_MAX;
    max = INT_MIN; 

    
    
    if((len % 2) == 0) {
        for(i=0; i<len; i+=2){
            if(a[i] < a[i+1]) {
                if(a[i] < min)
                    min = a[i];
                else if(a[i+1] > max)
                    max = a[i+1];
            }
            else {
                if(a[i+1] < min)
                    min = a[i+1];
                else if(a[i] > max)
                    max = a[i];
            }
        }
    }
    else {
        
        for(i=0; i<len-1; i+=2){
            if(a[i] < a[i+1]) {
                if(a[i] < min)
                    min = a[i];
                else if(a[i+1] > max)
                    max = a[i+1];
            }
            else {
                if(a[i+1] < min)
                    min = a[i+1];
                else if(a[i] > max)
                    max = a[i];
            }
        }
        
        if(a[len-1] > max)
            max = a[len-1];
        if(a[len-1] < min)
            min = a[len-1];
    }       
    
    
    result[0] = min;
    result[1] = max;
}

gint adjust(double q) {
    gint new_q = (int)(q+0.5);
    return MIN(255, MAX(0,new_q));
}


void bubblesort_int(gint *l, gint n) {

    gint i,j, tmp;

    for(j=n-1; j>0; j--) {
        for(i=0; i<j; i++) {
            if(l[i+1] < l[i]) {
                tmp = l[i];
                l[i] = l[i+1];
                l[i+1] = tmp;
            }
        }
    }

}

void bubblesort_char(guchar *l, gint n) {

    gint i,j;
    guchar tmp;

    for(j=n-1; j>0; j--) {
        for(i=0; i<j; i++) {
            if(l[i+1] < l[i]) {
                tmp = l[i];
                l[i] = l[i+1];
                l[i+1] = tmp;
            }
        }
    }

}


/* ************************************************************************ */
/* Function: select */
/*     Desc: select partitions the array L, of length n, around the
       element k so that smaller elements are on the left, and larger
       elements are on the right.*/
/* ************************************************************************ */

guchar select_char(guchar *L, gint n, gint k)
{
    gint left, right, i, j;
    guchar v, tmp;
    
    assert(n >= 0 && n <=n);
    
    left = 0; right = n-1;

    while(right > left)
    {
        v = L[right];
        i = left - 1; j = right;

        for(;;)
        {
            while(L[++i] < v);
            while(L[--j] > v);
            if(i >= j)
                break;
            tmp = L[i]; L[i] = L[j]; L[j] = tmp;
        }

        tmp = L[i]; L[i] = L[right]; L[right] = tmp;
        if(i >= k)
            right = i - 1;
        if(i <= k)
            left = i + 1;
    }
    return L[k];
}

gint select_int(gint *L, gint n, gint k)
{
    gint left, right, i, j;
    gint v, tmp;
    
    assert(n >= 0 && n <=n);
    
    left = 0; right = n-1;

    while(right > left)
    {
        v = L[right];
        i = left - 1; j = right;

        for(;;)
        {
            while(L[++i] < v);
            while(L[--j] > v);
            if(i >= j)
                break;
            tmp = L[i]; L[i] = L[j]; L[j] = tmp;
        }

        tmp = L[i]; L[i] = L[right]; L[right] = tmp;
        if(i >= k)
            right = i - 1;
        if(i <= k)
            left = i + 1;
    }
    return L[k];
}

