/***************************************************************************
* Filename: localOperator.h 
* Copyright (C) 2000-2001 Daniel X. Pape. 
* 
* This file contains the common functions that the plugins call in order
* to support the "doProcessing" framework.
* 
***************************************************************************
* Revision History: 
* Thursday, July 16, 1998 - Original 
***************************************************************************
* This program is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2 of the License, or 
* (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software 
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
***************************************************************************/

#ifndef _LOCALOPERATOR_H_
#define _LOCALOPERATOR_H_

#include "libgimp/gimp.h"


/* localOperator's public functions */

/************************************************************************
 * Sets the size of the operator kernel window which will be used to
 * process the image. Must be odd (i.e., ws=3 for a 3x3 kernel window).
 ************************************************************************ */
void setWindowSize(gint ws);

/************************************************************************
 * Is called from the plugin code to begin the processing of the image
 * by using an NxN "window" of the image at a time. Needs a GimpDrawable,
 * and a number of passes that the kernel window will be run over the
 * image.
 ************************************************************************ */
void doWindowProcessing(GimpDrawable *drawable, int numPasses);

/************************************************************************
 * Is called from the plugin code to begin the processing of the image
 * by using a pixel of the image at a time. Needs a GimpDrawable, and a
 * number of passes that the kernel window will be run over the image.
 ************************************************************************ */
void doPointProcessing(GimpDrawable *drawable, int numPasses);


#endif
