/***************************************************************************
* Filename: min-max-median.c
* Copyright (C) 2000-2001 Daniel X. Pape. 
* 
* This operator suppresses noise, smoothes the images, and attempts to
* preserve the edge sharpness. In contrast to the normal median
* operator, one-dimensional image features remain unaltered.
* 
***************************************************************************
* Revision History: 
* Sunday, January 16, 2000 - Original 
***************************************************************************
* This program is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2 of the License, or 
* (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software 
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
***************************************************************************/

#include "ipxPluginInterface.h"

#include "libgimp/gimp.h"


#include "localOperator.h"
#include "ipxUtils.h"

#include <math.h>

#define IPX_PLUGIN_DATE "v0.3.2 - July 09, 2001"

GimpPlugInInfo PLUG_IN_INFO =
{
  NULL,    /* init_proc */
  NULL,    /* quit_proc */
  query,   /* query_proc */
  run,     /* run_proc */
};

void
query ()
{
  static GimpParamDef args[] =
  {
    { GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive" },
    { GIMP_PDB_IMAGE, "image", "Input image (unused)" },
    { GIMP_PDB_DRAWABLE, "drawable", "Input drawable" },
  };

  static GimpParamDef *return_vals = NULL;
  static int nargs = sizeof (args) / sizeof (args[0]);
  static int nreturn_vals = 0;

  gimp_install_procedure ("plug_in_ipx-min-max-median",
			  "Min/max-median.",
			  "This plugin calculates ...",
			  "Daniel X. Pape",
			  "Daniel X. Pape",
              IPX_PLUGIN_DATE,
			  "<Image>/Filters/IPX/Min-Max-Median",
			  "RGB*, GRAY*",
			  GIMP_PLUGIN,
			  nargs, nreturn_vals,
			  args, return_vals);
}



void
run (gchar *name, gint nparams, GimpParam *param, gint *nreturn_vals,
GimpParam **return_vals)
{
    
    static GimpParam values[1];
    GimpDrawable *drawable;
    GimpRunModeType run_mode;
    GimpPDBStatusType status = GIMP_PDB_SUCCESS;
    
    run_mode = param[0].data.d_int32;
    
    *nreturn_vals = 1;
    *return_vals = values;
    
    values[0].type = GIMP_PDB_STATUS;
    values[0].data.d_status = status;
    
    switch (run_mode)
    {
        case GIMP_RUN_INTERACTIVE:
                /*  Possibly retrieve data  */
/*             gimp_get_data ("plug_in_ipx-extreme-value", &bvals); */
            
                /*  First acquire information with a dialog  */
/*             if (! extreme-value_dialog ()) */
/*                 return; */
            break;
            
        case GIMP_RUN_NONINTERACTIVE:
                /*  Make sure all the arguments are there!  */
            if (nparams != 3)
                status = GIMP_PDB_CALLING_ERROR;
            if (status == GIMP_PDB_SUCCESS)
            {
            }
            break;
            
        case GIMP_RUN_WITH_LAST_VALS:
                /*  Possibly retrieve data  */
/*             gimp_get_data ("plug_in_ipx-extreme-value", &bvals); */
            break;
            
        default:
            break;
    }
    
    
        /*  Get the specified drawable  */
    drawable = gimp_drawable_get (param[2].data.d_drawable);
    
        /*  Make sure that the drawable is gray or RGB color  */
    if (gimp_drawable_is_rgb (drawable->id) ||
        gimp_drawable_is_gray (drawable->id))
    {
        gimp_tile_cache_ntiles(2*(drawable->width / gimp_tile_width() + 1));

        doWindowProcessing(drawable, 1);
        
        if (run_mode != GIMP_RUN_NONINTERACTIVE)
            gimp_displays_flush ();
        
        
            /*  Store data  */
/*         if (run_mode == GIMP_RUN_INTERACTIVE) */
/*             gimp_set_data ("plug_in_ipx-extreme-value", &bvals, sizeof (bvals));     */
    }
    else
    {
        status = GIMP_PDB_EXECUTION_ERROR;
    }
    
    gimp_drawable_detach (drawable);
        
    values[0].data.d_status = status;
}

void initOperator(void)
{
    setWindowSize(5);
}

guchar processWindow(guchar **window, gint currentPassNumber)
{
    guchar m0, m1, m2, m3, m4;
    guchar max, min;
    guchar pf[numPixelsInWindow];
    guchar R[windowSize];

    gint i, j;
    gint pf_median_index = (numPixelsInWindow-1) / 2;
    gint R_median_index = (windowSize-1) / 2;
    
    for(i=0; i<windowSize; i++)
        for(j=0; j<windowSize; j++)
            pf[i + windowSize*j] = window[i][j];

    m0 = select_char(pf, numPixelsInWindow, pf_median_index);

    for(i=0; i<windowSize; i++)
        R[i] = pf[halfWindowSize + i*windowSize];
    m1 = select_char(R, windowSize, R_median_index);

    for(i=0; i<windowSize; i++)
        R[i] = pf[halfWindowSize*windowSize + i];
    m2 = select_char(R, windowSize, R_median_index);

    for(i=0; i<windowSize; i++)
        R[i] = pf[i * (windowSize + 1)];
    m3 = select_char(R, windowSize, R_median_index);

    for(i=0; i<windowSize; i++)
        R[i] = pf[(windowSize - 1) + i * (windowSize - 1)];
    m4 = select_char(R, windowSize, R_median_index);
    

    max = MAX(MAX(MAX(m1, m2), m3), m4);
    min = MIN(MIN(MIN(m1, m2), m3), m4);

    if(abs(max - m0) >= abs(min - m0))
        return max;
    else
        return min;
}

/* ******************************** unused ******************************** */

void cleanupOperator(void) {}

void preProcessWindowPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
void postProcessWindowPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}

void preProcessPointPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
guchar processPoint(guchar point, gint currentPassNumber) {return point;}
void postProcessPointPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
