/***************************************************************************
* Filename: morphological.c 
* Copyright (C) 2000-2001 Daniel X. Pape. 
*
* The morphological edge detection operator detects edges by performing
* a point-to-point combination of the resulting images from two
* morphological opertions, erosion and dilation. A simple smoothing
* operation is also included to help combat noise problems.
* 
***************************************************************************
* Revision History: 
* Friday, July 17, 1998 - Original 
***************************************************************************
* This program is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2 of the License, or 
* (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software 
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
***************************************************************************/


#include "ipxPluginInterface.h"

#include "libgimp/gimp.h"


#include "localOperator.h"
#include "ipxUtils.h"

#include <math.h>

#define IPX_PLUGIN_DATE "v0.3.2 - July 09, 2001"

GimpPlugInInfo PLUG_IN_INFO =
{
  NULL,    /* init_proc */
  NULL,    /* quit_proc */
  query,   /* query_proc */
  run,     /* run_proc */
};

void
query ()
{
  static GimpParamDef args[] =
  {
    { GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive" },
    { GIMP_PDB_IMAGE, "image", "Input image (unused)" },
    { GIMP_PDB_DRAWABLE, "drawable", "Input drawable" },
  };

  static GimpParamDef *return_vals = NULL;
  static int nargs = sizeof (args) / sizeof (args[0]);
  static int nreturn_vals = 0;

  gimp_install_procedure ("plug_in_ipx-morphological",
			  "Edge Detection with Morphological Operator",
			  "This plugin calculates ...",
			  "Daniel X. Pape",
			  "Daniel X. Pape",
              IPX_PLUGIN_DATE,
			  "<Image>/Filters/IPX/Morphological-Edge-Detect",
			  "RGB*, GRAY*",
			  GIMP_PLUGIN,
			  nargs, nreturn_vals,
			  args, return_vals);
}



void
run (gchar *name, gint nparams, GimpParam *param, gint *nreturn_vals,
GimpParam **return_vals)
{
    
    static GimpParam values[1];
    GimpDrawable *drawable;
    GimpRunModeType run_mode;
    GimpPDBStatusType status = GIMP_PDB_SUCCESS;
    
    run_mode = param[0].data.d_int32;
    
    *nreturn_vals = 1;
    *return_vals = values;
    
    values[0].type = GIMP_PDB_STATUS;
    values[0].data.d_status = status;
    
    switch (run_mode)
    {
        case GIMP_RUN_INTERACTIVE:
                /*  Possibly retrieve data  */
/*             gimp_get_data ("plug_in_ipx-morphological", &bvals); */
            
                /*  First acquire information with a dialog  */
/*             if (! morphological_dialog ()) */
/*                 return; */
            break;
            
        case GIMP_RUN_NONINTERACTIVE:
                /*  Make sure all the arguments are there!  */
            if (nparams != 3)
                status = GIMP_PDB_CALLING_ERROR;
            if (status == GIMP_PDB_SUCCESS)
            {
            }
            break;
            
        case GIMP_RUN_WITH_LAST_VALS:
                /*  Possibly retrieve data  */
/*             gimp_get_data ("plug_in_ipx-morphological", &bvals); */
            break;
            
        default:
            break;
    }
    
    drawable = gimp_drawable_get (param[2].data.d_drawable);
    
    if (gimp_drawable_is_rgb (drawable->id) ||
        gimp_drawable_is_gray (drawable->id))
    {
        gimp_tile_cache_ntiles(2*(drawable->width / gimp_tile_width() + 1));

        doWindowProcessing(drawable, 1);
        
        if (run_mode != GIMP_RUN_NONINTERACTIVE)
            gimp_displays_flush ();
        
        
            /*  Store data  */
/*  if (run_mode == GIMP_RUN_INTERACTIVE) */
/*     gimp_set_data ("plug_in_ipx-morphological", &bvals, sizeof (bvals)); */
    }
    else
    {
        status = GIMP_PDB_EXECUTION_ERROR;
    }
    
    gimp_drawable_detach (drawable);
    
    values[0].data.d_status = status;
}


void initOperator(void)
{
    setWindowSize(5);
}


guchar processWindow(guchar **window, gint currentPassNumber)
{
    int v, q[9], min, max;
    guchar f[25];
    gint result[2];

    f[0] = window[2][3];
    f[1] = window[1][3];
    f[2] = window[1][2];
    f[3] = window[1][1];
    f[4] = window[2][1];
    f[5] = window[3][1];
    f[6] = window[3][2];
    f[7] = window[3][3];
    f[8] = window[2][4];
    f[9] = window[1][4];
    f[10] = window[0][4];
    f[11] = window[0][3];
    f[12] = window[0][2];
    f[13] = window[0][1];
    f[14] = window[0][0];
    f[15] = window[1][0];
    f[16] = window[2][0];
    f[17] = window[3][0];
    f[18] = window[4][0];
    f[19] = window[4][1];
    f[20] = window[4][2];
    f[21] = window[4][3];
    f[22] = window[4][4];
    f[23] = window[3][4];
    f[24] = window[2][2];
    
    
    q[0] = f[2] + f[1] + f[9] + f[0] + f[8] + f[6] + f[7] +  f[23];
    q[1] = f[12] +  f[11] +  f[10] + f[2] + f[1] + f[9] + f[0] + f[8];
    q[2] = f[13] +  f[12] +  f[11] + f[3] + f[2] + f[1] + f[4] + f[0];
    q[3] = f[14] +  f[13] +  f[12] +  f[15] + f[3] + f[2] +  f[16] + f[4];
    q[4] = f[13] + f[3] + f[2] +  f[16] + f[4] +  f[17] + f[5] + f[6];
    q[5] = f[16] + f[4] +  f[17] + f[5] + f[6] +  f[18] +  f[19] +  f[20];
    q[6] = f[4] + f[0] + f[5] + f[6] + f[7] +  f[19] +  f[20] +  f[21];
    q[7] = f[0] + f[8] + f[6] + f[7] +  f[23] +  f[20] +  f[21] +  f[22];
    q[8] = f[3] + f[2] + f[1] + f[4] + f[0] + f[5] + f[6] + f[7];


    minmax_int(q, 9, result);
    min = result[0];
    max = result[1];

    v = MIN((int)((q[8]-min)/8), (int)((max-q[8])/8));

    v *= 2; /* based on personal testing, it doesn't look like the value
               at this point ever gets higher than 127, so let's go
               ahead and double it so that the final image isn't so
               dark. */
    
    return v;
        
}

/* ******************************** unused ******************************** */

void cleanupOperator(void) {}

void preProcessWindowPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
void postProcessWindowPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}

void preProcessPointPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
guchar processPoint(guchar point, gint currentPassNumber) {return point;}
void postProcessPointPass(GimpPixelRgn *srcPR, gint currentPassNumber) {}
