.ds Vn 2.1
.TH Vfile 5Vi "9 June 1994" "Vista Version \*(Vn"
.SH NAME
Vfile \- Vista data file format
.SH SYNOPSIS
\fIfile\fP ::= \fIheader attribute-list delimiter binary-data\fP
.PP
\fIheader\fP ::= \fBV-data\fP \fIversion\fP
.PP
\fIversion\fP ::= \fB2\fP
.PP
\fIattribute-list\fP ::= \fB{\fP \fIattribute\fP \fIattribute\fP ... \fB}\fP
.PP
\fIattribute\fP ::= \fIname\fP \fB:\fP \fIvalue\fP
.SB NEWLINE
.PP
\fIname\fP ::= \fIword\fP
.PP
\fIvalue\fP ::= \fIobject\fP | \fIword\fP | \fIstring\fP | \fIattribute-list\fP
.PP
\fIobject\fP ::= \fItype\fP \fIattribute-list\fP
.PP
\fItype\fP ::= \fIword\fP
.HP 10n
.na
.nh
\fIword\fP ::= a sequence of any characters except \fB}\fP, 
\fB^\fP, \fB:\fP,
.SB SPACE\c
,
.SB TAB\c
, or
.SB NEWLINE.
.hy
.ad
.HP 10n
.na
.nh
\fIstring\fP ::= \fB"\fP\ \ \ any characters, with \fB"\fP represented 
by \fB\\"\fP\ \ \  \fB"\fP
.hy
.ad
.PP
\fIdelimiter\fP ::=
.SB FORMFEED NEWLINE
.SH DESCRIPTION
.SS Introduction
The Vista data file format encodes data for storage and communication 
among programs. The format has the following properties: 
.RS 2n
.IP \(bu 2n
like Vista itself, the format is easily extended to accommodate new classes 
of objects and new object properties; 
.IP \(bu
the format encodes some data in human-readable form so that something can 
be learned about the contents of a data file, at a glance, using a text 
editor or \fBmore\fP(1); 
.IP \(bu
the format allows efficient representation of bulk data such as image pixel 
values; and 
.IP \(bu
the format allows a data file to contain multiple objects, such as a 
sequence of images or an image and an accompanying edge set. 
.RE
.SS "File Format"
A Vista data file is a sequence of four parts:
.RS 2n
.IP \(bu 2n
A header that identifies the file as containing Vista data, and specifies 
the version of the format used to encode the file. This manual page 
describes version 2; its header is the character sequence \fBV-data 2\fP. 
.IP \(bu
A list of attributes, each specifying an attribute name and an attribute 
value. This list is encoded in the human-readable form described below.
.IP \(bu
A delimiter that marks the end of the attribute list, which is the only 
human-readable section of the data file. This delimiter is a 
.SB FORMFEED
character followed by a 
.SB NEWLINE
character.
.IP \(bu
Any binary data associated with the attributes present in the file's 
attribute list.
.RE
.PP
The attribute list, \fIattribute-list\fP, is encoded as a sequence of 
attributes enclosed by \fB{\fP and \fB}\fP. The lexical tokens making up an 
\fIattribute-list\fP may be separated by one or more spaces and/or 
.SB TAB
characters.
.PP
Each attribute is encoded as an attribute name, a colon, an attribute 
value, and a 
.SB NEWLINE
character.
.PP
An attribute name is a word of up to 256 characters. It may include any
characters except \fB}\fP, \fB^\fP, \fB:\fP,
.SB SPACE\c
,
.SB TAB\c
, or
.SB NEWLINE.
.PP
An attribute value can have any of four forms:
.IP \fIobject\fP 15n
the name of a type (e.g., \fBimage\fP), followed by a value of that type 
encoded as an \fIattribute-list\fP.
.IP \fIword\fP
a sequence of characters, of any length, that doesn't include a
\fB}\fP, \fB^\fP, \fB:\fP,
.SB SPACE\c
,
.SB TAB\c
, or
.SB NEWLINE. 
A \fIword\fP can express an integer or floating-point number, a keyword, or
an alphanumeric string.
.IP \fIstring\fP
a sequence of characters enclosed in double quotes. Within a string, 
certain characters are encoded using a backslash as an escape character: a 
double quote is always encoded as ``\fB\\"\fP''; a backslash is always 
encoded as ``\fB\\\\\fP''; and a 
.SB NEWLINE
may be encoded as ``\fB\\n\fP'' or included literally. A 
.SB NEWLINE
that is not part of the value may be included in \fIstring\fP for 
readability by preceding it with a backslash. 
.IP "\fIattribute-list\fP"
a sequence of attributes enclosed by \fB{\fP and \fB}\fP.
.PP
The last section of the file contains binary data, encoding information 
that cannot be efficiently represented otherwise. Any attribute whose value 
is an \fIobject\fP can have a block of binary data associated with it. Two 
attributes, present in the \fIattribute-list\fP component of the 
\fIobject\fP, describe the block: 
.RS 2n
.IP \(bu 2n
one, named \fBdata\fP, specifies the byte offset of the data block from the 
start of the binary data section of the file
.IP \(bu
the other, named \fBlength\fP, specifies the length of the data block in 
bytes
.RE
.PP
A data file may contain more than one \fIobject\fP, each with its own block 
of binary data; one \fIobject\fP can even be nested in the 
\fIattribute-list\fP of another. In such cases the data blocks must appear 
in the same order as the objects that refer to them. An object's data block 
precedes the data blocks of the objects it contains.
.PP
The encoding of binary data, being particular to each type of object, is 
described by the appropriate section 5Vi manual pages. See the 
\fBVImage\fP(3Vi) page, for example, for a description of how image pixel 
values are encoded.
.SS "Generic Attributes"
Vista establishes conventions for certain attributes while allowing the 
programmer to define additional ones with other names. The attributes that 
Vista defines include generic ones, which apply to many types of objects 
(e.g., an object name attribute), and type-specific ones, which apply only 
to certain types of objects (e.g., an attribute specifying the number of 
rows in an image). Here is a list of the generic attributes currently used 
by Vista:
.IP \fBcomment\fP 10n
This string-valued attribute records a comment about an object.
.IP \fBdata\fP
This integer-valued attribute specifies the location of an object's binary 
data within a data file.
.IP \fBhistory\fP
This attribute records the series of operations that occurred to establish an
object. Its value is an attribute list containing a sequence of 
\fBcomment\fP attributes, each recording a single operation. The most
recent operation appears last in this sequence.
.IP \fBlength\fP
This integer-valued attribute specifies the length of an object's binary 
data within a data file.
.IP \fBname\fP
This string-valued attribute represents the name given an object.
.IP \fBrepn\fP
This attribute describes the representation of data such as image pixels. 
Its value is one of the numeric codes defined for the \fBVRepnKind\fP type 
(see \fBVlib\fP(7Vi)). A dictionary, \fBVNumericRepnDict\fP, is available 
for associating codes corresponding to numeric types (\fBVBitRepn\fP 
through \fBVDoubleRepn\fP) with keywords (\fBbit\fP through \fBdouble\fP). 
.PP
To prevent accidental misspellings of attribute names, you may wish to use 
the following symbolic constants in your code rather than actual character 
strings. These are defined in the \fB<vista/Vlib.h>\fP header file: 
.PP
.RS
.nf
.ft B
.ta 25n
#define VCommentAttr	"comment"
#define VDataAttr	"data"
#define VHistoryAttr	"history"
#define VLengthAttr	"length"
#define VNameAttr	"name"
#define VRepnAttr	"repn"
.DT
.RE
.fi
.ft
.PP
On the manual page for a particular object type (e.g., \fBVImage\fP(3Vi)) 
you will find a list of additional attributes that pertain specifically to 
objects of that type. 
.SS "Reading and Writing Data Files"
The Vista library includes routines for reading data files into memory and 
writing them from memory. In memory, the contents of a data file are 
represented by an \fIattribute list\fP data structure, which is described 
by the \fBVattribute\fP(3Vi) manual page. In general, each attribute in the 
file corresponds to an element in this attribute list data structure. 
However, an object whose type has been registered with the library (such as 
the image type, the edge set type, or some other one registered by the 
programmer) may have its value represented in memory not as an attribute 
list, but as some type-specific structure. This allows greater efficiency 
when working with such objects in memory. (An image, for example, is 
represented by the \fBVImageRec\fP structure described in \fBVImage\fP(3Vi).)
.PP
The following summarizes the mapping between an attribute value in a 
data file and the corresponding value in memory.
.IP \fIobject\fP 15n
If the object's type name is a registered one, then the object's value may 
be represented in memory by a type-specific structure and type-specific 
routines convert between the file- and memory-resident formats. 
\fBVtype\fP(3Vi) describes the mechanism for this. Otherwise, the object's 
type name, binary data, and attribute list are represented in memory by a 
\fBVNamedTypeRec\fP structure, also described in \fBVtype\fP(3Vi). In 
either case, library routines ensure that the object can be read in and 
written out again without loss of information. 
.IP \fIword\fP
.IP \fIstring\fP
Both \fIword\fP and \fIstring\fP values are represented in memory as 
strings. Routines for accessing attribute list data structures (e.g., 
\fBVGetAttr\fP(3Vi)) allow these strings to be interpreted as numbers,
keywords, or simply strings at the time of each access.
.IP \fIattribute-list\fP
An \fIattribute-list\fP value is represented in memory by an attribute list 
data structure. 
.SS Routines
The following summarizes some library routines that are available for 
reading and writing Vista data files.
.RS 2n
.IP \fBVReadFile\fP 15n
will read in a complete data file as an attribute list
.IP \fBVReadObjects\fP
will read a data file and extract objects of a specified type
.IP \fBVWriteFile\fP
will write out an attribute list as a complete data file
.IP \fBVWriteObjects\fP
will write a data file with objects of a specified type
.RE
.SH EXAMPLES
Here is an example of a data file containing two images, one an attribute 
of the other:
.RS
.nf
.ft B
V-data 2 {
.RS
ufo: image {
.RS
data: \fIoffset of first image's pixel values\fP
length: \fIlength of first image's pixel values\fP
nbands: 1
nrows: 256
ncolumns: 256
repn: float
name: "UFO sighted over Vancouver"
pixel_aspect_ratio: 1.25
icon: image {
.RS
data: \fIoffset of second image's pixel values\fP
length: \fIlength of second image's pixel values\fP
nbands: 1
nrows: 32
ncolumns: 32
repn: ubyte
name: "UFO icon"
.RE
}
.RE
}
.RE
}
^L
\fIfirst image's pixel values\fP
\fIsecond image's pixel values\fP
.SH "SEE ALSO"
.na
.nh
.BR Vattribute (3Vi),
.BR Vdictionary (3Vi),
.BR Vtype (3Vi),
.BR Vlib (7Vi),
.br
.BR VReadFile (3Vi),
.BR VReadObjects (3Vi),
.br
.BR VWriteFile (3Vi),
.BR VWriteObjects (3Vi),
.ad
.hy
.SH NOTES
Each block of binary data may be preceded or followed by padding bytes \(em 
to ensure, for example, that a block's offset and length are multiples of a 
machine's word size.
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
