.ds Vn 2.1
.TH Vpd 5Vi "6 June 1994" "Vista Version \*(Vn"
.SH NAME
Vpd \- format of a Vista parameter definition file
.SH SYNOPSIS
.nf
\fB#\fP \fIcomment\fP
\fImodule_name\fP
\fIname\fP \fItype\fP \fIvalue\fP \fIdescription\fP
.fi
.SH DESCRIPTION
A parameter definition file defines the name, type, value, and description
of a set of parameters. It provides a convenient way to supply a moderate
number of parameters to a Vista program. Vista includes a program for
compiling a parameter definition file into a file of C definitions
(\fBvpdc\fP(1Vi)), and for loading parameter values from parameter
definition files, Vista data files, and command line arguments.
.PP
A line beginning with a ``#'' character is ignored as a comment. Blank
lines are also ignored.
.PP
The first entry in the file, occupying a single line by itself, is the name
of the set of parameters. It must be a valid C identifier (i.e., begin with
a letter and consist of letters, numbers, and ``_'' characters).
.PP
Each subsequent line defines a single parameter. It contains four fields
separated by whitespace (\c
.SB TAB
characters and spaces):
.PP
.RS
.nf
\fIname\fP \fItype\fP \fIvalue\fP \fIdescription\fP
.fi
.RE
.PP
The name of the parameter is the valid C identifier specified by
\fIname\fP. It's type, \fItype\fP, is specified by one of the keywords
\fBVBit\fP, \fBVUByte\fP, \fBVSByte\fP, \fBVShort\fP, \fBVLong\fP,
\fBVFloat\fP, \fBVDouble\fP, \fBVBoolean\fP or \fBVString\fP. It's value is
the constant \fIvalue\fP, which must be a C initializer valid according to
the type of parameter. A brief description of the parameter occupies the
rest of the line.
.PP
A \fBVBit\fP, \fBVUByte\fP, \fBVSByte\fP, \fBVShort\fP or \fBVLong\fP
parameter value must be an integer constant. A \fBVFloat\fP or
\fBVDouble\fP parameter value must be a floating-point constant.  A
\fBVBoolean\fP parameter value must be one of the keywords \fBTRUE\fP or
\fBFALSE\fP. A \fBVString\fP parameter value must be a string
enclosed in double quotes; a double quote may be included in the string by
preceding it with a backslash character.
.PP
By convention, Vista parameter definition files have names ending in
``\fB.vpd\fP''.
.SH EXAMPLE
The following is an example of a parameter definition file:
.PP
.nf
.ft B
.RS
.ta 22n 32n 40n
#
# Parameters for feature graph matching
#
MatchPars

# Prior probabilities
model_match_prior	VFloat	0.1	Prior model match prob
feature_match_prior	VFloat	0.5	Prior feature match prob

# Search parameters
cutoff_time	VLong	0	Time to cutoff search (secs)
enforce_consistency	VBoolean	TRUE	Enforce graph topology constraints
max_branching	VShort	10	Max no. branching alternatives
version	VString	"V2.1"	Version string
.RE
.SH "SEE ALSO"
.na
.nh
.BR vpdc (1Vi),
.BR VLoadParameters (3Vi),
.BR VParseParamDefn (3Vi),
.BR VParseParamOptions (3Vi),
.BR VPrintParameters (3Vi),
.BR VReportValidParamOptions (3Vi),
.BR Vlib (7Vi)
.hy
.ad
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
