.ds Vn 2.1
.ds BD /usr/bin/vista
.ds ID /usr/include
.ds ML /usr/lib
.ds MR /usr/man
.ds SD /usr/src/vista
.TH Vista 7Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
Vista \- vision-related software
.SH DESCRIPTION
.SS Introduction
Vista is a collection of computer programs, subroutine libraries, and
data formats supporting research in computational vision. It includes
facilities for working with images as well as related types such as edge maps.
Vista is designed to be easily extended to support additional operations
and new types of objects.
.PP
Vista was developed at the University of British Columbia's Laboratory for
Computational Intelligence.
.SS "Manual Pages"
Manual pages pertaining to Vista have names that begin with ``v'' (for
programs) or ``V'' (for other things), and they have ``Vi'' appended to
their section numbers. Section 7Vi manual pages introduce major components of
Vista. Pages of that section also include information on how Vista is
installed in the local environment (the UBC Laboratory for Computational
Intelligence); they specify, for example, the locations of files. Section
5Vi manual pages describe file-resident and memory-resident data structures.
Section 1Vi and 3Vi manual pages describe programs and library routines or
macros, respectively.
.PP
The \fBVindex\fP(7Vi) manual page contains a list Vista programs.
.PP
You may view Vista manual pages on-line using \fBman\fP(1) or \fBxman\fP(1) 
by including the directory \fB\*(MR\fP in your \fBMANPATH\fP environment 
variable. A printed copy of the pages is kept in a binder labelled ``Vista 
man pages'', which resides on a bookshelf in room CSCI 311. 
.SS "Data File Format"
Vista employs a file format capable of representing a variety of different
types of objects, including images. This format is described in detail by
the \fBVfile\fP(5Vi) manual page.
.PP
The first section of any Vista data file
is human-readable, and can be directly inspected with a text editor or with
the \fBmore\fP(1) program. The
.SM ASCII\c
-encoded information in this section lists the objects contained in the
file, and specifies certain attributes of each. This information can be
safely edited using a text editor. A
.SB FORMFEED
character, ``^L'', separates this 
.SM ASCII
information from any binary-encoded data, which appears last in the file.
.PP
Three programs currently exist for inspecting the contents of Vista data
files, including any binary-encoded information they contain such as
images. The \fBvxview\fP(1Vi) program is the most general of these; it
allows you to browse the objects within a file and to view several files
simultaneously. The \fBvxviewe\fP(1Vi) and \fBvxviewi\fP(1Vi) programs
provide less functionality, being optimized to display just edge sets and
images, but they have simpler user interfaces.  All three programs must be
used from an X Windows display.  These and other Vista programs are
summarized on the \fBVindex\fP(7Vi) manual page.
.PP
By convention, Vista data files have names that end with ``\fB.v\fP''.
.PP
Some sample Vista data files are available in the directory
\fB\*(SD/data\fP.
.SS "Command Argument Conventions"
Vista programs share some conventions for how their options are specified 
on a command line. Each program option is entered as a keyword possibly 
followed by some values: 
.IP
.I \fB-\fPkeyword value1 value2 \fR...\fP
.PP
The keyword may be abbreviated provided that the abbreviation is
unambiguous.  Depending on the option, the keyword may be followed by
nothing, a certain number of values, or a variable number of values; values
may be numbers, strings, or keywords.
.PP
An option that is documented as simply a keyword followed by nothing (e.g.,
\fI-landscape\fP) is actually a Boolean option \(em by default the option
is off, and specifying the keyword turns it on. However, if the keyword is
followed by \fBfalse\fP, \fBno\fP, \fBoff\fP, \fBtrue\fP, \fByes\fP, or
\fBon\fP, that will be taken as its value. You can, for example, say either
.IP
.B vtops \fR...\fP -landscape yes \fIfilename\fP
.PP
or
.IP
.B vtops \fR...\fP -landscape \fIfilename\fP
.PP
provided \fIfilename\fP is not a name like \fByes\fP.
.PP
You can enter any string as a value for an option, but an escape convention
is needed to distinguish string values beginning with ``-'' (a hyphen)
from keywords specifying options. The following rules apply:
.RS 2n
.IP \(bu 2n
the command argument ``-'' is interpreted as the string value ``-''
.IP \(bu
a command argument that begins with ``--'' is interpreted as a string value
that begins with one fewer ``-''
.RE
.PP
If a program reads input from one or more files, those files can usually be
specified on the command line in any of three ways:
.RS 2n
.IP 1. 4n
by giving their names as values of a \fB-in\fP option (e.g., 
\fBvxview\ -in\ \fP\fIfile1.v\ file2.v\fP);
.IP 2.
by omitting the \fB-in\fP option and simply mentioning the names on the command
line (e.g., \fBvxview\ \fP\fIfile1.v\ file2.v\fP); or
.IP 3.
by directing a file to the standard input stream (e.g., 
\fBvxview\ <\ \fP\fIfile1.v\fP).
(Of course, only a single file can be supplied by this method.)
.RE
.PP
With method 1 or 2, you can indicate that a file should be read from the
standard input stream by specifying a name of ``-''. For example, 
\fBvxview\ \fP\fIfile1\fP\fB\ -\ \fP\fIfile3\fP\fB\ <\ \fP\fIfile2\fP 
will display \fIfile1\fP, then \fIfile2\fP, then \fIfile3\fP.
.PP
Similar conventions are used to identify a program's output file. They can
be specified:
.RS 2n
.IP 1. 4n
by giving their names as values of a \fB-out\fP option 
(e.g., \fBvget\ -out\ \fP\fIfile1.v\fP);
.IP 2.
by omitting the \fB-out\fP option and simply mentioning the names on the 
command line (e.g., \fBvget\ \fP\fIfile2.v\fP); or
.IP 3.
by directing the standard output stream to a file
(e.g., \fBvget\ >\ \fP\fIfile1.v\fP).
(Of course, only a single file can be written by this method.)
.RE
.PP
There is a restriction on how these methods can be combined in a single
command: If a program accepts a variable number of input files then you
cannot specify both its input files and its output files by simply
mentioning their names on the command line (method 2, above). If you were
to do so then all filenames appearing on the command line would be
interpreted as input files.
.PP
Each Vista program interprets the option \fB-help\fP as a request to print a
summary of its options. 
.SS Programs
See the \fBVindex\fP(7Vi) manual page for a list of Vista programs.
.PP
Include \fB\*(BD\fP in your \fBPATH\fP environment variable in order to
access them.
.PP
A typical Vista program will read objects from a file, manipulate them
somehow, and write them to a file. Since the output file may be opened
before the input file is entirely read you should not specify the same file
for both input and output.
.PP
A program that processes a certain type of object (e.g., images) will copy
any other types of objects it finds (e.g., edge sets) unchanged from its input
file to its output file. The ordering of objects is preserved.
.PP
Programs that are documented as reading images from Vista data files will
also automatically recognize and read most images from files that have the
(obsolete) UBC image file format (IFF). The mapping from IFF images to
Vista images is described by \fBVReadUbcIff\fP(3Vi).
.PP
Programs return a status of 0 if successful and 1 otherwise. Most programs 
report a completion message and/or any error messages to the standard error 
stream. 
.SS Library
The Vista library is a collection of C-callable routines for performing
operations that are common in computational vision applications. It defines
representations for both file-resident and memory-resident objects such as
images, and it includes routines for operating on those objects.
.PP
The \fBVlib\fP(7Vi) manual page introduces the library and points to 
sources of further information about the library. Individual routines, 
macros, and data structures are documented by various section 3Vi and 5Vi 
manual pages. There is also a writeup on the library, \fIIntroduction to
Vista Programming\fP.
.PP
A related library of routines exists for creating simple Vista applications
under the X Window System. See the \fBVX\fP(3Vi) manual page for more
information.
.\".LP
.\"Many Vista library routines are accessible from the Lucid Common Lisp 
.\"environment on Sun workstations. This access is provided by the \fBVLisp\fP 
.\"package. See the \fBVLisp\fP(7Vi) manual page for more information.
.PP
.SS "Source Code"
Source code for Vista resides in subdirectories of \fB\*(SD\fP. The library
is in \fBsrc/lib\fP and programs each have their own subdirectory within
\fBsrc/pgms\fP. You'll find some example programs in the \fBexamples\fP
subdirectory.
.SH FILES
.IP \fB\*(BD\fP
the directory containing Vista programs
.IP \fB\*(ID/vista\fP
the directory containing Vista library header files
.IP \fB\*(ML\fP
the directory containing Vista library archives
.SH "SEE ALSO"
.nh
.na
.BR Vtutorial (7Vi),
.BR Vindex (7Vi),
.BR Vlib (7Vi),
.BR Vfile (5Vi),
.BR VX (3Vi),
.BR vxview (1Vi)
.ad
.hy
.PP
.I Introduction to Vista Programming
.br
.I Vista Contributor's Guide
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
