.ds Vn 2.1
.ds ID /usr/include
.ds ML /usr/lib
.ds SD /usr/src/vista
.TH Vlib 7Vi "16 June 1994" "Vista Version \*(Vn"
.SH NAME
Vlib \- library of vision-related software
.SH SYNOPSIS
\fBcc\fP\0\0...\0\0\fB-I\*(ID\fP
.PP
.B #include <vista/Vlib.h>
.PP
.B ld\0\0\fR...\fP\0\0-lvista\0\0-lm\0\0-lgcc\0\0\fR...\fP
.SH DESCRIPTION
.SS Introduction
The Vista library is a collection of C-callable routines for performing
operations that are common in computational vision applications.
.PP
A tutorial introduction to the library can be found in the document
``Introduction to Vista Programming'', which is available on-line in the
directory \fB\*(SD/doc/intro\fP.
.PP
The library defines representations for both file-resident and
memory-resident objects such as images, and it includes routines for
operating on those objects. The library is designed to be easily extended
to support additional operations and new types of objects.
.PP
A machine-independent format is used for storing data objects in files. 
This format is described by the \fBVfile\fP(5Vi) manual page. 
.SS Usage
As a Vista library user you will need to direct the C compiler to header 
files located in the \fB\*(ID\fP directory (usually by specifying 
\fB-I\*(ID\fP), and you will need to include \fB<vista/Vlib.h>\fP in your C 
source code files. Various section 3Vi and 5Vi manual pages indicate any 
other header files that may be needed. If you are working with images, for 
example, you must also include \fB<vista/VImage.h>\fP, as indicated by 
\fBVImage\fP(3Vi). 
.PP
Three versions of the library are available. They are functionally 
equivalent but they differ in the manner in which they were compiled. To 
link with an optimized version of the library, specify the option 
\fB-lvista\fP to \fBcc\fP(1) or \fBld\fP(1). To link with a debugging 
version that includes symbol information for \fBdbx\fP(1), specify the 
option \fB-lvista_d\fP. To link with a profiling version that, because it 
was compiled with the \fB-pg\fP option, produces output for \fBgprof\fP(1), 
specify the option \fB-lvista_p\fP. All three library archives are found in 
the \fB\*(ML\fP directory. 
.PP
The Vista library should be followed in the linking sequence by the math
library, which is specified by \fB-lm\fP. Depending on what routines you
use you may need to include additional libraries; a routine's manual page
will say so if this is necessary.
.\"
.\" The following applies only if the library was compiled using gcc
.\"
Also, if you're using something other
than \fBgcc\fP(1) (the GNU C compiler) to perform the linking, you will
need to include a \fBgcc\fP-related library by specifying \fB-lgcc\fP.
(This is necessary because the Vista library was compiled using \fBgcc\fP;
when \fBgcc\fP links, it includes \fB-lgcc\fP automatically.)
.SS Conventions
Symbols defined by the library follow a naming convention that helps to
ensure that they are unique and easily recognized as belonging to Vista.
All type names, constant names, macro names, and externally-visible
function and data structure names begin with a V (capital v). When such
names are formed from multiple words they are constructed by capitalizing
the first letter of each word. A typical function name, for example, is
\fBVCreateImage\fP.
.PP
Names of structure members and names of function parameters mentioned in 
the documentation use entirely lower case letters. When such names are 
formed from multiple words the words are separated by underline characters. 
A typical structure member is \fIrow_index\fP. 
.PP
For those using ANSI C, function prototypes are included in Vista header 
files. The prototypes are included automatically if either of the 
preprocessor symbols \fB__STDC__\fP or \fBFUNCPROTO\fP is defined. ANSI C 
compilers such as \fBgcc\fP(1) normally predefine \fB__STDC__\fP, causing 
the prototypes to be included. 
.SS "Error Handling"
Vista library functions that encounter errors report them using the 
routines \fBVError\fP(3Vi) and \fBVWarning\fP(3Vi). The default behavior 
for these routines is to print an error message to the standard error 
stream, and, if the error is fatal, to halt the program. The programmer 
can, however, alter this behavior using \fBVSetErrorHandler\fP(3Vi) and 
\fBVSetWarningHandler\fP(3Vi). 
.PP
Typically, a Vista library function will return a non-zero result (such as
.SB TRUE 
or an address other than
.SB NULL\c
) if successful. If it encounters a recoverable error,
it will issue a warning message using \fBVWarning\fP(3Vi) and return a zero
result. If it encounters a fatal error, it will call \fBVError\fP(3Vi) to
issue a message and halt the program.
.PP
An error message produced by the Vista library identifies the routine that
encountered the error. The section 3Vi manual page for a routine
will usually list the routine's error messages and provide further information
about them.
.SS "Numeric Data Types"
The following types are used for representing numeric data such as pixel 
intensities, distances, and other measurements. They may be defined in 
different ways for different machine architectures, but each is guaranteed 
to have a certain minimum precision. There definitions are in 
\fB<vista/Vlib.h>\fP. 
.RS 2n
.IP \fBVBit\fP 10n
represents an unsigned integer in the range [0,1].
.IP \fBVUByte\fP
represents an unsigned integer in the range [0,255].
.IP \fBVSByte\fP
represents a two's-complement integer in the range [-128,127].
.IP \fBVShort\fP
represents a two's-complement integer in at least 16 bits.
.IP \fBVLong\fP
represents a two's-complement integer in at least 32 bits.
.IP \fBVFloat\fP
represents a floating-point number in at least 32 bits.
.IP \fBVDouble\fP
represents a floating-point number in at least 64 bits.
.RE
.PP
Each is at least as large as the minimum addressable unit on the present 
machine (usually a byte), and also large enough to represent its full range 
of values (e.g., at least 16 bits in the case of \fBVShort\fP). It may be 
larger. For example, \fBVBit\fP occupies a byte on most machines, and 
\fBVShort\fP might occupy 32 bits on some machines. 
.PP
\fINote for users of ``Kernighan and Ritchie'' (K&R) C\fP: In ANSI C,
\fBVSByte\fP is declared as \fBsigned char\fP and an operation involving a
\fBVSByte\fP value preserves its sign. However, in the older K&R C dialect
\fBVSByte\fP is declared as simply \fBchar\fP, which may be either signed
or unsigned depending on the machine. If you are using the library with a
K&R compiler that thinks \fBchar\fP is unsigned, take care to preserve the
sign of a \fBVSByte\fP value when promoting it to an \fBint\fP or to some
other signed type.
.SS "Other Generic Types"
The following types are used for various purposes. They are defined in
\fB<vista/Vlib.h>\fP.
.RS 2n
.IP \fBVAttrList\fP 15n
represents an \fIattribute list\fP, which is described by the
\fBVattribute\fP(3Vi) manual page.
.IP \fBVBoolean\fP
represents either
.SB TRUE
or
.SB FALSE\c
.
.IP \fBVList\fP
represents a list of pointers to opaque objects. It is described by the 
\fBVList\fP(3Vi) manual page.
.IP \fBVBundle\fP
represents an object as a type name, an attribute list value, and a block 
of binary data. It is used to contain objects of unfamiliar type found in data 
files. \fBVBundle\fP is described by the \fBVtype\fP(3Vi) manual page. 
.IP \fBVPointer\fP
represents a pointer to an unspecified type. In
.SM ANSI
C it is defined as \fBvoid\ *\fP.
.IP \fBVString\fP
represents a null-terminated string. It is defined as \fBchar\ *\fP.
.IP \fBVStringConst\fP
represents a null-terminated string whose contents are guaranteed
not to be changed. In
.SM ANSI
C it is defined as \fBconst\ char\ *\fP.
.RE
.SS "Standard Object Types"
The Vista library includes built-in support for some complex types of 
objects, such as images and edge sets. For each of these standard object 
types the library provides: 
.RS 2n
.IP \(bu 2n
a format for representing objects in a data file
.IP \(bu 
a format for representing objects in memory
.IP \(bu 
routines for transferring objects between data files and memory
.IP \(bu 
routines and macros for accessing objects in memory
.IP \(bu 
routines for manipulating objects in memory
.RE
.PP
There is a section 5Vi manual page for each standard object type. 
Currently, the standard object types are:
.RS 2n
.IP \fBcamera\fP 10n
Camera model. See \fBVCamera\fP(3Vi).
.IP \fBedges\fP
Set of edges. See \fBVEdges\fP(3Vi).
.IP \fBimage\fP
Multi-band two-dimensional array of pixels. See \fBVImage\fP(3Vi).
.RE
.PP
It is possible for the programmer using the Vista library to extend this 
list, adding support for his or her own, custom object types. The 
\fBVtype\fP(3Vi) manual page explains how. 
.SS "Identifying Types"
One sometimes needs to identify, to a routine, what representation a data 
value has. The \fB<vista/Vlib.h>\fP header file defines the following 
constants for this purpose: 
.PP
.RS
.nf
.B typedef enum {
.RS
.ft B
.fi
.ad l
.nh
VUnknownRepn,
.br
VBitRepn, VUByteRepn, VSByteRepn, VShortRepn, VLongRepn,
.br
VFloatRepn, VDoubleRepn,
.br
VAttrListRepn, VBooleanRepn, VBundleRepn, VListRepn, VPointerRepn,
VStringRepn,
.br
VEdgesRepn, VImageRepn
.hy
.ad
.nf
.ft
.RE
.B } VRepnKind;
.fi
.RE
.PP
The \fBVRepn\fP(3Vi) manual page describes some macros for obtaining 
information about data types.
.SS Constants
The following constants are defined in \fB<vista/Vlib.h>\fP, and used as
arguments or return values by some library routines:
.RS
.TP 10n
.SB FALSE
is defined to be 0.
.TP
.SB TRUE
is defined to be 1.
.TP
.SB NULL
is defined to be 0.
.RE
.SS "Other Facilities"
The Vista library also includes support for
.RS 2n
.IP \(bu 2n
parsing command line options; see \fBVoption\fP(3Vi).
.IP \(bu
obtaining parameter values from files; see \fBVpd\fP(5Vi).
.IP \(bu
generating PostScript documents; see \fBVpostscript\fP(3Vi).
.IP \(bu
writing X Windows applications; see \fBVX\fP(3Vi).
.RE
.\".LP
.\"The \fBVLisp\fP(7Vi) manual page describes an interface to the Vista 
.\"library for Lucid Common Lisp users. 
.SH FILES
.IP \fB\*(ID/vista\fP
the directory containing Vista library header files
.IP \fB\*(ML\fP
the directory containing Vista library archives
.RE
.SH "SEE ALSO"
.na
.nh
.BR Vista (7Vi),
.BR Vattribute (3Vi),
.BR Vfile (5Vi),
.BR Vtype (3Vi),
.br
.BR VCamera (3Vi),
.BR VEdges (3Vi),
.BR VImage (3Vi),
.BR VList (3Vi),
.br
.BR VError (3Vi),
.BR VWarning (3Vi),
.BR VRepn (3Vi),
.BR VConst (3Vi),
.br
.BR Voption (3Vi),
.BR Vpd (5Vi),
.BR Vpostscript (3Vi),
.\".BR VLisp (7Vi),
.BR VX (3Vi)
.hy
.ad
.PP
.I Introduction to Vista Programming
.br
.I Vista Contributor's Guide
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
