.ds Vn 2.1
.ds BD /usr/bin/vista
.ds MR /usr/man
.ds SD /usr/src/vista
.TH Vtutorial 7Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
Vtutorial \- Vista tutorial
.SH INTRODUCTION
Vista provides a set of software modules and an environment for doing
computer vision research.  It currently contains modules for a wide range
of image processing and higher-level vision operations.  Vista is not
limited to operating with images, but also provides a user-extensible set
of data types for manipulating vectors and other types of data.
.PP
A key aspect of Vista is that it does not just provide many existing
modules, but is also designed to make it easy for any user to add their own
modules and define new data types.  A large library of subroutines is
provided to simplify programming tasks.  All code is written in ANSI C, and
an interface from Common Lisp is provided.  Each module can typically be
run as a Unix filter or called as a subroutine.  In addition, a number of
modules provide graphical interfaces for inspecting and manipulating data,
and a toolkit is provided to simplify the task of creating interactive
programs running under the X Window system.
.PP
Vista was developed at the University of British Columbia's Laboratory for
Computational Intelligence.  The initial developer of the environment was
Art Pope, but other members of the laboratory are contributing to its
further development.  All code is available to the public in source form
without significant restrictions, so we hope that it will become a standard
for development and interchange in the vision community.
.SH GETTING STARTED
You need to ensure that:
.RS 2n
.IP \(bu 3n
the search path for commands executed by your shell (the \fBpath\fP shell
variable) includes the directory \fB\*(BD\fP, where Vista programs are
installed:
.PP
.RS 6n
.nf
.ft B
set path = ($path \*(BD)
.ft
.fi
.RE
.IP \(bu 3n
the search path for on-line manual pages (the \fBMANPATH\fP environment
variable) includes the directory \fB\*(MR\fP, where Vista manual pages are
installed
.PP
.RS 6n
.nf
.ft B
setenv MANPATH $MANPATH":\*(MR"
.ft
.fi
.RE
.IP \(bu 3n
the \fBVISTA\fP environment variable is defined as the location of the
Vista distribution, for your convenience in referring to the the source
code and sample data files:
.PP
.RS 6n
.nf
.ft B
setenv VISTA \*(SD
.ft
.fi
.RE
.RE
.SH DOCUMENTATION
The documentation for Vista is available in a considerable number of
online manual pages.  Start by looking at the following manual pages
to get a quick overview:
.RS 2n
.IP "\fBVista\fP(7Vi)" 15n
Introduction to Vista, including file formats and conventions.
.IP "\fBVindex\fP(7Vi)"
A list of the currently available program modules and other manual pages.
.RE
.PP
These two manual pages should give you enough background to understand
the examples given in the next section.  Of course, you will also want
to look at the manual page for each program module that you are using.
.SH EXAMPLES OF USING VISTA
Following are some examples that you can run to demonstrate various
existing Vista modules.  See the manual page for each command for further
details.  Most Vista modules operate as ordinary Unix filters and take data
from the standard input stream and place it on the standard output stream,
while also allowing file names to be specified on the command line.  See
the Vista manual page for full details on command line conventions.
.SS "Displaying an image:"
.rs
.IP
.nf
.B % vxviewe $VISTA/data/einstein.v
.fi
.PP
This program will pop up an X Window containing the first image from a 
Vista file. The program can be run on monochrome or color displays. The 
window can be resized in the same way as any other X window, and the image 
will be resampled to fit (maintaining the correct aspect ratio). Type 
Ctrl-N to see the next image in the file. See the manual page for 
information on how to zoom and display pixel values. Also, see the program 
\fBvxview\fP for a viewer with many more features. 
.SS "Looking at the header of an image file:"
.rs
.IP
.nf
.B % more $VISTA/data/einstein.v
.fi
.PP
The Vista file header is in 
.SM ASCII
and should be quite easy to understand. Full details are given in the 
\fBVfile\fP(5Vi) and \fBVImage\fP(3Vi) manual pages. The binary data 
follows a new page character, so you can quit \fBmore\fP before displaying 
the binary data. 
.SS "Output an image to a printer using PostScript:"
.rs
.IP
.nf
.B % vtops $VISTA/data/einstein.v | lpr
.fi
.PP
Run the \fBvtops\fP program to output an image to a printer in PostScript. 
This program will also output edge data files and can handle colour images 
and printers. It outputs encapsulated PostScript that can be included as 
a figure in a document.
.SS "Convolving an image with a convolution mask:"
.rs
.IP
.nf
.B % vconvolve -mask mask.v <input.v >output.v
.fi
.PP
There is a sample mask for a 3x3 Laplacian operator in
\fB$VISTA/data/laplace.v\fP.
.SS "Converting other image formats into Vista format:"
.rs
.IP
.nf
.B % vconvert <image.iff >image.v
.B % pnmtov <image.pgm >image.v
.fi
.PP 
Vista is upward compatible with the ``IFF'' image format used previously in
the Laboratory for Computational Intelligence, so all programs will read
images in the old format.  For explicit conversion, use the \fBvconvert\fP
program.  The \fBpnmtov\fP program will convert images in the portable PBM,
PGM, and PNM formats into the Vista format.  Use \fBvtopgm\fP to convert in
the other direction.
.SS "Detecting edges and linking them into edge lists:"
.rs
.IP
.nf
.B % vcanny image.v | vlink >edges.v
.fi
.PP
\fBvxviewe\fP can be used to display edge sets overlayed on images from which
they were obtained.  This can be done by running \fBvxviewe\fP on both files
and typing Ctrl-N to display the second file of edges on top of the image:
.IP
.nf
.B % vxviewe image.v edges.v
.fi
.SH WRITING YOUR OWN PROGRAMS
Vista has been specifically designed to make it as simple as possible to
add new modules to the system that integrate fully with all existing code.
All source code for the system is available free to the public, so you can
take the system to other sites and use whatever modules you have written.
A particularly useful piece of source code is the \fBVImageView\fP widget
that can display an image in a device- and size-independent way in an X
window.
.PP
There is a 30-page introduction to Vista programming in the directory
\fB$VISTA/doc/intro/\fP. The existing source code is in the
\fB$VISTA/src\fP directory.
.PP
There are some sample modules available in the \fB$VISTA/examples\fP
directory.  You can copy one of these examples into your own directory,
complete with \fBREADME\fP and \fBMakefile\fP, and modify it to create your
own program.  For example:
.IP
.nf
.B % cp -r $VISTA/examples/ramp \fImy-directory\fP
.fi
.PP
There are extensive manual pages available to describe the various library
routines and conventions used in Vista modules.  You should read the
\fBVlib\fP(7Vi) man page first, as this will tell you about some issues,
such as the C data types used in Vista programs, that you will need to know
immediately.
.PP
Following is a list of some manual pages for the Vista programmer
in rough order of importance:
.RS 2n
.na
.nh
.IP "Introduction:" 25n
\fBVista\fP(7Vi), \fBVlib\fP(7Vi)
.IP "File formats:"
\fBVfile\fP(5Vi), \fBVImage\fP(3Vi)
.IP "Programming conventions:"
\fBVattribute\fP(3Vi), \fBVoption\fP(3Vi)
.IP "Utility routines:"
\fBVError\fP(3Vi), \fBVWarning\fP(3Vi), \fBVMalloc\fP(3Vi), \fBVNew\fP(3Vi),
\fBVGetAttr\fP(3Vi), \fBVCreateAttrList\fP(3Vi)
.IP "Display widgets:"
\fBVImageView\fP(3Vi)
.IP "Image handling routines:"
\fBVCreateImage\fP(3Vi), \fBVCreateImageLike\fP(3Vi), \fBVCopyImage\fP(3Vi)
.IP "File I/O:"
\fBVReadFile\fP(3Vi), \fBVWriteFile\fP(3Vi), \fBVReadImages\fP(3Vi),
\fBVWriteImages\fP(3Vi)
.ad
.hy
.RE
.SH "THE VX LIBRARY"
Programming interactive graphical programs for the X Window system has a
very steep learning curve.  Vista includes a library called VX that
automates the task of creating X Window applications that display images
and lines, incorporate menus and dialog boxes, and allow for mouse
interaction.  See the \fBVX\fP(3Vi) manual page for further details.
.SH AUTHOR
David Lowe <lowe@cs.ubc.ca>
