/*
 *  $Id: ConvolveSep.c,v 1.1 1994/06/10 01:32:35 pope Exp pope $
 *
 *  Convolve an image with a separable filter.
 */

/*
 *  Copyright 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Arthur Pope, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include "vista/Vlib.h"
#include "vista/os.h"
#include "vista/VImage.h"

/* File identification string: */
VRcsId ("$Id: ConvolveSep.c,v 1.1 1994/06/10 01:32:35 pope Exp pope $");


/*
 *  VConvolveImageSep
 *
 *  Convolve an image with a separable filter.
 *  The filter is specified as separate band-wise, row-wise, and column-
 *  filters. Each is applied in sucession in accordance with its respective
 *  pad method and shift. If a specific band is specified, a one-band
 *  image is produced by centering the kernel on that band of src.
 */

VImage VConvolveImageSep (VImage src, VImage dest, VBand band, VImage masks[3],
			  VConvolvePadMethod pad_methods[3], int shifts[3])
{
    int last_filter, i;
    VImage result, prev_image;

    /* Identify the last in the series of filters: */
    for (last_filter = 2; last_filter >= 0; last_filter--)
	if (masks[last_filter])
	    break;
    if (last_filter == -1) {
	VWarning ("VConvolveSep: No mask specified");
	return NULL;
    }

    result = src;
    for (i = 0; i < 3; i++) {
	prev_image = result;
	if (! masks[i])
	    continue;
	result = VConvolveImage (prev_image, (i == last_filter) ? dest : NULL,
				 band, masks[i], pad_methods[i], shifts[i]);
	if (prev_image != src)
	    VDestroyImage (prev_image);
	band = VAllBands;
    }

    return result;
}
