/*
 * $Id: EdgesToPS.c,v 1.6 1994/06/15 23:14:49 pope Exp pope $
 *
 *  This file contains routines for rendering Vista edges in PostScript.
 */

/*
 *  Copyright 1993, 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Daniel Ko, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include "vista/Vlib.h"
#include "vista/mu.h"
#include "vista/os.h"
#include "vista/postscript.h"
#include "vista/VEdges.h"

/* File identification string: */
VRcsId ("$Id: EdgesToPS.c,v 1.6 1994/06/15 23:14:49 pope Exp pope $");


/*
 *  VEdgesToPS 
 *
 * Render an edge set using PostScript.
 */

VBoolean VEdgesToPS (FILE *f, VEdges edge_set, VBooleanPromoted endpoints)
{
    VEdge e;
    VFloat *point, nrows = VEdgesNRows (edge_set);
    int i, j, k, n;

    /* Draw each edge as a poly-line: */
    for (e = VFirstEdge (edge_set); VEdgeExists (e); e = VNextEdge (e)) {
	if (VEdgeNPoints (e) == 0)
	    continue;
	point = VEdgePointArray (e)[0];
	fprintf (f, "np %g %g mt", nrows - point[1], point[0]);

	/* Stack points in reverse order, as required by the `pl' PostScript
	   procedure for drawing polylines. Stack no more than 100 at
	   a time. */
	for (j = 1; j < VEdgeNPoints (e); j += n) {
	    n = VMin (VEdgeNPoints (e) - j, 100);

	    /* j indexes the next point to be drawn */
	    /* n is the number of points to be drawn in this batch */
	    
	    for (i = j + n - 1, k = 0; i >= j; i--, k++) {
		point = VEdgePointArray (e)[i];
		fprintf (f, "%c%g %g", k % 5 ? ' ' : '\n',
			 nrows - point[1], point[0]);
	    }
	    fprintf (f, "\n%d pl", n);
	}

	if (VEdgeClosed (e))
	    fputs (" cp", f);
	fputs (" st\n", f);

	if (endpoints && ! VEdgeClosed (e)) {
	    point = VEdgePointArray (e)[0];
	    fprintf (f, "%g %g ep\n", nrows - point[1], point[0]);
	    point = VEdgePointArray (e)[VEdgeNPoints (e) - 1];
	    fprintf (f, "%g %g ep\n", nrows - point[1], point[0]);
	}
    }

    return TRUE;
}
