/*
 *  $Id: ForkImageDpy.c,v 1.1 1994/06/10 01:32:49 pope Exp pope $
 *
 *  This file contains a function for forking a copy of vxviewi to display
 *  an image.
 */

/*
 *  Copyright 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Art Pope, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include "vista/Vlib.h"
#include "vista/mu.h"
#include "vista/os.h"
#include "vista/VImage.h"

/* File identification string: */
VRcsId ("$Id: ForkImageDpy.c,v 1.1 1994/06/10 01:32:49 pope Exp pope $");


/*
 *  VForkImageDisplay
 *
 *  Fork a process to display an image. Returns the process id of
 *  the process if successful, and 0 otherwise.
 */

pid_t VForkImageDisplay (VImage image, VStringConst program,
			 VStringConst title)
{
    char *filename;
    FILE *f;
    VBoolean result;
    int pid;
    
    filename = tmpnam (NULL);
    if ((f = fopen (filename, "w")) == NULL) {
	VWarning ("VForkImageDisplay: Couldn't open tmp file %s", filename);
	return FALSE;
    }
    result = VWriteImages (f, NULL, 1, & image);
    fclose (f);
    if (! result)
	return FALSE;
    
    switch (pid = fork ()) {
	
    case -1:
	VSystemWarning ("VForkImageDisplay: fork() failed");
	return 0;
	
    case 0:
	/* Child process. */
	execlp (program, program, "-in", filename,
		"-title", title ? title : "Image", (char *) 0);
	VSystemError ("VForkImageDisplay: Couldn't execute %s", program);
	/* UNREACHED */
	
    default:
	return pid;
    }
}
