/*
 *  $Id: Sample.c,v 1.1 1994/06/10 01:33:17 pope Exp pope $
 *
 *  This file contains routines for subsampling images.
 */

/*
 *  Copyright 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Art Pope, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include "vista/Vlib.h"
#include "vista/mu.h"
#include "vista/os.h"
#include "vista/VImage.h"

/* File identification string: */
VRcsId ("$Id: Sample.c,v 1.1 1994/06/10 01:33:17 pope Exp pope $");


/*
 *  VSampleImage
 *
 *  Sample the source image, selecting every row_scale'th pixel in the row
 *  direction and every column_scale'th pixel in the column direction.,
 */

VImage VSampleImage (VImage src, VImage dest, VBand band,
		     int row_scale, int column_scale)
{
    int dest_nrows, dest_ncolumns, src_band, dest_band, i, j;

    #define Sample(type)						\
    {									\
        type *dest_pp = VPixelPtr (dest, dest_band, 0, 0);		\
        for (i = 0; i < VImageNRows (src); i += row_scale)		\
	    for (j = 0; j < VImageNColumns (src); j += column_scale)	\
		*dest_pp++ = VPixel (src, src_band, i, j, type);	\
    }

    /* Ensure that the specified band exists: */
    if (! VSelectBand ("VSampleImage", src, band, NULL, NULL))
	return NULL;

    /* Validates row_scale and column_scale: */
    if (row_scale <= 0 || column_scale <= 0) {
	VWarning ("VSampleImage: Scaling factors are not positive");
	return NULL;
    }
    
    /* If scales are 1, just copy: */
    if (row_scale == 1 && column_scale == 1)
	return VCopyImage (src, dest, band);
    
    /* Create or validate the destination image: */
    dest_nrows = (VImageNRows (src) + row_scale - 1) / row_scale;
    dest_ncolumns = (VImageNColumns (src) + column_scale - 1) / column_scale;
    dest = VSelectDestImage ("VSampleImage", dest,
			     band == VAllBands ? VImageNBands (src) : 1,
			     dest_nrows, dest_ncolumns, VPixelRepn (src));
    if (! dest)
	return NULL;

    /* Subsample band by band: */
    src_band = (band == VAllBands) ? 0 : band;
    for (dest_band = 0; dest_band < VImageNBands (dest);
	 src_band++, dest_band++)
	switch ((int) VPixelRepn (src)) {
	case VBitRepn: Sample(VBit); break;
	case VUByteRepn: Sample(VUByte); break;
	case VSByteRepn: Sample(VSByte); break;
	case VShortRepn: Sample(VShort); break;
	case VLongRepn:  Sample(VLong); break;
	case VFloatRepn: Sample(VFloat); break;
	case VDoubleRepn: Sample(VDouble); break;
	}

    VCopyImageAttrs (src, dest);
    return dest;

    #undef Sample
}
