/*
 *  $Id: postscript.h,v 1.7 1994/06/15 23:14:49 pope Exp pope $
 *
 *  Definitions associated with PostScript routines.
 */

#ifndef V_postscript_h
#define V_postscript_h 1

/*
 *  Copyright 1993, 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Daniel Ko, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include "vista/Vlib.h"

/* From the standard C library: */
#include <stdio.h>

/* For portability: */
#include <X11/Xfuncproto.h>

#ifdef __cplusplus
extern "C" {
#endif


/*
 *  Data structure recording page layout options and
 *  PostScript generation state.
 *
 *  (All linear measurements are in PostScript units.)
 */

typedef struct {
    VBoolean eps;		/* 0: PostScript; 1: Encapsulated PostScript */
    int npages;			/* number of pages processed */
    VStringConst prog_name;	/* program name */
    VBoolean landscape;		/* 0: portrait; 1: landscape */
    float page_width;		/* page dimensions */
    float page_height;
    float top_margin, bottom_margin; /* margin widths */
    float left_margin, right_margin;
    float frame_width;		/* frame width (negative: no frame) */
    float caption_height;	/* height of caption font */
    VStringConst caption;	/* caption text */
    VBoolean bb_set;		/* whether bounding box has been noted */
    double bb_top, bb_bottom;	/* bounding box dimensions */
    double bb_left, bb_right;
} VPSDescRec;


/*
 *  Assorted constants.
 */

/* Name and version numbers of PostScript proc set used by Vista: */
#define VPSProcSetName		"vista_procs"
#define VPSProcSetVersion	"1.1"
#define VPSProcSetRevision	"1.1"

/* Font and spacing for drawing caption: */
#define VPSCaptionFont		"Times-Roman"
#define VPSAboveCaption		1.0	/* lines of space above and below */
#define VPSBelowCaption		1.0


/*
 *  Declarations of library routines.
 */

/* From PostScript.c: */

extern VBoolean VParsePSOptions (
#if NeedFunctionPrototypes
    int *		/* argc */,
    char **		/* argv */,
    VPSDescRec *	/* ps */
#endif
);

VBoolean VParsePSDistance (
#if NeedFunctionPrototypes
    VStringConst	/* str */,
    VStringConst	/* keyword */,
    float *		/* value */
#endif
);				

extern void VReportValidPSOptions (
#if NeedFunctionPrototypes
    void
#endif
);

extern void VBeginPSDoc (
#if NeedFunctionPrototypes
    FILE *		/* f */,
    VPSDescRec *	/* ps */,
    VBooleanPromoted	/* eps */,
    VStringConst	/* title */
#endif
);

extern void VEndPSDoc (
#if NeedFunctionPrototypes
    FILE *		/* f */,
    VPSDescRec *	/* ps */
#endif
);

extern void VEndPSHeader (
#if NeedFunctionPrototypes
    FILE *		/* f */,
    VPSDescRec *	/* ps */
#endif
);

extern void VBeginPSPage (
#if NeedFunctionPrototypes
    FILE *		/* f */,
    VPSDescRec *	/* ps */
#endif
);

extern void VEndPSPage (
#if NeedFunctionPrototypes
    FILE *		/* f */,
    VPSDescRec *	/* ps */
#endif
);

extern VBoolean VIncludePSFile (
#if NeedFunctionPrototypes
    FILE *		/* f */,
    VStringConst	/* include file */
#endif
);

extern VBoolean VIncludePSProcSet (
#if NeedFunctionPrototypes
    FILE *		/* f */,
    VPSDescRec *	/* ps */,
    VStringConst	/* filename */,
    VStringConst	/* proc_set_name */,
    VStringConst	/* version */,
    VStringConst	/* revision */
#endif
);

extern void VExtendPSBoundingBox (
#if NeedFunctionPrototypes
    VPSDescRec *	/* ps */,
    double		/* left */,
    double		/* bottom */,
    double		/* right */,
    double		/* top */
#endif
);

extern VBoolean VBeginVPSDoc (
#if NeedFunctionPrototypes
    FILE *		/* f */,
    VPSDescRec *	/* ps */,
    VBooleanPromoted	/* eps */,
    VStringConst	/* title */
#endif
);

extern void VEndVPSDoc (
#if NeedFunctionPrototypes
    FILE *		/* f */,
    VPSDescRec *	/* ps */
#endif
);

extern VBoolean VIncludeVPSProcSet (
#if NeedFunctionPrototypes
    FILE *		/* f */,
    VPSDescRec *	/* ps */
#endif
);

extern void VBeginVPSPage (
#if NeedFunctionPrototypes
    FILE *		/* f */,
    VPSDescRec *	/* ps */,
    double		/* nrows */,
    double		/* ncolumns */
#endif
);

extern void VEndVPSPage (
#if NeedFunctionPrototypes
    FILE *		/* f */,
    VPSDescRec *	/* ps */
#endif
);

#ifdef __cplusplus
}
#endif

#endif /* V_postscript_h */
