.ds Vn 2.1
.TH VAdjustImage 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VAdjustImage \- adjust image brightness and contrast
.SH SYNOPSIS
.nf
.B #include <vista/VImage.h>
.PP
.ft B
VImage VAdjustImage (\fIsrc\fP, \fIdest\fP, \fIband\fP, \fIb\fP, \fIc\fP)
.RS
VImage \fIsrc\fP, \fIdest\fP;
VBand \fIband\fP;
double \fIb\fP, \fIc\fP;
.RE
.fi
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies the image to operate on (the source image).
.IP \fIdest\fP
May specify a destination image, or may be
.SB NULL
to indicate that a new destination image should be created.
.IP \fIband\fP
May specify a particular band of the source image to be operated on,
or may be the constant \fBVAllBands\fP to indicate that all bands 
of the source image should be operated on.
.IP \fIb\fP
Specifies a brightness adjustment. Positive values increate brightness;
negative ones decrease it; 0 causes no change in brightness.
.IP \fIc\fP
Specifies a contrast adjustment. Positive values increase contrast;
negative ones decrease it; 0 causes no change in contrast.
.SH DESCRIPTION
\fBVAdjustImage\fP adjusts the brightness and/or contrast of an image.
.PP
First brightness is adjusted by a transformation analogous to gamma
correction. Source pixel values are mapped linearly into the range [0,1],
then raised to an exponent:
.IP
\fIy\fP = \fIx\fP ** (2 ** -\fIb\fP)
.PP
where \fIx\fP is the source pixel value (already mapped to [0,1]) and
\fIy\fP is the brightness-adjusted pixel value.
.PP
Then contrast is adjusted by a linear scaling of pixel values centered
about 0.5:
.IP
\fIz\fP = (\fIy\fP - 0.5) * (2 ** \fIc\fP) + 0.5
.PP
Finally, the original linear mapping to [0,1] is reversed and pixel values
are clipped if they exceed the range permitted by the image's pixel
representation.
.PP
If a destination image is provided by the caller then it must have the same
numbers of rows and columns, and the same pixel representation, as the
source image; and it must have either the same number of bands as the
source image (when \fIband\fP is \fBVAllBands\fP) or a single band
(otherwise). If no destination image is provided, one with these properties
will be created and returned.
.SH "RETURN VALUES"
\fBVAdjustImage\fP returns the destination image if it is successful and 
.SB NULL
otherwise.
.SH "SEE ALSO"
.na
.nh
.BR VConvertImage (3Vi),
.BR VOpImage (3Vi),
.BR VImage (3Vi),
.BR Vlib (7Vi)
.ad
.hy
.SH DIAGNOSTICS
.IP "``Band \fIband\fP referenced in image of \fInbands\fP band(s).''"
The \fIband\fP argument is invalid given the number of bands in \fIsrc\fP.
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIsrc_prop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``bands'', ``rows'', or ``columns''.
A destination image was specified but it has the wrong properties.
.SH AUTHOR
Ralph Horstmann <horstman@cs.ubc.ca>
