.ds Vn 2.1
.TH VBuildComplexImage 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VBuildComplexImage \- build complex image from real and imaginary components
.SH SYNOPSIS
.nf
.B #include <vista/VImage.h>
.fi
.HP 10n
.na
.nh
.B VImage VBuildComplexImage (\fIr_src\fP, \fIr_band\fP,
.B \fIi_src\fP, \fIi_band\fP, \fIdest\fP, \fIpixel_repn\fP)
.hy
.nf
.PD 0
.PP
.PD
.RS
.B VImage \fIr_src\fP, \fIi_src\fP, \fIdest\fP;
.B VBand \fIr_band\fP, \fIi_band\fP;
.B VRepnKind \fIpixel_repn\fP;
.RE
.fi
.SH ARGUMENTS
.IP \fIr_src\fP 10n
Specifies an image containing real pixel components, or is
.SB NULL
to indicate that the real components should be zero.
.IP \fIr_band\fP
Specifies one band or all bands of \fIr_src\fP.
.IP \fIi_src\fP
Specifies an image containing imaginary pixel components, or is
.SB NULL
to indicate that the imaginary components should be zero.
.IP \fIr_band\fP
Specifies or band or all bands of \fIi_src\fP.
.IP \fIdest\fP
May specify a destination image, or may be
.SB NULL
to indicate that a new destination image should be created.
.IP \fIpixel_repn\fP
Specifies the pixel representation that the destination image is to have.
.SH DESCRIPTION
\fBVBuildComplexImage\fP builds a complex image from one or more bands
of real pixel components and/or imaginary pixel components.
.PP
The real components are obtained from \fIr_src\fP, if provided; the
imaginary components are obtained from \fIi_src\fP, if provided. At least
one of these two images must be provided, and if they are both provided
they must have the same numbers of rows and columns. If either is omitted,
the corresponding components are set to zero.
.PP
If \fIr_band\fP and \fIi_band\fP each specify a single band of their 
respective source images then the destination image will have two bands, 
one of real components and the other of imaginary. On the other hand, if 
\fIr_band\fP and \fIi_band\fP are \fBVAllBands\fP then the destination 
image will have twice as many bands as the \fIr_src\fP and/or \fIi_src\fP, 
with alternating bands of real and imaginary components. 
.PP
The \fIpixel_repn\fP argument specifies what kind of pixel representation
is to be used for the destination image; it is usually either
\fBVFloatRepn\fP or \fBVDoubleRepn\fP.
.PP
If \fIdest\fP is provided it must have the same number of rows
and columns as the source image(s), a pixel representation of
\fIpixel_repn\fP, and an appropriate number of bands. If
.SB NULL
is passed, instead, a suitable destination image will be created by
\fBVBuildComplexImage\fP.
.PP
The destination image will be given band interpretation attributes
describing it as a complex image (i.e., \fBncomponents:\ 2\fP and
\fBcomponent_interp: complex\fP). 
.SH "RETURN VALUES"
\fBVBuildComplexImage\fP returns the destination image if it is successful
and 
.SB NULL
otherwise.
.SH "SEE ALSO"
.BR VImage (3Vi),
.BR Vlib (7Vi)
.SH DIAGNOSTICS
.IP "``Neither real nor imaginary component given.''"
Both \fIr_src\fP and \fIi_src\fP are
.SB NULL\c
.
.IP "``Band \fIband\fP referenced in image of \fInbands\fP band(s).''"
Either \fIr_band\fP or \fIi_band\fP refers to a band not found in 
\fIr_src\fP or \fIi_src\fP, respectively.
.IP "``Destination image has \fIdest_nbands\fP bands; \fIsrc_nbands\fP expected.''"
A destination image was specified but it has the wrong number of bands.
.IP "``Real and imaginary images have different sizes.''"
\fIR_src\fP and \fIi_src\fP do not have the same numbers of rows and columns.
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIsrc_prop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``bands'', ``rows'', or ``columns''.
A destination image was specified but it does not have the correct pixel
representation, number of bands, number of rows, or number of columns.
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
