.ds Vn 2.1
.TH VCanny 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VCanny \- detect edges in an image using a Canny operator
.SH SYNOPSIS
.nf
.B #include <vista/VImage.h>
.PP
.ft B
VImage VCanny (\fIsrc\fP, \fIdest\fP, \fIband\fP, \fIsigma\fP, \fIori\fP, \
\fInoise\fP)
.RS
VImage \fIsrc\fP, \fIdest\fP, *\fIori\fP;
VBand \fIband\fP;
double \fIsigma\fP, *\fInoise\fP;
.RE
.fi
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies the source image.
.IP \fIdest\fP
May specify a destination image in which to return the edge magnitude map, 
or may be \fBNULL\fP to indicate that a new destination image should be 
created. 
.IP \fIband\fP
Specifies a particular band of the source image.
.IP \fIsigma\fP
Specifies the standard deviation of the Gaussian filter used in the edge 
operator. 
.IP \fIori\fP
May 
.RS 12n
.IP (a) 4n
point to an image in which to return the edge orientation map;
in this case *\fIori\fP should have a non-\fBNULL\fP value;
.IP (b)
point to a \fBNULL\fP value to indicate that a new image should be created 
for returning the edge orientation map; in this case *\fIori\fP should be 
\fBNULL\fP; or 
.IP (c)
be \fBNULL\fP to indicate that no edge orientation map is to be returned.
.RE
.IP \fInoise\fP
If not 
.SB NULL\c
, it specifies that an edge magnitude threshold is to be computed from an
estimate of noise in the image. It passes the fraction, \fIq\fP, of image
pixels to be considered noise, and returns the edge magnitude \fIr\fP such
that \fIq\fP of the pixels have edge magnitudes below \fIr\fP.
.SH DESCRIPTION
\fBVCanny\fP detects edges in an image by marking the local maxima of the
gradient of the Gaussian smoothed image. The magnitude of the gradient at
these local maxima is scaled to the range [1,255] to produce the edge
magnitude map, which is returned as the destination image. Pixels that are
not local maxima have values of zero in this map. If requested an edge
orientation map, indicating the edge orientation at the local maxima, is
also returned.
.PP
\fBVCanny\fP uses the non-maximum suppression technique described in 
Section 5.3 of J.F. Canny, ``Finding Edges and Lines in Images'', 
AI-TR-720, M.I.T.. A rotationally symmetric Gaussian filter is used, and the 
2D edge operator is implemented as two 1D filters to minimize computation. 
.PP
The argument \fIsrc\fP specifies the source image. The argument \fIband\fP 
specifies a single band of that image whose edges are to be detected. 
.PP
If \fIdest\fP specifies a destination image, it must have exactly one band, 
the same number of rows and columns as the source image, and \fBVUByte\fP 
pixels. If, on the other hand, \fIdest\fP is
.SB NULL\c
, \fBVCanny\fP first creates a destination image with the correct
properties.
.PP
The argument \fIsigma\fP specifies the standard deviation of the Gaussian 
filter used in the edge operator. It also determines the filter size, which 
is roughly 8 times \fIsigma\fP plus 1. 
.PP
If *\fIori\fP specifies an image in which to return the edge orientation 
map, it must have exactly one band, the same number of rows and columns as 
the source image, and \fBVFloat\fP pixels. If *\fIori\fP is \fBNULL\fP, 
\fBVCanny\fP first creates an image with the correct properties and stores 
it in *\fIori\fP. If \fIori\fP is \fBNULL\fP, no edge orientation map will be 
returned.
.PP
The orientation of an edge is defined as the direction of greatest increase
in image intensity. It is measured in radians, counterclockwise from the
positive x-axis (direction of increasing column index) and it is in the
range [-Pi,+Pi]. Locations that are not local edge maxima will have pixel
values of zero in the orientation map. Note that because zero is a legal
orientation value one cannot use the orientation map alone to identify edge
pixels.
.PP
If \fInoise\fP is not 
.SB NULL\c
, \fBVCanny\fP also computes and returns a threshold that can be used to
select the most significant edges in the image. The threshold is based on
an estimate of the noise in the image. It is the smallest value, \fIr\fP,
such that at least a fraction \fIq\fP of the pixels have edge magnitudes
less than \fIr\fP. A fraction \fIq\fP in the range [0,1] is passed in
*\fInoise\fP; the threshold \fIr\fP, which is derived from \fIq\fP and from
the gradient of the Gaussian smoothed image, is returned in *\fInoise\fP.
To perform Canny's hysteresis thresholding on the edge magnitude map
produced by \fBVCanny\fP, one typically uses \fIr\fP as the low threshold,
and 2\fIr\fP as the high threshold.
.SH "RETURN VALUES"
\fBVCanny\fP returns the destination image if it is successful and 
\fBNULL\fP otherwise. 
.SH "SEE ALSO"
.na
.nh
.BR VLinkImage (3Vi),
.BR VImage (3Vi),
.BR Vlib (7Vi)
.ad
.hy
.SH DIAGNOSTICS
.IP "``Band \fIband\fP referenced in image of \fInbands\fP bands.''"
The \fIband\fP argument is invalid given the number of bands in \fIsrc\fP.
.IP "``Cannot operate on all bands of the image.''"
The \fIband\fP  argument must specify a particular band, not
\fBVAllBands\fP.
.IP "``Destination image has \fIdest_nbands\fP bands; \fInbands\fP expected.''"
A destination image (or the image specified by *\fIori\fP) was specified but 
it has the wrong number of bands.
A destination image must have a single band.
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIprop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``rows'', or ``columns''.  A
destination image (or the image specified by *\fIori\fP) was specified but
it does not have the correct size or pixel representation.
.IP "``Noise estimation fraction (\fIq\fP) is outside [0,1].''"
The value passed in *\fInoise\fP is outside [0,1]; 0.6 will be used instead.
.IP "``Sigma \fIsigma\fP is not a positive number.''"
The \fIsigma\fP argument must be positive.
.SH AUTHORS
.na
.nh
Richard\ Pollock\ <pollock@cs.ubc.ca>,
Daniel\ Ko\ <ko@cs.ubc.ca>,
Art\ Pope\ <pope@cs.ubc.ca>
