.ds Vn 2.1
.TH VConvertImageCopy 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VConvertImageCopy \- convert an image's pixel representation
.SH SYNOPSIS
.PP
.nf
.B #include <vista/VImage.h>
.fi
.PP
.nf
.B VImage VConvertImageCopy (\fIsrc\fP, \fIdest\fP, \fIband\fP, \fIpixel_repn\fP)
.RS
.ft B
VImage \fIsrc\fP, \fIdest\fP;
VBand \fIband\fP;
VRepnKind \fIpixel_repn\fB;
.ft
.RE
.fi
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies the image to be converted.
.IP \fIdest\fP
May specify a destination image, or may be
.SB NULL
to indicate that a new destination image should be created.
.IP \fIband\fP
May specify a particular band of the source image to be converted,
or may be the constant \fBVAllBands\fP to indicate that all bands 
of the source image should be converted.
.IP \fIpixel_repn\fP
Specifies the pixel representation to which the source image should
be converted.
.SH DESCRIPTION
\fBVConvertImageCopy\fP converts a source image to a destination image
having the pixel representation specified by \fIpixel_repn\fP.  Each source
pixel value is simply copied to the destination image without being scaled
according to the destination image's representation.
.PP
A floating point pixel value is converted to an integer
representation by rounding it to the nearest integer. Also, if a pixel
value falls outside the range representable in the destination image, it is
clipped and a warning message is issued.
.PP
\fISrc\fP specifies the source image. \fIBand\fP specifies either a single band
of that image to be converted, or \fBVAllBands\fP if all bands are to be 
converted.
.PP
If \fIdest\fP specifies a destination image, it must have the appropriate
number of bands, the same number of rows and columns as the source
image, and a pixel representation of \fIpixel_repn\fP.
If, on the other hand, \fIdest\fP is 
.SB NULL\c
, \fBVConvertImageCopy\fP first creates a destination image with the correct
properties.
.SH "RETURN VALUES"
\fBVConvertImageCopy\fP returns the destination image if it is successful
and 
.SB NULL
otherwise.
.SH "SEE ALSO"
.na
.nh
.BR VConvertImageLinear (3Vi),
.BR VConvertImageOpt (3Vi),
.BR VConvertImageRange (3Vi),
.BR VImage (3Vi),
.BR Vlib (7Vi)
.hy
.ad
.SH DIAGNOSTICS
.IP "``Band \fIband\fP referenced in image of \fInbands\fP band(s).''"
The \fIband\fP argument is invalid given the number of bands in \fIsrc\fP.
.IP "``Destination image has \fIdest_nbands\fP bands; \fIsrc_nbands\fP expected.''"
A destination image was specified but it has the wrong number of bands.
If \fIband\fP is \fBVAllBands\fP, the destination image must have the same
number of bands as the source image. Otherwise, it must have a single
band.
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIsrc_prop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``rows'', or ``columns''.
A destination image was specified but it does not have the correct size or
pixel representation.
.IP "``Source pixel value exceeds destination range.''"
One or more pixel values were clipped because they fell outside the range
representable in the destination image.
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
