.ds Vn 2.1
.TH VConvertImageOpt 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VConvertImageOpt \- convert an image's pixel representation
.SH SYNOPSIS
.nf
.B #include <vista/VImage.h>
.PP
.ft B
VImage VConvertImageOpt (\fIsrc\fP, \fIdest\fP, \fIband\fP, \
\fIpixel_repn\fP, \fImethod\fP)
.RS
VImage \fIsrc\fP, \fIdest\fP;
VBand \fIband\fP;
VRepnKind \fIpixel_repn\fP;
int \fImethod\fP;
.RE
.fi
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies the image to be converted
.IP \fIdest\fP
May specify a destination image, or may be
.SB NULL
to indicate that a new destination image should be created.
.IP \fIband\fP
May specify a particular band of the source image to be converted,
or may be the constant \fBVAllBands\fP to indicate that all bands 
of the source image should be converted.
.IP \fIpixel_repn\fP
Specifies the pixel representation to which the source image should
be converted.
.IP \fImethod\fP
Specifies the mapping to use in converting source pixels
to destination pixels. Possible values are described below.
.SH DESCRIPTION
\fBVConvertImageOpt\fP converts a source image to a destination image
having the pixel representation specified by \fIpixel_repn\fP. The
conversion may be thought of as enhancing the contrast of the image 
because the actual range of source pixel
values is mapped to a full range of possible destination pixel values.
.PP
\fISrc\fP specifies the source image. \fIBand\fP specifies either a single band
of that image to be converted, or \fBVAllBands\fP if all bands are to be 
converted.
.PP
If \fIdest\fP specifies a destination image, it must have the appropriate 
number of bands, the same number of rows and columns as the source image, 
and a pixel representation of \fIpixel_repn\fP. If, on the other hand, 
\fIdest\fP is 
.SB NULL\c
, \fBVConvertImageOpt\fP first creates a destination image with the correct
properties.
.PP
The \fImethod\fP argument specifies the mapping between source and 
destination pixel values. In the description that follows the actual range 
of pixel values present in the source image is denoted 
[\fIAmin\fP,\ \fIAmax\fP] and the full range of pixel values capable of being 
represented in the destination image is denoted [\fIDmin\fP,\ \fIDmax\fP]. 
(For the \fBVFloat\fP and \fBVDouble\fP pixels the range 
[\fIDmin\fP,\ \fIDmax\fP] is taken to be [-1,\ 1].) 
Define \fIAabs\fP\ =\ max (\ |\fIAmin\fP|,\ |\fIAmax\fP|\ ).
.PP
The \fImethod\fP argument takes one of three values:
.RS 2n
.IP 1 2n
This enhances the contrast of an image while increasing all intensity
levels by a constant multiple. It maps [-\fIAabs\fP,\ \fIAabs\fP] to 
[-\fIDmax\fP,\ \fIDmax\fP].
.IP 2
This enhances the contrast of an image. It not only increases intensity
levels by a constant multiple, it also shifts the range of intensity levels
by subtracting a constant term. The exact mapping depends on the
values of \fIAmin\fP and \fIAmax\fP:
.RS 4n
.IP "when \fIAmin\fP = \fIAmax\fP < 0" 25n
each destination pixel is \fIDmin\fP
.IP "when \fIAmin\fP = \fIAmax\fP = 0"
each destination pixel is 0
.IP "when 0 < \fIAmin\fP = \fIAmax\fP"
each destination pixel is \fIDmax\fP
.IP "when \fIAmin\fP < \fIAmax\fP < 0"
it maps [\fIAmin\fP, \fIAmax\fP] to [\fIDmin\fP, -\fIe\fP]
.IP "when \fIAmin\fP < \fIAmax\fP = 0"
it maps [\fIAmin\fP, 0] to [\fIDmin\fP, 0]
.IP "when \fIAmin\fP < 0 < \fIAmax\fP"
it maps [-\fIAabs\fP, \fIAabs\fP] to [-\fIDmax\fP, \fIDmax\fP]
.IP "when 0 = \fIAmin\fP < \fIAmax\fP"
it maps [0, \fIAmax\fP] to [0, \fIDmax\fP]
.IP "when 0 < \fIAmin\fP < \fIAmax\fP"
it maps [\fIAmin\fP, \fIAmax\fP] to [\fIe\fP, \fIDmax\fP]
.RE
.IP
Here, \fIe\fP is 1 / (\fIAmax\fP - \fIAmin\fP + 1) if destination pixels 
are \fBVFloat\fP or \fBVDouble\fP, and it is 1 otherwise. 
.IP 3
This enhances the contrast of an image but, unlike \fImethod\fP\ =\ 1 and
\fImethod\fP\ =\ 2 it doesn't necessarily preserve the signs of pixel values.
The exact mapping depends on the values of \fIAmin\fP and \fIAmax\fP:
.RS 4n
.IP "when \fIAmin\fP = \fIAmax\fP < 0" 25n
each destination pixel is \fIDmin\fP
.IP "when \fIAmin\fP = \fIAmax\fP = 0"
each destination pixel is 0
.IP "when 0 < \fIAmin\fP = \fIAmax\fP"
each destination pixel is \fIDmax\fP
.IP "when \fIAmin\fP < \fIAmax\fP"
it maps [\fIAmin\fP, \fIAmax\fP] to [\fIDmin\fP, \fIDmax\fP]
.RE
.PP
When floating point pixel values are converted to an integer representation
each is rounded to the nearest integer after being mapped. Also, if a
destination pixel value falls outside the range [\fIDmin\fP,\ \fIDmax\fP],
it is set to \fIDmin\fP or \fIDmax\fP and a warning message is issued.
.SH "RETURN VALUES"
\fBVConvertImageOpt\fP returns the destination image if it is successful
and 
.SB NULL
otherwise.
.SH "SEE ALSO"
.na
.nh
.BR VConvertImageCopy (3Vi),
.BR VConvertImageLinear (3Vi),
.BR VConvertImageRange (3Vi),
.BR VImage (3Vi),
.BR Vlib (7Vi)
.hy
.ad
.SH DIAGNOSTICS
.IP "``Band \fIband\fP referenced in image of \fInbands\fP band(s).''"
The \fIband\fP argument is invalid given the number of bands in \fIsrc\fP.
.IP "``Destination image has \fIdest_nbands\fP bands; \fIsrc_nbands\fP expected.''"
A destination image was specified but it has the wrong number of bands.
If \fIband\fP is \fBVAllBands\fP, the destination image must have the same
number of bands as the source image. Otherwise, it must have a single
band.
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIsrc_prop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``rows'', or ``columns''.
A destination image was specified but it does not have the correct size or
pixel representation.
.IP "``Pixel value exceeds destination range.''"
One or more pixel values were clipped because they fell outside the range
representable in the destination image.
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
