.ds Vn 2.1
.TH VConvolveImage 3Vi "11 April 1994" "Vista Version \*(Vn"
.SH NAME
VConvolveImage \- convolve an image with a mask
.SH SYNOPSIS
.nf
.B #include <vista/VImage.h>
.fi
.HP 10n
.na
.nh
.ft B
VImage VConvolveImage (VImage\ \fIsrc\fP, VImage\ \fIdest\fP,
VBand\ \fIband\fP, VImage\ \fImask\fP, VConvolvePadMethod\ \fIpad_method\fP, 
int\ \fIshift\fP);
.ft
.hy
.nf
.PP
.B "typedef enum {"
.RS
.ta 20n
\fBVConvolvePadNone\fP	/* set border pixels to zero */
\fBVConvolvePadZero,\fP	/* pad with zero */
\fBVConvolvePadBorder,\fP	/* pad with border pixel values */
\fBVConvolvePadWrap\fP	/* pad with wrapped around values */
\fBVConvolvePadTrim\fP	/* don't pad, trim destination image */
.DT
.RE
.B "} VConvolvePadMethod;"
.fi
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies one of the two images to be convolved (the source).
.IP \fIdest\fP
Specifies a destination image, or is
.SB NULL
to indicate that a new destination image should be created.
.IP \fIband\fP
Specifies a particular band of the source image to be convolved, or is
\fBVAllBands\fP to indicate that all bands should be convolved.
.IP \fImask\fP
Specifies the other image to be convolved (the mask).
.IP \fIpad_method\fP
Specifies how the source image is to be padded beyond its borders when
computing destination pixel values near the borders.
.IP \fIshift\fP
Specifies an amount by which to right-shift each destination pixel value
before storing it.
.SH DESCRIPTION
\fBVConvolveImage\fP convolves a source image with a mask image to produce
a destination image.
.PP 
The convolution is performed in the spatial domain using inner products.  
Conceptually, a destination pixel value is computed by centering the mask 
over the corresponding source pixel. Then each mask pixel value is 
multiplied by its underlying source pixel value, and these terms are summed 
to produce the destination pixel value.
.PP
The mask can have any number of bands, rows, and columns. If the size of 
a mask dimension is \fIm\fP, the mask's center pixel has an index in that 
dimension of floor(\fIm/2\fP).
.PP
The routine supports several alternative methods for computing pixels near 
the border of a destination image, where the mask extends beyond the source 
image.  These include padding the source image with zeros, ``wrapping'' the 
source image, and producing a destination image that is trimmed of any 
pixels so affected.  The \fIpad_method\fP argument determines which method 
is used:
.RS 2n
.IP \fBVConvolvePadBorder\fP 20n
Pad the source image using values from the nearest border pixels. 
.IP \fBVConvolvePadNone\fP
The source image isn't padded. Instead, affected destination pixels near the 
border are set to zero.
.IP \fPVConvolvePadTrim\fP
The source image isn't padded. Instead, the destination image is trimmed so 
that it includes none of the affected border pixels.
.IP \fBVConvolvePadWrap\fP
Wrap the source image around on itself
(e.g., the top-left pixel is padded above using the value of the
bottom-left pixel).
.IP \fBVConvolvePadZero\fP 20n
Pad the input image with zeros.
.RE
.PP
To avoid overflow, each destination pixel value can be scaled down by a 
specified amount before it is stored. The \fIshift\fP argument determines 
the amount of scaling.
.PP
In general, the destination image has the same size and pixel representation 
as the source image. There are three exceptions:
.IP \(bu 2n
If the source image has unsigned pixels and the mask has signed pixels, 
then the destination image has signed pixels.  Specifically, if the 
source image has \fBbit\fP or \fBubyte\fP pixels and the mask has something 
other than \fBbit\fP or \fBubyte\fP pixels, then the destination image has 
\fBsbyte\fP pixels.
.IP \(bu
If \fBVConvolvePadTrim\fP is specified, then the destination image is
smaller than the source image by an amount that depends on the mask size.  
If the size of an source image dimension is \fIn\fP and the size of the 
corresponding mask dimension if \fIm\fP, then the size of that destination 
image dimension is \fIn\fP-\fIm\fP+1.
.IP \(bu
If \fIband\fP specifies a particular band (rather than \fBVAllBands\fP),
then the destination image has a single band whose pixel values are 
computed by centering the mask on the specified band of the source image.
.SH "RETURN VALUES"
\fBVConvolveImage\fP returns the destination image if it is successful and
.SB NULL
otherwise.
.SH NOTES
The operation performed by \fBVConvolveImage\fP is correctly termed 
cross-correlation rather than convolution, but it is commonly referred
to as convolution. A proper convolution can be be performed by first 
reflecting the mask about each of its dimensions (see \fBVFlipImage\fP(3Vi)).
.PP
If both source and mask images have integer pixel representations, pixel 
arithmetic is done using \fBVLong\fP values; otherwise \fBVDouble\fP vales 
are used (see \fBVlib\fP(7Vi), ``Numeric Data Types'').
.PP
In ANSI C, the result of right-shifting a negative signed integer is
implementation dependent. Generally a one bit is shifted into the sign
position so that the shift is equivalent to a division by two. This is the
case on all platforms we've tested, but ANSI C doesn't insist on that
behavior.
.SH "SEE ALSO"
.na
.nh
.BR VConvolveImageSep (3Vi),
.BR VGaussianConvolveImage (3Vi),
.BR VImage (3Vi),
.BR Vlib (7Vi)
.ad
.hy
.SH DIAGNOSTICS
.IP "``Band \fIband\fP referenced in image of \fInbands\fP band(s).''"
The \fIband\fP argument is invalid given the number of bands in \fIsrc\fP,
the padding method, and the number of bands in \fImask\fP.
.IP "``Destination image has \fIdest_nbands\fP bands; \fIsrc_nbands\fP expected.''"
A destination image was specified but it has the wrong number of bands.
If \fIband\fP is \fBVAllBands\fP, the destination image must have the same
number of bands as the source image. Otherwise, it must have a single
band.
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIsrc_prop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``rows'', or ``columns''.
A destination image was specified but it does not have the same
pixel representation, number of rows, and number of columns as the
source image.
.IP "``Destination pixel value(s) clipped.''"
One or more pixel values were clipped because they fell outside the range
representable in the destination image.
.IP "``Image (\fIa\fPx\fIb\fP) is smaller than mask (\fIc\fIx\fId\fP).''"
\fBVConvolvePadTrim\fP was specified with a source image smaller than the
mask.
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
