.ds Vn 2.1
.TH VFlipImage 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VFlipImageHorizontal, VFlipImageVertical \- flip image horizontally or vertically
.SH SYNOPSIS
.nf
.ft B
#include <vista/VImage.h>
.PP
.ft B
VImage VFlipImage (\fIsrc\fP, \fIdest\fP, \fIband\fP, \fIvertical\fP)
.RS
VImage \fIsrc\fP, \fIdest\fP;
VBand \fIband\fP;
VBoolean \fIvertical\fP;
.RE
.fi
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies the image to be flipped.
.IP \fIdest\fP
May specify a destination image, or may be
.SB NULL
to indicate that a new destination image should be created.
.IP \fIband\fP
May specify a particular band of the source image to be flipped,
or may be the constant \fBVAllBands\fP to indicate that all bands 
of the source image should be flipped.
.IP \fIvertical\fP
Specifies whether the image is to be flipped horizontally (about a vertical
centerline) or vertically (about a horizontal centerline).
.SH DESCRIPTION
If \fIvertical\fP is
.SB FALSE\c
, \fBVFlipImage\fP flips an image horizontally so that the left side of the
original image becomes the right side of the flipped image, etc.
.PP
If, on the other hand, \fIvertical\fP is
.SB TRUE\c
, \fBVFlipImage\fP flips an image vertically so that such that the top of the
original image becomes the bottom of the flipped image, etc.
.PP
\fISrc\fP specifies the source image. \fIBand\fP either specifies a single 
band of the image to be flipped, or has the value \fBVAllBands\fP if all 
bands are to be flipped. 
.PP
\fIDest\fP specifies where to put the flipped image. If \fIdest\fP is not 
\fBNULL\fP, then the image it refers to must have the following properties: 
.RS 2n
.IP \(bu 2n
the same number of rows, number of columns, and pixel representation as 
\fIsrc\fP 
.IP \(bu
the same number of bands as \fIsrc\fP if \fIband\fP equals \fBVAllBands\fP, 
or exactly one band otherwise 
.PP
.RE
If \fIdest\fP is \fBNULL\fP, then a new image with the above properties 
will be created to hold the flipped image. 
.SH "RETURN VALUES"
\fBVFlipImage\fP returns the destination image if successful and
.SB NULL
otherwise.
.SH "SEE ALSO"
.na
.hy
.BR VRotateImage (3Vi),
.BR VTransposeImage  (3Vi),
.BR VImage (3Vi),
.BR Vlib (7Vi)
.ad
.hy
.SH DIAGNOSTICS
.IP "``Band \fIband\fP referenced in image of \fInbands\fP bands.''"
The \fIband\fP argument is invalid given the number of bands in \fIsrc\fP.
.IP "``Destination image has \fIdest_nbands\fP bands; \fIsrc_nbands\fP expected.''"
A destination image was specified but it has the wrong number of bands.
If \fIband\fP is \fBVAllBands\fP, the destination image must have the same
number of bands as the source image. Otherwise, it must have a single
band.
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIsrc_prop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``rows'', or ``columns''.
A destination image was specified but it does not have the same
pixel representation, number of rows, and number of columns as the
source image.
.SH AUTHOR
Daniel Ko <ko@cs.ubc.ca>
