.ds Vn 2.1
.TH VGaussianConvolveImage 3Vi "11 April 1994" "Vista Version \*(Vn"
.SH NAME
VGaussianConvolveImage \- convolve an image with a Gaussian filter or its derivative
.SH SYNOPSIS
.nf
.B "#include <vista/VImage.h>"
.HP 10n
.na
.nh
.ft B
VImage VGaussianConvolveImage (VImage\ \fIsrc\fP, VImage\ \fIdest\fP,
VBand\ \fIband\fP, double\ \fIsigma\fP, int\ \fIfilter_size\fP,
VFilterKind\ \fIfilter_kind\fP);
.ft
.hy
.ad
.PP
.B "typedef enum {"
.RS
.ta 20n
\fBVFilterGaussian,\fP	/* 2D Gaussian smoothing */
\fBVFilterGaussianDx,\fP	/* d / dX */
\fBVFilterGaussianDy\fP	/* d / dY */
.DT
.RE
.B "} VFilterKind;"
.fi
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies the source image to be convolved.
.IP \fIdest\fP
May specify a destination image, or may be 
.SB NULL
to indicate that a new destination image should be created. 
.IP \fIband\fP
May specify a particular band of the source image, or may be 
\fBVAllBands\fP to specify all bands of the image. 
.IP \fIsigma\fP
Specifies the standard deviation of the Gaussian on which the filter is 
based (in pixels).
.IP \fIfilter_size\fP
Specifies the finite diameter at which the filter is truncated (in pixels).
.IP \fIfilter_kind\fP
Specifies the kind of filter. It's value must be one of:
.RS 12n
.IP \fBVFilterGaussian\fP 18n
to convolve with a circularly-symmetric Gaussian filter.
.IP \fBVFilterGaussianDx\fP
to convolve with the partial derivative with respect to x of a Gaussian
filter.
.IP \fBVFilterGaussianDy\fP
to convolve with the partial derivative with respect to y of a Gaussian
filter.
.RE
.RE
.SH DESCRIPTION
\fBVGaussianConvolveImage\fP convolves an image with a Gaussian filter or a 
filter that is the partial derivative of a Gaussian. The 2D filter is 
implemented as two 1D filters to minimize computation.
.PP
If \fIdest\fP specifies a destination image, it must have the appropriate 
number of bands, and the same numbers of rows and columns as the source 
image.  In most cases, \fIdest\fP should also have the same pixel 
representation as \fIsrc\fP.  However, if \fIfilter_type\fP is either 
\fBVFilterGaussianDx\fP or \fBVFilterGaussianDy\fP \fIand\fP the source 
image has either \fBVBit\fP or \fBVUByte\fP pixels, then the destination 
image must have \fBVSByte\fP pixels.  When \fIdest\fP is 
.SB NULL\c
, \fBVGaussianConvolveImage\fP creates and returns a  destination image 
with the correct properties.
.PP
The \fIsigma\fP argument specifies the standard deviation of the Gaussian 
used, and \fIfilter_size\fP specifies the diameter at which the filter is 
truncated. For good results is should be at least 6*\fIsigma\fP, and odd.
.PP
When computing destination pixel values near the image borders, the
region outside of the source image is padded with border pixel values.
.SH "RETURN VALUES"
\fBVGaussianConvolveImage\fP returns the destination image if it is 
successful and 
.SB NULL
otherwise.
.SH "SEE ALSO"
.na
.nh
.BR VConvolveImage (3Vi),
.BR VImage (3Vi),
.BR Vlib (7Vi)
.hy
.ad
.SH DIAGNOSTICS
.IP "``Band \fIband\fP referenced in image of \fInbands\fP bands.''"
The \fIband\fP argument is invalid given the number of bands in \fIsrc\fP.
.IP "``Destination image has \fIdest_nbands\fP bands; \fInbands\fP expected.''"
A destination image was specified but it has the wrong number of bands. If 
\fIband\fP is \fBVAllBands\fP, the destination image must have the same 
number of bands as the source image. Otherwise, it must have a single band. 
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIprop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``rows'', or ``columns''. A 
destination image was specified but it does not have the correct size or 
pixel representation. 
.IP "``Filter size (\fIfilter_size\fP) is not positive.''"
The \fIfilter_size\fP argument must be greater than zero.
.IP "``Sigma (\fIsigma\fP) is not positive.''"
The \fIsigma\fP argument must be greater than zero.
.IP "``Unknown filter type (\fIfilter_type\fP).''"
The \fIfilter_type\fP argument is not one of the values listed above.
.SH AUTHOR
Daniel Ko <ko@cs.ubc.ca>
