.ds Vn 2.1
.TH VImage 3Vi "12 April 1994" "Vista Version \*(Vn"
.SH NAME
VImage \- image representation
.SH SYNOPSIS
.nf
.B #include <vista/VImage.h>
.PP
.B VImage \fIimage\fP;
.fi
.SH DESCRIPTION
.SS Introduction
Vista represents an image as one or more two-dimensional arrays of pixel
values. Each two-dimensional array, called a \fIband\fP, has the same size,
which is expressed as a number of rows and a number of columns. Multiple
bands may, for example, be used to represent multiple color components,
multiple frames of a motion sequence, the real and imaginary components of
a complex image, or some combination of these. Besides pixel data, an image
may also has an arbitrary list of attributes associated with it.
.PP
An image in memory is referred to by the C pointer type \fBVImage\fP. In 
data files it's identified by the type name \fBimage\fP, and in memory, by 
the \fBVRepnKind\fP code \fBVImageRepn\fP. Since it is a standard object 
type with built-in support in the Vista library, images can be read from 
data files, written to data files, and manipulated as part of attribute 
lists by routines such as \fBVReadFile\fP(3Vi), \fBVWriteFile\fP(3Vi), and 
\fBVGetAttr\fP(3Vi). 
.SS "Creating and Destroying Images"
The following routines create and destroy images.
.HP 10n
.na
.nh
.B VImage VCreateImage (int \fInbands\fP, int\ \fInrows\fP, 
.B int\ \fIncolumns\fP, VPixelRepn \fIpixel_repn\fP)
.ad
.hy
.IP "" 0.5i
\fBVCreateImage\fP allocates memory for a new image with the specified
numbers of bands, rows, and columns. Its pixel values are not initialized,
and it is given an empty attribute list. The \fIpixel_repn\fP argument is
described below, under ``Pixel Representation''. (See
\fBVCreateImage\fP(3Vi) for details.)
.PP
.B void VDestroyImage (VImage \fIimage\fP)
.IP
\fBVDestroyImage\fP releases the memory occupied by \fIimage\fP, including
that occupied by its pixel data and attribute list.
.SS "Image Size"
An image's numbers of bands, row, and columns are sometimes abbreviated as 
\fBnbands\fP, \fBnrows\fP, and \fBncolumns\fP. The following macros access 
those numbers.
.PP
.B int VImageNBands (VImage \fIimage\fP)
.PP
.B int VImageNRows (VImage \fIimage\fP)
.PP
.B int VImageNColumns (VImage \fIimage\fP)
.IP
\fBVImageNBands\fP, \fBVImageNRows\fP, \fBVImageNColumns\fP return
the number of bands, rows, and columns that \fIimage\fP has. 
.PP
.B int VImageNPixels (VImage \fIimage\fP)
.IP
\fBVImageNPixels\fP returns the number of pixel values \fIimage\fP has 
altogether (i.e., the product \fBnbands\fP * \fBnrows\fP * \fBncolumns\fP). 
.SS "Pixel Representation"
An image's pixel values may have any of the following types: \fBVBit\fP, 
\fBVUByte\fP, \fBVSByte\fP, \fBVShort\fP, \fBVLong\fP, \fBVFloat\fP, or 
\fBVDouble\fP. These are described in the ``Numeric Data Types'' section of 
the \fBVlib\fP(7Vi) manual page. Variables meant to hold pixel values or 
pointers to pixel values should be defined using these same types (rather 
than built-in C types such as \fBshort\fP and \fBlong\fP) to ensure 
software portability. 
.PP
The following macros return information about an image's pixel 
representation: 
.PP
.B VRepnKind VPixelRepn (VImage \fIimage\fP)
.IP
\fBVPixelRepn\fP returns a code indicating \fIimage\fP's pixel 
representation. (See \fBVlib\fP(7Vi) for a definition of the 
\fBVRepnKind\fP codes.) If \fIimage\fP has \fBVBit\fP pixels, for example, 
it returns \fBVBitRepn\fP. 
.PP
.B size_t VPixelSize (VImage \fIimage\fP)
.IP
\fBVPixelSize\fP returns the size, in bytes, used for storing a single 
pixel of \fIimage\fP (i.e, a pixel within a single band, row, and column). 
This value depends on machine architecture. 
.PP
.B int VPixelPrecision (VImage \fIimage\fP)
.IP
\fBVPixelPrecision\fP returns the minimum size, in bits, needed for 
representing a single pixel of \fIimage\fP. This value does not depend on 
machine architecture, but only on the kind of pixel value representation 
used by \fIimage\fP. If \fIimage\fP has \fBVBit\fP pixels, for example, 
this macro returns 1. 
.PP
.B VStringConst VPixelRepnName (VImage \fIimage\fP)
.IP
\fBVPixelRepnName\fP returns a string describing the type of pixel value
representation used by \fIimage\fP. For example, if \fIimage\fP has
\fBVBit\fP pixels, the string ``bit'' is returned.
.PP
.B VDouble VPixelMaxValue (VImage \fIimage\fP)
.IP
\fBVPixelMaxValue\fP returns the maximum value that a pixel of \fIimage\fP
may have. For example, if \fIimage\fP has \fBVSByte\fP pixels, it returns
127.0.
.PP
.B VDouble VPixelMinValue (VImage \fIimage\fP)
.IP
\fBVPixelMinValue\fP returns the minimum value that a pixel of \fIimage\fP
may have. For example, if \fIimage\fP has \fBVSByte\fP pixels, it returns
-128.0.
.PP
.B size_t VImageSize (VImage \fIimage\fP)
.IP
\fBVImageSize\fP returns the size, in bytes, of \fIimage\fP's block of 
pixel data, which contains the values of all pixels in all bands. This 
value depends on machine architecture. 
.SS "Pixel Values"
A particular pixel value is indexed by a band number, a row number, and a
column number. Bands, rows, and columns are numbered consecutively from
zero. Rows number down and columns number to the right so that row 0,
column 0 is at the upper-left corner of the image.
.HP 10n
.na
.nh
.B \fItype\fP VPixel (VImage \fIimage\fP, int \fIband\fP,
.B int\ \fIrow\fP, int\ \fIcolumn\fP, \fItype\fP)
.ad
.hy
.IP "" 0.5i
\fBVPixel\fP accesses the pixel at a specified band, row and column in an 
image whose pixels are of a specified type. The pixel type, \fItype\fP, 
must be one of \fBVBit\fP, \fBVUByte\fP, \fBVSByte\fP, \fBVShort\fP, 
\fBVLong\fP, \fBVFloat\fP, or \fBVDouble\fP. The macro may be used either 
as an rvalue (to fetch the pixel's value) or as an lvalue (to modify it 
or obtain its address). The ``EXAMPLES'' section, below, demonstrates its 
use as an rvalue.
.HP 10n
.na
.nh
.B VPointer VPixelPtr (VImage \fIimage\fP, int \fIband\fP, 
.B int\ \fIrow\fP, int\ \fIcolumn\fP)
.ad
.hy
.PP
.RS
\fBVPixelPtr\fP returns the address of a pixel specified by its
band, row, and column coordinates. The pixel at that address can be fetched
or modified by first coercing the pointer to the appropriate type for the
pixel. For example:
.RS
.PP
.nf
.ft B
pixel = VPixelPtr (image, band, row, column);
if (VPixelRepn (image) == VBitRepn)
.RS
* (VBit *) pixel = new_value;
.RE
else \fR...
.fi
.RE
.RE
.PP
.nf
.B \fItype\fP *** VPixelArray (VImage \fIimage\fP, \fItype\fP)
.fi
.RS
.PP
\fBVPixelArray\fP returns a pointer that can be used to access any image
pixel by indexing it first with the pixel's band number, then its row
number, and then its column number. For example:
.RS
.PP
.nf
.B VUByte ***pixels = VPixelArray (image, VUByte);
.B pixels[band][row][column] = new_value;
.fi
.RE
.RE
.PP
.B VPointer VImageData (VImage \fIimage\fP)
.IP
\fBVImageData\fP returns a pointer to \fIimage\fP's block of pixel data. 
The block has the size \fBVImageSize\ (\fIimage\fB)\fR bytes.
.SS "Other Image Attributes"
In addition to its pixel values an image may have any number of other
attributes; they are represented as an attribute list.
.PP
.B VAttrList VImageAttrList (VImage \fIimage\fP)
.RS
.PP
\fBVImageAttrList\fP is a macro for accessing the attribute list associated
with an image. The macro may be used as an rvalue to reach attributes
within the list:
.RS
.PP
.nf
.B VGetAttr (VImageAttrList (image), VNameAttr, \fR...\fP)
.fi
.RE
.PP
or as an lvalue to manipulate the entire list:
.RS
.PP
.nf
.B VDestroyAttrList (VImageAttrList (image));
.B VImageAttrList (image) = VCopyAttrList (other_list);
.RE
.RE
.fi
.SS "Band Interpretation"
The bands of a multi-band image might represent the successive frames of a 
motion sequence, the left and right halves of a stereo pair, or the red, 
green, and blue channels of a color image. They might even represent a 
combination of such dimensions \(em for example, a stereo pair of color 
images. Because a great many different uses of the bands are possible, each 
image includes information describing how its bands are to be interpreted. 
This information assigns each band an interpretation in terms of four 
dimensions:
.RS 2n
.IP \fIframe\fP 10n
covers the various frames of a motion sequence
.IP \fIviewpoint\fP
covers, for example, the left and right channels of a stereo pair
.IP \fIcolor\fP
covers, for example, the red, green, and blue channels of a color image
.IP \fIcomponent\fP
covers, for example, the real and imaginary components of a complex image
.RE
.PP
Although these four dimensions do not account for all conceivable ways one 
might wish to use the bands of a multi-band image, they do cover most 
cases. The scheme can usually be adapted to cover cases not explicitly 
handled. For example, a collection of images obtained under various 
lighting conditions could be represented by using the frame dimension to 
index the lighting condition. 
.PP
Each image specifies the sizes of its four band interpretation dimensions. 
For a stereo pair of color images, for example, the size of the viewpoint 
dimension is two, the size of the color dimension is three, and the 
remaining dimensions have sizes of one. The dimension sizes are often 
abbreviated as \fBnframes\fP, \fBnviewpoints\fP, \fBncolors\fP, and 
\fBncomponents\fP. Their product should always equal the number of bands in 
the image. These macros exist for accessing the sizes: 
.RS
.PP
.B int VImageNFrames (VImage \fIimage\fP)
.PP
.B int VImageNViewpoints (VImage \fIimage\fP)
.PP
.B int VImageNColors (VImage \fIimage\fP)
.PP
.B int VImageNComponents (VImage \fIimage\fP)
.RE
.PP
Each macro may be used as rvalue to obtain the size of a dimension, or as 
an lvalue to set it. 
.PP
The four dimensions are mapped onto the single dimension, band, according 
to a particular ordering: component varies most rapidly, then color, then 
viewpoint, and finally frame. A macro is available for computing this 
mapping: 
.RS
.HP 10n
.na
.nh
.ft B
int VBandIndex (VImage \fIimage\fP, int \fIframe\fP, int\ \fIviewpoint\fP,
int\ \fIcolor\fP, int\ \fIcomponent\fP)
.ft
.ad
.hy
.RE
.PP
\fBVBandIndex\fP returns the index of the band that corresponds to a
particular combination of frame, viewpoint, channel, and component indices.
.PP
An image's attribute list may include attributes indicating how the image
employs the four band interpretation dimensions. One attribute for each
dimension provides the first level of description. Its value is an integer
code, with some standard codes defined for common uses. Below is a list of
the four attributes and their standard values. Shown in are both the
symbolic constants that can be used C programs to refer to the integer
values, and (in parentheses) the keywords used to represent the values
externally, in Vista data files.
.RS 4n
.PP
\fBframe_interp\fP, which currently has no standard values defined for it
.PP
\fBviewpoint_interp\fP, which has this standard value:
.RS 4n
.IP "\fBVBandInterpStereoPair\fP (\fBstereo_pair\fP)"
The two viewpoints represent (respectively) the left and right images of a 
stereo pair.
.RE
.PP
\fBcolor_interp\fP, which has these standard values:
.RS 4n
.IP "\fBVBandInterpRGB\fP (\fBrgb\fP)"
The three colors represent (respectively) red, green, and blue color 
channels.
.RE
.PP
\fBcomponent_interp\fP, which has these standard values:
.RS 4n
.IP "\fBVBandInterpComplex\fP (\fBcomplex\fP)"
The two components represent (respectively) the real and imaginary 
components of a complex image.
.IP "\fBVBandInterpGradient\fP (\fBgradient\fP)"
There is a single component representing gradient magnitude, two
components representing (respectively) the x (rightward) and y (upward)
components of gradient, or three representing the x, y, and z (forward in
band sequence) components of gradient.
.IP "\fBVBandInterpIntensity\fP (\fBintensity\fP)"
There is a single component representing image intensity. For integer pixel
representations, pixel values normally span the range between zero and the
maximum representable pixel values (e.g., [0,127] for \fBVSByte\fP pixels).
For floating point pixel representations, pixel values normally span the
range [\-1,+1].
.IP "\fBVBandInterpOrientation\fP (\fBorientation\fP)"
There is a single component representing orientation. Pixel values express
orientation in radians, counterclockwise from the direction of increasing
column number (rightward).
.RE
.RE
.PP
To prevent accidental misspellings of the attribute names you should use 
these symbolic constants: 
.RS
.PP
.nf
.ft B
#define VFrameInterpAttr "frame_interp"
#define VViewpointInterpAttr "viewpoint_interp"
#define VColorInterpAttr "color_interp"
#define VComponentInterpAttr "component_interp"
.ft
.fi
.RE
.PP
A dictionary, \fBVBandInterpDict\fP, defines the mapping between the band
interpretation codes (e.g., \fBVBandInterpComplex\fP) and their keywords
(e.g., \fBcomplex\fP):
.RS
.PP
.B VDictEntry VBandInterpDict[];
.RE
.PP
For each of the four band interpretation dimensions there is a routine you 
can use to simultaneously check both the dimension's size and, if present, 
its interpretation attribute. The \fBVBandInterp\fP(3Vi) man page describes 
the four routines. 
.SS "Pixel Aspect Ratio"
Pixel aspect ratio is defined as the ratio of pixel width to pixel height. 
An image may include, on its attribute list, an attribute specifying this 
ratio as a floating point value. The attribute has the name 
\fBpixel_aspect_ratio\fP and a symbolic constant exists for referring to 
it: 
.RS
.PP
.nf
.ft B  
#define VPixelAspectRatioAttr "pixel_aspect_ratio"
.fi
.RE
.PP
When the attribute is absent, a pixel aspect ratio of 1.0 is assumed.
.SS "Calling Conventions"
A Vista library routine that processes one image to produce another usually
takes three or more arguments, like this:
.RS
.HP 10n
.na
.nh
.B VImage VProcessImage (VImage\ \fIsrc\fP, VImage\ \fIdest\fP,
.B VBand\ \fIband\fP)
.ad
.hy
.RE
.PP
The first argument specifies the \fIsource\fP image to be processed. The
second specifies the \fIdestination\fP image, to contain the result. You
generally have three choices for how this destination image is provided:
.RS 2n
.IP a) 4n
you can have the routine create a destination image. If you pass
.SB NULL
for \fIdest\fP, the routine will create and return a destination image that
is appropriate for the source image and for the type of operation being
performed on it:
.RS
.PP
.B 	dest = VProcessImage (src, NULL, band);
.RE
.IP b)
you can supply a destination image provided it has the correct properties
(usually, an appropriate size and pixel representation):
.RS
.PP
.nf
.B 	dest = VCreateImageLike (src);
.B 	VProcessImage (src, dest, band);
.fi
.RE
.IP c)
you can specify the same image as both source and destination, in which 
case the source pixel values will be replaced by destination pixel values. 
This choice is only available for operations where the source and 
destination images have the same size and pixel representation. 
.RS
.PP
.B 	VProcessImage (src, src, band);
.RE
.RE
.PP
In all cases, the routine will return the destination image if successful,
and
.SB NULL
otherwise. Moreover, if
.SB NULL 
is returned, the routine will already have called \fBVWarning\fP(3Vi) to
issue a warning message describing the problem.
.PP
The third argument to most image processing routines, \fIband\fP, may
specify a particular band of the source image to be processed; or it may be
the constant \fBVAllBands\fP (defined as -1) to indicate that all bands of
the source image are to be processed. When a particular band is specified,
\fIband\fP must be at least 0 and less than the number of bands in the
source image; the destination image will usually have a single band to
contain the result. When \fBVAllBands\fP is specified, the destination
image will usually have the same number of bands as the source image.
.PP
Normally a routine will copy the source image's attribute list to the 
destination image, replacing any attributes the destination image had 
already. The sizes of the four band interpretation dimensions 
(\fBnframes\fP, \fBnviewpoints\fP, etc.) are also copied. When appropriate, 
however, a routine may delete or modify the attributes and dimensional 
sizes as it copies them from source to destination. For example, a routine 
that converts an RGB color image to a grey-scale one 
(\fBVRGBImageToGray\fP(3Vi)) expects a source image with the properties 
\fBncolors:\ 3\fP and \fBcolor_interp: rgb\fP; the destination image 
it produces has \fBncolors:\ 1\fP and no \fBcolor_interp\fP attribute. 
.SS "Image Representation in Memory"
.nf
.ft B
.ta 25n
typedef struct {
.ft
.RS
.ft B
int nbands;	/* number of bands */
int nrows;	/* number of rows */
int ncolumns;	/* number of columns */
VRepnKind pixel_repn;	/* representation of pixel values */
unsigned long flags;	/* various flags */
VAttrList attributes;	/* list of other image attributes */
VPointer data;	/* array of image pixel values */
VPointer *row_index;	/* ptr to first pixel of each row */
VPointer **band_index;	/* ptr to first row of each band */
int nframes;	/* number of motion frames */
int nviewpoints;	/* number of camera viewpoints */
int ncolors;	/* number of color channels */
int ncomponents;	/* number of vector components */
.ft
.RE
.ft B
} VImageRec, *VImage;
.DT
.ft
.fi
.PP
An image is represented in memory by a value of type \fBVImage\fP, which 
points to a \fBVImageRec\fP structure. Programmers using images will 
usually not need to access this structure's fields directly from their code 
since there are Vista library routines and macros available for most 
purposes. However, when debugging, one may occasionally need to examine a 
\fBVImageRec\fP structure's fields directly. The following paragraphs 
describe the fields.
.PP
The \fBnbands\fP, \fBnrows\fP, and \fBncolumns\fP fields of the structure 
describe how many bands, rows, and columns the image has. Its 
\fBpixel_repn\fP field specifies how the image's pixel values are 
represented; it contains one of the constants \fBVBitRepn\fP, 
\fBVUByteRepn\fP, \fBVSByteRepn\fP, \fBVShortRepn\fP, \fBVLongRepn\fP, 
\fBVFloatRepn\fP, or \fBVDoubleRepn\fP. Its \fBattributes\fP field contains 
the list of attributes associated with the image. 
.PP
An image's pixel values are stored in a contiguous block of memory where 
they are arranged as follows. Within each band pixel values are ordered, 
beginning with the pixel at row 0, column 0, proceeding to row 0, column 1, 
etc., and ending with the highest numbered row and column. Each band's 
array of pixel values occupies a separate block of memory, and these blocks 
are arranged contiguously. All of band 0's pixel values are first, followed 
by all of band 1's, etc. In summary, pixel values are ordered by band, 
then by row within each band, and then by column. The pixel values 
themselves are of type \fBVBit\fB, \fBVUByte\fP, ..., or \fBVDouble\fP.
.PP
The \fBdata\fP field of a \fBVImageRec\fP points to the block of memory
containing pixel values. The \fBrow_index\fP field points to a vector of
pointers to the first pixel value of each row within each band; this vector
is of length (\fBnbands\fP * \fBnrows\fP) and it is indexed by an
expression of the form (\fIband\fP\ *\ \fBnrows\fP\ +\ \fIrow\fP). The
\fBband_index\fP field points to a vector of pointers to the first
\fBrow_index\fP entry of each band; this vector is of length \fBnbands\fP
and it is indexed by band number.
.PP
Individual bits of a \fBVImageRec\fP's \fBflags\fP field are used to denote 
various things. One bit is currently defined: 
.IP \fBVImageSingleAlloc\fP 21n
This bit, if nonzero, indicates that the \fBVImageRec\fP, the block of pixel
values, the row index, and the band index were all obtained with a single
call to \fBVMalloc\fP(3Vi). If the bit is zero, these things were allocated
separately and therefore they must be released separately.
.PP
The \fBnframes\fP, \fBnviewpoints\fP, \fBncolors\fP, and \fBncomponents\fP
fields describe the number of motion frames, camera positions, color
channels, and vector components that the image's various bands represent.
Together they should account for all bands of the image \(em i.e., their
product should equal \fBnbands\fP. The image's bands are ordered according
to these fields, with component index incrementing most rapidly and frame
index incrementing least rapidly.
.SS "Image Representation in a File"
.nf
.ft B
\fIattribute-name\fP: image {
.RS
nbands: \fInbands\fP
nrows: \fInrows\fP
ncolumns: \fIncolumns\fP
repn: \fIpixel-representation\fP
nframes: \fInframes\fP
nviewpoints: \fInviewpoints\fP
ncolors: \fIncolors\fP
ncomponents: \fIncomponents\fP
data: \fIdata-offset\fP
length: \fIdata-length\fP
\fIother attributes\fP
.RE
}
.PP
\fIpixel-representation\fP ::= 
.RS
\fBbit\fP | \fBubyte\fP | \fBsbyte\fP | \fBshort\fP | \fBlong\fP | \
\fBfloat\fP | \fBdouble\fP
.RE
.fi
.PP
An image is represented in a Vista data file as an attribute. The 
attribute's value is an object of type \fBimage\fP containing the 
following attributes in any order:
.RS 2n
.IP \fBnbands\fP 10n
an integer-valued attribute specifying the number of bands in the image.
.IP \fBnrows\fP
an integer-valued attribute specifying the number of rows in the image.
.IP \fBncolumns\fP
an integer-valued attribute specifying the number of columns in the image.
.IP \fBrepn\fP
a string-valued attribute specifying the image's pixel value representation 
with one of the following keywords: \fBbit\fP, \fBubyte\fP, \fBsbyte\fP, 
\fBshort\fP, \fBlong\fP, \fBfloat\fP, or \fBdouble\fP. 
.IP \fBnframes\fP
an integer-valued attribute specifying the number of motion frames
represented by the image's bands.
.IP \fBnviewpoints\fP
an integer-valued attribute specifying the number of camera viewpoints
represented by the image's bands.
.IP \fBncolors\fP
an integer-valued attribute specifying the number of color
channels represented by the image's bands.
.IP \fBncomponents\fP
an integer-valued attribute specifying the number of vector
components represented by the image's bands.
.IP \fBdata\fP
an integer-valued attribute specifying the offset of the image's pixel 
data within the binary data section of the file
.IP \fBlength\fP
an integer-valued attribute specifying the size, in bytes, of the image's 
pixel data
.RE
.PP
An \fBnbands\fP, \fBnframes\fP, \fBnviewpoints\fP, \fBncolors\fP, or 
\fBncomponents\fP attribute may be omitted if its value is 1. Moreover, the 
\fBimage\fP object's attribute list may contain any other attributes 
associated with the image. See, for example, the data file listed under 
``EXAMPLES'', below.
.SS "Pixel Representation in a File"
Image pixel values are represented in binary form in the binary data 
section of a file. They occupy a contiguous block in which pixel values are 
ordered by band, row, and column, as in memory. 
.PP
Single-bit pixel values are packed eight to a byte, beginning with
the most-significant bit of each byte. Pixel values of more than 8 bits are
packed into multiple bytes, beginning with the most-significant byte of the
pixel value. Consecutive pixel values are packed together, without
intervening bits for padding or alignment. If necessary, however, the
entire block of pixel values is padded so that its length is a multiple of
8 bits.
.PP
Each kind of pixel is represented as follows in the Vista data file:
.RS
.IP \fBVBit\fP 10n
as a 1-bit unsigned integer
.IP \fBVUByte\fP
as an 8-bit unsigned integer
.IP \fBVSByte\fP
as an 8-bit two's-complement integer
.IP \fBVShort\fP
as a 16-bit two's-complement integer
.IP \fBVLong\fP
as a 32-bit two's-complement integer
.IP \fBVFloat\fP
as a 32-bit IEEE floating-point number
.IP \fBVDouble\fP
as a 64-bit IEEE floating point number
.RE
.PP
Note that a pixel's representation in a Vista data file is independent of 
the form it takes when stored in memory on any particular machine. Thus a 
\fBVBit\fP pixel may occupy 8 bits in the main memory of a Sun SPARCstation 
32 bits on a Cray machine, but it always occupies a single bit in a Vista 
data file. 
.SH EXAMPLES
This code fragment sets all pixels with 1 in a one-band image of 
single-bit pixels:
.PP
.RS
.ft B
for (i = 0; i < VImageNRows (image); i++)
.RS
for (j = 0; j < VImageNColumns (image); j++)
.RS
VPixel (image, 0, i, j, VBit) = 1;
.RE
.RE
.ft
.RE
.fi
.PP
The previous example may be made more efficient by avoiding the
repeated computation of pixel addresses:
.PP
.RS
.ft B
VBit *p = & VPixel (image, 0, 0, 0, VBit);
.PP
.ft B
for (i = 0; i < VImageNPixels (image); i++)
.RS
*p++ = 1;
.RE
.ft
.RE
.PP
This code fragment creates an image of complex pixel values:
.PP
.RS
.ft B
.na
.nh
.ft B
image = VCreateImage (2, 256, 256, VFloatRepn);
.HP 10n
.ft B
VImageNFrames\ (image)\ = VImageNViewpoints\ (image)\ = 
VImageNColors\ (image)\ =\ 1;
.PP
.ft B
VImageNComponents (image) = 2;
.HP 10n
.ft B
VSetAttr (VImageAttrList (image), VComponentInterpAttr,
NULL, VLongRepn, (VLong)\ VBandInterpComplex);
.ad
.hy
.RE
.PP
Here is an example of a Vista data file containing two images:
.PP
.nf
.RS
.ft B
V-data 2 {
.RS
one: image {
.RS
nbands: 2
nrows: 256
ncolumns: 256
ncomponents: 2
component_interp: complex
repn: float
data: \fIoffset of first image's pixel values\fP
length: 524288
name: "UFO sighted over Vancouver"
pixel_aspect_ratio: 1.25
.RE
}
two: image {
.RS
nrows: 32
ncolumns: 32
repn: ubyte
data: \fIoffset of second image's pixel values\fP
length: 128
name: "UFO icon"
.RE
}
.RE
}
^L
.I first image's pixel values
.I second image's pixel values
.fi
.SH "SEE ALSO"
.na
.nh
.BR Vattribute (3Vi),
.BR Vfile (5Vi),
.BR Vlib (7Vi)
.ad
.hy
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
.SH "LIST OF ROUTINES"
The following table summarizes the Vista library routines that operate on
images. Many of these routine are documented elsewhere, by a section 3Vi
man page named for the routine.
.PP
For creating and destroying images in memory:
.RS 2n
.IP \fBVCreateImage\fP 20n
Create an image.
.IP \fBVCreateImageLike\fP
Create one image like another.
.IP \fBVDestroyImage\fP
Release memory occupied by an image.
.RE
.PP
For fetching and storing pixel values:
.RS 2n
.IP \fBVGetPixel\fP 20n
Fetch a pixel value with any pixel representation.
.IP \fBVSetPixel\fP
Store a pixel value with any pixel representation.
.IP \fBVSelectBand\fP
Select bands of image pixels for processing.
.IP \fBVSelectDestIamge\fP
Select a destination for an image processing operation.
.RE
.PP
For getting band interpretation information:
.RS 2n
.IP \fBVImageFrameInterp\fP 20n
Report meaning of frame dimension.
.IP \fBVImageColorInterp\fP
Report meaning of color dimension.
.IP \fBVImageComponentInterp\fP
Report meaning of component dimension.
.IP \fBVImageViewpointInterp\fP
Report meaning of viewpoint dimension.
.RE
.PP
For reading and writing images:
.RS 2n
.IP \fBVReadImages\fP 20n
Read a set of images from a data file.
.IP \fBVReadPlain\fP
Read an image in Vista plain file format.
.IP \fBVReadPnm\fP
Read an image in Portable Anymap (PNM) format.
.IP \fBVReadUbcIff\fP
Read an image in UBC image file format.
.IP \fBVWriteImages\fP
Write a set of images to a data file.
.IP \fBVWriteUbcIff\fP
Write an image in UBC image file format.
.RE
.PP
For comparing images:
.RS 2n
.IP \fBVSameImageRange\fP 20n
Test whether two images have the same size and pixel representation.
.IP \fBVSameImageSize\fP
Test whether two images have the same size.
.RE
.PP
For copying images:
.RS 2n
.IP \fBVCombineBands\fP 20n
Copy selected bands of pixel values from various source images to a single
destination image.
.IP \fBVCopyBand\fP
Copy one or all bands of pixel values from one image to another.
.IP \fBVCopyImage\fP
Copy one image to another.
.IP \fBVCopyImageAttrs\fP
Copy one image's attributes to another.
.IP \fBVCopyImagePixels\fP
Copy one image's pixel values to another.
.RE
.PP
For converting an image's pixel representation:
.RS 2n
.IP \fBVConvertImageCopy\fP 20n
Convert an image's pixel representation while simply copying pixel values.
.IP \fBVConvertImageLinear\fP
Convert an image's pixel representation using some linear mapping of pixel
values.
.IP \fBVConvertImageOpt\fP
Convert an image's pixel representation while mapping the actual range of
source pixel values to the full range of possible destination pixel values.
.IP \fBVConvertImageRange\fP
Convert an image's pixel representation while mapping the full range of
possible source pixel values to the full range of possible destination
pixel values.
.RE
.PP
For rotating or transposing an image:
.RS 2n
.IP \fBVFlipImage\fP 20n
Flip an image horizontally or vertically.
.IP \fBVRotateImage\fP
Rotate an image by any angle.
.IP \fBVTransposeImage\fP
Transpose the rows and columns of an image.
.RE
.PP
For changing the size of an image:
.RS 2n
.IP \fBVCropImage\fP 20n
Extract a rectangular region from an image.
.IP \fBVExpandImage\fB
Increase the size of an image by an integer factor. 
.IP \fBVReduceImage\fB
Decrease the size of an image by an integer factor.
.IP \fBVScaleImage\fB
Scale the size of an image, up or down, by any factor.
.RE
.PP
For filtering and transforming images:
.RS 2n
.IP \fBVAdjustImage\fP 20n
Adjust image brightness and/or contrast.
.IP \fBVCanny\fP
Detect edges in an image using a Canny operator.
.IP \fBVConvolveImage\fP
Convolve an image with a mask.
.IP \fBVGaussianConvolveImage\fP
Convolve an image with a Gaussian filter or its derivative.
.IP \fBVImageFFT\fP
Compute a forward or inverse Fourier transform.
.IP \fBVImageGradient\fP
Compute the horizontal and vertical components of image gradient.
.IP \fBVImageOp\fP
Perform an arithmetic or logical operation on an image's pixel values.
.IP \fBVInvertImage\fP
Invert an image's pixel values, swapping black and white.
.IP \fBVLinkImage\fP
Create an edge set by linking connected, non-zero image pixels.
.IP \fBVNegateImage\fP
Negate an image's pixel values.
.IP \fBVZeroCrossings\fP
Mark the zero crossings in an image.
.RE
.PP
For computing image statistics:
.RS 2n
.IP \fBVImageStats\fP 20n
Compute the minimum, maximum, mean, and variance of an image's pixel values.
.RE
.PP
For working with complex images:
.RS 2n
.IP \fBVBuildComplexImage 20n
Build a complex image from separate images of real and imaginary
components.
.IP \fBVImageMagnitude\fP
Compute the magnitude of a complex image.
.IP \fBVImagePhase\fP
Compute the phase of a complex image, or the gradient direction of a 
two-component gradient image.
.RE
.PP
For displaying or rendering images:
.RS 2n
.IP \fBVImageBandToPS\fP 20n
Render an image band using PostScript.
.IP \fBVRGBImageToPS\fP
Render an RGB color image using PostScript.
.IP \fBVImageView\fP
An X Toolkit widget for displaying an image.
.IP \fBVImageWindowSize\fP
Compute appropriate window dimensions for displaying an image.
.RE
.PP
For filling images with patterns:
.RS 2n
.IP \fBVFillImage\fP 20n
Fill an image with a constant pixel value.
.IP \fBVRampImage\fP
Fill an image with an intensity ramp.
.IP \fBVSineGratingImage\fP
Fill an image with a sine grating.
.IP \fBVZonePlateImage\fP
Fill an image with a zone plate pattern.
.IP \fBVBilinearNoiseImage\fP
Fill an image with a random mixture of two pixel values.
.IP \fBVNormalNoiseImage\fP
Fill an image with pixel values drawn from a normal distribution.
.IP \fBVUniformNoiseImage\fP
Fill an image with pixel values drawn from a uniform distribution.
.RE
.PP
Miscellaneous:
.RS 2n
.IP \fBVOptFlowWLS\fP 20n
compute optical flow by weighted least squares.
.RE
