.ds Vn 2.1
.TH VImageFFT 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VImageFFT \- compute the Fourier transform of an image
.SH SYNOPSIS
.nf
.B #include <vista/VImage.h>
.PP
.ft B
VImage VImageFFT (\fIsrc\fP, \fIdest\fP, \fIinverse\fP, \fInorm\fP)
.RS
VImage \fIsrc\fP, \fIdest\fP;
VBoolean \fIinverse\fP;
double \fInorm\fP;
.RE
.fi
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies the image to transform (the source image).
.IP \fIdest\fP
May specify a destination image, or may be
.SB NULL
to indicate that a new destination image should be created.
.IP \fIinverse\fP
Indicates whether the forward (\c
.SB FALSE\c
) or inverse (\c
.SB TRUE\c
) transform is to be computed.
.IP \fInorm\fP
A normalizing constant used to scale each pixel of the result.
.SH DESCRIPTION
\fBVImageFFT\fP computes the Fourier transform, or inverse Fourier
transform, of a complex image using the Fast Fourier Transform algorithm.
.PP
\fISrc\fP is a two-band, \fBVFloat\fP or \fBVDouble\fP image whose numbers 
of rows and columns are both powers of two. Band 0 contains the real 
component, and band 1, the imaginary component, of each pixel. 
.PP
If \fIdest\fP is provided it must have the same properties as \fIsrc\fP; if 
.SB NULL
is passed, instead, a suitable destination image will be created by
\fBVImageFFT\fP. Again, its two bands represent the real and imaginary
components of each pixel.
.PP
Once the transform or its inverse is computed, each pixel of the result is
divided by \fInorm\fP. Typically, a value of \fInorm\fP is derived from
the number of rows (\fIN\fP) and columns (\fIM\fP) of the image. Common
choices are:
.RS 2n
.IP \(bu 2n
1.0 for the forward transform, and \fIN\fP*\fIM\fP for the inverse transform;
.IP \(bu
.B sqrt(\fIN\fP*\fIM\fP) 
for both the forward and inverse transforms.
.RE
.PP
In the transformed image, frequencies are arranged in wrap-around order,
independently in each dimension. The rows correspond to frequencies in the
vertical direction. For example:
.RS 2n
.IP \(bu 2n
row 0 represents 0 cycles per image height
.IP \(bu
row 1 represents 1/\fIN\fP cycles per image height
.IP \(bu
row \fIN\fP/2 represents both 1/2 and -1/2 cycles per image height
.IP \(bu
row \fIN\fP-1 represents -1/\fIN\fP cycles per image height
.RE
.PP
Similarly, the columns correspond to frequencies in the horizontal direction.
.SH "RETURN VALUES"
\fBVImageFFT\fP returns the destination image if it is successful
and 
.SB NULL
otherwise.
.SH "SEE ALSO"
.na
.nh
.BR VBuildComplexImage (3Vi),
.BR VImage (3Vi),
.BR Vlib (7Vi)
.ad
.hy
.SH DIAGNOSTICS
.IP "``Source image must have VFloat or VDouble pixels.''"
The source image has the wrong pixel representation.
.IP "``Source image must have 2 bands.''"
The source image has something other than two bands.
.IP "``Source image must have 2**n rows.''"
The number of rows in the source image is not a power of two.
.IP "``Source image must have 2**n columns.''"
The number of columns in the source image is not a power of two.
.IP "``Destination image has \fIdest_nbands\fP bands; 2 expected.''"
A destination image was specified but it has the wrong number of bands.
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIsrc_prop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``rows'', or ``columns''.
A destination image was specified but it does not have the same
pixel representation, number of rows, and number of columns as the
source image.
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
.SH ACKNOWLEDGMENT
The algorithm used by this routine is from \fINumerical Recipes in C\fP,
pp. 467-470.
