.ds Vn 2.1
.TH VImageGradient 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VImageGradient \- estimate image gradient components
.SH SYNOPSIS
.nf
.B #include <vista/VImage.h>
.PP
.ft B
VImage VImageGradient (\fIsrc\fP, \fIdest\fP, \fIband\fP)
.RS
VImage \fIsrc\fP, \fIdest\fP;
VBand \fIband\fP;
.RE
.fi
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies a source image whose gradient is to be estimated.
.IP \fIdest\fP
May specify a destination image, or may be
.SB NULL
to indicate that a suitable destination image should be created.
.IP \fIband\fP
Specifies the bands of the source image to be processed.
.SH DESCRIPTION
\fBVImageGradient\fP estimates the horizontal and vertical components of
the intensity gradient at each pixel of the source image.
.PP
For a source pixel \fIs\fP with vertical neighbours \fIa\fP and \fIb\fP,
and horizontal neighbours \fIc\fP and \fId\fP:
.PP
.nf
.ta 20n 25n 30n
		\fIa\fP
	\fIc\fP	\fIs\fP	\fId\fP
		\fIb\fP
.DT
.fi
.PP
the horizontal component of the gradient is estimated as
 (\fId\fP\ \-\ \fIc\fP)\ /\ 2
and the vertical component is estimated as 
(\fIa\fP\ \-\ \fIb\fP)\ /\ 2. Both components of the gradient are 
taken to be zero at border pixels.
.PP
The source image is specified by \fIsrc\fP and \fIband\fP, which may be a 
particular band number or \fBVAllBands\fP. It may have any size and pixel 
representation, but it must have a single component 
(\fBncomponents:\ 1\fP). 
.PP 
For each band of the source image that is processed, the destination image
contains a pair of consecutive bands. The first band of the pair contains
the gradient's horizontal component, and the second contains the vertical
component.
.PP
Ths destination image has the same numbers of rows and columns as the source
image.
.PP
If the source image has \fBVBit\fP or \fBVUByte\fP pixels, the destination
image has \fBVSByte\fP pixels; otherwise the destination image has the same
pixel representation as the source image.
.PP
When the destination image is given a copy of the source image's 
attributes, band interpretation attributes are changed so that the 
destination image is described as a gradient image (\fBncomponents: 2\fP
and \fBcomponent_interp:\ gradient\fP). 
.SH "RETURN VALUES"
\fBVImageGradient\fP returns the destination image if it is successful
and 
.SB NULL
otherwise.
.SH "SEE ALSO"
.na
.nh
.BR VImageMagnitude (3Vi),
.BR VImagePhase (3Vi)
.BR VImage (3Vi),
.BR Vlib (7Vi)
.ad
.hy
.SH NOTES
\fBVImageMagnitude\fP(3Vi) and \fBVImagePhase\fP(3Vi) can be used
to compute the magnitude and direction of the intensity gradient from
the output of \fBVImageGradient\fP.
.SH DIAGNOSTICS
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIsrc_prop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``bands'', ``rows'', or ``columns''.
A destination image was specified but it does not have the correct properties.
.IP "``Destination pixel(s) out of range.''"
One or more destination pixel values were clipped because they exceeded the
range that can be represented in the destination image.
.IP "``Source image must have one component.''"
\fISrc\fP must have \fBncomponents:\ 1\fP.
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
