.ds Vn 2.1
.TH VImageOp 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VImageOp \- perform arithmetic or logical pixel operation
.SH SYNOPSIS
.nf
.B #include <vista/VImage.h>
.PP
.ft B
VImage VImageOpI (\fIsrc1\fP, \fIdest\fP, \fIband1\fP, \fIop\fP, \
\fIsrc2\fP, \fIband2\fP, \fIminp\fP, \fImaxp\fP)
.RS
VImage \fIsrc1\fP, \fIdest\fP, \fIsrc2\fP;
VBand \fIband1\fP, \fIband2\fP;
VImageOpKind \fIop\fP;
VDouble *\fIminp\fP, *\fImaxp\fP;
.RE
.PP
.ft B
VImage VImageOpU (\fIsrc1\fP, \fIdest\fP, \fIband1\fP, \fIop\fP, \
\fIminp\fP, \fImaxp\fP)
.RS
VImage \fIsrc1\fP, \fIdest\fP;
VBand \fIband1\fP;
VImageOpKind \fIop\fP;
VDouble *\fIminp\fP, *\fImaxp\fP;
.RE
.PP
.ft B
VImage VImageOpV (\fIsrc1\fP, \fIdest\fP, \fIband1\fP, \fIop\fP, \
\fIvalue\fP, \fIminp\fP, \fImaxp\fP)
.RS
VImage \fIsrc1\fP, \fIdest\fP;
VBand \fIband1\fP;
VImageOpKind \fIop\fP;
double \fIvalue\fP;
VDouble *\fIminp\fP, *\fImaxp\fP;
.RE
.PP
.fi
.na
.nh
.ft B
typedef enum {
.RS
.sp
/* Unary operations: */
.br
VImageOpAbs, VImageOpExp, VImageOpLog, VImageOpNot,
VImageOpSqrt, VImageOpSquare,
.sp
/* Binary (dyadic) operations: */
.br
VImageOpAdd, VImageOpAnd, VImageOpDist, VImageOpDiv, 
VImageOpMax, VImageOpMin, VImageOpMult, VImageOpOr, 
VImageOpSub, VImageOpXor
.sp
.RE
} VImageOpKind;
.ad
.hy
.SH ARGUMENTS
.IP "\fIsrc1\fP, \fIsrc2\fP" 10n
Specify images to process (the source images).
.IP \fIdest\fP
May specify a destination image, or may be
.SB NULL
to indicate that a new destination image should be created.
.IP "\fIband1\fP, \fIband2\fP"
Specify particular bands of the source images to process, or may be the
constant \fBVAllBands\fP to indicate that all bands of a source image 
should be processed. 
.IP \fIop\fP
Specifies the operation to perform.
.IP \fIvalue\fP
Specifies a second operand for a dyadic operation. 
.IP \fIminp\fP
.IP \fImaxp\fP
May specify bounds for clipping destination pixel values, or may be
.SB NULL
to specify no clipping.
.SH DESCRIPTION
The three \fBVImageOp\fP routines perform arithmetic and logical 
operations on image pixels.
.RS 2n
.IP \(bu 2n
\fBVImageOpI\fP performs a binary (two-operand) operation between each 
pixel of the image \fIsrc1\fP and a corresponding pixel of the image 
\fIsrc2\fP. 
.IP \(bu
\fBVImageOpU\fP performs a unary (single-operand) operation with each pixel 
of the image \fIsrc1\fP. 
.IP \(bu
\fBVImageOpV\fP performs a binary (two-operand) operation between each 
pixel of the image \fIsrc1\fP and the single value \fIvalue\fP. 
.RE
.PP
The destination image has the same number of rows and columns, and the same 
pixel representation as the source image \fIsrc1\fP. Also, it has either 
the same number of bands as the \fIsrc1\fP (if \fIband1\fP is 
\fBVAllBands\fP), or a single bands (otherwise).
.PP
If \fIminp\fP or \fImaxp\fP isn't
.SB NULL\c
, it points to a lower or upper bound for clipping destination pixel 
values. For example, if \fIminp\fP is &1.0 then pixels less than 1.0 will 
be set to 1.0. If no clipping bound is specified, pixels will be clipped to 
the minimum or maximum value that can be represented by the pixel type. A 
warning message is produced if pixels are clipped, but no error indication 
is returned. 
.PP
\fBVImageOpI\fP provides three different ways of operating on bands from 
the two source images:
.RS 2n
.IP \(bu 2n
\fIband1\fP and \fIband2\fP can each specify a particular band of their 
respective source images \(em then the destination image has one band 
formed from \fIband1\fP of \fIsrc1\fP and \fIband2\fP of \fIsrc2\fP.
.IP \(bu
\fIband1\fP and \fIband2\fP can both be \fBVAllBands\fP \(em then the 
destination image and both source images all have the same number of bands, 
and band \fIi\fP of the destination image is formed from band \fIi\fP of 
both source images. 
.IP \(bu
\fIband1\fP can be \fBVAllBands\fP while \fIband2\fP specifies a 
particular band \(em then the destination image has the same number of bands as
\fIsrc1\fP, and band \fIi\fP of the destination image is formed from band
\fIi\fP of \fIsrc1\fP and band \fIband2\fP of \fIsrc2\fP.
.RE
.PP
Operation codes \fBVImageOpAbs\fP through \fBVImageOpSquare\fP may be 
specified to \fBVImageOpU\fP; codes \fBVImageOpAdd\fP through 
\fBVImageOpXor\fP may be specified to \fBVImageOpI\fP or \fBVImageOpV\fP. 
These codes have the following interpretations in terms of the operands 
\fIu\fP (a pixel value drawn from src1) and \fIv\fP (either a pixel value 
drawn from \fIsrc2\fP or the \fIvalue\fP argument): 
.RS 2n
.PP
.nf
.ta 17n
\fBVImageOpAbs\fP	absolute value of \fIu\fP
\fBVImageOpExp\fP	exponential function \fIe\fP\ **\ \fIu\fP
\fBVImageOpLog\fP	natural logarithm \fIlog\fP\ (\fIu\fP)
\fBVImageOpNot\fP	negation !\ \fIu\fP
\fBVImageOpSqrt\fP	square root of \fIu\fP
\fBVImageOpSquare\fP	\fIu\fP\ *\ \fIu\fP
\fBVImageOpAdd\fP	\fIu\fP\ +\ \fIv\fP
\fBVImageOpAnd\fP	\fIu\fP\ &\ \fIv\fP
\fBVImageOpDist\fP	absolute value of (\ \fIu\fP\ \-\ \fIv\fP\ )
\fBVImageOpDiv\fP	\fIu\fP\ /\ \fIv\fP
\fBVImageOpMax\fP	\fImax\fP\ (\fIu\fP,\ \fIv\fP)
\fBVImageOpMin\fP	\fImin\fP\ (\fIu\fP,\ \fIv\fP)
\fBVImageOpMult\fP	\fIu\fP\ *\ \fIv\fP
\fBVImageOpOr\fP	\fIu\fP\ |\ \fIv\fP
\fBVImageOpSub\fP	\fIu\fP\ \-\ \fIv\fP
\fBVImageOpXor\fP	\fIu\fP\ ^\ \fIv\fP
.DT
.fi
.RE
.SH "RETURN VALUES"
These routines return the destination image if successful and 
.SB NULL
otherwise.
.SH "SEE ALSO"
.nh
.na
.BR VAdjustImage (3Vi),
.BR VInvertImage (3Vi),
.BR VNegateImage (3Vi),
.br
.BR VImage (3Vi),
.BR Vlib (7Vi)
.hy
.ad
.SH NOTES
The \fBVImageOpAnd\fP, \fBVImageOpOr\fP, and \fBVImageOpXor\fP operations 
can only be performed with images having integer pixel representations (not 
\fBVFloat\fP or \fBVDouble\fP). 
.PP
A floating point result (such as \fBVImageOpExp\fP produces) is rounded to 
the nearest integer for storage in an integer destination pixel.
.PP
The destination image inherits band interpretation properties and other 
image attributes from \fIsrc1\fP.
.PP
For the operations \fBVImageOpExp\fP, \fBVImageOpLog\fP, and
\fBVImageOpSqrt\fP, certain math library routines are used (see
\fBintro\fP(3M)).  They signal errors using the \fBmatherr\fP() mechanism
(see \fBmatherr\fP(3M)). Other operations may report exceptions by sending
.SB SIGFPE
signals (see \fBsignal\fP(3)).  The caller may arrange to
detect exceptions by, for example, establishing a \fBmatherr\fP() routine
and a 
.SB SIGFPE
signal handler.
.SH DIAGNOSTICS
.IP "``Band \fIband\fP referenced in image of \fInbands\fP band(s).''"
One of the band arguments, \fIband1\fP or \fIband2\fP, was inappropriate 
for its corresponding source image, \fIsrc1\fP or \fIsrc2\fP.
.IP "``Clipping bound out of range; set to \fIw\fP.''"
Either *\fIminp\fP or *\fImaxp\fP is outside the range that can be 
represented by destination pixel values so the default clipping bound, 
\fIw\fP, is being used. 
.IP "``Clipping range is empty.''"
The \fIminp\fP and \fImaxp\fP arguments specify an empty clipping range.
.IP "``Destination image has \fIdest_nbands\fP bands; \fIsrc_nbands\fP expected.''"
A destination image was specified but it has the wrong number of bands.
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIsrc_prop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``rows'', or ``columns''. A 
destination image was specified but it does not have the same pixel 
representation, number of rows, and number of columns as the source image(s).
.IP "``Incorrect combination of band1, band2.''"
The \fIband1\fP and \fIband2\fP arguments of \fBVImageOpI\fP are do not 
specify one of the combinations of \fBVAllBands\fP and specific band 
numbers listed above. 
.IP "``Invalid operation (\fIop\fP) for \fIrepn\fP pixels.''"
The operation specified by \fIop\fP is not appropriate for the routine 
called (e.g., \fBVImageOpAdd\fP was passed to \fBVImageOpU\fP), or not 
appropriate for the pixel type (e.g., a logical operation was specified 
for \fBVFloat\fP pixels).
.IP "``Source images have differing numbers of bands.''"
Corresponding bands of the two source images are to be processed but the 
images don't have an identical number of bands. 
.IP "``Source images have dissimilar properties.''"
The two source images don't have the same numbers of rows and columns, and 
the same pixel representation. 
.SH AUTHORS
.na
.nh
Ralph\ Horstmann\ <horstman@cs.ubc.ca>,
Art\ Pope\ <pope@cs.ubc.ca>
