.ds Vn 2.1
.TH VImagePhase 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VImagePhase \- compute phase of a complex image
.SH SYNOPSIS
.nf
.B #include <vista/VImage.h>
.PP
.ft B
VImage VImagePhase (\fIsrc\fP, \fIdest\fP, \fIband\fP)
.RS
VImage \fIsrc\fP, \fIdest\fP;
VBand \fIband\fP;
.RE
.fi
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies a complex source image.
.IP \fIdest\fP
May specify a destination image, or may be
.SB NULL
to indicate that a suitable destination image should be created.
.IP \fIband\fP
Specifies the bands of \fIsrc\fP whose magnitude is to be computed.
.SH DESCRIPTION
\fBVImagePhase\fP treats the source image as having complex pixel values,
with alternating bands of real and imaginary pixel components. It creates a
destination image in which each pixel value is the phase of the
corresponding source pixel, expressed in radians.
.PP
If a source pixel has real component \fIa\fP and imaginary component
\fIb\fP then the corresponding destination pixel will have the value
\fIatan2\fP\ (\fIb\fP,\ \fIa\fP). (If \fIa\fP and \fIb\fP are both zero then
the corresponding destination pixel is set to zero.)
.PP
If \fIband\fP is \fBVAllBands\fP then the source image should have 2\fIn\fP
bands, for some integer \fIn\fP. The destination image will have \fIn\fP
bands, and band \fIi\fP of the destination image is computed from bands
2\fIi\fP and 2\fIi\fP\ +\ 1 of the source image.
.PP
Otherwise \fIband\fP specifies a particular band of the source image. The
destination image will have a single band whose pixel values are obtained
from bands \fIband\fP and \fIband\fP\ +\ 1 of the source image.
.PP
The destination image has the same numbers of rows and columns as the
source image. If the source image has \fBVDouble\fP pixels then so does the
destination image; otherwise, the destination image has \fBVFloat\fP pixels.
.PP
When the destination image is given a copy of the source image's 
attributes, band interpretation attributes are changed so that the 
destination image is described as an orientation image 
(\fBncomponents:\ 1\fP and \fBcomponent_interp: orientation\fP). 
.SH "RETURN VALUES"
\fBVImagePhase\fP returns the destination image if it is successful
and 
.SB NULL
otherwise.
.SH "SEE ALSO"
.na
.nh
.BR VBuildComplexImage (3Vi),
.BR VImageMagnitude (3Vi),
.BR VImage (3Vi),
.BR Vlib (7Vi)
.ad
.hy
.SH DIAGNOSTICS
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIsrc_prop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``bands'', ``rows'', or ``columns''.
A destination image was specified but it does not have the correct
pixel representation, number of bands, number of rows, or number of columns.
.IP "``Source image must have two components.''"
\fISrc\fP must have \fBncomponents:\ 2\fP.
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
