.ds Vn 2.1
.TH VImageView 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VImageView \- widget for displaying an image.
.SH SYNOPSIS
.nf
.ft B
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <vista/VImageView.h>
.PP
\fIwidget\fB = XtCreateWidget (\fIname\fB, vImageViewWidgetClass, \fR...\fB);
.PP
\fBld\0\0\fR...\fR\0\0-lvista\0\0-lXmu\0\0-lXt\0\0\-lXext\0\0-lX11\0\0\fR...
.fi
.SS "CLASS HIERARCHY"
Core -> VImageView
.SH DESCRIPTION
The VImageView widget displays an image within a window. The image
must be of the form described by the \fBVImage\fP(3Vi) manual page.
.PP
The widget displays both RGB color images and gray-scale images.
.RS 2n
.IP \(bu 2n
If the image is a standard RGB color image (\fBncolors:\ 3\fP and
\fBcolor_interp: rgb\fP) with a single component dimension 
(\fBncomponents:\ 1\fP), then each multiple of three bands will be
displayed in color
.IP \(bu
Otherwise, each band is displayed individually as a single, gray-scale image.
.RE
.PP
The widget allows the image to be overlaid with other information by
invoking a callback list whenever the image is redrawn. A procedure
registered with that callback list can perform additional drawing overtop
of the image.
.PP
Another callback list is invoked in response to mouse or keyboard input.
A procedure can be registered with that callback list to handle any such
input.
.PP
The widget also supports zooming. The zooming parameters (i.e. the
zoom center and the zoom level) can be specified by setting the
appropriate resources or by invoking certain action procedures.
Three predefined action procedures are available for handling
zooming, and each of these action procedures is associated with
a callback list.
.SS "Image Rendering"
The number of colors or gray shades used in rendering an image
depends generally on the capabilities of the display device.
Various facilities within the X Window System attempt to characterize
the device and model its features in a consistent and portable way.
.PP
Basic use of the VImageView widget does not require any special knowledge
of this model, since the default behavior of the widget will produce a
rendered image which makes conservative use of the display features.
.PP
More advanced use requires some familiarity with X Windows and with
other facilities of Vista.  An example would be if the application
required the image to be rendered using all available colors or gray
shades rather than the default selection.  In use, this might result
in colormap flashing, depending on the capabilities of the display
and on how the display is used by other applications.
See the \fBVInitColormap\fP(3Vi) and \fBVFreeColormap\fP(3Vi) manual pages
for details on the use of VColormap objects.
.PP
The VImageView widget will attempt to use colors or gray shades defined
by what are known in X Windows as ``standard colormaps'' for shared use
between applications.  The idea is that the window manager or a related
client such as \fBxstdcmap\fP would have registered these standard
colormaps as properties of the root window of a given screen, and would
also have initialized appropriate sets of colors and gray shades to suit
the needs of typical applications.  The VImageView widget decides which
of these properties to use and performs its rendering accordingly.
.PP
Dithering is used to accommodate an image's range of pixel values
to the possibly smaller range of colors or gray shades that have been
allocated.
.PP
In the current state of development of X Windows, it is likely that the
window manager has no capability to register standard colormaps, and
that no other client has been run for the purpose.  Therefore, if no
appropriate standard colormap exists already, the widget will attempt a
series of steps in order to allocate the necessary colors or gray
shades.  The first attempt in this series is to register the standard
colormap exactly as the window manager would have done, for shared use
by all applications.  Failing this, colors will be allocated as if the
appropriate standard colormap existed.
.PP
The default rendering done by the widget therefore closely tracks the
colormap conventions used by X Windows.  These are designed to depend in
turn on the default visual class of the root window and the default
depth of the screen.  Based on the current X Windows release, some
rendering characteristics can be described.
.RS 2n
.IP \(bu 2n
On a typical 8-bit (per pixel) color screen under X11R5,
a color image is rendered using 5 shades each of red, green,
and blue,
and a grayscale image is rendered using 16 shades of gray.
.IP \(bu
On a typical 24-bit (per pixel) color screen under X11R5,
a color image is rendered using 256 shades each of red, green,
and blue,
and a grayscale image is rendered using 256 shades of gray.
.IP \(bu
On a grayscale (including monochrome) screen, a color image
is first converted into a grayscale image, and this image
is then rendered using all available gray shades of the screen.
.RE
.PP
For an image with signed (i.e. \fBVSByte\fP, \fBVShort\fP, \fBVLong\fP,
\fBVFloat\fP, or \fBVDouble\fP) pixel values, the interpretation
of the pixels depends on the value of the \fBabsolute\fP resource.
If \fBabsolute\fP is \fBTrue\fP, intensity corresponds to absolute
pixel values with 0 indicating minimum intensity. If \fBabsolute\fP
is \fBFalse\fP, zero indicates medium intensity, negative values indicate
darker shades, and positive values indicate lighter shades.
For an image with unsigned (i.e. \fBVBit\fP or \fBVUByte\fP)
pixels, intensity always corresponds to pixel values with zero
indicating minimum intensity.
.SS Resources
When creating a VImageView widget instance, the following resources are
retrieved from the argument list or from the resource database:
.PP
.ta 1.85i 3.6i 4.85i
.nf
Name	Class	Type	Default	Value
VxNabsolute	VxCAbsolute	Boolean	True
XtNaccelerators	XtCAccelerators	AcceleratorTable	NULL
XtNancestorSensitive	XtCAncestorSensitive	Boolean	True
XtNbackground	XtCBackground	Pixel	XtDefaultBackground
XtNbackgroundPixmap	XtCPixmap	Pixmap	XtUnspecifiedPixmap
VxNband	VxCBand	Int	0
XtNborderColor	XtCBorderColor	Pixel	XtDefaultForeground
XtNborderPixmap	XtCPixmap	Pixmap	XtUnspecifiedPixmap
XtNborderWidth	XtCBorderWidth	Dimension	1
XtNcolormap	XtCColormap	Pointer	Parent's Colormap
VxNcolumnCenter	VxCColumnCenter	Int	0
VxNcursor	VxCCursor	Cursor	``crosshair''
XtNdepth	XtCDepth	int	Parent's Depth
XtNdestroyCallback	XtCCallback	XtCallbackList	NULL
VxNexposeCallback	VxCCallback	XtCallbackList	NULL
XtNheight	XtCHeight	Dimension	0
VxNimage	VxCImage	Pointer	NULL
XtNinitialResourcesPersistent	XtCInitialResourcesPersistent	Boolean	True
VxNinputCallback	VxCCallback	XtCallbackList	NULL
XtNmappedWhenManaged	XtCMappedWhenManaged	Boolean	True
VxNmoveZoomCenterCallback	VxCCallback	XtCallbackList	NULL
VxNproportion	VxCProportion	Boolean	True
VxNresize	VxCResize	Boolean	False
VxNrowCenter	VxCRowCenter	Int	0
XtNscreen	XtCScreen	Pointer	Parent's Screen
XtNsensitive	XtCSensitive	Boolean	True
XtNtranslations	XtCTranslations	TranslationTable	See below
VxNusePixmap	VxCUsePixmap	Boolean	True
VxNvColormap	VxCVColormap	VColormap	NULL
XtNwidth	XtCWidth	Dimension	0
XtNx	XtCX	Position	0
XtNy	XtCY	Position	0
VxNzoomInCallback	VxCCallback	XtCallbackList	NULL
VxNzoomLevel	VxCZoomLevel	Int	100
VxNzoomOutCallback	VxCCallback	XtCallbackList	NULL
.DT
.fi
.PP
Note that names beginning with \fBVx\fP are defined in 
\fB<vista/VImageView.h>\fP, whereas those beginning with \fBXt\fP are defined
in \fB<X11/StringDefs.h>\fP.
.PP
The new resources associated with VImageView are:
.IP \fBVxNabsolute\fP 15n
If it is \fBTrue\fP, intensity corresponds to absolute
pixel values with zero indicating minimum intensity. If it
is \fBFalse\fP, zero indicates medium intensity, negative values indicate
darker shades, and positive values indicate lighter shades.
.IP \fBVxNband\fP
Specifies which band or bands of the image to display.
If the image is being shown as a color image then this resource specifies the
first of the three consecutive bands representing red, green, and blue. 
Otherwise, this resource specifies the single band being shown as a
grey-scale image.
.IP \fBVxNcolumnCenter\fP
Specifies the column coordinate of the zoom center.
.IP \fBVxNcursor\fP
Specifies the image that will be displayed as the pointer cursor whenever
it is in the window.
.IP \fBVxNexposeCallback\fP
All procedures on this list are called after any portion of the image is
redrawn.
.IP \fBVxNimage\fP
Specifies the image to be displayed. If its value is 
.SB NULL
then none is displayed.
.IP \fBVxNinputCallback\fP
All procedures on this list are called whenever the \fBInput\fP action is
invoked.
.IP \fBVxNmoveZoomCenterCallback\fP
All procedures on this list are called whenever the \fBMoveZoomCenter\fP 
action is invoked.
.IP \fBVxNproportion\fP
Specifies whether the correct ratio of image width to height is to be 
retained. If its value is 
.SB TRUE
then the image is scaled in both directions so that it both fits within
the window and is correctly proportioned. The image's 
\fBVPixelAspectRatioAttr\fP attribute is used, if it exists, to determine
the correct proportions.
If the resource's value is
.SB FALSE
then the image is scaled independently in width and height to precisely
fill the window.
.IP \fBVxNresize\fP
Specifies whether the VImageView widget should attempt to resize to its
preferred dimensions whenever \fBXtSetValues\fP is called for it or
it is managed by its parent.
.IP \fBVxNrowCenter\fP
Specifies the row coordinate of the zoom center.
.IP \fBVxNusePixmap\fP
Specifies whether to cache the rendered image in a pixmap on the server.
Doing so, which is the default, generally gives the best performance but
requires more server memory.
.IP \fBVxNvColormap\fP
Specifies the \fBVColormap\fP object used to render the image.
If its value is
.SB NULL
then the image will be rendered using default values.
.IP \fBVxNzoomInCallback\fP
All procedures on this list are called whenever the \fBZoomIn\fP action is
invoked.
.IP \fBVxNzoomLevel\fP
Specifies the zoom level in magnification units of 1/100 times.
A value greater than 100 will expand the image (causing the visible
portion of the image to shrink). Values less than 100 are invalid.
.IP \fBVxNzoomOutCallback\fP
All procedures on this list are called whenever the \fBZoomOut\fP action is
invoked.
.PP
.SS "Translations and Actions"
The VImageView widget has four predefined actions, which are
described below:
.IP \fBInput()\fP 15n
This action simply invokes the callback list \fBVxNinputCallback\fP.
.IP \fBZoomIn()\fP
This action multiplies the zoom level by 2 and sets the zoom center
to the cursor position if the event causing its invocation is
associated with a key-press/release or button-press/release.
In any case, the callback list \fBVxNzoomInCallback\fP will be
invoked.
.IP \fBZoomOut()\fP
This action divides the zoom level by 2 (or sets the zoom level to
100 if the resulting zoom level will be less than 100) 
and sets the zoom center
to the cursor position if the event causing its invocation is
associated with a key-press/release or button-press/release.
In any case, the callback list \fBVxNzoomOutCallback\fP will be
invoked.
.IP \fBMoveZoomCenter()\fP
This action sets the zoom center
to the cursor position if the event causing its invocation is
associated with a key-press/release or button-press/release.
In any case, the callback list \fBVxNmoveZoomCenterCallback\fP will be
invoked.
.PP
The following is the default translation table used by the VImageView
widget:
.PP
.RS
.nf
.B !Ctrl<Btn2Down>: MoveZoomCenter()
.B None<Btn2Down>: ZoomIn()
.B None<Btn3Down>: ZoomOut()
.B <Key>: Input()
.B <BtnDown>: Input()
.fi
.RE
.PP
.SS "Programmatic Interface"
.IP \(bu 2n
To create a VImageView widget instance, use \fBXtCreateWidget\fP
and specify the class variable \fBvImageViewWidgetClass\fP.
.IP \(bu
To destroy a VImageView widget instance, use \fBXtDestroyWidget\fP
and specify the widget ID of the VImageView widget.
.IP \(bu
There is no way to specify a value for the \fBVxNimage\fP resource in a
resource file.  Its value must be set in the argument list to
\fBXtCreateWidget\fP or \fBXtSetValues\fP.
.IP \(bu
There is no way to specify a value for the \fBVxNvColormap\fP resource in a 
resource file. Its value must be set in the argument list to 
\fBXtCreateWidget\fP. Because of its relationship to the \fBXtNvisual\fP 
and \fBXtNcolormap\fP resources, which can only be specified at creation 
time, it is recommended not to set \fBVxNvColormap\fP in \fBXtSetValues\fP. 
See the \fBVInitColormap\fP(3Vi) and \fBVFreeColormap\fP(3Vi) manual pages 
for details on the use of VColormap objects. 
.IP \(bu
The VImageView widget supports six callback lists:
.PP
.RS
.IP \fBXtNdestroyCallback\fP
is invoked when the widget is destroyed.
.IP \fBVxNexposeCallback\fP
is invoked immediately after any portion of the image is redrawn in
response to, for example, an expose event. 
The \fBcall_data\fP argument passed to callbacks on this list is the 
\fBRegion\fP to be redrawn.
.IP \fBVxNinputCallback\fP
is invoked by the \fBInput\fP action.
The \fBcall_data\fP argument passed to callbacks on this list is a
pointer to the \fBXEvent\fP describing the input event.
.IP \fBVxNzoomInCallback\fP
is invoked by the \fBZoomIn\fP action.
The \fBcall_data\fP argument passed to callbacks on this list is a
pointer to the \fBXEvent\fP describing the input event.
.IP \fBVxNzoomOutCallback\fP
is invoked by the \fBZoomOut\fP action.
The \fBcall_data\fP argument passed to callbacks on this list is a
pointer to the \fBXEvent\fP describing the input event.
.IP \fBVxNmoveZoomCenterCallback\fP
is invoked by the \fBMoveZoomCenter\fP action.
The \fBcall_data\fP argument passed to callbacks on this list is a
pointer to the \fBXEvent\fP describing the input event.
.RE
.IP \(bu 2n
To redraw an image, use:
.PP
.RS
.nf
.B void VImageViewRedraw (\fIw\fP)
.RS
.B Widget \fIw\fP;
.RE
.RE
.fi
.IP " "
\fBVImageViewRedraw\fP will redraw the whole image and call
all callbacks in the \fBVxNexposeCallback\fP list.
.IP \(bu 2n
To map window coordinates to image coordinates while an image is displayed, 
use: 
.PP
.RS
.nf
.ft B
VBoolean VImageViewWindowToImage (\fIw\fP, \fIx\fP, \fIy\fP, \
\fIrowp\fP, \fIcolumnp\fP)
.RS
Widget \fIw\fP;
int \fIx\fP, \fIy\fP;
double *\fIrowp\fP, *\fIcolumnp\fP;
.RE
.RE
.fi
.IP " "
Pass, as \fIx\fP and \fIy\fP, the window coordinates. If the point 
described by these coordinates lies within the displayed image, 
\fBVImageViewWindowToImage\fP will set *\fIrowp\fP and *\fIcolumnp\fP to be 
the corresponding image coordinates, and it will return 
.SB TRUE\c
. Otherwise, it will return
.SB FALSE\c
.
.RE
.IP " " 2n
Or use:
.PP
.RS
.nf
.ft B
VBoolean VImageViewClipToImage (\fIw\fP, \fIx\fP, \fIy\fP, \
\fIrowp\fP, \fIcolumnp\fP)
.RS
Widget \fIw\fP;
int \fIx\fP, \fIy\fP;
double *\fIrowp\fP, *\fIcolumnp\fP;
.RE
.RE
.fi
.IP " "
\fBVImageViewClipToImage\fP is similar to \fBVImageViewWindowToImage\fP, 
except that when the point described by the given coordinates lies outside 
the displayed image, \fBVImageViewClipToImage\fP will set *\fIrowp\fP and 
*\fIcolumnp\fP to be the image coordinates that are closest to the given 
coordinates , and it will return 
.SB TRUE
in this case.
.RE
.IP \(bu 2n
To map image coordinates to window coordinates while an image is displayed, 
use: 
.PP
.RS
.nf
.ft B
VBoolean VImageViewImageToWindow (\fIw\fP, \fIrow\fP, \fIcolumn\fP, \
\fIxp\fP, \fIyp\fP)
.RS
Widget \fIw\fP;
double \fIrow\fP, \fIcolumn\fP;
int *\fIxp\fP, *\fIyp\fP;
.RE
.RE
.fi
.IP " "
Pass, as \fIrow\fP and \fIcolumn\fP, the image coordinates. If the point 
described by these coordinates lies within the displayed image, 
\fBVImageViewImageToWindow\fP will set *\fIxp\fP and *\fIyp\fP to be the 
corresponding window coordinates, and it will return 
.SB TRUE\c
. Otherwise, it will return
.SB FALSE\c
.
.RE
.IP \(bu 2n
To find the size of an image pixel in terms of window pixels, use:
.PP
.RS
.nf
.ft B
VBoolean VImageViewPixelSize (\fIw\fP, \fIwidth\fP, \fIheight\fP)
.RS
Widget \fIw\fP;
double *\fIwidth\fP, *\fIheight\fP;
.RE
.RE
.fi
.IP " "
If the image resource of the widget \fIw\fP is not 
.SB NULL\c
, \fBVImageViewPixelSize\fP will set *\fIwidth\fP and *\fIheight\fP to be the 
width and height (expressed in window pixels) of an image pixel, and it 
will return
.SB TRUE\c
. Otherwise, it will return 
.SB FALSE\c
.
.RE
.SH "SEE ALSO"
.BR VImage (3Vi),
.BR VInitColormap (3Vi),
.BR VFreeColormap (3Vi),
.BR XStandardColormap (3X11),
.BR xstdcmap (1)
.SH DIAGNOSTICS
.IP "``Band number \fIband\fP is out of range.''"
The value of the \fBVxNband\fP resource is not valid for the current image.
.IP "``Image has bad pixel_aspect_ratio attribute.''"
The attribute's value can't be interpreted as a number.
.IP "``rowCenter/columnCenter (\fIi\fP) is out of image.''"
The value of the \fBVxNrowCenter\fP or \fBVxNcolumnCenter\fP resource
specifies a location outside the image.
.IP "``Widget has NULL image resource.''"
A routine such as \fBVImageViewWindowToImage\fP was called while the
widget's \fBVxNimage\fP resource was
.SB NULL.
.IP "``zoomLevel is less than 100.''"
The value of the \fBVxNzoomLevel\fP resource must be >= 100.
.SH AUTHORS
.nf
Art Pope <pope@cs.ubc.ca>,
Daniel Ko <ko@cs.ubc.ca>,
Dan Razzell <razzell@cs.ubc.ca>
