.ds Vn 2.1
.TH VInvertImage 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VInvertImage \- invert image pixel values
.SH SYNOPSIS
.nf
.B #include <vista/VImage.h>
.fi
.PP
.ft B
VImage VInvertImage (\fIsrc\fP, \fIdest\fP, \fIband\fP)
.RS
VImage \fIsrc\fP, \fIdest\fP;
VBand \fIband\fP;
.RE
.fi
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies the image to invert (the source image).
.IP \fIdest\fP
May specify a destination image, or may be
.SB NULL
to indicate that a new destination image should be created.
.IP \fIband\fP
May specify a particular band of the source image to be inverted,
or may be the constant \fBVAllBands\fP to indicate that all bands 
of the source image should be inverted.
.SH DESCRIPTION
\fBVInvertImage\fP inverts each source pixel value to produce the
corresponding destination pixel value. The operation is suitable for
swapping black and white in a grey-scale image.
.PP
Each destination pixel is computed from the corresponding source pixel by
.PP
.RS
\fIdest-pixel\fP = \fIM\fP - \fIsrc-pixel\fP
.RE
.PP
where \fIM\fP is the maximum value allowed by the pixel representation, or
1 in the case of a \fBVFloat\fP or \fBVDouble\fP image. Source pixel values
must lie in the range [0,\ \fIM\fP].
.PP
The destination image has the same number of rows and columns, and the
same pixel representation, as the source image. Also, it has either the same
number of bands as the source image (if \fIband\fP is \fBVAllBands\fP), or
a single band (otherwise).
.SH "RETURN VALUES"
\fBVInvertImage\fP returns the destination image if it is successful
and 
.SB NULL
otherwise.
.SH "SEE ALSO"
.BR VNegateImage (3Vi),
.BR VImage (3Vi),
.BR Vlib (7Vi)
.SH DIAGNOSTICS
.IP "``Band \fIband\fP referenced in image of \fInbands\fP band(s).''"
The \fIband\fP argument is invalid given the number of bands in \fIsrc\fP.
.IP "``Destination image has \fIdest_nbands\fP bands; \fIsrc_nbands\fP expected.''"
A destination image was specified but it has the wrong number of bands.
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIsrc_prop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``rows'', or ``columns''.
A destination image was specified but it does not have the same
pixel representation, number of rows, and number of columns as the
source image.
.IP "``Source pixel(s) out of range.''"
One or more source pixels were negative, or, in the case of a \fBVFloat\fP
or \fBVDouble\fP image, larger than 1. They were clipped to the required
range.
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
