'\" e
.ds Vn 2.1
.TH VKernel 3Vi "21 April 1994" "Vista Version \*(Vn"
.SH NAME
VGaussianKernel,VGaussianD1Kernel,VGaussianSplineKernel \- compute kernel coefficients
.SH SYNOPSIS
.nf
.B "#include <vista/VImage.h>
.fi
.HP 10n
.na
.nh
.ft B
VBoolean VGaussianKernel (int\ \fIncoeffs\fP, VDouble\ \fIcoeffs\fP[],
double\ \fIs\fP);
.ft
.hy
.HP 10n
.na
.nh
.ft B
VBoolean VGaussianD1Kernel (int\ \fIncoeffs\fP, VDouble\ \fIcoeffs\fP[],
double\ \fIs\fP);
.ft
.hy
.HP 10n
.na
.nh
.ft B
VBoolean VGaussianSplineKernel (int\ \fIncoeffs\fP, VDouble\ \fIcoeffs\fP[],
double\ \fIs\fP);
.ft
.hy
.nf
.SH ARGUMENTS
.IP \fIncoeffs\fP 10n
Specifies the number of coefficients desired.
.IP \fIcoeffs\fP
Returns the coefficients.
.IP \fIs\fP
Specifies the Gaussian's standard deviation.
.SH DESCRIPTION
These functions compute the values of a kernel function at a sequence of
discrete points, returning those values in the vector \fIcoeffs\fP. The
element \fIcoeffs\fP[\fIi\fP] is set to the value of the function at
\fIi\fP\ -\ (\fIncoeffs\fP-1)/2.
.PP
\fBVGaussianKernel\fP computes coefficients for a Gaussian kernel, which is
defined as
.PP
.RS
.EQ
f(x)~=~{1 over Z}~e sup {-x sup 2 / 2 s sup 2 }
.EN
.RE
.PP
where \fIZ\fP is a normalizing factor chosen so that the discrete kernel
sums to 1.
.PP
\fBVGaussianD1Kernel\fP computes coefficients for a first derivative of a
Gaussian, which is defined as
.PP
.RS
.EQ
f'(x)~=~{x over { sqrt {2 pi} s sup 3 }}~e sup {-x sup 2 / 2 s sup 2 }
.EN
.RE
.PP
\fBVGaussianSplineKernel\fP computes the coefficients of a Gaussian bump
whose tails are splined to zero. The kernel is Gaussian out to a
radius of 2\fIs\fP, it splines to zero between 2\fIs\fP and 4\fIs\fP, and
beyond 4\fIs\fP it is zero. It is defined as
.PP
.RS
.ta 25n
.EQ
define aa % { e sup {-x sup 2 / 2 s sup 2 } } %
define ab % { roman "for" ~ | x |~<=~2s } %
define ba % { { 1 over {16 e sup 2}} (4 - { { | x | } over s}) sup 4 } %
define bb % { roman "for" ~ 2s~<=~|x|~<=~4s, roman and} %
define ca % { 0 } %
define cb % { roman "otherwise", } %
g(x)~=~{1 over Z}~left{ 
lpile { aa above ba above ca }	lpile { ab above bb above cb }
.EN
.RE
.PP
where \fIZ\fP is a normalizing factor chosen so that the discrete kernel
sums to 1. This kernel is twice continuously differentiable. Note, however,
that \fIncoeffs\fP should be at least 8\fIs\fP+1.
.SH "RETURN VALUES"
These routines return
.SB TRUE
if successful, and
.SB FALSE
if there is some problem with arguments.
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
.SH ACKNOWLEDGMENT
The Gaussian spline kernel is from a book by Mark Nitzberg, David Mumford,
and Takahiro Shiota, ``Filtering, Segmentation and Depth'', Lecture
Notes in Computer Science 662, Springer-Verlag, 1993.
