.ds Vn 2.1
.TH VList 3Vi "3 June 1994" "Vista Version \*(Vn"
.SH NAME
VList \- representation of a list of opaque items
.SH SYNOPSIS
.nf
.B #include <vista/VList.h>
.PP
.B VList \fIvlist\fP;
.PP
.B VList VListCreate (void);
.PP
.B int VListCount (VList \fIvlist\fP);
.PP
.B VPointer VListFirst (VList \fIvlist\fP);
.PP
.B VPointer VListLast (VList \fIvlist\fP);
.PP
.B VPointer VListNext (VList \fIvlist\fP);
.PP
.B VPointer VListPrev (VList \fIvlist\fP);
.PP
.B VPointer VListCurr (VList \fIvlist\fP);
.PP
.B void VListAdd (VList \fIvlist\fP, VPointer \fIitem\fP);
.PP
.B void VListInsert (VList \fIvlist\fP, VPointer \fIitem\fP);
.PP
.B void VListAppend (VList \fIvlist\fP, VPointer \fIitem\fP);
.PP
.B void VListPrepend (VList \fIvlist\fP, VPointer \fIitem\fP);
.PP
.B VPointer VListRemove (VList \fIvlist\fP);
.PP
.B void VListConcat (VList \fIvlist1\fP, VList \fIvlist2\fP);
.PP
.B void VListDestroy (VList \fIvlist\fP, void (*\fIitem_free\fP)());
.PP
.B VPointer VListTrim (VList \fIvlist\fP);
.PP
.B "VPointer VListSearch (VList \fIvlist\fP, int (*\fIcomp\fP)(), VPointer \fIcomp_arg\fP);"
.PP
.B VPointer VListGetCurr (VList \fIvlist\fP);
.PP
.B void VListSetCurr (VList \fIvlist\fP, VPointer \fIcurr\fP);
.fi
.SH DESCRIPTION
.SS Introduction
The \fBVList\fP data type and its associated routines allow programmers to 
create and manipulate lists of opaque items. A \fBVList\fP is an ordered 
list whose elements are pointers to opaque data structures. 
.PP
The elements of a \fBVList\fP are arranged in a sequential order so that it 
is meaningful to say that an element is before (or after) another element. 
A \fBVList\fP has two pseudo-elements: BEFORE-ALL and AFTER-ALL. BEFORE-ALL 
is located before all other elements of the list. The element before 
BEFORE-ALL is still BEFORE-ALL itself; the element after BEFORE-ALL is the 
first element of the list (or AFTER-ALL if the list is empty). Similarly, 
AFTER-ALL is located after all other elements of the list. The element 
after AFTER-ALL is still AFTER-ALL itself; the element before AFTER-ALL is 
the last element of the list (or BEFORE-ALL if the list is empty). 
.PP
A \fBVList\fP also has the notion of a \fIcurrent item\fP. The current item 
of a \fBVList\fP may be a particular element of the list, the 
pseudo-element BEFORE-ALL, or the pseudo-element AFTER-ALL. The value of 
the current item is simply the value of the element it represents, or 
\fBNULL\fP if it is the pseudo-element BEFORE-ALL or AFTER-ALL. 
.SS "Routines and Macros"
\fBVListCreate\fP creates an empty \fBVList\fP, makes
BEFORE-ALL the current item, and returns the new \fBVList\fP.
.PP
\fBVListCount\fP returns the number of elements in \fIvlist\fP.
.PP
\fBVListFirst\fP makes the first element of \fIvlist\fP the current
item if \fIvlist\fP is not empty, and makes BEFORE-ALL the current item
otherwise. It returns the value of the new current item.
.PP
\fBVListLast\fP makes the last element of \fIvlist\fP the current
item if \fIvlist\fP is not empty, and makes AFTER-ALL the current item
otherwise. It returns the value of the new current item.
.PP
\fBVListNext\fP makes the element after the current item of \fIvlist\fP 
the new current item, and returns the value of the new current item.
.PP
\fBVListPrev\fP makes the element before the current item of \fIvlist\fP
the new current item, and returns the value of the new current item.
.PP
\fBVListCurr\fP returns the value of the current item of \fIvlist\fP.
.PP
\fBVListAdd\fP adds \fIitem\fP to \fIvlist\fP immediately after
the current item if the current item is not AFTER-ALL.
Otherwise, \fIitem\fP is added to \fIvlist\fP immediately before 
AFTER-ALL.
\fBVListAdd\fP makes \fIitem\fP the new current item.
.PP
\fBVListInsert\fP adds \fIitem\fP to \fIvlist\fP immediately before
the current item if the current item is not BEFORE-ALL.
Otherwise, \fIitem\fP is added to \fIvlist\fP immediately after 
BEFORE-ALL.
\fBVListInsert\fP makes \fIitem\fP the new current item.
.PP
\fBVListAppend\fP adds \fIitem\fP to \fIvlist\fP immediately before
AFTER-ALL, and makes \fIitem\fP the new current item.
.PP
\fBVListPrepend\fP adds \fIitem\fP to \fIvlist\fP immediately after
BEFORE-ALL, and makes \fIitem\fP the new current item.
.PP
\fBVListRemove\fP returns \fBNULL\fP if the current item of \fIvlist\fP
is either BEFORE-ALL or AFTER-ALL.
Otherwise, \fBVListRemove\fP removes the current item from \fIvlist\fP,
returns the value of the element being removed, and makes the element after the
original current item the new current item.
.PP
\fBVListConcat\fP concatenates \fIvlist2\fP to the end of \fIvlist1\fP.
If the current item of \fIvlist1\fP is AFTER-ALL, then \fBVListConcat\fP 
will make the element after BEFORE-ALL of \fIvlist2\fP the current item
of the concatenated list; otherwise, the current item of \fIvlist1\fP 
will become the current item of the concatenated list.  
The concatenated list will be stored in \fIvlist1\fP, and \fIvlist2\fP will
cease to exist after the concatenation.
.PP
\fBVListDestroy\fP frees all the storage area occupied by \fIvlist\fP,
including the storage area occupied by the opaque data objects pointed to
by the elements of \fIvlist\fP. \fIitem_free\fP is a pointer to a function
that knows how to free an opaque data object. \fIitem_free\fP must have the
following prototype:
.RS
\fBvoid \fIitem_free\fP (VPointer \fIopaque_object\fP)\fP
.RE
\fRwhere \fIopaque_object\fP is the object to be freed.
.PP
\fBVListTrim\fP removes the last element of \fIvlist\fP,
returns the value of the element being removed, and makes
the new last element the current item (by calling \fBVListLast\fP)
if \fIvlist\fP is not empty. Otherwise, \fBVListTrim\fP makes
AFTER-ALL the current item and
returns \fBNULL\fP.
.PP
\fBVListSearch\fP searches for the first occurrence (starting
from the current item) of an element \fIe\fP
in \fIvlist\fP such that a call to (*\fIcomp\fP)(\fIe\fP, \fIcomp_arg\fP) 
returns a non-zero value. If such an element exists, \fBVListSearch\fP 
makes that element the current item and returns the value of that element.
Otherwise, \fBVListSearch\fP makes AFTER-ALL the current item and returns
\fBNULL\fP.
.PP
\fBVListGetCurr\fP returns an opaque pointer denoting the current item.
This pointer is only intended for use in a subsequent invocation of
\fBVListSetCurr\fP.
.PP
\fBVListSetCurr\fP sets the identity of the current item using a value
previously supplied by \fBVListGetCurr\fP. Care must be taken to ensure
that this isn't an item that has been removed from the list since
\fBVListGetCurr\fP was used.
.SH "SEE ALSO"
.BR Vlib (7Vi)
.SH AUTHOR
Daniel Ko <ko@cs.ubc.ca>
