.ds Vn 2.1
.TH VLoadParameters 3Vi "6 June 1994" "Vista Version \*(Vn"
.SH NAME
VLoadParameters \- load values for a set of parameters
.SH SYNOPSIS
.B "#include <vista/option.h>"
.HP 10n
.ft B
.na
.nh
VBoolean VLoadParameters (int\ \fInparams\fP, VOptionDescRec\ \fIparams\fP[],
VStringConst\ \fIfilename\fP, VStringConst\ \fIobject\fP, 
VPointer\ \fIvalues\fP, VBoolean\ \fIignore_unrecog\fP);
.hy
.ad
.ft
.SH ARGUMENTS
.IP \fInparams\fP 10n
Specifies the number of entries in the table of option descriptors.
.IP \fIparams\fP
Specifies a table of option descriptors. Each entry describes one parameter.
.IP \fIfilename\fP
Specifies the file from which parameter values are to be loaded.
.IP \fIobject\fP
In the case where the file is a Vista data file, this specifies the name of
the attribute list within that file containing the list of parameter
values.
.IP \fIvalues\fP
Specifies the location of the structure supplying storage for the parameter
values loaded.
.IP \fIignore_unrecog\fP
Specifies whether unrecognized parameters encountered in the file should be
ignored (\c
.SB TRUE\c
), or whether they should result in an error (\c
.SB FALSE\c
).
.SH DESCRIPTION
\fBVLoadParameters\fP loads parameter values from a parameter definition
file (see \fBVpd\fP(5Vi)) or a Vista data file (see \fBVfile\fP(5Vi)).
.PP
The parameters are described by a table of option descriptors (see
\fBVoption\fP(3Vi)). The \fBvpdc\fP(1Vi) man page shows how such a table
can be created automatically from the contents of a parameter definition
file.
.PP
If \fIfilename\fP has the suffix \fB.vpd\fP, it is taken to be a parameter
definition file. If it has the suffix \fB.v\fP, it is taken to be a Vista
data file. Any other suffix produces an error.
.PP
In loading parameter values from a Vista data file, \fBVLoadParameters\fP
looks for a top-level attribute whose name is specified by \fIobject\fP.
The value of this attribute must be an attribute list. Each element on that
attribute list supplies the name and value of one parameter.
.PP
Parameter values are stored in the structure pointed to by \fIvalues\fP.
The offset at which a parameter lies within that structure must be supplied
by the \fIvalue\fP field of the parameter's \fIparams\fP table entry.
Appropriate definitions having this property are produced by the \fBvpdc\fP
compiler.
.SH "RETURN VALUES"
\fBVLoadParameters\fP returns 
.SB TRUE
if successful, and 
.SB FALSE
on encountering an error.
.SH EXAMPLES
The \fBVpd\fP(5Vi) man page shows an example of a parameter definition file
for the \fBMatchPars\fP set of parameters. A file of that form can be used
to supply values for the \fBMatchPars\fP set. The following Vista data file
can also supply values for those parameters:
.PP
.nf
.ft B
.RS
V-data 2 {
	parameters: {
		model_match_prior: 0.2
		feature_match_prior: 0.5
		cutoff_time: 10
		enforce_consistency: false
		max_branching: 10
		version: "V2.2"
	}
}
.RE
.ft
.fi
.SH "SEE ALSO"
.na
.nh
.BR vpdc (1Vi),
.BR Voption (3Vi),
.BR VParseParamOptions (3Vi),
.BR VPrintParameters (3Vi),
.BR VReportValidParamOptions (3Vi),
.BR Vpd (5Vi),
.BR Vlib (7Vi)
.hy
.ad
.SH DIAGNOSTICS
.IP "``EOF expected at line \fIi\fP.''"
A parameter definition file contains garbage.
.IP "``Error at line \fIi\fP.''"
Reports the line at which the previously-reported error was encountered.
.IP "``Expected \fIname\fP to be a \fIrepn1\fP, not a \fIrepn2\fP.''"
In parsing a parameter definition file, \fBVLoadParameters\fP encountered a
conflicting definition for the \fIname\fP parameter.
.IP "``Failed to process \fIfilename\fP.''"
Reports the name of the file in which the previously-reported error was
encountered.
.IP "``\fIname\fP attribute has bad value.''"
An attribute supplying a value for the \fIname\fP parameter has a value
that cannot be recognized.
.IP "``Parameter \fIname\fP not recognized.''"
The file contains a value for a parameter not among those listed in
\fIparams\fP. This error is reported only if \fIignore_unrecog\fP is
.SB FALSE.
.IP "``Parameter \fIname\fP takes multiple values.''"
The option description record for parameter \fIname\fP specifies that it
takes two or more values; parameters can only take one value each.
.IP "``Suffix of \fIfilename\fP not recognized.''"
The suffix of \fIfilename\fP is neither \fB.vpd\fP nor \fB.v\fP.
.IP "``The \fIobject\fP attribute's value isn't an attribute list.''"
The \fIobject\fP argument specifies an attribute in \fIfilename\fP
whose value isn't a attribute list.
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
