'\" e
.ds Vn 2.1
.TH VOptFlowWLS 3Vi "22 April 1994" "Vista Version \*(Vn"
.SH NAME
VOptFlowWLS \- compute optical flow by weighted least squares
.SH SYNOPSIS
.nf
.B "#include <vista/VImage.h>
.fi
.HP 10n
.na
.nh
.ft B
VImage VOptFlowWLS (\fBVImage\fP\ \fIsrc\fP, \fBVImage\fP\ \fIdest\fP, 
\fBint\fP\ \fImethod\fP, \fBdouble\fP\ \fIsmoothing\fP, 
\fBdouble\fP\ \fIthreshold\fP, \fB double\fP\ \fInoise\fP\fB[2]\fP,
\fBdouble\fP\ \fIprior_variance\fP);
.ft
.hy
.nf
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies the motion sequence image from which optical flow is to be
computed.
.IP \fIdest\fP
Specifies a destination image, or is
.SB NULL
to indicate that a new destination image should be created.
.IP \fImethod\fP
Specifies the method to be used to compute optical flow. See ``Method'',
below.
.IP \fIsmoothing\fP
Specifies the standard deviation, in pixel-frames, of the Gaussian kernel
used for spatiotemporal smoothing.
.IP \fIthreshold\fP
Specifies a threshold that must be exceeded for a pixel's image velocity to
be considered significant. This corresponds to
.EQ
tau
.EN
in [3].
.IP \fInoise\fP
Specifes the variances of two sources of noise in the spatiotemporal
derivatives. These correspond to 
.EQ
sigma sub 1
.EN
and 
.EQ
sigma sub 2
.EN
in [2]. If \fInoise\fP[0] is
zero, modeling of derivative noise is disabled.
.IP \fIprior_variance\fP
Specifies the variance of the prior distribution of image velocities. This
corresponds to 
.EQ
sigma sub p
.EN
in [2]. If \fIprior_variance\fP is zero, no prior is used.
.SH DESCRIPTION
\fBVOptFlowWLS\fP computes optical flow from a motion sequence. It
implements three methods, all based on the use of first-order
spatiotemporal derivatives and the gradient constraint equation.
.PP
A motion sequence is expected to be a multi-frame gray-scale image with
a single component, color, and viewpoint (i.e., \fBncomponents:\ 1\fP,
etc.). It may have any pixel type.
.PP
Optical flow will be returned as a multi-frame gradient image (i.e.,
\fBncomponents:\ 2\fP and \fBcomponent_interp:\ gradient\fP) with
\fBfloat\fP pixels. The two components represent the x and y components of
image velocity, in pixels per frame.  At pixels where flow cannot be
computed, the gradient is set to [Inf,Inf].
.SS Method
\fBVOptFlowWLS\fP first smoothes the motion sequence with a spatiotemporal
Gaussian filter, and then estimates derivatives using four-point central
differences.  What happens next depends on the value of \fImethod\fP, which
should be one of the following:
.RS 2n
.IP 0 4n
This method involves the simplest application of the gradient constraint
equation, but it produces the least impressive results.  The image velocity
at each pixel is given the same direction as the intensity gradient, and
its magnitude is computed from the gradient constraint equation.  Image
velocity is set to [Inf,Inf] at pixels where the intensity gradient
magnitude is less than \fIthreshold\fP.
.IP 1
This method solves for the velocity at each pixel by fitting a
constant-velocity model to the pixel's 5x5 neighborhood of derivatives,
using the gradient constraint equation to dictate the relation between
velocity and derivatives. The fit it found by weighted least squares
optimization, with weights used to give most influence to pixels near the
center of the neighbourhood. Image velocity is set to [Inf,Inf] at pixels
where either eigenvalue of the least squares solution's characteristic
equation is less than \fIthreshold\fP (see [3]).
This method generally produces the best results.
.IP 2
Like method 0, this solves only for the magnitude of the velocity, taking
its direction to be that of the intensity gradient. But like method 1, this
uses a weighted least squares fit to a 5x5 neighborhood to find the
solution. Image velocity is set to [Inf,Inf] at pixels where both
eigenvalues of the least squares solution's characteristic equation are less
than \fIthreshold\fP.
.RE
.PP
The use of weighted least squares optimization follows Lucas and Kanade
[1]. Methods 1 and 2 incorporate an extension, due to Simoncelli et al.
[2], that more fully models the effects of noise in the spatiotemporal
derivatives.  More information on all three methods may be found in a
survey by Barron et al. [3].
.SH "RETURN VALUES"
\fBVOptFlowWLS\fP returns the destination image if it is successful and
.SB NULL
otherwise.
.SH NOTES
Because smoothing and differencing are performed in all three dimensions,
several consecutive frames of motion sequence are needed to compute each
frame of optical flow. The number of motion frames participating in each
flow frame is 6\fIsmoothing\fP+5 or 6\fIsmoothing\fP+6 (whichever is odd).
Consequently, there will be fewer flow frames than motion frames, and the
flow frames produced will correspond to the middle frames of the motion
sequence.
.PP
Central differences are computed using the kernel
.EQ
{1 over 12} roman "[ 1 -8 0 8 -1 ]".
.EN
.PP
Weights for the 5x5 neighborhoods are defined by the outer product of
[ 0.0625 0.25 0.375 0.25 0.0625 ] with itself.
.SH DIAGNOSTICS
.IP "``Source and destination must be different.''"
You cannot supply the same image for both \fIsrc\fP and \fIdest\fP.
.IP "``Source image must be a pure motion sequence.''"
The source image isn't a multi-frame, single-viewpoint, single-component,
gray-scale image. See ``Band Interpretation'' in \fBVImage\fP(3Vi).
.IP "``Source image needs \fIn\fP frames for smoothing=\fIs\fP.''"
The source image has too few frames for the specified degree of smoothing.
See the first remark under ``NOTES'', above.
.SH AUTHORS
Original implementation by John Barron.  
Vista implementation by Art Pope <pope@cs.ubc.ca>.
.SH REFERENCES
.IP [1] 4n
B.D. Lucas, T. Kanade, ``An iterative image registration technique with an
application to stereo vision'', \fBProc. DARPA Image Understanding
Workshop\fP (1981), pp.\ 121\-130.
.IP [2]
E.P. Simoncelli, E.H. Adelson, D.J. Heeger, ``Probability distributions of
optical flow'', \fBProc. CVPR\fP (1991), pp.\ 310\-315.
.IP [3]
J.L. Barron, D.J. Fleet, and S.S. Beauchemin, ``Performance of optical flow
techniques'', \fBInt. J. of Computer Vision 12\fP (1994), pp.\ 43\-77.
