.ds Vn 2.1
.ds Un inches
.TH VParsePSOptions 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VParsePSOptions \- parse PostScript-specific command line options
.SH SYNOPSIS
.nf
.B "#include <vista/postscript.h>"
.PP
.ft B
VBoolean VParsePSOptions (\fIargc\fP, \fIargv\fP, \fIps\fP)
.RS
int *\fIargc\fP;
char **\fIargv\fP;
VPSDescRec *\fIps\fP;
.RE
.PP
.B "void VReportValidPSOptions ()"
.fi
.SH ARGUMENTS
.IP \fIargc\fP 10n
Specifies the number of command line arguments to be parsed, and returns
the number of arguments that were not recognized as valid options.
.IP \fIargv\fP
Specifies a vector of command line arguments to be parsed, and returns
a vector of the arguments that were not recognized as valid options.
.IP \fIps\fP
Points to a \fBVPSDescRec\fP record for storing the PostScript parameters 
obtained.
.SH DESCRIPTION
\fBVParsePSOptions\fP parses command line options describing page layout.
It is intended to be used by Vista programs that generate PostScript
output. \fBVParsePSOptions\fP uses the same parsing rules as
\fBVParseCommand\fP; for more information, see \fBVParseCommand\fP(3Vi).
.PP
\fBVReportValidPSOptions\fP prints a message on the standard error stream
describing the options accepted by \fBVParsePSOptions\fP. It's intended to
be used in conjunction with \fBVReportUsage\fP(3Vi), as shown in the
``EXAMPLES'' section of the \fBVpostscript\fP(3Vi) man page.
.PP
\fBVParsePSOptions\fP recognizes the following command line options: 
.IP \fB-landscape\fP 15n
Gives the document a landscape orientation. If omitted, it has a
portrait orientation. 
.IP "\fB-width\fP \fIdistance\fP"
Specifies the width of the physical page. Default: 8.5in.
.IP "\fB-height\fP \fIdistance\fP"
Specifies the height of the physical page. Default: 11in.
.PP
.nf
\fB-left\fP \fIdistance\fP
\fB-right\fP \fIdistance\fP
\fB-top\fP \fIdistance\fP
.fi
.ns
.IP "\fB-bottom\fP \fIdistance\fP" 15n
Specify the width of the page margins. All default to 1in.
.IP "\fB-frame\fP \fIdistance\fP"
Specifies that a frame is to be drawn around pages of the document. The 
frame will have width \fIdistance\fP, which may be 0 to produce a hairline. 
Default: no frame. 
.IP "\fB-caption\fP \fIcaption\fP"
Specifies a caption to be drawn at the bottom of each page. Default: no
caption is drawn.
.IP "\fB-capheight\fP \fIdistance\fP"
Specifies the height of the caption font. Default: 14pt.
.PP
Distances can be specified in any of several units using these suffixes
adopted from TeX: pt (points), pc (picas), in (inches), bp (big points, of
1/72 in), cm (centimeters), mm (millimeters), dd (didot points), cc
(ciceros), sp (scaled points, of 1/65536 pt). If no units are specified,
\*(Un are assumed. The conversion to PostScript units is done by
\fBVParsePSDistance\fP(3Vi).
.PP
The \fIargc\fP and \fIargv\fP arguments to \fBVParsePSOptions\fP are 
normally identical to the parameters of the program's \fBmain\fP function. 
(Note, however, that the address of \fBmain\fP's \fIargc\fP is passed 
rather than the actual value of \fIargc\fP.) 
.SH "RETURN VALUES"
If successful, \fBVParsePSOptions\fP returns 
.SB TRUE
and fills in the fields of \fIps\fP according to the command line options 
it parsed; otherwise it returns 
.SB FALSE.
.PP
\fBVParsePSOptions\fP also returns a revised \fIargv\fP containing only the 
command line arguments that it did not recognize as valid options. In 
*\fIargc\fP it returns the number of such arguments. \fBVParsePSOptions\fP 
moves elements of the \fIargv\fP vector of strings so that all arguments 
that are not part of valid options are in the first *\fIargc\fP elements. 
At minimum, there will be one: the program's name.
.SH "SEE ALSO"
.na
.nh
.BR VParseCommand (3Vi),
.BR VParsePSDistance (3Vi),
.BR Vpostscript (3Vi),
.BR Vlib (7Vi)
.ad
.hy
.SH DIAGNOSTICS
\fBVParsePSOptions\fP reports errors in command line options by printing
directly to the standard error stream. Error reports include the program
name obtained from \fIargv\fP[0]. The following messages may be produced:
.IP "``Duplicate -\fIoption\fP option; ignoring all but last.''"
The specified option was found more than once on the command line.
.IP "``Option -\fIoption\fP has incorrect value \fIarg\fP.''"
One of the values associated with -\fIoption\fP on the command line
is \fIarg\fP, which is of the wrong form with respect to the type of value
required.
.IP "``Option -\fIoption\fP requires \fIm\fP value(s); found only \fIn\fP.''"
The option keyword -\fIoption\fP must be followed by \fIn\fP values on the
command line. Only \fIm\fP were present, however.
.IP "``Option -\fIoption\fP units (\fIxx\fP) not recognized.''"
The letter pair \fIxx\fP is not a valid unit abbreviation, such as ``in''.
.IP "``Option -\fIoption\fP value too long.''"
The argument following -\fIoption\fP must have less than 100 characters.
.SH AUTHORS
Daniel Ko <ko@cs.ubc.ca>,
Art Pope <pope@cs.ubc.ca>
