.ds Vn 2.1
.TH VParseParamOptions 3Vi "6 June 1994" "Vista Version \*(Vn"
.SH NAME
VParseParamOptions \- parse command line options specifying parameter values
.SH SYNOPSIS
.B "#include <vista/param>"
.HP 10n
.na
.nh
.ft B
VBoolean VParseParamOptions (int\ \fInparams\fP,
VOptionDescRec\ \fIparams\fP[],
int\ *\fIargc\fP,
char\ **\fIargv\fP,
VPointer\ \fIvalues\fP);
.ft
.hy
.ad
.HP 10n
.na
.nh
.ft B
void VReportValidParamOptions (int\ \fInparams\fP,
VOptionDescRec\ \fIparams\fP[],
VPointer\ \fIvalues\fP);
.ft
.hy
.ad
.SH ARGUMENTS
.IP \fInparams\fP 10n
Specifies the number of entries in the table of option descriptors.
.IP \fIparams\fP
Specifies a table of option descriptors. Each entry describes one parameter.
.IP \fIargc\fP
Specifies the number of command line arguments to be parsed, and returns
the number of arguments that were not recognized as valid options.
.IP \fIargv\fP
Specifies a vector of command line arguments to be parsed, and returns
a vector of the arguments that were not recognized as valid options.
.IP \fIvalues\fP
Specifies the location of the structure containing the parameter values.
.SH DESCRIPTION
\fBVParseParamOptions\fP parses command line options that set parameter values.
.PP
The parameters are described by a table of option descriptors (see
\fBVoption\fP(3Vi)). The \fBvpdc\fP(1Vi) man page shows how such a table
can be created automatically from the contents of a parameter definition
file.
.PP
The \fIargc\fP and \fIargv\fP arguments to \fBVParseParamOptions\fP are
normally identical to the parameters of the program's \fBmain\fP function.
(Note, however, that the address of \fBmain\fP's \fIargc\fP is passed
rather than the actual value of \fIargc\fP.)
.PP
\fBVParseParamOptions\fP stores parameter values in the structure pointed
to by \fIvalues\fP.  The offset at which a parameter lies within that
structure must be supplied by the \fIvalue\fP field of the parameter's
\fIparams\fP table entry.  Appropriate definitions having this property are
produced by the \fBvpdc\fP compiler.
.PP
\fBVReportValidParamOptions\fP prints a message on the standard error
stream describing the options accepted by \fBVParseParamOptions\fP for the
specified set of parameters. It is intended to be used in conjunction with
\fBVReportUsage\fP(3Vi). Default values for the parameters, which are
included in the printed message, are supplied by \fIvalues\fP.
.SH "RETURN VALUES"
If successful, \fBVParseParamOptions\fP returns 
.SB TRUE
and fills in the fields of \fIvalue\fP according to the command line options 
it parsed; otherwise it returns 
.SB FALSE.
.PP
\fBVParseParamOptions\fP also returns a revised \fIargv\fP containing only
the command line arguments that it did not recognize as valid options. In
*\fIargc\fP it returns the number of such arguments. \fBVParseParamOptions\fP
moves elements of the \fIargv\fP vector of strings so that all arguments
that are not part of valid options are in the first *\fIargc\fP elements.
At minimum, there will be one: the program's name.
.SH "SEE ALSO"
.na
.nh
.BR vpdc (1Vi),
.BR Voption (3Vi),
.BR VParseParamOptions (3Vi),
.BR VLoadParameters (3Vi),
.BR Vpd (5Vi),
.BR Vlib (7Vi)
.hy
.ad
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
