.ds Vn 2.1
.TH VPatternImage 3Vi "1 April 1994" "Vista Version \*(Vn"
.SH NAME
VPatternImage \- fill an image with an intensity pattern
.SH SYNOPSIS
.B "#include <vista/VImage.h>"
.HP 10n
.na
.nh
.ft B
VBoolean VRampImage (VImage\ \fIimage\fP, VBand\ \fIband\fP,
double\ \fIorigin\fP[2], double\ \fIorientation\fP, double\ \fIbase\fP, 
double\ \fIslope\fP);
.ft
.hy
.ad
.HP 10n
.na
.nh
.ft B
VBoolean VSineGratingImage (VImage\ \fIimage\fP, VBand\ \fIband\fP,
double\ \fIorigin\fP[2], double\ \fIamplitude\fP, double\ \fIbase\fP,
double\ \fIfrequency\fP, double\ \fIorientation\fP, double\ \fIphase\fP);
.ft
.hy
.ad
.HP 10n
.na
.nh
.ft B
VBoolean VZonePlateImage (VImage\ \fIimage\fP, VBand\ \fIband\fP, 
double\ \fIorigin\fP[2], double\ \fIamplitude\fP, double\ \fIbase\fP, 
double\ \fIfrequency\fP, double\ \fIphase\fP);
.ft
.hy
.ad
.SH ARGUMENTS
.IP \fIimage\fP 10n
Specifies the image to be filled.
.IP \fIband\fP
May specify a particular image band to be filled with the pattern, or may be the
constant \fBVAllBands\fP to indicate that each band should be filled with 
a copy of the pattern.
.IP \fIorigin\fP
Specifies the pattern's origin as a pair of row and column coordinates.
.IP \fIorientation\fP
Specifies the orientation of the pattern, measured in radians CCW from the
direction of increasing row number.
.IP \fIamplitude\fP
Specifies the magnitude of the variable component of the pattern.
.IP \fIbase\fP
Specifies the magnitude of the constant (DC) component of the pattern.
.IP \fIslope\fP
Specifies the slope of the ramp pattern in units of intensity change per 
pixel.
.IP \fIfrequency\fP
Specifies the frequency of the sine grating or zone plate pattern, in
cycles per pixel.
.IP \fIphase\fP
Specifies the phase of the sine grating or zone plate pattern at the 
origin, in radians.
.SH DESCRIPTION
Each routine fills one or more bands of \fIimage\fP with an intensity 
pattern. If \fIband\fP is \fBVAllBands\fP, each band of the image gets a 
copy of the pattern; otherwise, only the specified band gets the pattern 
and other bands are left unchanged.
.PP
\fBVRampImage\fP creates an intensity ramp whose slope is \fIslope\fP units 
per pixel.  The ramp is centered at row \fIorigin\fP[0], column 
\fIorigin\fP[1], where it has the value \fIbase\fP.
.PP
\fBVSineGratingImage\fP creates a sine grating.  The grating has a DC 
component of \fIbase\fP and an amplitude of \fIamplitude\fP (or twice that 
peak-to-peak).  Its frequency and direction are specified by 
\fIfrequency\fP and \fIorientation\fP.  The grating is shifted so that at 
\fIorigin\fP it has phase \fIphase\fP.
.PP
\fBVZonePlateImage\fP creates the zone plate pattern defined by 
\fIf\fP(\fIr\fP)\ =\ \fIamplitude\fP\ cos(\fIr\fP^2*\fIfrequency\fP +\ \fIphase\fP)\ +\ \fIbase\fP,
where \fIr\fP is the radial distance from \fIorigin\fP.
.SH "RETURN VALUES"
These routines return
.SB TRUE
if successful, and 
.SB FALSE
otherwise.
.SH "SEE ALSO"
.na
.nh
.BR VNoiseImage (3Vi),
.BR VFillImage (3Vi),
.BR VImage (3Vi),
.BR Vlib (7Vi)
.ad
.hy
.SH DIAGNOSTICS
.IP "``Band \fIband\fP referenced in image of \fInbands\fP band(s).''"
The \fIband\fP argument is invalid given the number of bands in \fIsrc\fP.
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
