.ds Vn 2.1
.TH VRGBImageToGray 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VRGBImageToGray \- convert RGB color image to gray-scale
.SH SYNOPSIS
.nf
.B #include <vista/VImage.h>
.PP
.ft B
VImage VRGBImageToGray (\fIsrc\fP, \fIdest\fP, \fIband\fP)
.RS
VImage \fIsrc\fP, \fIdest\fP;
VBand \fIband\fP;
.RE
.fi
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies the RGB color image to be converted.
.IP \fIdest\fP
May specify a destination image, or may be
.SB NULL
to indicate that a new destination image should be created.
.IP \fIband\fP
Specifies the bands of \fIsrc\fP to be converted.
.SH DESCRIPTION
\fBVRGBImageToGray\fP converts an RGB color image to a gray-scale image.
.PP
The band interpretation attributes of \fIsrc\fP must describe it as a
standard RGB color image (\fBncolors:\ 3\fP and \fBcolor_interp:\ rgb\fP)
with a single component dimension (\fBncomponents:\ 1\fP).
.PP
If \fIband\fP is \fBVAllBands\fP, each consecutive three bands of \fIsrc\fP
is converted to a single band in \fIdest\fP; consequently \fIdest\fP has
one third as many bands as \fIsrc\fP. When \fIband\fP is not
\fBVAllBands\fP the three consecutive bands of \fIsrc\fP beginning with
\fIband\fP are converted to a single-band in \fIdest\fP, which has only one
band.
.PP
The destination image has the same number of rows and columns, and the same
pixel representation, as the source image. When the destination image is
given a copy of the source image's attributes, attributes referring to
color (\fBncolors\fP and \fBcolor_interp\fP) are deleted.
.PP
The following conversion formula is used:
.RS
.PP
\fIgray\fP = 0.299\ \fIred\fP + 0.587\ \fIgreen\fP + 0.114\ \fIblue\fP
.RE
.PP
For images with integer-valued pixel representations, the truncated
value of \fIgray\fP is used.
.SH "RETURN VALUES"
\fBVRGBImageToGray\fP returns the destination image if it is successful
and 
.SB NULL
otherwise.
.SH "SEE ALSO"
.BR VImage (3Vi),
.BR Vlib (7Vi)
.SH DIAGNOSTICS
.IP "``Band \fIband\fP referenced in image of \fInbands\fP band(s).''"
The \fIband\fP argument is invalid given the number of bands in \fIsrc\fP.
.IP "``Image isn't single-component RGB.''"
\fISrc\fP isn't an RGB color image with a single component dimension, as 
described above. 
.IP "``Destination image has \fIdest_nbands\fP bands; \fInbands\fP expected.''"
A destination image was provided but it has the wrong number of bands.
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIsrc_prop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``rows'', or ``columns''.
A destination image was specified but it does not have the same
pixel representation, number of rows, and number of columns as the
source image.
.SH AUTHOR
Daniel Ko <ko@cs.ubc.ca>
