.ds Vn 2.1
.TH VReadUbcIff 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VReadUbcIff \- read an image in UBC image file format
.SH SYNOPSIS
.nf
.ft B
#include \fB<stdio>\fP
#include \fB<vista/VImage.h>\fP
.PP
.ft B
VImage VReadUbcIff (\fIfile\fP)
.RS
FILE *\fIfile\fP;
.RE
.fi
.SH ARGUMENTS
.IP \fIfile\fP 10n
Specifies the open stream from which the image should be read.
.SH DESCRIPTION
\fBVReadUbcIff\fP reads, from an open stream, a single image in UBC image
file format (IFF). A data structure representing the image is created 
and returned. The new image has a single band and the same numbers of rows
and columns as the IFF image. Its pixel values are of type:
.RS
.PP
.IP \fBVBit\fP 10n
if the IFF image has 1 bit per pixel
.IP \fBVUByte\fP
if the IFF image has 8 bits per pixel and is unsigned
.IP \fBVSByte\fP
if the IFF image has 8 bits per pixel and is signed
.IP \fBVShort\fP
if the IFF image has 16 bits per pixel and is signed
.IP \fBVLong\fP
if the IFF image has 32 bits per pixel and is signed
.RE
.PP
No IFF image types other than those listed above are supported.
.PP
Any comment string found in the image's IFF header is retained as a
\fBcomment\fP attribute associated with the new image.
.SH "RETURN VALUES"
\fBVReadUbcIff\fP returns a newly created \fBVImage\fP, or 
.SB NULL
if an error is encountered. In the case of an error, the stream may not 
have been completely read.
.SH "SEE ALSO"
.BR image (3-UBC),
.BR VReadImages (3Vi),
.BR stdio (3),
.br
.BR VImage (3Vi),
.BR Vlib (7Vi)
.SH NOTES
Data is read from the stream sequentially, permitting the stream to be
a pipe. No rewind is performed either before reading or after.
.SH DIAGNOSTICS
.IP "``No images present in stream.''"
No images were encountered in the specified stream.
.IP "``Read from stream failed.''"
An \fBfread\fP(3) call failed because of an I/O error or a corrupt
data structure.
.IP "``Unrecognizable UBC IFF header.''"
The first bytes of the image could not be decoded as a UBC IFF Version 2
header.
.IP "``Obsolete UBC IFF header.''"
The image's IFF header indicates a version other than 2, which is the only
version supported by \fBVReadUbcIff\fP.
.IP "``Negative image will be considered positive.''"
The image's IFF header indicates that pixel values are complemented. This
is being ignored.
.IP "``[Un]signed image of \fIn\fP bpp not supported.''"
The image is of a type not supported by \fBVReadUbcIff\fP.
.IP "``Could only read \fIm\fP out of \fIn\fP bytes of image data.''"
An attempt to read the expected amount of pixel data failed with only
\fIm\fP bytes obtained. The image file may have been truncated.
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
