.ds Vn 2.1
.TH VRotateImage 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VRotateImage \- rotate an image by any angle
.SH SYNOPSIS
.nf
.B "#include <vista/VImage.h>"
.PP
.ft B
VImage VRotateImage (\fIsrc\fP, \fIdest\fP, \fIband\fP, \fIangle\fP)
.RS
VImage \fIsrc\fP, \fIdest\fP;
VBand \fIband\fP;
double \fIangle\fP;
.RE
.fi
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies the image to be rotated (the source image).
.IP \fIdest\fP
May specify a destination image, or may be
.SB NULL
to indicate that a new destination image should be created.
.IP \fIband\fP
May specify a particular band of the source image to be rotated, or may be 
the constant \fBVAllBands\fP to indicate that all bands of the source image 
should be rotated. 
.IP \fIangle\fP
Specifies the angle of rotation in radians. A positive value rotates 
counter-clockwise and a negative one rotates clockwise. 
.SH DESCRIPTION
\fBVRotateImage\fP rotates an image about its center by \fIangle\fP 
radians. The rotation is done by first rotating the image by a multiple of 
90 degrees (using flips and transpositions) to an angle nearest to the 
specified angle, and then by completing the rotation using Alan Paeth's 
three-shear method.
.PP
Because each pixel in the rotated image is a weighted average of the 
corresponding pixels in the original image, the set of colors (or gray 
shades) used by the original image will not be preserved in the rotated 
image. 
.PP
In general, the edges of the rotated image will not be parallel to the 
vertical and horizontal axes. In such cases, the destination image returned 
will be the smallest rectangular region containing the whole rotated image. 
Portions of the destination image not covered by the rotated image will be 
filled with zeros. 
.PP
If \fIdest\fP specifies a destination image, it must have the appropriate 
number of bands, rows and columns, and the same pixel representation as the 
source image. Otherwise, (when \fIdest\fP is 
.SB NULL\c
) a destination image with the correct properties will be created. Since 
determining the appropriate size of the destination image in advance is 
non-trivial, the \fIdest\fP argument should almost always be \fBNULL\fP. 
.SH "RETURN VALUES"
\fBVRotateImage\fP returns the destination image if successful and 
.SB NULL
otherwise.
.SH "SEE ALSO"
.BR VImage (3Vi),
.BR Vlib (7Vi)
.PP
``A Fast Algorithm for General Raster Rotation'' by Alan Paeth,
\fIGraphics Interface '86\fP, pp. 77-81.
.SH DIAGNOSTICS
.IP "``Band \fIband\fP referenced in image of \fInbands\fP band(s).''"
The \fIband\fP argument is invalid given the number of bands in \fIsrc\fP.
.IP "``Destination image has \fIdest_nbands\fP bands; \fIsrc_nbands\fP expected.''"
A destination image was specified but it has the wrong number of bands.
If \fIband\fP is \fBVAllBands\fP, the destination image must have the same
number of bands as the source image. Otherwise, it must have a single
band.
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIsrc_prop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``rows'', or ``columns''.
A destination image was specified but it does not have the 
appropriate properties.
.SH AUTHOR
Daniel Ko <ko@cs.ubc.ca>
