.ds Vn 2.1
.TH VScaleImage 3Vi "8 June 1994" "Vista Version \*(Vn"
.SH NAME
VScaleImage \- scale the dimensions of an image
.SH SYNOPSIS
.B "#include <vista/VImage.h>"
.na
.nh
.HP 10n
.ft B
VImage VExpandImage (VImage\ \fIsrc\fP, VImage\ \fIdest\fP,
VBand\ \fIband\fP, int\ \fIrow_scale\fP, int\ \fIcolumn_scale\fP);
.HP 10n
.ft B
VImage VReduceImage (VImage\ \fIsrc\fP, VImage\ \fIdest\fP, 
VBand\ \fIband\fP, int\ \fIrow_scale\fP, int\ \fIcolumn_scale\fP);
.HP 10n
.ft B
VImage VSampleImage (VImage\ \fIsrc\fP, VImage\ \fIdest\fP, 
VBand\ \fIband\fP, int\ \fIrow_scale\fP, int\ \fIcolumn_scale\fP);
.HP 10n
.ft B
VImage VScaleImage (VImage\ \fIsrc\fP, VImage\ \fIdest\fP, 
VBand\ \fIband\fP, double\ \fIrow_scale\fP, double\ \fIcolumn_scale\fP);
.ft
.hy
.ad
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies the image to be scaled (the source image).
.IP \fIdest\fP
May specify a destination image, or may be
.SB NULL
to indicate that a new destination image should be created.
.IP \fIband\fP
May specify a particular band of the source image to be scaled,
or may be the constant \fBVAllBands\fP to indicate that all bands 
of the source image should be scaled.
.IP \fIrow_scale\fP
Specifies the scaling factor for the row dimension.
.IP \fIcolumn_scale\fP
Specifies the scaling factor for the column dimension.
.SH DESCRIPTION
These routines produce an image that is a scaled version of \fIsrc\fP.
.RS 2n
.IP \(bu 2n
\fBVExpandImage\fP expands the row and column dimensions by the integer 
factors \fIrow_scale\fP and \fIcolumn_scale\fP. It simply replicates 
pixels. The size of the destination image is: 
.RS 2n
.nf
\fBVImageNRows\fP(\fIdest\fP) = \fBVImageNRows\fP(\fIsrc\fP) * \
\fIrow_scale\fP
.br
\fBVImageNColumns\fP(\fIdest\fP) = \fBVImageNColumns\fP(\fIsrc\fP) * \
\fIcolumn_scale\fP
.fi
.RE
.IP \(bu
\fBVReduceImage\fP shrinks the row and column dimensions by the integer 
factors \fIrow_scale\fP and \fIcolumn_scale\fP. It simply averages blocks 
of pixels. The size of the destination image is: 
.RS 2n
.nf
\fBVImageNRows\fP(\fIdest\fP) = \
\fIfloor\fP ( \fBVImageNRows\fP(\fIsrc\fP) / \fIrow_scale\fP )
.br
\fBVImageNColumns\fP(\fIdest\fP) = \
\fIfloor\fP ( \fBVImageNColumns\fP(\fIsrc\fP) / \fIcolumn_scale\fP )
.fi
.RE
.IP \(bu
\fBVSampleImage\fP shrinks the row and column dimensions by the same amount
as \fBVReduceImage\fP. Instead of averaging blocks of pixels, however, it
simply copies every \fIrow_scale\fP'th and \fIcolumn_scale\fP'th pixel.
.IP \(bu
\fBVScaleImage\fP scales the row dimension by factor of \fIrow_scale\fP and 
the column dimension by a factor of \fIcolumn_scale\fP. \fIrow_scale\fP and 
\fIcolumn_scale\fP can be any positive, floating point numbers. A value 
less than 1 shrinks the image; one greater than 1 expands it. 
\fBVScaleImage\fP uses bilinear interpolation to perform the required 
scaling. However, if either \fIrow_scale\fP or \fIcolumn_scale\fP is less 
than or equal to 0.5, it uses \fBVReduceImage\fP to reduce the image by an 
integer factor before performing the interpolation. The size of the 
destination image is: 
.RS 2n
.nf
\fBVImageNRows\fP(\fIdest\fP) = \
\fIfloor\fP ( \fBVImageNRows\fP(\fIsrc\fP) * \fIrow_scale\fP )
.br
\fBVImageNColumns\fP(\fIdest\fP) = \
\fIfloor\fP ( \fBVImageNColumns\fP(\fIsrc\fP) * \fIcolumn_scale\fP ) 
.fi
.RE
.RE
.PP
If \fIdest\fP specifies a destination image, it must have the appropriate 
number of bands, rows and columns, and the same pixel representation as the 
source image. Otherwise, (when \fIdest\fP is 
.SB NULL\fP)
a destination image with the correct
properties will be created.
.SH "RETURN VALUES"
Each routine returns the destination image if it is successful and 
.SB NULL
otherwise.
.SH "SEE ALSO"
.BR VImage (3Vi),
.BR Vlib (7Vi)
.SH NOTES
\fBVExpandImage\fP and \fBVReduceImage\fP are faster than 
\fBVScaleImage\fP, and they are to be preferred when the situation permits.
.SH DIAGNOSTICS
.IP "``Band \fIband\fP referenced in image of \fInbands\fP band(s).''"
The \fIband\fP argument is invalid given the number of bands in \fIsrc\fP.
.IP "``Destination image has \fIdest_nbands\fP bands; \fIsrc_nbands\fP expected.''"
A destination image was specified but it has the wrong number of bands.
If \fIband\fP is \fBVAllBands\fP, the destination image must have the same
number of bands as the source image. Otherwise, it must have a single
band.
.IP "``Destination image has \fIdest_prop\fP \fIproperty\fP; \fIsrc_prop\fP expected.''"
\fIProperty\fP is one of ``pixels'', ``rows'', or ``columns''.  A
destination image was specified but it does not have the same pixel
representation, number of rows, and number of columns as the source image.
.IP "``Destination image would have zero dimension.''"
If scaled as requested, the image would have zero rows and/or zero columns.
.IP "``Scaling factors are not positive.''"
The scaling factors \fIrow_scale\fP and \fIcolumn_scale\fP are not both
positive numbers.
.SH AUTHORS
Daniel Ko <ko@cs.ubc.ca>, 
Art Pope <pope@cs.ubc.ca>
