.ds Vn 2.1
.TH VSegEdgesIntoLines 3Vi "23 April 1994" "Vista Version \*(Vn"
.SH NAME
VSegEdgesIntoLines \- segment edges into straight lines
.SH SYNOPSIS
.nf
.B "#include <vista/VEdges.h>
.fi
.HP 10n
.na
.nh
.ft B
VEdges VSegEdgesIntoLines (\fBVEdges\fP\ \fIedge_set\fP, 
\fBdouble\fP\ \fIaccuracy\fP, \fBint\fP\ \fIgranularity\fP, 
\fBdouble\fP\ \fImagnitude\fP, \fBint\fP\ \fIproduct\fP);
.ft
.hy
.nf
.SH ARGUMENTS
.IP \fIedge_set\fP 10n
Specifies the set of edges to be segmented.
.IP \fIaccuracy\fP
Specifies a limit on measurement accuracy, which will be used as the lower
bound for the maximum deviation of any fragment.
.IP \fIgranularity\fP
Specifies the minimum segment length, in number of edge points.  Recursive
subdivision proceeds until fragments have at most this many points.
.IP \fImagnitude\fP
A segment is discarded if its average gradient magnitude is less than
\fImagnitude\fP. Gradient magnitude is assumed to be stored in the third
field of each edge point (where, for example, \fBvlink\fP(1Vi) places it).
.IP \fIproduct\fP
Specifies the form in which segments are to be produced, as described under
``RETURN VALUES''.
.SH DESCRIPTION
\fBVSegEdgesIntoLines\fP divides edges into segments that can be
approximated reasonably well be straight lines. Each is segmented
individually to yield one or more new, straight edges. These edges are
incorporated into a new edge set, which is returned by the routine.
.PP
The segmentation algorithm is fully described in [1] and [2]. Briefly, an
edge is first subdivided at the point where it deviates most from a
straight line joining its endpoints. Subdivision repeats recursively until
the edge is reduced to short fragments. Pairs of fragments are then
re-joined wherever the best length-to-deviation ratio of the two fragments
is exceeded by the length-to-deviation ratio of their catenation. The
resulting segments represent a partition of the entire edge into sections
that approximate straight lines.
.SH "RETURN VALUES"
\fBVSegEdgesIntoLines\fP returns the new set of edges.
.PP
The \fIproduct\fP argument determines how each new segment is represented
in the returned edge set. It's value must be one of the following:
.RS 2n
.IP 0 4n
A segment retains all of its points.
.IP 1
A segment is reduced to its two endpoints; internal points are discarded.
.IP 2
A segment's points are fit with a straight line by least squares
optimization. The segment's endpoints are projected onto this line to 
obtain two new endpoints, which form the segment that is output. This is
the default.
.RE
.SH "SEE ALSO"
.na
.nh
.BR VLinkImage (3Vi),
.BR VEdges (3Vi),
.BR Vlib (7Vi)
.hy
.ad
.SH NOTES
Each of the resulting edges (points) retains the edge (point) fields of the
edge (point) from which it is derived.
.SH LIMITATIONS
Closed edges are currently treated as if open \(em there is no attempt to fit
a straight line across the edge closure.
.SH AUTHORS
Original implementation by David Lowe <lowe@cs.ubc.ca>. Vista
implementation by Jeff Beis <beis@cs.ubc.ca> and Art Pope <pope@cs.ubc.ca>.
.SH REFERENCES
.IP [1] 4n
D.G. Lowe, \fBPerceptual Organization and Visual Recognition\fP, (Kluwer,
1985).
.IP [2]
D.G. Lowe, ``Three-dimensional object recognition from single
two-dimensional images'', \fBArtif. Intel. 31\fP (1987) 355-395.
