.ds Vn 2.1
.TH VWarning 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VWarning, VSystemWarning \- report a nonfatal error
.SH SYNOPSIS
.nf
.ft B
void VWarning (\fIformat\fP \fR[\fB, \fIarg\fR...]\fB)
.RS
VStringConst \fIformat\fP;
.RE
.PP
.ft B
void VSystemWarning (\fIformat\fR \fR[\fB, \fIarg\fR...]\fB)
.RS
VStringConst \fIformat\fP;
.RE
.fi
.SH ARGUMENTS
.IP \fIformat\fP 10n
Specifies the format of the message that is to be reported.
.PP
.IP \fIarg\fP 10n
Specifies values that are to be included in the formatted message.
.SH DESCRIPTION
\fBVWarning\fP is called to report a nonfatal error. It composes an error 
message and passes it to the nonfatal error handler, which typically prints 
the message and returns (see \fBVSetWarningHandler\fP(3Vi)). 
.PP
The message composed by \fBVWarning\fP is of the form:
.PP
.RS
.nf
\fIProgram\fB: Warning: \fIDescription\fB.
.fi
.RE
.PP
\fIProgram\fP, the program's name, is included if it has been previously 
established with a call to \fBVSetProgramName\fP. \fIDescription\fP is a 
string that has been formatted by \fBsprintf\fP using \fIformat\fP, plus 
all of the optional arguments, \fIarg\fP. 
.PP
\fBVSystemWarning\fP is identical to \fBVWarning\fP except that it also 
includes in the message a phrase based on the current value of the external 
variable \fBerrno\fP, which is used by system calls to return diagnostic 
codes. The message composed by \fBVSystemWarning\fP is of the form: 
.PP
.RS
.nf
\fIProgram\fB: Warning: \fIDescription\fB: \fIDiagnostic\fB.
.fi
.RE
.PP
\fIDiagnostic\fP is a short phrase corresponding to the value of 
\fBerrno\fP. This phrase is drawn from the table used by \fBperror\fP(3). 
.SH EXAMPLES
In this first example, the values of \fBattr_name\fP, \fBact_value\fP,
and \fBexp_value\fP are incorporated into the error message produced by
\fBVWarning\fP.
.PP
.RS
.nf
.ft B
char *attr_name;
float act_value, exp_value;

if (act_value != exp_value)
	VWarning ("MyRoutine: Attribute %s is %f rather than %f",
		attr_name, act_value, exp_value);
.fi
.RE
.PP
In this second example, the diagnostic phrase corresponding to the value of 
\fBerrno\fP is incorporated into the error message produced by
\fBVSystemWarning\fP.
.PP
.RS
.nf
.ft B
if (ioctl (fd, request, arg) == -1)
	VSystemWarning ("MyRoutine: ioctl failed");
.fi
.RE
.SH "SEE ALSO"
.na
.nh
.BR VError (3Vi),
.BR VSetWarningHandler (3Vi),
.BR VSetProgramName (3Vi),
.BR perror (3),
.BR Vlib (7Vi)
.ad
.hy
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
