.ds Vn 2.1
.TH VWriteUbcIff 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VWriteUbcIff \- write an image in UBC image file format
.SH SYNOPSIS
.nf
.ft B
#include \fB<stdio>\fP
#include \fB<vista/VImage.h>\fP
.PP
.ft B
VBoolean VWriteUbcIff (\fIfile\fP, \fIimage\fP, \fIband\fP)
.RS
FILE *\fIfile\fP;
VImage \fIimage\fP;
VBand \fBband\fP;
.RE
.fi
.SH ARGUMENTS
.IP \fIfile\fP 10n
Specifies the open stream to which the image should be written.
.IP \fIimage\fP
Specifies the image to be written.
.IP \fIband\fP
Specifies the band of \fIimage\fP to be written.
.SH DESCRIPTION
\fBVWriteUbcIff\fP writes, to an open stream, a single image band in UBC 
image file format (IFF). The IFF image will be ``positive'', and it will 
have the same numbers of rows and columns as \fIimage\fP. If \fIimage\fP's 
pixel values are of type: 
.RS
.PP
.IP \fBVBit\fP 10n
the IFF image will have 1 bit per pixel
.IP \fBVUByte\fP
the IFF image will have 8 bits per pixel and be unsigned
.IP \fBVSByte\fP
the IFF image will have 8 bits per pixel and be signed
.IP \fBVShort\fP
the IFF image will have 16 bits per pixel and be signed
.IP \fBVLong\fP
the IFF image will have 32 bits per pixel and be signed
.RE
.PP
The \fBVFloat\fP and \fBVDouble\fP pixel representations are not supported.
.PP
If \fIimage\fP has a \fBcomment\fP attribute, its value will be
included in the IFF header as a comment string.
.SH "RETURN VALUES"
\fBVWriteUbcIff\fP returns
.SB TRUE
if successful, and 
.SB FALSE
otherwise. In the case of an error, the file may be incomplete.
.SH "SEE ALSO"
.na
.nh
.BR image (3-UBC),
.BR VImage (3Vi),
.BR VWriteImages (3Vi),
.BR stdio (3),
.BR Vlib (7Vi)
.ad
.hy
.SH NOTES
Data is written to the stream sequentially, permitting the stream to be
a pipe. No rewind is performed either before writing or after.
.SH DIAGNOSTICS
.IP "``Band \fIband\fP referenced in image of \fInbands\fP band(s).''"
The \fIband\fP argument is invalid given the number of bands in \fIimage\fP.
.IP "``Floating-point pixel values not supported.''"
\fIImage\fP has pixels of type \fBVFloat\fP or \fBVDouble\fP,
which \fBVWriteUbcIff\fP does not support.
.IP "``Write to stream failed.''"
An \fBfwrite\fP(3) call failed because of an I/O error or a corrupt
data structure.
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
