.ds Vn 2.1
.ds GL /usr/lib
.TH Vpostscript 3Vi "24 March 1994" "Vista Version \*(Vn"
.SH NAME
Vpostscript \- routines for producing PostScript documents
.SH SYNOPSIS
.nf
.B #include <vista/postscript.h>
.PP
.B VPSDescRec \fIps\fP;
.fi
.SH DESCRIPTION
.SS Introduction
The \fBVpostscript\fP routines are designed to assist programmers in 
producing PostScript documents that conform to Version 3.0 of the 
PostScript Document Structuring Conventions (DSC). The routines parse 
command line options having to do with page layout, produce DSC comments 
delineating the structure of a PostScript document, set up coordinate 
systems on the page, and copy boilerplate text into the document.
.PP
There are two groups of \fBVpostscript\fP routines: a group of basic ones 
suitable for generating almost any PostScript document, and a group of 
specialized ones for generating documents depicting images. The two groups 
are described in separate sections below.
.PP
All of these routines share information via a \fBVPSDescRec\fP structure, 
which is described below under ``Data Structure''. Although the programmer 
is responsible for allocating this structure, its initialization can be 
left to the \fBVParsePSOptions\fP, \fBVBeginPSDoc\fP, and 
\fBVBeginVPSDoc\fP routines. 
.PP
Most routines take a \fBFILE\ *\fP argument, \fIf\fP, specifying the stream 
to which the PostScript document is being written. The programmer is 
responsible for opening and closing this stream. 
.SS "Basic Routines"
The following routines are suitable for generating almost any PostScript 
document. They are described here in the order that they are typically 
used. 
.HP 10n
.na
.nh
.B VBoolean VParsePSOptions (int *\fIargc\fP, char **\fIargv\fP, 
.B VPSDescRec\ *\fIps\fP)
.ad
.hy
.IP "" 0.5i
\fBVParsePSOptions\fP parses command line options and uses them to 
initialize a \fBVPSDescRec\fP record. (See \fBVParsePSOptions\fP(3Vi) for 
details.) 
.PP
.B void VReportValidPSOptions (void)
.IP
This prints a message on the standard error stream describing the options
accepted by \fBVParsePSOptions\fP. It's intended to be used in conjunction
with \fBVReportUsage\fP(3Vi), as shown below under ``EXAMPLES''.
.HP 10n
.na
.nh
.B VBoolean VParsePSDistance (VStringConst \fIstr\fP, 
.B VStringConst \fIkeyword\fP, float *\fIvalue\fP)
.ad
.hy
.IP "" 0.5i
VParsePSDistance converts a distance (e.g., ``11.5in'' or ``14pt'') to
PostScript units of 1/72 inch. (See \fBVParsePSDistance\fP(3Vi) for 
details.)
.HP 10n
.na
.nh
.B "void VBeginPSDoc (FILE *\fIf\fP, VPSDescRec *\fIps\fP,"
.B "VBoolean \fIeps\fP, VStringConst \fItitle\fP)"
.ad
.hy
.IP "" 0.5i
\fBVBeginPSDoc\fP begins outputing a PostScript document to the stream 
\fIf\fP. Pass 
.SB TRUE
for \fIeps\fP if the document is to be in Encapsulated PostScript (EPS)
format;
.SB FALSE
indicates normal PostScript format. Certain fields of \fIps\fP, such as a 
page counter and bounding box, are initialized and the following PostScript 
code is emitted: 
.PP
.RS
.ft B
.nf
%!PS-Adobe-3.0 \fR[\fP EPSF-3.0 \fR]\fP
%%BoundingBox: (atend)
%%Creator: \fIprogram_name\fP
%%CreationDate: \fIdate_and_time\fP
%%For: \fRusername@host\fP
%%Orientation: Portrait \fRor\fP Landscape
%%PageOrder: Ascend \fR(if not an EPS document)\fP
%%Pages: (atend) \fRor\fP 0 \fRdepending on\fP \fIeps\fP
%%Title: \fItitle\fP
.fi
.ft
.RE
.PP
.B void VEndPSHeader (FILE *\fIf\fP, VPSDescRec *\fIps\fP)
.IP
\fBVEndPSHeader\fP signals the end of the PostScript document's header
section by emitting
.PP
.RS
.B %%EndComments
.RE
.HP 10n
.na
.nh
.B VBoolean VIncludePSProcSet (FILE *\fIf\fP, VPSDescRec *\fIps\fP,
.B VStringConst\ \fIfilename\fP, VStringConst\ \fIproc_set_name\fP,
.B VStringConst\ \fIversion\fP, VStringConst\ \fIrevision\fP)
.ad
.hy
.IP "" 0.5i
\fBVIncludePSProcSet\fP copies a set of PostScript procedures (a ``proc 
set'' in PostScript terminology) into the PostScript document. 
\fIfilename\fP names the file containing the proc set to be included. The 
parameters \fIproc_set_name\fP, \fIversion\fP, and \fIrevision\fP specify 
the name, version number, and revision number by which the proc set is to 
be identified in the document. \fBVIncludePSProcSet\fP emits
.PP
.RS
.nf
.B %%BeginResource: procset \fIproc_set_name\fP \fIversion\fP \fIrevision\fP
contents of \fIfilename\fP
.B %%EndResource
.fi
.RE
.IP
It returns
.SB TRUE
if successful and
.SB FALSE
otherwise.
.PP
.B void VBeginPSPage (FILE *\fIf\fP, VPSDescRec *\fIps\fP)
.IP
\fBVBeginPSPage\fP starts a new page by incrementing the page counter,
\fInpages\fP, recorded in \fIps\fP, and emitting
.PP
.RS
.nf
.B "%%Page: \fInpages npages\fP \fR(if not an EPS document)\fP"
.B /VistaEnteredState save def
.fi
.RE
.HP 10n
.na
.nh
.B void VExtendPSBoundingBox (VPSDescRec *\fIps\fP, double \fIleft\fP,
.B double\ \fIbottom\fP, double\ \fIright\fP, double\ \fItop\fP)
.ad
.hy
.IP "" 0.5i
The \fBVpostscript\fP routines write a \fB%%BoundingBox\fP comment at the
end of the document to record the maximum extent of the graphics on the
document's pages. \fBVExtendPSBoundingBox\fP provides the information
needed to generate that comment. Whenever you output PostScript code that
draws on the page, call \fBVExtendPSBoundingBox\fP to specify the page area
marked. The parameters \fIleft\fP, \fIbottom\fP, \fIright\fP, and \fItop\fP
are all expressed in the coordinate system of the PostScript default user
space.
.PP
.B void VEndPSPage (FILE *\fIf\fP, VPSDescRec *\fIps\fP)
.IP
\fBVEndPSPage\fP ends the current page by emitting
.PP
.RS
.nf
.B VistaEnteredState restore
.B "showpage \fR(if not an EPS document)\fP"
.B "%%PageTrailer \fR(if not an EPS document)\fP"
.fi
.RE
.PP
.B void VEndPSDoc (FILE *\fIf\fP, VPSDescRec *\fIps\fP)
.IP
\fBVEndPSDoc\fP ends the PostScript document by emitting
.PP
.RS
.nf
.B %%Trailer
.B "%%BoundingBox: \fIleft bottom right top\fP"
.B "%%Pages: \fInpages\fP \fR(if not an EPS document)\fP"
.B %%EOF
.fi
.RE
.PP
.B VBoolean VIncludePSFile (FILE *\fIf\fP, VStringConst \fIfilename\fP)
.IP
\fBVIncludePSFile\fP 
copies the contents of the file \fIfilename\fP, unchanged, into the 
PostScript document. In can be used at any step in the production of the 
document.
.SS "Specialized Routines"
Routines in this group are specialized for creating PostScript documents 
depicting images. They provide support for setting up a coordinate system 
on the page that matches the row/column coordinate system used by an image, 
drawing a frame around the image, and labelling it with a caption. The 
``EXAMPLES'' section, below, demonstrates how the routines are used.
.PP
The PostScript code generated by these routines relies on a set of 
PostScript definitions. These are automatically included in the generated 
code as a PostScript proc set named ``vista_procs''.
.HP 10n
.na
.nh
.B "VBoolean VBeginVPSDoc (FILE *\fIf\fP, VPSDescRec *\fIps\fP,"
.B "VBoolean \fIeps\fP, VStringConst \fItitle\fP)"
.ad
.hy
.IP "" 0.5i
\fBVBeginVPSDoc\fP begins outputing a PostScript document to the stream 
\fIf\fP. Pass 
.SB TRUE
for \fIeps\fP if the document is to be in Encapsulated PostScript (EPS)
format;
.SB FALSE
indicates normal PostScript format. Certain fields of \fIps\fP, such as a
page counter and bounding box, are initialized and the following PostScript
code is emitted:
.PP
.RS
.ft B
.nf
%!PS-Adobe-3.0 \fR[\fP EPSF-3.0 \fR]\fP
%%BoundingBox: (atend)
%%Creator: \fIprogram_name\fP
%%CreationDate: \fIdate_and_time\fP
%%For: \fRusername@host\fP
%%Orientation: Portrait \fRor\fP Landscape
%%PageOrder: Ascend \fR(if not an EPS document)\fP
%%Pages: (atend) \fRor\fP 0 \fRdepending on\fP \fIeps\fP
%%Title: \fItitle\fP
%%EndComments
%%BeginProlog
%%BeginResource: vista_procs 1.1 1.1
\fRvista_procs definitions\fP
%%EndResource
%%EndProlog
%%BeginSetup
%%IncludeResource: font Times-Roman
%%EndSetup
.fi
.ft
.RE
.IP
\fBVBeginVPSDoc\fP returns
.SB TRUE
if successful and
.SB FALSE
otherwise.
.HP 10n
.na
.nh
.B "VBoolean VIncludeVPSProcSet (FILE *\fIf\fP, VPSDescRec *\fIps\fP)"
.ad
.hy
.IP "" 0.5i
\fBVIncludeVPSProcSet\fP performs a portion of the same work done by
\fBVBeginVPSDoc\fP. It outputs a copy of the ``vista_procs'' proc set:
.PP
.RS
.ft B
.nf
%%BeginResource: vista_procs 1.1 1.1
\fRvista_procs definitions\fP
%%EndResource
.fi
.ft
.RE
.IP
You might use \fBVIncludeVPSProcSet\fP if you don't use \fBVBeginVPSDoc\fP,
but you still want to use the routines listed below.
\fBVIncludeVPSProcSet\fP returns
.SB TRUE
if successful and
.SB FALSE
otherwise.
.HP 10n
.na
.nh
.B void VBeginVPSPage (FILE *\fIf\fP, VPSDescRec *\fIps\fP,
.B double\ \fInrows\fP, double\ \fIncolumns\fP)
.ad
.hy
.IP "" 0.5i
\fBVBeginVPSPage\fP starts a new page by incrementing the page counter,
\fInpages\fP, recorded in \fIps\fP, and emitting
.PP
.RS
.nf
.B "%%Page: \fInpages npages\fP \fR(if not an EPS document)\fP"
.B \VistaEnteredState save
.B "%%PageResources: font Times-Roman \fR(if not an EPS document)\fP"
.B "%%BeginPageSetup \fR(if not an EPS document)\fP"
.B VistaDict begin
.fi
.RE
.IP
It then emits code to transform the PostScript coordinate system so that
the X axis corresponds to an image ``row'' axis, and the Y axis corresponds
to an image ``column'' axis:
.RS
.IP \(bu 2n
the origin is at the upper left-hand corner
.IP \(bu
the positive X axis extends downward a distance of \fInrows\fP units
.IP \(bu
the positive Y axis extends to the right a distance of \fIncolumns\fP units
.IP \(bu
the X and Y dimensions are scaled by the same amount
.IP \(bu
it is centered within the region of the PostScript page defined by the page 
size and margin widths recorded in \fIps\fP 
.RE
.IP
Depending on various fields of the \fBVPSDescRec\fP structure, \fIps\fP, 
\fBVBeginVPSPage\fP may also draw a framing rectangle around the region of 
the page covered by this coordinate system, and a caption beneath it.
The region of the page spanned by the coordinate system, any framing 
rectangle, and any caption are all included by \fBVBeginVPSPage\fP in the 
document's bounding box.
.PP
Finally, \fBVBeginVPSPage\fP emits
.PP
.RS
.nf
.B "%%EndPageSetup \fR(if not an EPS document)\fP"
.fi
.RE
.HP 10n
.na
.nh
.B VBoolean VImageBandToPS (FILE *\fIf\fP, VImage \fIimage\fP, 
.B VBand\ \fIband\fP)
.HP 10n
.B VBoolean VRGBImageToPS (FILE *\fIf\fP, VImage \fIimage\fP, 
.B VBand\ \fIband\fP)
.HP 10n
.B VBoolean VEdgesToPS (FILE *\fIf\fP, VEdges \fIedges\fP,
.B VBoolean\ \fIdraw_endpoints\fP)
.ad
.hy
.IP "" 0.5i
These routines emit PostScript code for rendering images and edge sets. 
They assume a PostScript coordinate system has already been established by 
a prior call to \fBVBeginVPSPage\fP. The routines are described fully, 
elsewhere, by the \fBVImageToPS\fP(3Vi) and \fBVEdgesToPS\fP(3Vi) man 
pages. 
.PP
.B void VEndVPSPage (FILE *\fIf\fP, VPSDescRec *\fIps\fP)
.IP
\fBVEndVPSPage\fP ends the current page by emitting
.PP
.RS
.nf
.B end
.B VistaEnteredState restore
.B "showpage \fR(if not an EPS document)\fP"
.B "%%PageTrailer \fR(if not an EPS document)\fP"
.fi
.RE
.PP
.B void VEndVPSDoc (FILE *\fIf\fP, VPSDescRec *\fIps\fP)
.IP
\fBVEndVPSDoc\fP ends the PostScript document by emitting
.PP
.RS
.nf
.B %%Trailer
.B %%BoundingBox: \fIleft bottom right top\fP
.B "%%Pages: \fInpages\fP \fR(if not an EPS document)\fP"
.B %%EOF
.fi
.RE
.SS "Data Structure"
.nf
.ft B
.ta 25n
typedef struct {
.RS
VBoolean eps;	/* 0: PostScript; 1: Encapsulated PostScript */
int npages;	/* number of pages processed */
VStringConst prog_name;	/* program name */
VBoolean landscape;	/* 0: portrait; 1: landscape */
float page_width;	/* page dimensions */
float page_height;
float top_margin, bottom_margin;	/* margin widths */
float left_margin, right_margin;
float frame_width;	/* frame width (negative: no frame) */
float caption_height;	/* height of caption font */
VStringConst caption;	/* caption text */
VBoolean bb_set;	/* whether bounding box has been noted */
float bb_top, bb_bottom;	/* bounding box dimensions */
float bb_left, bb_right;
.RE
} VPSDescRec;
.DT
.ft
.fi
.PP
The \fBVpostscript\fP routines share information via a \fBVPSDescRec\fP 
structure. This information includes page layout parameters (e.g., paper 
size and margin widths), the current state of document generation (e.g., 
number of pages and bounding box), and options expressing page decorations 
(e.g., caption text). 
.PP
Distances such as page height are all expressed in PostScript default 
units, which are 1/72 of an inch. If the document has landscape 
orientation, the page dimensions and margin widths are adjusted accordingly 
(e.g., page height and width are exchanged). 
.SH EXAMPLES
This example uses the basic routines to produce a PostScript document: 
.PP
.nf
.RS
.ft B
int main (int argc, char **argv)
{
.RS
VPSDescRec ps;
FILE *f;

if (! VParseCommand (...) ||
.RS
! VParsePSOptions (& argc, argv, & ps)) {
VReportUsage (argv[0], \fR...\fP);
VReportValidPSOptions ();
exit (1);
.RE
}
\fRparse remaining command line options\fP
\fRopen f for output\fP

VBeginPSDoc (f, & ps, FALSE);
VEndPSHeader (f, & ps);
fputs ("%%BeginProlog\\n", f);
VIncludePSProcSet (f, & ps, "myprocs.ps", 
.RS
"myprocs", "1.0", "1.0");
.RE
fputs ("%%EndProlog\\n", f);

VBeginPSPage (f, & ps);
fputs ("newpath 100 100 moveto 200 200 lineto stroke", f);
VExtendPSBoundingBox (& ps, 100.0, 100.0, 200.0, 200.0);
\fRmore code to draw graphics on page 1\fP
VEndPSPage (f, & ps);

VBeginPSPage (f, & ps);
\fRcode to draw graphics on page 2\fP
VEndPSPage (f, & ps);

VEndPSDoc (f, & ps);
.RE
}
.ft
.RE
.fi
.PP
This example uses the specialized routines to produce an Encapsulated 
PostScript document depicting a single image: 
.PP
.nf
.RS
.ft B
int main (int argc, char **argv)
{
.RS
VPSDescRec ps;
FILE *f;
VImage image;
VBand band;
double nrows, ncolumns;

\fRparse command line options\fP
\fRobtain image and band\fP
\fRopen f for output\fP

VBeginVPSDoc (f, & ps, TRUE);

nrows = VImageNRows (image);
ncolumns = VImageNColumns (image);
VBeginVPSPage (f, & ps, nrows, ncolumns);
VImageBandToPS (f, image, band);
VEndVPSPage (f, & ps);

VEndVPSDoc (f, & ps);
.RE
}
.ft
.RE
.fi
.SH FILES
.IP \fB\*(GL/vista_procs.ps\fP
the proc set used by \fBVBeginVPSDoc\fP, \fBVBeginVPSPage\fP, etc.
.SH "SEE ALSO"
.na
.nh
.BR VParsePSOptions (3Vi),
.BR VParsePSDistance (3Vi),
.BR VReportValidPSOptions (3Vi),
.br
.BR VImageToPS (3Vi),
.BR VRGBImageToPS (3Vi),
.BR VEdgesToPS (3Vi)
.br
.I PostScript Language Reference Manual, Second Edition
.ad
.hy
.SH AUTHORS
Daniel Ko <ko@cs.ubc.ca>,
Art Pope <pope@cs.ucs.ca>
