/*
 * $Id: vadjust.c,v 1.5 1994/06/15 23:14:49 pope Exp pope $
 * 
 * The vadjust program, which adjusts image brightness and contrast.
 */

/*
 *  Copyright 1993, 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Ralph Horstmann, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include "vista/Vlib.h"
#include "vista/mu.h"
#include "vista/option.h"
#include "vista/os.h"
#include "vista/VImage.h"

/* File identification string: */
VRcsId ("$Id: vadjust.c,v 1.5 1994/06/15 23:14:49 pope Exp pope $");


/*
 *  Program entry point.
 */

int main (int argc, char **argv)
{
    static VFloat brightness = 0.0, contrast = 0.0;
    static VOptionDescRec options[] = {
        { "brightness", VFloatRepn, 1, & brightness, VOptionalOpt, NULL,
	     "Positive to increase; negative to decrease" },
        { "contrast", VFloatRepn, 1, & contrast, VOptionalOpt, NULL,
	     "Positive to increase; negative to decrease" }
    };
    FILE *in_file, *out_file;
    VAttrList list;
    VAttrListPosn posn;
    VImage image;
    int nimages = 0;

    /* Parse command line arguments: */
    VParseFilterCmd (VNumber (options), options, argc, argv,
		     & in_file, & out_file);

    /* Read the input file: */
    if (! (list = VReadFile (in_file, NULL)))
	exit (EXIT_FAILURE);

    /* Adjust each image: */
    for (VFirstAttr (list, & posn); VAttrExists (& posn); VNextAttr (& posn)) {
	if (VGetAttrRepn (& posn) != VImageRepn)
	    continue;
	VGetAttrValue (& posn, NULL, VImageRepn, & image);
	if (! VAdjustImage (image, image, VAllBands,
			    (double) brightness, (double) contrast))
            exit (EXIT_FAILURE);
	nimages++;
    }

    /* Write the output file: */
    if (! VWriteFile (out_file, list))
	exit (EXIT_FAILURE);

    fprintf (stderr, "%s: Adjusted %d image%s.\n",
	     argv[0], nimages, nimages == 1 ? "" : "s");
    return EXIT_SUCCESS;
}
