.ds Vi Vista
.ds Vn 2.1
.TH vcanny 1Vi "24 April 1993" "\*(Vi Version \*(Vn"
.SH NAME
vcanny \- detect edges in an image using a Canny operator
.SH SYNOPSIS
\fBvcanny\fR [\fB-s \fIsigma\fR] [\fB-\fIoption\fR ...] \
[\fIinfile\fR] [\fIoutfile\fR]
.SH DESCRIPTION
\fBvcanny\fP detects edges in each input image to produce output images 
that represent edge magnitude and, optionally, edge orientation.
.PP
Edges are found by marking the local maxima of the gradient of the Gaussian
smoothed image. The magnitude of the gradient at these local maxima is
scaled to the range [1,255] to produce an edge magnitude map. Pixels that
are not local maxima have values of zero in this map. If requested an edge
orientation map, indicating the edge orientation at the local maxima, is
produced as well.
.PP
The program uses the non-maximum suppression technique described in Section 
5.3 of J. F. Canny, ``Finding Edges and Lines in Images'', AI-TR-720, M.I.T.
A rotationally symmetric Gaussian filter is used, and the 2D edge operator 
is implemented as two 1D filters to minimize computation.
.PP
An input image may have any type of pixel representation. Only band 0 of 
each input image is processed. Edge magnitude maps are generated with 
\fBubyte\fP pixels, and edge orientation maps are generated with 
\fBfloat\fP pixels. 
.PP
The orientation of an edge is defined as the direction of greatest increase
in image intensity. It is measured in radians, counterclockwise from the
positive x-axis (direction of increasing column index) and it is in the
range [-Pi,+Pi]. Locations that are not local edge maxima will have
orientation map pixel values of zero. Note that because zero is a legal
orientation value one cannot use the orientation map alone to identify edge
pixels.
.PP
The program also estimates low and high threshold values for Canny's
hysteresis thresholding (see \fBvlink\fP(1Vi)). These estimates are stored
in the \fBlow_threshold\fP and \fBhigh_threshold\fP attributes of the edge
magnitude map. The low threshold is the smallest value, \fIr\fP, such that
at least a fraction \fIq\fP of the image pixels have edge magnitudes below
\fIr\fP. The high threshold is simply twice the low threshold. The
parameter \fIq\fP can be supplied by a \fB-noise\fP option, or allowed to
default. Note that the estimation method is only a heuristic, and in most
cases human judgement may find better threshold values.
.SH COMMAND LINE OPTIONS
\fBvcanny\fP accepts the following options:
.IP \fB-help\fP 15n
Prints a message describing options.
.IP "\fB-in\fP \fIinfile\fP"
Specifies the Vista data file containing the input images.
.IP "\fB-out\fP \fIoutfile\fP"
Specifies where to write the output images as a Vista data file.
.IP "\fB-sigma\fP \fIsigma\fP"
Specifies the standard deviation of the Gaussian filter used in the edge 
operator. Default: 1.0.
.IP "\fB-noise\fP \fIq\fP"
Specifies a fraction of the pixels to be considered edge noise when
estimating hysteresis thresholds. The value \fIq\fP must be in [0,1].
Default: 0.6.
.IP \fB-orientation\fP
Specifies that edge orientation maps are to be produced as well as edge 
magnitude maps. Default: edge orientation maps are not produced.
.IP "\fB-result\fP \fBafter\fP | \fBinsert\fP | \fBreplace\fP"
Specifies what to put in the output file:
.RS 17n
.IP \fBafter\fP 9n
Each input image is copied to the output file followed immediately by its
edge maps.
.IP \fBinsert\fP
Each input image is copied to the output file with the image's edge maps
embedded in its attribute list.
.IP \fBreplace\fP
Each input image is replaced by its edge maps in the output file (the default).
.RE
.PP
Input and output files can be specified on the command line or allowed to
default to the standard input and output streams.
.SH "SEE ALSO"
.na
.nh
.BR vlink (1Vi),
.BR VCanny (3Vi),
.BR Vista (7Vi)
.SH AUTHOR
Richard\ Pollock\ <pollock@cs.ubc.ca>,
Daniel\ Ko\ <ko@cs.ubc.ca>,
Art\ Pope\ <pope@cs.ubc.ca>
