/*
 *  $Id: vcomplex.c,v 1.5 1994/06/15 23:14:49 pope Exp pope $
 *
 *  The vcomplex program creates a complex image from real and/or imaginary
 *  components.
 */

/*
 *  Copyright 1993, 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Arthur Pope, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include "vista/Vlib.h"
#include "vista/mu.h"
#include "vista/option.h"
#include "vista/os.h"
#include "vista/VImage.h"

/* From the standard C library: */
#include <math.h>

/* File identification string: */
VRcsId ("$Id: vcomplex.c,v 1.5 1994/06/15 23:14:49 pope Exp pope $");

/* Later in this file: */
VImage ReadImage (VStringConst);

/* Command line options: */
static VStringConst real_name, imag_name;
static VBoolean real_found, imag_found;
static VLong repn = VFloatRepn;
static VDictEntry repn_dict[] = {
    { "float",	VFloatRepn },
    { "double",	VDoubleRepn }, 
    { NULL }
};
static VStringConst object_name = "image";
static VOptionDescRec options[] = {
    { "real", VStringRepn, 1, & real_name, & real_found, NULL,
	  "Real component of image" },
    { "imaginary", VStringRepn, 1, & imag_name, & imag_found, NULL,
	  "Imaginary component of image" },
    { "repn", VLongRepn, 1, & repn, VOptionalOpt, repn_dict,
	  "Desired pixel representation" },
    { "name", VStringRepn, 1, & object_name, VOptionalOpt, NULL,
	  "Name to be given created image" }
};


/*
 *  Program entry point.
 */

int main (int argc, char *argv[])
{
    FILE *out_file;
    VImage real_image, imag_image, result;
    VAttrList list;

    /* Parse command line arguments and identify the input and output files: */
    VParseFilterCmd (VNumber (options), options, argc, argv,
		     NULL, & out_file);

    /* Read the real and imaginary components: */
    real_image = real_found ? ReadImage (real_name) : NULL;
    imag_image = imag_found ? ReadImage (imag_name) : NULL;

    /* Build a complex image from them: */
    if (! (result = VBuildComplexImage (real_image, VAllBands,
					imag_image, VAllBands,
					NULL, (VRepnKind) repn)))
	exit (EXIT_FAILURE);

    /* Put it on an attribute list with the specified name: */
    list = VCreateAttrList ();
    VAppendAttr (list, object_name, NULL, VImageRepn, result);

    /* Open and write the output file: */
    if (! VWriteFile (out_file, list))
	exit (EXIT_FAILURE);

    fprintf (stderr, "%s: 1 complex image created.\n", argv[0]);
    return EXIT_SUCCESS;
}


/*
 *  ReadImage
 *
 *  Read a single image from a named file.
 */

VImage ReadImage (VStringConst filename)
{
    FILE *f;
    int n;
    VAttrList attributes;
    VImage *images;

    /* Open the file: */
    f = VOpenInputFile (filename, TRUE);

    /* Read images from it: */
    if ((n = VReadImages (f, & attributes, & images)) == 0)
	exit (EXIT_FAILURE);	/* no images found */
    if (n > 1)
	VError ("Input file %s contains more than 1 image");

    /* Close the file and discard things not needed: */
    fclose (f);
    VDestroyAttrList (attributes);
    return images[0];
}
