/*
 *  $Id: vconvolve.c,v 1.8 1994/06/15 23:14:49 pope Exp pope $
 *
 *  The vconvolve program, which convolves two images found in
 *  Vista data files.
 */

/*
 *  Copyright 1993, 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Arthur Pope, UBC Laboratory for Computational Intelligence
 */

/* From the V library: */
#include "vista/Vlib.h"
#include "vista/mu.h"
#include "vista/option.h"
#include "vista/os.h"
#include "vista/VImage.h"

/* File identification string: */
VRcsId ("$Id: vconvolve.c,v 1.8 1994/06/15 23:14:49 pope Exp pope $");


int main (int argc, char **argv)
{
    static VString mask_filename;
    static VLong pad_method = VConvolvePadZero;
    static VDictEntry pad_dict[] = {
        { "border",	VConvolvePadBorder }, 
        { "none",	VConvolvePadNone },
        { "trim",	VConvolvePadTrim },
        { "wrap",	VConvolvePadWrap },
        { "zero",	VConvolvePadZero },
        { NULL }
    };
    static VLong shift = 0;
    static VOptionDescRec options[] = {
	{ "mask", VStringRepn, 1, & mask_filename, VRequiredOpt, NULL,
	      "File containing mask image" },
        { "pad", VLongRepn, 1, & pad_method, VOptionalOpt, pad_dict,
	      "Method of padding at image borders" },
        { "shift", VLongRepn, 1, & shift, VOptionalOpt, NULL,
	      "Bits to right-shift resulting pixel values" }
    };
    FILE *in_file, *out_file, *f;
    VAttrList list;
    VImage *masks, src, dest;
    int nmasks, nimages = 0;
    VAttrListPosn posn;

    /* Parse command line arguments: */
    VParseFilterCmd (VNumber (options), options, argc, argv,
		     & in_file, & out_file);

    /* Read the mask image: */
    f = VOpenInputFile (mask_filename, TRUE);
    if ((nmasks = VReadImages (f, & list, & masks)) == 0)
	exit (EXIT_FAILURE);
    if (nmasks > 1)
	VError ("Mask file %s contains %d rather than 1 image",
		mask_filename, nmasks);
    fclose (f);

    /* Read source image(s): */
    if (! (list = VReadFile (in_file, NULL)))
	exit (EXIT_FAILURE);

    /* Convolve each source image with the mask: */
    for (VFirstAttr (list, & posn); VAttrExists (& posn); VNextAttr (& posn))
	if (VGetAttrRepn (& posn) == VImageRepn) {
	    VGetAttrValue (& posn, NULL, VImageRepn, & src);
	    dest = VConvolveImage (src, NULL, VAllBands, masks[0],
				   (VConvolvePadMethod) pad_method, shift);
	    if (! dest)
		exit (EXIT_FAILURE);
	    VSetAttrValue (& posn, NULL, VImageRepn, dest);
	    VDestroyImage (src);
	    nimages++;
	}

    /* Write the results to the output file: */
    if (! VWriteFile (out_file, list))
	exit (EXIT_FAILURE);
    fprintf (stderr, "%s: Convolved %d image%s with the mask.\n",
	     argv[0], nimages, nimages == 1 ? "" : "s");
    return EXIT_SUCCESS;
}
