.ds Vn 2.1
.TH vconvolve 1Vi "11 April 1994" "Vista Version \*(Vn"
.SH NAME
vconvolve \- convolve or cross-correlate a pair of images
.SH SYNOPSIS
\fBvconvolve \fB-mask\fP \fImaskfile\fP [-\fIoption\fP ...] \
[\fIinfile\fP] [\fIoutfile\fP]
.SH DESCRIPTION
\fBvconvolve\fP convolves a file of input images with a mask image to 
produce a file of output images.
.PP 
The convolutions are performed in the spatial domain using inner products.  
Conceptually, an output pixel value is computed by centering the mask 
over the corresponding input pixel. Then each mask pixel value is 
multiplied by its underlying input pixel value, and these terms are summed 
to produce the output pixel value.
.PP
The mask can have any number of bands, rows, and columns. If the size of 
a mask dimension is \fIm\fP, the mask's center pixel has an index in that 
dimension of floor(\fIm/2\fP).
.PP
The program supports several alternative methods for computing output 
pixels near the border of an output image, where the mask extends beyond 
the input image.  These include padding the input image with zeros, 
``wrapping'' the input image, and producing an output image that is trimmed 
of any pixels so affected.  The \fB-pad\fP option determines which method 
is used.
.PP
To avoid overflow, each output pixel value can be scaled down by a 
specified amount before it is stored. The \fB-shift\fP option determines 
the amount of scaling. 
.SH "COMMAND LINE OPTIONS"
\fBvconvolve\fP accepts the following options:
.IP \fB-help\fP 15n
Prints a message describing options.
.IP "\fB-in\fP \fIinfile\fP
Specifies a Vista data file containing input images to be convolved.
.IP "\fB-mask\fP \fImaskfile\fP"
Specifies a Vista data file containing the mask image. The option must 
be specified, and \fImaskfile\fP must contain a single image. If 
\fImaskfile\fP is ``\fB-\fP'', the mask image is read from the standard 
input stream.
.IP "\fB-out\fP \fIoutfile\fP"
Specifies where the output images should be written as a Vista data file. 
.IP "\fB-pad\fP \fIkeyword\fP"
Specifies how to compute output pixel values for which the mask extends 
beyond the border of the input image. \fIkeyword\fP is one of:
.RS 17n
.IP \fBborder\fP 8n
Pad the input image using values from the nearest border pixels. 
.IP \fBnone\fP
The input image isn't padded.  Instead, affected output pixels near the 
border are set to zero.
.IP \fPtrim\fP
The input image isn't padded.  Instead, the output image is trimmed so 
that it includes none of the affected border pixels.
.IP \fBwrap\fP
Wrap the input image around on itself
(e.g., the top-left pixel is considered to lie below the bottom-left 
pixel, and to the right of the top-right pixel).
.IP \fBzero\fP
Pad the input image with zeros. This is the default.
.RE
.IP "\fB-shift\fP \fIshift\fP" 15n
Specifies the number of bits by which to right-shift each output pixel 
value before storing it. Default: 0.
.PP
Input and output files can be specified on the command line or allowed to
default to the standard input and output streams.
.SH OUTPUT
In general, each output image has the same size and pixel representation 
as the corresponding input image. There are two exceptions:
.IP \(bu 2n
If the input image has unsigned pixels and the mask has signed pixels, then 
the output image has signed pixels.  Specifically, if the input image 
has \fBbit\fP or \fBubyte\fP pixels and the mask has something other than 
\fBbit\fP or \fBubyte\fP pixels, then the output image has \fBsbyte\fP pixels.
.IP \(bu
If \fB-pad trim\fP is specified, then the output image is smaller than 
the input image by an amount that depends on the mask size.  If the size of 
an input image dimension is \fIn\fP and the size of the corresponding mask 
dimension if \fIm\fP, then the size of that output image dimension is 
\fIn\fP-\fIm\fP+1.
.SH NOTES
The operation performed by \fBvconvolve\fP is correctly termed 
cross-correlation rather than convolution, but it is commonly referred
to as convolution. A proper convolution can be be performed by first 
reflecting the mask about each of its dimensions (see \fBvflip\fP(1Vi)).
.PP
If both input and mask images have integer pixel representations, pixel 
arithmetic is done using \fBVLong\fP values; otherwise \fBVDouble\fP vales 
are used (see \fBVlib\fP(7Vi), ``Numeric Data Types'').
.PP
In ANSI C, the result of right-shifting a negative signed integer is
implementation dependent. Generally a one bit is shifted into the sign
position so that the shift is equivalent to a division by two. This is the
case on all platforms we've tested, but ANSI C doesn't insist on that
behavior.
.SH "SEE ALSO"
.na
.nh
.BR vgauss (1Vi),
.BR VConvolveImage (3Vi),
.BR VConvolveImageSep (3Vi),
.BR VImage (3Vi),
.BR Vista (7Vi)
.ad
.hy
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>
