/*
 * $Id: vcrop.c,v 1.5 1994/06/15 23:14:49 pope Exp pope $
 * 
 * The vcrop program crops an image an creates a new image.
 */

/*
 *  Copyright 1993, 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Ralph Horstmann, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include "vista/Vlib.h"
#include "vista/mu.h"
#include "vista/option.h"
#include "vista/os.h"
#include "vista/VImage.h"

/* File identification string: */
VRcsId ("$Id: vcrop.c,v 1.5 1994/06/15 23:14:49 pope Exp pope $");


int main(int argc, char *argv[])
{
    static VLong left, right, top, bottom, height, width;
    static VBoolean top_found, bottom_found, height_found;
    static VBoolean left_found, right_found, width_found;
    static VOptionDescRec  options[] = {
	{"left", VLongRepn, 1, & left, & left_found, NULL,
	     "First (leftmost) column of cropped region" },
	{"right", VLongRepn, 1, & right, & right_found, NULL,
	     "Last (rightmost) column of cropped region" },
	{"width", VLongRepn, 1, & width, & width_found, NULL,
	     "Width of cropped region in columns" },
	{"top", VLongRepn, 1, & top, & top_found, NULL,
	     "First (top) row of cropped region" },
	{"bottom", VLongRepn, 1, & bottom, & bottom_found, NULL,
	     "Last (bottom) row of cropped region"},
	{"height", VLongRepn, 1, & height, & height_found, NULL,
	     "Height of cropped region in rows"}
    };
    FILE *in_file, *out_file;
    VAttrList list;
    VAttrListPosn posn;
    VImage src, result;
    int nimages = 0;

    /* Parse command line arguments: */
    VParseFilterCmd (VNumber (options), options, argc, argv,
		     & in_file, & out_file);
    
    /* Require exactly two of -top, -bottom, -height: */
    if (top_found + bottom_found + height_found != 2)
	VError ("Specify two of -top, -bottom, and -height");
    if (! top_found)
	top = bottom - height + 1;
    else if (! height_found)
	height = bottom - top + 1;
    if (height <= 0)
	VError ("Height of cropped region must be positive");
	
    /* Require exactly two of -left, -right, -width: */
    if (left_found + right_found + width_found != 2)
	VError ("Specify two of -left, -right, and -width");
    if (! left_found)
	left = right - width + 1;
    else if (! width_found)
	width = right - left + 1;
    if (width <= 0)
	VError ("Width of cropped region must be positive");

    /* Read the input file: */
    if (! (list = VReadFile (in_file, NULL)))
	exit (EXIT_FAILURE);

    /* Replace each image with a cropped version: */
    for (VFirstAttr (list, & posn); VAttrExists (& posn); VNextAttr (& posn)) {
	if (VGetAttrRepn (& posn) != VImageRepn)
	    continue;
	VGetAttrValue (& posn, NULL, VImageRepn, & src);
	result = VCropImage (src, NULL, VAllBands, top, left, height, width);
	if (! result)
	    exit (EXIT_FAILURE);
	VSetAttrValue (& posn, NULL, VImageRepn, result);
	VDestroyImage (src);
	nimages++;
    }

    /* Write the output file: */
    if (! VWriteFile (out_file, list))
	exit (EXIT_FAILURE);

    fprintf(stderr, "%s: Cropped %d image%s.\n",
	    argv[0], nimages, nimages == 1 ? "" : "s");
    return EXIT_SUCCESS;
}
