/*
 *  $Id: vflip.c,v 1.4 1994/06/15 23:14:49 pope Exp pope $
 *
 *  The vflip program, which performs one or more flip operations
 *  on Vista images.
 */

/*
 *  Copyright 1993, 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Daniel Ko, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include "vista/Vlib.h"
#include "vista/mu.h"
#include "vista/option.h"
#include "vista/os.h"
#include "vista/VImage.h"

/* File identification string: */
VRcsId ("$Id: vflip.c,v 1.4 1994/06/15 23:14:49 pope Exp pope $");


int main (int argc, char **argv)
{
    static VBoolean horizontal_flip = FALSE, vertical_flip = FALSE;
    static VOptionDescRec options[] = {
	{ "horizontal", VBooleanRepn, 1, & horizontal_flip, VOptionalOpt, NULL,
	      "Flip horizontally" },
	{ "vertical", VBooleanRepn, 1, & vertical_flip, VOptionalOpt, NULL,
	      "Flip vertically" }
    };
    FILE *in_file, *out_file;
    VAttrList list;
    VAttrListPosn posn;
    VImage image;
    int nimages = 0;

    /* Parse command line arguments: */
    VParseFilterCmd (VNumber (options), options, argc, argv,
		     & in_file, & out_file);

    /* Default to horizontal flip if no flipping is specified: */
    if (! horizontal_flip && ! vertical_flip)
	horizontal_flip = TRUE;

    /* Read source image(s): */
    if (! (list = VReadFile (in_file, NULL)))
	exit (EXIT_FAILURE);
    
    /* Perform flip operations on each image: */
    for (VFirstAttr (list, & posn); VAttrExists (& posn); VNextAttr (& posn)) {
	
	if (VGetAttrRepn (& posn) != VImageRepn)
	    continue;
	VGetAttrValue (& posn, NULL, VImageRepn, & image);
	
	/* First flip horizontal: */
	if (horizontal_flip && ! VFlipImage (image, image, VAllBands, FALSE))
	    exit (EXIT_FAILURE);
	
	/* Then flip vertical: */
	if (vertical_flip && ! VFlipImage (image, image, VAllBands, TRUE))
	    exit (EXIT_FAILURE);

	nimages++;
    }

    /* Write the results to the output file: */
    if (! VWriteFile (out_file, list))
	exit (EXIT_FAILURE);

    fprintf (stderr, "%s: Flipped %d image(s) %s%s%s.\n",
	     argv[0], nimages,
	     horizontal_flip ? "horizontally" : "",
	     (horizontal_flip && vertical_flip) ? " and " : "",
	     vertical_flip ? "vertically" : "");

    return EXIT_SUCCESS;
}
