/*
 *  $Id: vflow.c,v 1.1 1994/06/10 01:37:18 pope Exp pope $
 *
 *  The vflow program, which estimates optical flow from an image sequence.
 */

/*
 *  Copyright 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Arthur Pope, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include "vista/Vlib.h"
#include "vista/mu.h"
#include "vista/option.h"
#include "vista/os.h"
#include "vista/VImage.h"

/* From the standard C library: */
#include <math.h>

/* File identification string: */
VRcsId ("$Id: vflow.c,v 1.1 1994/06/10 01:37:18 pope Exp pope $");


/* Command line options: */
static VFloat smoothing = 1.5, threshold = 1.0;
static VShort method = 1;
static VDictEntry methodDict[] = {
    { "simple", 0 },
    { "ls", 1 },
    { "lsnorm", 2 },
    NULL
};
static VFloat noise[2] = { 0.08, 1.0}, prior = 2.0;
static VOptionDescRec options[] = {
    { "method", VShortRepn, 1, & method, VOptionalOpt, methodDict,
	  "Method" },
    { "smoothing", VFloatRepn, 1, & smoothing, VOptionalOpt, NULL,
	  "Std dev of spatial-temporal smoothing (pixel-frames)" },
    { "threshold", VFloatRepn, 1, & threshold, VOptionalOpt, NULL,
	  "Velocity significance threshold" },
    { "noise", VFloatRepn, 2, & noise, VOptionalOpt, NULL,
	  "Noise in derivative measurements" },
    { "prior", VFloatRepn, 1, & prior, VOptionalOpt, NULL,
	  "Variance of velocity prior" }
};


/*
 *  Program entry point.
 */

int main (int argc, char *argv[])
{
    FILE *in_file, *out_file;
    VAttrList list;
    VAttrListPosn posn;
    VImage src, result;
    int nimages = 0;
    double noise_dbl[2];

    /* Parse command line arguments and identify the input and output files: */
    VParseFilterCmd (VNumber (options), options, argc, argv,
		     & in_file, & out_file);
    noise_dbl[0] = noise[0];
    noise_dbl[1] = noise[1];

    /* Read the input file: */
    if (! (list = VReadFile (in_file, NULL)))
	exit (EXIT_FAILURE);

    /* Process each image: */
    for (VFirstAttr (list, & posn); VAttrExists (& posn); VNextAttr (& posn)) {
	if (VGetAttrRepn (& posn) != VImageRepn)
	    continue;
	VGetAttrValue (& posn, NULL, VImageRepn, & src);

	if (! (result = VOptFlowWLS (src, NULL, method, smoothing, threshold,
				     noise_dbl, prior)))
	    exit (EXIT_FAILURE);
	VDestroyImage (src);
	VSetAttrValue (& posn, NULL, VImageRepn, result);
	nimages++;
    }

    /* Write the output file: */
    if (! VWriteFile (out_file, list))
	exit (EXIT_FAILURE);

    fprintf (stderr, "%s: Processed %d image%s.\n",
	     argv[0], nimages, nimages == 1 ? "" : "s");
    return EXIT_SUCCESS;
}
