'\" e
.ds Vn 2.1
.TH vflow 1Vi "21 April 1994" "Vista Version \*(Vn"
.SH NAME
vflow \- compute optical flow
.SH SYNOPSIS
\fBvflow\fR [\fB-\fIoption\fR ...] [\fIinfile\fR] [\fIoutfile\fR]
.SH DESCRIPTION
\fBvflow\fP computes optical flow from a motion sequence. The current
version implements three methods, all based on the use of first-order
spatiotemporal derivatives and the gradient constraint equation.
.PP
A motion sequence is expected to be a multi-frame gray-scale image with
a single component, color, and viewpoint (i.e., \fBncomponents:\ 1\fP,
etc.). It may have any pixel type.
.PP
Optical flow will be produced as a multi-frame gradient image (i.e.,
\fBncomponents:\ 2\fP and \fBcomponent_interp:\ gradient\fP) with
\fBfloat\fP pixels. The two components represent the x and y components of
image velocity, in pixels per frame.  At pixels where flow cannot be
computed, the gradient is set to [Inf,Inf].
.PP
\fBvflow\fP first smoothes the motion sequence with a spatiotemporal
Gaussian filter, and then estimates derivatives using four-point central
differences.  What happens next depends on the \fB-method\fP option:
.RS 2n
.IP "\fB-method simple\fP"
This method involves the simplest application of the gradient constraint
equation, but it produces the least impressive results.  The image velocity
at each pixel is given the same direction as the intensity gradient, and
its magnitude is computed from the gradient constraint equation.
.IP "\fB-method ls\fP"
This method solves for the velocity at each pixel by fitting a
constant-velocity model to the pixel's 5x5 neighborhood of derivatives,
using the gradient constraint equation to dictate the relation between
velocity and derivatives. The fit it found by weighted least squares
optimization, with weights used to give most influence to pixels near the
center of the neighbourhood. This method generally produces the best results.
.IP "\fB-method lsnorm\fP"
Like the \fBsimple\fP method, this method solves only for the magnitude of
the velocity, taking its direction to be that of the intensity gradient.
But like the \fBls\fP method, this uses a weighted least squares fit to a
5x5 neighborhood to find the solution.
.RE
.PP
The \fBls\fP method is due to Lucas and Kanade [1]. Both the \fBls\fP and
\fBlsnorm\fP implementations incorporate an extension, due to Simoncelli et
al. [2], that more fully models the effects of noise in the spatiotemporal
derivatives.  More information on all three methods may be found in a
survey by Barron et al. [3].
.SH "COMMAND LINE OPTIONS"
\fBvflow\fP accepts the following options:
.IP \fB-help\fP 15n
Prints a message describing options.
.IP "\fB-in\fP \fIinfile\fP ..."
Specifies a Vista data file containing the motion sequence.
.IP "\fB-out\fP \fIoutfile\fP"
Specifies where to write the optical flow as a Vista data file.
.IP "\fB-method simple\fP | \fBls\fP | \fBlsnorm\fP"
Specifies the method to use to compute optical flow. Default: \fBls\fP.
.IP "\fB-smoothing\fP \fIs\fP"
Specifies the standard deviation, in pixel-frames, of the Gaussian kernel
used for spatiotemporal smoothing. Default: 1.5 pixel-frames.
.IP "\fB-threshold\fP \fIt\fP"
Specifies a threshold that must be exceeded for a pixel's image velocity to
be considered significant. Under the \fBsimple\fP method, \fIt\fP is a
lower bound for the magnitude of intensity gradient. Under the other two
methods, \fIt\fP is a lower bound for one (\fBlsnorm\fP) or both (\fBls\fP)
eigenvalues of the least squares solution's characteristic equation (see
[3]). Default: 1.
.IP "\fB-noise\fP \fIn1 n2\fP"
Specify the variances of two sources of noise in the spatiotemporal
derivatives. These correspond to 
.EQ
sigma sub 1
.EN
and 
.EQ
sigma sub 2
.EN
in [2]. If \fIn1\fP is
zero, modeling of derivative noise is disabled.  Defaults: 0.08 and 1.
.IP "\fB-prior\fP \fIp\fP"
Specifies the variance of the prior distribution of image velocities. This
corresponds to 
.EQ
sigma sub p
.EN
in [2]. If \fIp\fP is zero, no prior is used.
Default: 2.
.PP
Input and output files can be specified on the command line or allowed to
default to the standard input and output streams.
.SH "SEE ALSO"
.BR VImage (3Vi),
.BR Vista (7Vi)
.SH NOTES
Because smoothing and differencing are performed in all three dimensions,
several consecutive frames of motion sequence are needed to compute each
frame of optical flow. The number of motion frames participating in each
flow frame is 6\fIs\fP+5 or 6\fIs\fP+6 (whichever is odd).  Consequently,
there will be fewer flow frames than motion frames, and the flow frames
produced will correspond to the middle frames of the motion sequence.
.PP
Central differences are computed using the kernel
.EQ
{1 over 12} roman "[ 1 -8 0 8 -1 ]".
.EN
.PP
Weights for the 5x5 neighborhoods are defined by the outer product of
[ 0.0625 0.25 0.375 0.25 0.0625 ] with itself.
.SH AUTHORS
Original implementation by John Barron.  
Vista implementation by Art Pope <pope@cs.ubc.ca>.
.SH REFERENCES
.IP [1] 4n
B.D. Lucas, T. Kanade, ``An iterative image registration technique with an
application to stereo vision'', \fBProc. DARPA Image Understanding
Workshop\fP (1981), pp.\ 121\-130.
.IP [2]
E.P. Simoncelli, E.H. Adelson, D.J. Heeger, ``Probability distributions of
optical flow'', \fBProc. CVPR\fP (1991), pp.\ 310\-315.
.IP [3]
J.L. Barron, D.J. Fleet, and S.S. Beauchemin, ``Performance of optical flow
techniques'', \fBInt. J. of Computer Vision 12\fP (1994), pp.\ 43\-77.
