/*
 *  $Id: vgauss.c,v 1.4 1994/06/15 23:14:49 pope Exp pope $
 *
 *  The vgauss program, which prefroms Gaussian smoothing on Vista images.
 */

/*
 *  Copyright 1993, 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Daniel Ko, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include <vista/Vlib.h>
#include <vista/mu.h>
#include <vista/option.h>
#include <vista/os.h>
#include <vista/VImage.h>

/* File identification string: */
VRcsId ("$Id: vgauss.c,v 1.4 1994/06/15 23:14:49 pope Exp pope $");


int main (int argc, char **argv)
{
    static VDouble sigma = 1.0;
    static VLong size = 0;
    static VBoolean size_found;
    static VDictEntry dict[] = {
	{ "smooth", VFilterGaussian },
	{ "xgrad", VFilterGaussianDx },
	{ "ygrad", VFilterGaussianDy },
	NULL
    };
    static VLong type = VFilterGaussian;
    static VOptionDescRec options[] = {
	{ "sigma", VDoubleRepn, 1, & sigma, VOptionalOpt, NULL,
	      "Standard deviation of Gaussian filter" },
	{ "size", VLongRepn, 1, & size, & size_found, NULL,
	      "Filter size (pixels)" },
	{ "type", VLongRepn, 1, & type, VOptionalOpt, dict,
	      "Filter type" }
    };
    FILE *in_file, *out_file;
    VAttrList list;
    VImage src, dest;
    VAttrListPosn posn;
    int nimages;
    
    /* Parse command line arguments: */
    VParseFilterCmd (VNumber (options), options, argc, argv,
		     & in_file, & out_file);
    
    /* Filter size defaults to 6.0 * sigma + 1: */
    if (! size_found) {
	size = 6.0 * sigma + 1;
	if ((size & 1) == 0)
	    size++;
    }

    /* Read source image(s): */
    if (! (list = VReadFile (in_file, NULL)))
	exit (EXIT_FAILURE);
    
    /* Performs Gaussian smoothing on each image: */
    nimages = 0;
    for (VFirstAttr (list, & posn); VAttrExists (& posn); VNextAttr (& posn)) {
	if (VGetAttrRepn (& posn) != VImageRepn)
	    continue;
	VGetAttrValue (& posn, NULL, VImageRepn, & src);

	dest = VGaussianConvolveImage (src, NULL, VAllBands, sigma, size,
				       (VFilterKind) type);
	if (dest == NULL)
	    exit (EXIT_FAILURE);
	
	VSetAttrValue (& posn, NULL, VImageRepn, dest);
	VDestroyImage (src);
	nimages++;
    }

    /* Write the results to the output file: */
    if (VWriteFile (out_file, list))
	fprintf (stderr, "%s: Convolved %d image%s with a Gaussian filter.\n",
		 argv[0], nimages, nimages == 1 ? "" : "s");
    return EXIT_SUCCESS;
}
