/*
 *  $Id: vistat.c,v 1.5 1994/06/15 23:14:49 pope Exp pope $
 *
 *  The vistat program, which reports image statistics.
 */

/*
 *  Copyright 1993, 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Arthur Pope, UBC Laboratory for Computational Intelligence
 */

/* From the V library: */
#include "vista/Vlib.h"
#include "vista/mu.h"
#include "vista/option.h"
#include "vista/os.h"
#include "vista/VImage.h"

/* From the standard C library: */
#include <math.h>

/* File identification string: */
VRcsId ("$Id: vistat.c,v 1.5 1994/06/15 23:14:49 pope Exp pope $");


int main (int argc, char **argv)
{
    FILE *in_file, *out_file;
    VAttrList list, stat_list;
    VAttrListPosn posn;
    VImage image;
    int nimages = 0;
    VDouble min_pixel, max_pixel, mean, variance;
    VStringConst name;

    /* Parse command line arguments and identify the input and output files: */
    VParseFilterCmd (NULL, 0, argc, argv, & in_file, & out_file);

    /* Read the input file: */
    if (! (list = VReadFile (in_file, NULL)))
	exit (EXIT_FAILURE);

    /* For each image in the input file: */
    for (VFirstAttr (list, & posn); VAttrExists (& posn); VNextAttr (& posn)) {
	if (VGetAttrRepn (& posn) != VImageRepn)
	    continue;
	VGetAttrValue (& posn, NULL, VImageRepn, & image);

	/* Measure its statistics: */
	if (! VImageStats (image, VAllBands,
			   & min_pixel, & max_pixel, & mean, & variance))
	    exit (EXIT_FAILURE);
	    
	/* Assemble the statistics into an attribute list: */
	stat_list = VCreateAttrList ();
	if (VGetAttr (VImageAttrList (image), VNameAttr, NULL,
		      VStringRepn, & name) == VAttrFound)
	    VSetAttr (stat_list, VNameAttr, NULL, VStringRepn, name);
	VSetAttr (stat_list, "min", NULL, VDoubleRepn, min_pixel);
	VSetAttr (stat_list, "max", NULL, VDoubleRepn, max_pixel);
	VSetAttr (stat_list, "mean", NULL, VDoubleRepn, mean);
	VSetAttr (stat_list, "std_dev", NULL, VDoubleRepn,
		  sqrt (variance));
	    
	/* Replace the image with the statistics: */
	VDestroyImage (image);
	VSetAttrValue (& posn, NULL, VAttrListRepn, stat_list);
	nimages++;
    }


    /* Write the output file: */
    if (! VWriteFile (out_file, list))
	exit (EXIT_FAILURE);

    fprintf (stderr, "%s: Reported statistics for %d image%s.\n",
	     argv[0], nimages, nimages == 1 ? "" : "s");
    return EXIT_SUCCESS;
}
