.ds Vn 2.1
.TH vlink 1Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
vlink \- link connected non-zero image pixels to form edges
.SH SYNOPSIS
\fBvlink\fR [\fB-\fIoption\fR ...] [\fIinfile\fR] [\fIoutfile\fR]
.SH DESCRIPTION
\fBvlink\fP joins up connected non-zero pixels in an image to form a set of
edges. Each input image yields a corresponding output edge set. Only
images described as single-component gradient images (\fBncomponents:\ 1\fP
and \fBcomponent_interp: gradient\fP) are processed.
.PP
Most edge detection methods used to produce an input image will mark each 
pixel with a gradient or strength measure, such that only pixels with high 
values are likely to correspond to true edges rather than noise. Rather 
than using just a single threshold on this gradient value to select edge 
pixels, \fBvlink\fP performs Canny's hysteresis thresholding which uses 
both high and low threshold values. This method prevents breaking an edge 
into many disconnected pieces by only starting an edge when a pixel is 
above the high threshold and keeping all pixels above the low threshold in 
intervals bounded by pixels above the high threshold.
.PP
Command line options \fB-low\fP and \fB-high\fP can be used to specify the
two thresholds. But if neither option has been specified \fBvlink\fP will
look for \fBlow_threshold\fP and \fBhigh_threshold\fP attributes in the
input image. If they're present, \fBvlink\fP will use the thresholds they
specify. This allows \fBvlink\fP to work well with \fBvcanny\fP because
\fBvcanny\fP determines thresholds based on an estimate of noise in the
image and records those thresholds as \fBlow_threshold\fP and
\fBhigh_threshold\fP attributes.
.PP
Edge points in the set of edges produced each have three fields, giving the 
x,y location and gradient value of each point. 
.PP
The program discards any edges that have a length less than \fIminlength\fP 
pixels, where \fIminlength\fP is specified by a command line option. 
.SH "COMMAND LINE OPTIONS"
\fBvlink\fP accepts the following options:
.IP \fB-help\fP 15n
Prints a message describing options.
.IP "\fB-in\fP \fIinfile\fP"
Specifies a Vista data file containing the input images.
.IP "\fB-out\fP \fIoutfile\fP"
Specifies where to write the output edge sets as a Vista data file.
.IP "\fB-minlength\fP \fInum\fP"
All edges with a length in pixels less than this number will be
discarded. Default: 4.
.IP "\fB-low\fP \fInum\fP"
The low threshold value for Canny's hysteresis thresholding. Default: 15.
.IP "\fB-high\fP \fInum\fP"
The high threshold value for Canny's hysteresis thresholding. Default: 30.
.IP "\fB-result\fP \fBafter\fP | \fBinsert\fP | \fBreplace\fP"
Specifies what to put in the output file:
.RS 17n
.IP \fBafter\fP 9n
Each input image is copied to the output file followed immediately by its
edge set.
.IP \fBinsert\fP
Each input image is copied to the output file with the image's edge set
embedded in its attribute list.
.IP \fBreplace\fP
Each input image is replaced by its edge set in the output file (the default).
.RE
.PP
Input and output files can be specified on the command line or allowed to
default to the standard input and output streams.
.SH "SEE ALSO"
.na
.nh
.BR vcanny (1Vi),
.BR vconvolve (1Vi),
.BR vzeroc (1Vi),
.BR VImage (3Vi),
.BR Vista (7Vi)
.ad
.hy
.SH LIMITATIONS
Input images must have \fBubyte\fP pixels and a single band.
.SH AUTHOR
David Lowe <lowe@cs.ubc.ca>
