.ds Vn 2.1
.TH vpdc 1Vi "6 June 1994" "Vista Version \*(Vn"
.SH NAME
vpdc \- compile a Vista parameter definition file
.SH SYNOPSIS
\fBvpdc\fR [\fB-\fIoption\fR ...] \fIinfile\fR [\fIoutfile\fR]
.SH DESCRIPTION
\fBvpdc\fP read a Vista parameter definition file and produces from it a C
header file.
.PP
In the following description, \fImodule_name\fP refers to the name of the
set of parameters as specified in the parameter definition file. The header
file produced by \fBvpdc\fP contains:
.RS 2n
.IP \(bu 2n
a definition of a C structure having a field for each parameter. The
structure is given the typedef name \fImodule_name\fP.
.IP \(bu
a definition of a macro that expands to a list of the parameters' values.
This macro is given the name \fImodule_name\fB_Defaults\fR.
.IP \(bu
a definition of a macro that expands to a list of option definitions for
the parameters. This macro is given the name \fImodule_name\fB_Defns\fR.
Option definitions are described by \fBVoption\fP(3Vi).
.IP \(bu
a definition of a mcro that expands to the number of parameters. This macro
is given the name \fImodule_name\fB_N\fR.
.RE
.SH "COMMAND LINE OPTIONS"
.IP \fB-help\fP 10n
Prints a message describing options.
.IP "\fB-comment_column\fP \fIc\fP"
Specifies the column at which comments should begin in the output file.
Default: 36.
.IP "\fB-continuation_column \fId\fP"
Specifies the column at which continuation characters should be placed in
th eoutput file. Default: 78.
.IP "\fB-line true\fP | \fBfalse\fP"
Specifies whether \fB#line\fP preprocessor directives should be included in
the output file. They make it easier to locate errors in the definitions
caught by the C compiler, but the directives aren't supported by some older
compilers. Default: \fBtrue\fP.
.PP
The input file must be specified on the command line; the output file need
not be. If the output file is not specified, output is placed in a file
with the same name as the input file but with a \fB.h\fP extension
replacing the usual \fB.vpd\fP.
.SH EXAMPLE
The Vpd(5Vi) man page shows an example of a parameter definition file
for the \fBMatchPars\fP set of parameters. This file may be compiled with
.PP
.RS
.B "vpdc match_pars.vpd"
.RE
.PP
The following C program uses the C header file produced by \fBvpdc\fP to
declare storage for the parameters, initialize that storage with the
parameters' default values, and accept command line options modifying those
defaults.  The parameter values in effect are then reported to
\fBstdout\fP.
.PP
.RS
.nf
.ft B
#include "match_pars.h"

/* Declare storage for parameters and supply default values: */
MatchPars matchPars = MatchPars_Defaults;

/* Table of parameter-setting options: */
VOptionDescRec matchParDefns[MatchPars_N] = MatchPars_Defns;

/* Parse command line arguments for parameter-setting options: */
VParseParamOptions (MatchPars_N, matchParDefns, & argc, argv, & matchPars);

/* Report parameter values: */
VPrintParameters (stdout, MatchPars_N, matchParDefns, & matchPars);

/* Example of accessing a parameter value: */
if (matchPars.enforce_consistency) ...
.RE
.SH "SEE ALSO"
.na
.nh
.BR Vpd (5Vi),
.BR VLoadParameters (3Vi),
.BR VParseParamDefn (3Vi),
.BR VParseParamOptions (3Vi),
.BR VPrintParameters (3Vi),
.BR VReportValidParamOptions (3Vi),
.BR Voption(3Vi),
.BR Vista (7Vi)
.hy
.ad
.SH AUTHOR
Art Pope <pope@cs.ubc.ca>.
