/*
 *  $Id: vrgbtogray.c,v 1.4 1994/06/15 23:14:49 pope Exp pope $
 *
 *  The vrgbtogray program, which converts RGB colors images to grey-scale.
 */

/*
 *  Copyright 1993, 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Daniel Ko, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include "vista/Vlib.h"
#include "vista/mu.h"
#include "vista/option.h"
#include "vista/os.h"
#include "vista/VImage.h"

/* File identification string: */
VRcsId ("$Id: vrgbtogray.c,v 1.4 1994/06/15 23:14:49 pope Exp pope $");


int main (int argc, char **argv)
{
    FILE *in_file, *out_file;
    VAttrList list;
    VAttrListPosn posn;
    VImage src, result;
    int nimages = 0;

    /* Parse command line arguments: */
    VParseFilterCmd (0, NULL, argc, argv, & in_file, & out_file);
    
    /* Read source image(s): */
    if ((list = VReadFile (in_file, NULL)) == NULL)
	exit (EXIT_FAILURE);

    /* Performs conversion on each RGB image: */
    for (VFirstAttr (list, & posn); VAttrExists (& posn); VNextAttr (& posn)) {
	if (VGetAttrRepn (& posn) != VImageRepn)
	    continue;
	VGetAttrValue (& posn, NULL, VImageRepn, & src);

	if (VImageColorInterp (src) != VBandInterpRGB ||
	    VImageNComponents (src) != 1) {
	    VWarning ("Encountered image that's not "
		      "single-component, RGB color");
	    continue;
	}

	result = VRGBImageToGray (src, NULL, VAllBands);
	if (! result)
	    exit (EXIT_FAILURE);
	VSetAttrValue (& posn, NULL, VImageRepn, result);
	VDestroyImage (src);
	nimages++;
    }
    
    /* Write the results to the output file: */
    if (! VWriteFile (out_file, list))
	exit (EXIT_FAILURE);

    fprintf (stderr, "%s: Converted %d RGB color image%s to gray-scale.\n",
	     argv[0], nimages, nimages == 1 ? "" : "s");
    return EXIT_SUCCESS;
}
