/*
 *  $Id: vrotate.c,v 1.4 1994/06/15 23:14:49 pope Exp pope $
 *
 *  The vrotate program, which rotates an image by an arbitrary angle.
 */

/*
 *  Copyright 1993, 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Daniel Ko, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include "vista/Vlib.h"
#include "vista/mu.h"
#include "vista/option.h"
#include "vista/os.h"
#include "vista/VImage.h"

/* From the standard C libaray: */
#include <math.h>

/* File identification string: */
VRcsId ("$Id: vrotate.c,v 1.4 1994/06/15 23:14:49 pope Exp pope $");


int main (int argc, char **argv)
{
    static VDouble angle = 90.0;
    static VOptionDescRec options[] = {
	{ "angle", VDoubleRepn, 1, & angle, VOptionalOpt, NULL,
	      "Angle of rotation (degrees CCW)" }
    };
    FILE *in_file, *out_file;
    VAttrList list;
    VImage src, result;
    VAttrListPosn posn;
    int nimages = 0;
    double angle_rads;

    /* Parse command line arguments: */
    VParseFilterCmd (VNumber (options), options, argc, argv,
		     & in_file, & out_file);

    /* Convert angle to radians: */
    angle_rads = angle * M_PI / 180.0;

    /* Read source image(s): */
    if (! (list = VReadFile (in_file, NULL)))
	exit (EXIT_FAILURE);

    /* Rotate each image: */
    for (VFirstAttr (list, & posn); VAttrExists (& posn); VNextAttr (& posn)) {
	if (VGetAttrRepn (& posn) != VImageRepn)
	    continue;
	VGetAttrValue (& posn, NULL, VImageRepn, & src);

	result = VRotateImage (src, NULL, VAllBands, angle_rads);
	if (result == NULL)
	    exit (EXIT_FAILURE);
	
	VSetAttrValue (& posn, NULL, VImageRepn, result);
	VDestroyImage (src);
	nimages++;
    }

    /* Write the results to the output file: */
    if (! VWriteFile (out_file, list))
	exit (EXIT_FAILURE);

    fprintf (stderr, "%s: Rotated %d image%s by %g degrees.\n",
	     argv[0], nimages, nimages == 1 ? "" : "s", (double) angle);
    return EXIT_SUCCESS;
}
