.ds Vn 2.1
.TH vsegedges 1Vi "23 April 1994" "Vista Version \*(Vn"
.SH NAME
vsegedges \- segment edges into straight lines
.SH SYNOPSIS
\fBvsegedges\fR [\fB-\fIoption\fR ...] [\fIinfile\fR] [\fIoutfile\fR]
.SH DESCRIPTION
\fBvsegedges\fP divides edges into segments that can be approximated
reasonably well be straight lines. Each is segmented individually to yield
one or more new, straight edges. These new edges replace the original ones
in the program's output.
.PP
The segmentation algorithm is fully described in [1] and [2]. Briefly, an
edge is first subdivided at the point where it deviates most from a
straight line joining its endpoints. Subdivision repeats recursively until
the edge is reduced to short fragments. Pairs of fragments are then
re-joined wherever the best length-to-deviation ratio of the two fragments
is exceeded by the length-to-deviation ratio of their catenation. The
resulting segments represent a partition of the entire edge into sections
that approximate straight lines.
.SH "COMMAND LINE OPTIONS"
\fBvsegedges\fP accepts the following options:
.IP \fB-help\fP 15n
Prints a message describing options.
.IP "\fB-in\fP \fIinfile\fP ..."
Specifies a Vista data file containing the set of edges to be segmented.
.IP "\fB-out\fP \fIoutfile\fP"
Specifies where to write the set of segmented edges.
.IP "\fB-product points\fP | \fBlines\fP | \fBfits\fP"
Specifies the form in which each segment is output:
.RS
.IP \fBpoints\fP 8n
A segment retains all of its points.
.IP \fBlines\fP
A segment is reduced to its two endpoints; internal points are discarded.
.IP \fBfits\fP
A segment's points are fit with a straight line by least squares
optimization. The segment's endpoints are projected onto this line to 
obtain two new endpoints, which form the segment that is output. This is
the default.
.RE
.IP "\fB-accuracy\fP \fIa\fP"
Specifies a limit on measurement accuracy, which will be used as the lower
bound for the maximum deviation of any fragment.
Default:\ 2.
.IP "\fB-granularity\fP \fIn\fP"
Specifies the minimum segment length, in number of edge points.  Recursive
subdivision proceeds until fragments have at most this many points.
Default:\ 4.
.IP "\fB-magnitude\fP \fIm\fP"
A segment is discarded if its average gradient magnitude is less than
\fIm\fP. Gradient magnitude is assumed to be stored in the third field of
each edge point (where, for example, \fBvlink\fP(1Vi) places it). Default:\ 0.
.PP
Input and output files can be specified on the command line or allowed to
default to the standard input and output streams.
.SH "SEE ALSO"
.BR vlink (1Vi),
.BR VEdges (3Vi),
.BR Vista (7Vi)
.SH NOTES
Each of the output edges (points) retains the edge (point) fields of the
edge (point) from which it is derived.
.SH LIMITATIONS
Closed edges are currently treated as if open \(em there is no attempt to fit
a straight line across the edge closure.
.SH AUTHORS
Original implementation by David Lowe <lowe@cs.ubc.ca>. Vista
implementation by Jeff Beis <beis@cs.ubc.ca> and Art Pope <pope@cs.ubc.ca>.
.SH REFERENCES
.IP [1] 4n
D.G. Lowe, \fBPerceptual Organization and Visual Recognition\fP, (Kluwer,
1985).
.IP [2]
D.G. Lowe, ``Three-dimensional object recognition from single
two-dimensional images'', \fBArtif. Intel. 31\fP (1987) 355-395.
