/*
 *  $Id: vtopgm.c,v 1.8 1994/06/15 23:14:49 pope Exp pope $
 *
 *  The vtopgm program, which converts Vista image files to portable graymap
 *  (PGM) files.
 */

/*
 *  Copyright 1993, 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: David Lowe, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include "vista/Vlib.h"
#include "vista/mu.h"
#include "vista/option.h"
#include "vista/os.h"
#include "vista/VImage.h"

/* File identification string: */
VRcsId ("$Id: vtopgm.c,v 1.8 1994/06/15 23:14:49 pope Exp pope $");

/* Later in this file: */
void WritePgmFile (FILE *, VImage, VBand);

int main (int argc, char *argv[])
{
    static VShort band;
    static VOptionDescRec options[] = {
	{ "band", VShortRepn, 1, & band, VOptionalOpt, NULL,
	      "Image band to convert" }
    };
    FILE *infile, *outfile;
    int nimages;
    VAttrList attributes;
    VImage *images, image;

    /* Parse command line arguments and identify the input and output files */
    VParseFilterCmd (VNumber (options), options, argc, argv,
		     & infile, & outfile);

    /* Read all images from the input file */
    if ((nimages = VReadImages (infile, & attributes, & images)) == 0)
	exit (EXIT_FAILURE);	/* no images found */
    fclose (infile);

    /* Ensure that the specified band exists: */
    if (nimages != 1)
	VWarning ("Only the first of %d images will be converted", nimages);
    if (band < 0 || band >= VImageNBands (images[0]))
	VError ("Image of %d band(s) has no band %d",
		VImageNBands (images[0]), band);

    /* Convert the image to UByte: */
    if (VPixelRepn(images[0]) != VUByteRepn) {
	image = VConvertImageRange (images[0], NULL, (VBand) band, VUByteRepn);
	if (! image)
	    exit (EXIT_FAILURE);
	band = 0;
    } else image = images[0];

    /* Output the first image as a pgm file. */
    WritePgmFile (outfile, image, (VBand) band);

    return EXIT_SUCCESS;
}

/* Output an image in the pgm format (see pgm man page for details).
 */
void WritePgmFile (FILE *outfile, VImage image, VBand band)
{
    VUByte *ppixel;
    int i;
    int rows = VImageNRows (image),
        columns = VImageNColumns (image);

    /* Output header information for pgm. */
    fprintf (outfile, "P2\n%d %d\n255\n", columns, rows);

    ppixel = (VUByte *) VPixelPtr (image, band, 0, 0);
    
    for (i = 0; i < rows * columns; i++) {
	fprintf (outfile, " %3d", *ppixel++);
	if (i%10 == 9)
	    fprintf (outfile, "\n");
    }
    fprintf (outfile, "\n");
}
