.ds Vn 2.1
.ds Un inches
.TH vtops 1Vi "21 April 1994" "Vista Version \*(Vn"
.SH NAME
vtops \- render image and edge sets using PostScript
.SH SYNOPSIS
\fBvtops\fR [\fB-\fIoption\fR ...] [\fIinfile\fR ...] [\fB>\fP \fIoutfile\fR]
.SH DESCRIPTION
\fBvtops\fP uses PostScript to render one image, one or more 
edge sets, or an image overlaid with one or more edge sets. Any image found 
among the input files is rendered as a background, and then any edge sets 
are drawn overtop. The output of \fBvtops\fP can be sent directly to a 
PostScript printer or included in another document. 
.PP
\fBvtops\fP examines an image's band interpretation attributes to decide
whether to render it as a gray-scale image, a color image, or a flow field.
It is rendered in color if it is a standard RGB color image (\fBncolors:\
3\fP and \fBcolor_interp:\ rgb\fP) and it has a single component dimension
(\fBncomponents: 1\fP). It is rendered as a flow field if it is a
two-component gradient image (\fBncomponents:\ 2\fP and
\fBcomponent_interp:\ gradient\fP). Otherwise it is rendered in gray.
.PP
The PostScript image produced by \fBvtops\fP has 8-bit unsigned pixels. 
This means that it can have a maximum of 256 shades of gray (for a 
gray-scale image) or a maximum of 256 levels for each primary color (for a 
color image). If the input image is not represented in 8-bit unsigned 
pixels, then an appropriate conversion will be performed to preserve the 
appearance of the image. 
.PP
\fBvtops\fP uses the PostScript's RGB color model to specify colors. Since 
the model is machine-dependent to a certain degree, hardcopies and screen 
previews of \fBvtops\fP's output may vary in appearance among different 
printers and displays. 
.SH "COMMAND LINE OPTIONS"
These are general options accepted by \fBvtops\fP:
.IP \fB-help\fP 15n
Prints a message describing options.
.IP "\fB-in\fP \fIinfile\fP ..."
Specifies Vista data files containing the objects to be rendered.
.IP "\fB-out\fP \fIoutfile\fP"
Specifies where to write the PostScript output.
.IP "\fB-band\fP \fIband\fP"
Specifies the band of a gray-scale image to be rendered, the first of
three consecutive bands of an RGB color image to be rendered, or the first
of two consecutive bands of a gradient image to be rendered. Default: 0.
.IP "\fB-eps false\fP"
Normally \fBvtops\fP produces Encapsulated PostScript, which is suitable
for inclusion in other documents. Specifying \fB-eps false\fP causes it to
instead produce a PostScript page that can be sent directly to a printer.
.PP
These options control rendering of edge sets:
.IP "\fB-colors\fP \fIcolorspec...\fP"
Specifies colors to be used for drawing edges. The color of the first set 
of edges is specified by the first \fIcolorspec\fP, the second by the 
second \fIcolorspec\fP, and so on. Each \fIcolorspec\fP is a color name or 
a hexadecimal RGB triple, with the format described by the \fBX\fP(1) man 
page (e.g., \fBred\fP or \fB#F00\fP). If this option is absent or the number 
of \fIcolorspec\fPs is less than the number of edge sets, then those edge 
sets with no corresponding \fIcolorspec\fP will have default colors.
.IP \fB-endpoints\fP
Causes edge endpoints to be drawn as small circles. Default: edge 
endpoints are not drawn.
.IP "\fB-linewidths\fP \fIwidth\fP ..."
Specifies the widths of the lines used to draw the edges. The width of the 
first set of edges is specified by the first \fIwidth\fP, the second by the 
second \fIwidth\fP, and so on. A width of 0 produces a hairline. If this 
option is absent or the number of \fIwidth\fPs is less than the number of 
edge sets, then those edge sets with no corresponding \fIwidth\fP will have 
the default width of 0. A \fIwidth\fP is expressed in image pixels. 
.PP
This option controls rendering of gradient images:
.IP "\fB-arrow_width\fP \fIwidth\fP"
Specifies the width of the arrows used to draw flow fields, in units of
image pixels. Default: 0.2.
.PP
These options control page layout:
.IP \fB-landscape\fP
Gives the generated page a landscape orientation. If omitted, the
page has a portrait orientation.
.IP "\fB-width\fP \fIdistance\fP"
Specifies the width of the physical page. Default: 8.5in.
.IP "\fB-height\fP \fIdistance\fP"
Specifies the height of the physical page. Default: 11in.
.PP
.nf
\fB-left\fP \fIdistance\fP
\fB-right\fP \fIdistance\fP
\fB-top\fP \fIdistance\fP
.fi
.ns
.IP "\fB-bottom\fP \fIdistance\fP" 15n
Specify the width of the page margins. All default to 1in.
.PP
These options control decorations:
.IP "\fB-frame\fP \fIdistance\fP"
If specified, a frame is drawn around the image and edge sets. The frame 
has width \fIdistance\fP, which may be 0 to produce a hairline. Default: no 
frame. 
.IP "\fB-caption\fP \fIcaption\fP"
Specifies the text of a caption to be drawn at the bottom of the page.
Default: no caption is drawn.
.IP "\fB-capheight\fP \fIdistance\fP"
Specifies the height of the font used to draw the caption. Default: 14pt.
.PP
Input files can be specified on the command line or allowed to default to 
the standard input stream. The output file can be specified by the \fB-out\fP 
option or allowed to default to the standard output stream. 
.PP
Distances can be specified in any of several units using these suffixes
adopted from TeX: pt (points), pc (picas), in (inches), bp (big points, of
1/72 in), cm (centimeters), mm (millimeters), dd (didot points), cc
(ciceros), sp (scaled points, of 1/65536 pt). If no units are specified,
\*(Un are assumed.
.SH "SEE ALSO"
.BR VEdges (3Vi),
.BR VImage (3Vi),
.BR Vista (7Vi)
.SH NOTES
Any \fBpixel_aspect_ratio\fP attribute that an image has will be taken into
account when rendering it. Edge sets, on the other hand, are always assumed
to have coordinates that scale equally in both the x and y directions.
.PP
A gradient image may look best if it is subsampled so that the flow field
arrows are not too dense (with \fBvscale\ \-sample\fP), and scaled so that
the arrows are of reasonable length (with \fBvop\ \-op\ mult\fP).
.SH AUTHORS
Daniel Ko <ko@cs.ubc.ca>,
Art Pope <pope@cs.ubc.ca>
