/*
 *  $Id: vtranspose.c,v 1.4 1994/06/15 23:14:49 pope Exp pope $
 *
 *  The vtranspose program, which transposes the rows and columns 
 *  of Vista images.
 */

/*
 *  Copyright 1993, 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Daniel Ko, UBC Laboratory for Computational Intelligence
 */

/* From the Vista library: */
#include "vista/Vlib.h"
#include "vista/mu.h"
#include "vista/option.h"
#include "vista/os.h"
#include "vista/VImage.h"

/* File identification string: */
VRcsId ("$Id: vtranspose.c,v 1.4 1994/06/15 23:14:49 pope Exp pope $");


int main (int argc, char **argv)
{
    FILE *in_file, *out_file;
    VAttrList list;
    VImage src;
    VImage result;
    VAttrListPosn posn;
    int nimages = 0;

    /* Parse command line arguments: */
    VParseFilterCmd (0, NULL, argc, argv, & in_file, & out_file);

    /* Read source image(s): */
    if (! (list = VReadFile (in_file, NULL)))
	exit (EXIT_FAILURE);

    /* Transpose each image: */
    for (VFirstAttr (list, & posn); VAttrExists (& posn); VNextAttr (& posn)) {
	if (VGetAttrRepn (& posn) != VImageRepn)
	    continue;
	VGetAttrValue (& posn, NULL, VImageRepn, & src);

	result = VTransposeImage (src, NULL, VAllBands);
	if (result == NULL)
	    exit (EXIT_FAILURE);
	
	VSetAttrValue (& posn, NULL, VImageRepn, result);
	VDestroyImage (src);
	nimages++;
    }

    /* Write the results to the output file: */
    if (! VWriteFile (out_file, list))
	exit (EXIT_FAILURE);

    fprintf (stderr, "%s: Transposed %d image%s.\n",
	     argv[0], nimages, nimages == 1 ? "" : "s");
    return EXIT_SUCCESS;
}


/* end */
