/*
 *  $Id: FuncMin.h,v 1.1 1994/06/10 01:39:22 pope Exp pope $
 *
 *  Definitions associated with routines in FuncMin.c for locating a
 *  minimum of a multidimension function. They're based on ideas and
 *  routines in Press, et al., `Numerical Recipes in C', Cambridge
 *  University Press, 1988.
 */

/*
 *  Copyright 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Adapated by Arthur Pope, UBC Laboratory for Computational Intelligence
 */

#ifndef V_FuncMin_h
#define V_FuncMin_h 1

/*
 *  Usage:
 *      #include "FuncMin.h"
 *
 *      int dims;
 *	VDouble x[nvars];
 *      VPointer my_data;
 *      extern VDouble f (VPointer my_data, VDouble x[nvars]);
 *
 *      dims <- bit mask of dimensions to search over
 *      x <- initial guess
 *      my_data <- anything you'd like passed to your function
 *      if (V_PowellMin (f, my_data, dims, x, max_iterations, tolerance))
 *          x is now the location of a minimum;
 *      else no minimum could be located.
 */

/* From the Vista library: */
#include <vista/Vlib.h>

/* Maximum dimensionality supported: */
#define fmMaxNVars 12

/* Type declaration for the multidimensional function minimized: */
typedef VDouble FMFunctionN (VPointer, VDouble []);

/* Routines implemented in funcmin.c: */
VBoolean V_PowellMin (FMFunctionN *, VPointer, int, VDouble [], int, VDouble);

#endif /* V_FuncMin_h */
