/*
 *  $Id: LinSysMin.h,v 1.1 1994/06/10 01:39:26 pope Exp pope $
 *
 *  Definitions associated with a recursive least-squares estimator.
 *
 *  The estimator is implemented using the square-root information filter
 *  algorithm as described in Gerald J Bierman's book "Factorization Methods 
 *  for Discrete Sequential Estimation", Academic Press, 1977.
 */

/*
 *  Copyright 1994 University of British Columbia
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appears in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation. UBC makes no representations about the suitability of this
 *  software for any purpose. It is provided "as is" without express or
 *  implied warranty.
 *
 *  Author: Arthur Pope, UBC Laboratory for Computational Intelligence
 */

#ifndef V_LinSysMin_h
#define V_LinSysMin_h 1

/*
 *  Usage:
 *      #include "LinSysMin.h"
 *
 *	Estimator est;
 *	VDouble a[nvars], b;
 *
 *	V_InitEstimator (& est, nvars);
 *	for (i = 0; i < ...; i++) {
 *	    set a[0..nvars-1], b
 *	    V_ExtendEstimator (& est, a, b);
 *	}
 *	V_UpdateEstimator (& est);
 *	reference est.x[0..nvars-1], est.e
 */

/* From the Vista library: */
#include <vista/Vlib.h>

/* The maximum dimension of the state vector: */
#define lsmMaxNVars	7

/* The maximum number of measurements batched between state updates: */
#define lsmMaxBatch	100

/* Current state of an estimator: */
typedef struct {
    int nvars;				/* no. variables in system */
    VDouble x[lsmMaxNVars];		/* lastest state vector estimate */
    VDouble e;				/* estimated residual */
    int nrows;				/* no. rows valid in s */
    VDouble s[lsmMaxBatch][lsmMaxNVars + 1];/* a priori information array */
} Estimator;

/* Routines implemented in linsysmin.c: */
extern void V_InitEstimator (Estimator *, int);
extern void V_ExtendEstimator (Estimator *, VDouble [], VDouble);
extern VBoolean V_UpdateEstimator (Estimator *);

#endif /* V_LinSysMin_h */
