.ds Vn 2.1
.TH VCalibrateCamera 3Vi "10 June 1994" "Vista Version \*(Vn"
.SH NAME
VCalibrateCamera \- calibrate a camera model using data points
.SH SYNOPSIS
.B "#include <vista/VCamera.h>"
.HP 10n
.na
.nh
.ft B
VCamera VCalibrateCamera (VCamera\ \fIsrc\fP, VBoolean\ \fIextrinsics_only\fP,
VBoolean\ \fIcoplanar\fP, int\ \fInpoints\fP,
VDouble\ \fIpw\fP[][3], VDouble\ \fIpid\fP[][2],
int\ \fImax_iterations\fP, double\ \fItolerance\fP);
.ft
.hy
.ad
.SH ARGUMENTS
.IP \fIsrc\fP 10n
Specifies the camera model to be calibrated.
.IP \fIextrinsics_only\fP
Specifies whether all camera model parameters are to be estimated, or only
extrinsic ones.
.IP \fIcoplanar\fP
Specifies 
.SB TRUE
if the data points lie on a single world plane.
.IP \fInpoints\fP
Specifies the number of data points.
.IP \fIpw\fP
Specifies the world coordinates of \fInpoints\fP data points.
.IP \fIpid\fP
Specifies the distorted image coordinates of \fInpoints\fP data points
(i.e., the location at which the points are actually detected by the camera
and frame grabber, as subject to various distortions).
.IP \fImax_iterations\fP
Specifies a limit on iterations performed by the calibration procedure
during its function-minimization steps. Failure to locate a minima in this
many iterations signals an ill-conditioned problem. A reasonable value is
200.
.IP \fItolerance\fP
Specifies the desired accuracy for the function-minimization steps
performed by the calibration procedure. Failure to decrease by this
fractional amount during an iteration signals successful localization of a
minima. (A reasonable value is 1e-10).
.SH DESCRIPTION
\fBVCalibrateCamera\fP returns a calibrated copy of the camera model
\fIsrc\fP.  The copy differs from \fIsrc\fP only in that certain parameters
will have been estimated using the corresponding world and image
coordinates of a set of data points. The estimation procedure used is
essentially that described by Tsai [1], with enhancements by Reg Willson.
.PP
The data points are specified by \fInpoints\fP and by two arrays, \fIpw\fP and
\fIpid\fP, each with \fInpoints\fP elements. See ``Camera Calibration''
on the \fBVCamera\fP(3Vi) man page for restrictions on data points.
.PP
If \fIextrinsics_only\fP is 
.SB FALSE\c
, both intrinsic parameters (\fIC\fP, \fID\fP, \fIk1\fP, \fIsx\fP, and
\fIf\fP) and extrinsic ones (\fIR\fP and \fIT\fP) are estimated.
Otherwise, only extrinsic parameters are estimated.
.PP
The \fIcoplanar\fP argument must indicate whether the data points are
coplanar. If they are, they must all have a Z coordinate of zero.  The
horizontal scale factor, \fIsx\fP, can be estimated only if the data points
are not coplanar.
.PP
.PP
Parameters not estimated and other attributes are copied unchanged from
\fIsrc\fP to the result.
.PP
Even when \fBVCalibrateCamera\fP is being used to estimate intrinsic
parameters, \fIsrc\fP should contain initial guesses for \fIC\fP and
\fIsx\fP. Reasonable guesses are the image coordinates of the frame buffer
center and 1.0, respectively.
.SH "RETURN VALUES"
\fBVCalibrateCamera\fP returns the calibrated camera model if successful, and
.SB NULL
otherwise.
.SH "SEE ALSO"
.na
.nh
.BR VCamera (3Vi),
.BR VCameraError (3Vi),
.BR vcalsyn (1Vi),
.BR Vlib (7Vi)
.ad
.hy
.SH DIAGNOSTICS
.IP "``Focal length undefined.''"
\fBVCalibrateCamera\fP was asked to solve for extrinsic parameters only of a
camera model whose focal length is undefined.
.IP "``Numerically ill-conditioned problem.''"
Camera model parameters cannot be estimated from the data provided.
.IP "``Possible handedness problem with data.''"
Camera rotation ambiguity implies that the data points were measured in an
improper coordinate system.
.IP "``Z coordinate supplied for coplanar data.''"
The coplanar argument indicates that the data points are coplanar; however,
some have world Z coordinates that are non-zero.
.SH AUTHORS
Calibration procedure by Roger Tsai [1].
Original implementation by Reg Willson <rgw@cs.cmu.edu>.
Vista implementation by Art Pope <pope@cs.ubc.ca>.
.SH REFERENCES
.IP [1] 4n
R. Tsai, ``A versatile camera calibration technique for high-accuracy 3D
machine vision metrology using off-the-shelf TV cameras and lenses'',
\fBIEEE J. Robotics and Automation\fP (1987), pp.\ 323-344.
