.ds Vn 2.1
.TH VCamera 3Vi "31 May 1994" "Vista Version \*(Vn"
.SH NAME
VCamera \- camera model representation
.SH SYNOPSIS
.nf
.B "#include <vista/VCamera.h>"
.PP
.B "void VRegisterCameraType (void);"
.PP
.B "extern VRepnKind VCameraRepn;"
.PP
.B "VCamera \fIcamera\fP;"
.PP
.B "ld\0\0\fR...\fP\0\0-lvcam\0\0-lvista\0\0-lm\0\0-lgcc\0\0\fR...\fP"
.fi
.SH DESCRIPTION
.SS Introduction
A Vista camera model represents the characteristics and position of a
camera. By recording the camera's internal geometric and optical
characteristics (intrinsic parameters) as well as its 3D position relative
to a world coordinate system (extrinsic parameters), a camera model allows
conversion between 3D world coordinates and 2D image coordinates.
.PP
Vista implements a camera model based on the work of Roger Tsai [1]. The
model actually describes a combination of camera and frame grabber, and it
accounts for three of the most significant sources of image distortion:
radial distortion, displacement of the image center, and mismatch between
camera and frame grabber horizontal scan rates.
.SS "Coordinate Systems"
A Vista camera model describes the transformation from 3D world coordinates
to 2D image coordinates in four steps via intermediate coordinate systems.
All coordinate systems are right-handed with axes XY or XYZ. World, camera,
and sensor coordinates are expressed in millimeters; image coordinates are
in pixels. The four transformation steps are:
.ta 10n
.RS 2n
.IP 1. 4n
The 3D world coordinates of a point P, denoted \fIPw\fP, are mapped to 3D
camera coordinates \fIPc\fP by a rotation \fIR\fP about the world origin
followed by a translation \fIT\fP. The 3D camera coordinate system is
centered at the camera's optical center (where optical rays converge in the
absence of distortion).  Viewed in the direction the camera is pointing,
the camera coordinate system's +X axis points right, its +Y axis points
down, and its Z axis lies along the optical axis. The transformation from
world to camera coordinates is
.PP
.nf
	\fIPc\fP = \fIR\fP \fIPw\fP + \fIT\fP
.fi
.IP 2. 4n
The point \fIPc\fP is projected onto the camera sensor as if the camera
were an ideal pinhole camera. The focal length is \fIf\fP and the projected
point, in an ideal undistorted sensor coordinate system, has coordinates
\fIPsu\fP given by
.PP
.nf
	\fIPsu\fP[0] = \fIf\fP * \fIPc\fP[0] / \fIPc\fP[2]
	\fIPsu\fP[1] = \fIf\fP * \fIPc\fP[1] / \fIPc\fP[2]
.fi
.IP 3. 4n
The projected point is subjected to radial lens distortion, mapping it from
undistorted sensor coordinates \fIPsu\fP to distorted sensor coordinates
\fIPsd\fP. This distortion is modeled by a single factor proportional to
the square of the distance from the optical axis to the projected point.
This factor is parameterized by \fIk1\fP:
.PP
.nf
	\fIPsu\fP[0] = \fIPsd\fP[0] * (1 + \fIk1\fP * \fIrr\fP)
	\fIPsu\fP[1] = \fIPsd\fP[1] * (1 + \fIk1\fP * \fIrr\fP)
.fi
.IP "" 4n
where \fIrr\fP = \fIPsd\fP[0] * \fIPsd\fP[0] + \fIPsd\fP[1] * \fIPsd\fP[1]
.IP 4. 4n
The point is mapped from distorted sensor coordinates \fIPsd\fP to
distorted image coordinates \fIPid\fP. This mapping is determined by the
image coordinates of the image center, \fIC\fP, the size of image pixels in
sensor coordinates, \fID\fP, and a factor \fIsx\fP accounting for mismatch
between sensor and frame grabber horizontal scan rates. Unlike the sensor
and camera, the image's coordinate system has its origin at the lower left
corner of the image, its +X axis points right, and its +Y axis points up.
.PP
.nf
	\fIPid\fP[0] = \fIC\fP[0] + \fIPsd\fP[0] / \fID\fP[0] * \fIsx\fP
	\fIPid\fP[1] = \fIC\fP[1] - \fIPsd\fP[1] / \fID\fP[1]
.fi
.RE
.PP
An additional coordinate system represents the image coordinates a point
would have if the camera were to have no radial distortion. Undistorted
image coordinates are denoted by \fIPiu\fP:
.PP
.nf
	\fIPiu\fP[0] = \fIC\fP[0] + \fIPsu\fP[0] / \fID\fP[0] * \fIsx\fP
	\fIPiu\fP[1] = \fIC\fP[1] - \fIPsu\fP[1] / \fID\fP[1]
.fi
.DT
.PP
A fuller description of this camera model and an associated calibration
procedure may be found in [1].
.SS "Camera Model Components"
A Vista camera model specifies the following information about a camera.
.RS 2n
.IP \(bu 2n
the numbers of rows and columns of sensor elements
.IP \(bu
the numbers of rows and columns of frame grabber pixels
.IP \(bu
the dimensions of a sensor element
.IP \(bu
the effective dimensions of a frame grabber pixel, \fID\fP
.IP \(bu
the image coordinates, \fIC\fP, of the intersection between the
optical axis and the sensor
.IP \(bu
a radial distortion coefficient, \fIk1\fP
.IP \(bu
a horizontal scale factor, \fIsx\fP
.RE
.PP
Optionally, the camera model may also specify:
.RS 2n
.IP \(bu 2n
the position of the camera as defined by a rotation \fIR\fP and 
translation \fIT\fP
.IP \(bu
the camera's focal length, \fIf\fP
.IP \(bu
any other attributes one may wish to associate with the camera, such as
comments identifying the type of camera and frame grabber
.RE
.SS "The VCamera Type"
A camera model is represented in memory by an object of type \fBVCamera\fP.
Before using this type in your program (and before reading a \fBVCamera\fP
object from a data file) you must register the type by calling
\fBVRegisterCameraType\fP:
.PP
.RS
.B "VRegisterCameraType ();"
.RE
.PP
Following this call, the external variable \fBVCameraRepn\fP will contain
the representation code assigned to identify the \fBVCamera\fP type (see
\fBVtype\fP(3Vi)).
.PP
Your program must be linked with the \fBvcam\fP library, as shown above in
the SYNPOSIS section.
.SS "Creating, Copying, and Destroying Camera Models"
The following routines create, copy, and destroy \fBVCamera\fP objects.
.PP
.B "VCamera VCreateCamera (void);"
.IP
\fBVCreateCamera\fP allocates memory for a camera model. All camera
parameters are initialized to zero, and the camera model is given an empty
attribute list.
.PP
.B "VCamera VCopyCamera (VCamera \fIsrc\fP);"
.IP
\fBVCopyCamera\fP returns a copy of \fIsrc\fP. Included in the copy are
both camera parameters and any other attributes associated with \fIsrc\fP.
.PP
.B "void VDestroyCamera (VCamera \fIc\fP);"
.IP
\fBVDestroyCamera\fP releases the memory occupied by a camera model
and its attributes.
.SS "Accessing Camera Model Parameters"
The following table lists macros used to access fields of a \fBVCamera\fP
object. Each macro takes a single argument of type \fBVCamera\fP, and
accesses a value of the indicated type.
.PP
.RS 2n
.ta 25n 40n
.I "Macro	Type	Description"
.TP 40n
.B "VCameraNSels	VLong [2]"
numbers of columns and rows, respectively, of camera sensor elements
.TP
.B "VCameraNPixels	VLong [2]"
numbers of columns and rows of frame buffer pixels
.TP
.B "VCameraSelSize	VDouble [2]"
width and height of camera sensor elements, in millimeters
.TP
.B "VCameraPixelSize	VDouble [2]"
effective width and height of frame grabber pixels, \fID\fP, in millimeters
.TP
.B "VCameraImageCenter	VDouble [2]"
the image coordinates, \fIC\fP, of the intersection between the optical
axis and the sensor
.TP
.B "VCameraRadialDistortion	VDouble"
the radial distortion coefficient, \fIk1\fP
.TP
.B "VCameraScaleX	VDouble"
the horizontal scale factor, \fIsx\fP
.TP
.B "VCameraExtrinsicsDefd	VBoolean"
whether the camera model's extrinsic parameters are defined
.TP
.B "VCameraRotation	VDouble [3]"
the world-to-camera coordinate system rotation, represented as Euler angles
denoting successive rotations about the world X, Y, and Z axes; each
rotation angle is in radians, measured according to the right-hand
convention (clockwise looking along the axis in the positive direction)
.TP
.B "VCameraRotMatrix	VDouble [3][3]"
the world-to-camera coordinate system, represented as a rotation matrix,
\fIR\fP, for premultiplying location vectors
.TP
.B "VCameraTranslation	VDouble [3]"
the world-to-camera coordinate system translation, in millimeters
.TP
.B "VCameraFocalLengthDefd	VBoolean"
whether the camera model's focal length is defined
.TP
.B "VCameraFocalLength	VDouble"
the focal length, \fIf\fP, in millimeters
.TP
.B "VCameraAttrList	VAttrList"
a list of other attributes associated with the camera model
.RE
.PP
Each macro may be used as either an rvalue (e.g., on the right hand side of
an assignment operator) or as an lvalue (on the left hand side of an
assignment operator). See the EXAMPLES section, below, for an illustration
of their use.
.PP
The Euler-angle and rotation-matrix representations of the world-to-camera
rotation must be kept consistent. The \fBVSetCameraRotation\fP(3vi) routine
can be used to ensure consistency.
.SS "Data File Representation"
.nf
.ft B
\fIattribute-name\fP: camera {
.RS
nsels_x: \fInsels\fR[0]\fB
nsels_y: \fInsels\fR[1]\fB
npixels_x: \fInpixels\fR[0]\fB
npixels_y: \fInpixels\fR[1]\fB
sel_size_x: \fIsel_size\fR[0]\fB
sel_size_y: \fIsel_size\fR[1]\fB
pixel_size_x: \fID\fR[0]\fB
pixel_size_y: \fID\fR[1]\fB
image_center_x: \fIC\fR[0]\fB
image_center_y: \fIC\fR[1]\fB
radial_distortion: \fIk1\fP
scale_x: \fIsx\fP
rotation_x: \fIA\fR[0]\fB
rotation_y: \fIA\fR[1]\fB
rotation_z: \fIA\fR[2]\fB
translation_x: \fIT\fR[0]\fB
translation_y: \fIT\fR[1]\fB
translation_z: \fIT\fR[2]\fB
focal_length: \fIf\fP
\fIother attributes\fP
.RE
}
.fi
.ta 10n
.IP \(bu 2n
The attributes within a \fBcamera\fP object may be listed in any order.
.IP \(bu 2n
Distances, such as \fIsel_size\fP[0], are in millimeters; angles are in 
radians.
.IP \(bu
The \fBnpixels_x\fP and \fBnpixels_y\fP attributes are optional; 
if omitted, their values default to
.PP
.nf
	\fInpixels\fP[0] = 512
	\fInpixels\fP[1] = 480
.fi
.IP \(bu 2n
The \fBpixel_size_x\fP and \fBpixel_size_y\fP attributes are optional; 
if omitted, their values default to
.PP
.nf
	\fID\fP[0] = \fIsel_size\fP[0] * \fInsels\fP[0] / \fInpixels\fP[0]
	\fID\fP[1] = \fIsel_size\fP[1] * \fInsels\fP[1] / \fInpixels\fP[1]
.fi
.IP \(bu 2n
The \fBimage_center_x\fP and \fBimage_center_y\fP attributes are optional; 
if omitted, their values default to
.PP
.nf
	\fIC\fP[0] = \fInpixels\fP[0] / 2
	\fIC\fP[1] = \fInpixels\fP[1] / 2
.fi
.IP \(bu 2n
The \fBradial_distortion\fP attribute is optional;
if omitted, its value defaults to 0.
.IP \(bu
The \fBscale_x\fP attribute is optional; if omitted, its value defaults to 1.
.IP \(bu
The attributes \fBrotation_x\fP through \fBtranslation_z\fP may be omitted, 
indicating that the extrinsic parameters are undefined.
.IP \(bu
The \fBrotation_x\fP, \fBrotation_y\fP, and \fBrotation_z\fP attributes
describe the world-to-camera coordinate system rotation as successive
rotations about the world X, Y, and Z axes; each rotation angle is in
radians, measured according to the right-hand convention (clockwise looking
along the axis in the positive direction).
.IP \(bu
The \fBfocal_length\fP attribute may be omitted, indicating that the 
focal length is undefined.
.DT
.SS "Camera Calibration"
Given a series of data points whose positions are known in both world 
coordinates and distorted image coordinates, several of a camera model's 
parameters can be estimated. Using \fBvcamcal\fP(1Vi) or 
\fBVCalibrateCamera\fP(3Vi), you can
.RS 2n
.IP \(bu 2n
estimate both intrinsic parameters (\fIC\fP, \fID\fP, \fIk1\fP, \fIsx\fP, 
and \fIf\fP) and extrinsic ones (\fIR\fP and \fIT\fP);
.IP \(bu
estimate extrinsic parameters only, retaining the existing values of 
intrinsic ones; or
.IP \(bu
measure the accuracy of an existing camera model.
.RE
.PP
The estimation procedure differs slightly depending on whether the 
data points lie in a single world plane.
.RS 2n
.IP \(bu 2n
Coplanar points must all have a Z world coordinate of zero. In addition, 
for numerical stability, all points should lie some distance from the 
origin of the world coordinate system. The plane containing the
points should not be parallel to the imaging plane; a relative angle
of 30 degrees is recommended. The horizontal scale factor, \fIsx\fP, 
cannot be estimated from coplanar points.
.IP \(bu
Noncoplanar points must not lie on a single world plane. The horizontal
scale factor, \fIsx\fP, is among the parameters that can be estimated.
.RE
.PP
Various programs produce or accept lists of data points in 
Vista data files. A list of data points is represented as an attribute 
list with each data point occupying a single attribute. Data point values 
are, in turn, attribute lists of coordinate values. This is best 
illustrated by an example, here showing a list of (only) two data points:
.PP
.RS
.nf
.ft B
points: {
	0: {
		w_x: 10
		w_y: 10
		w_z: 30
		i_x: 193.45
		i_y: 185.20
	}
	1: {
		w_x: 10
		w_y: 30
		w_z: 30
		i_x: 192.68
		i_y: 148.59
	}
}
.ft
.fi
.RE
.PP
The \fBw_x\fP, \fBw_y\fP, and \fBw_z\fP attributes record the world
coordinates of a data point; \fBi_x\fP and \fBi_y\fP record its image
coordinates. The names of the data points (``0'' and ``1'' in the example)
are not significant.
.SH EXAMPLES
This Vista data file contains a minimal, uncalibrated camera model:
.PP
.RS
.nf
.ft B
V-data 2 {
	camera1: camera {
		camera_name: "Sony XC-77RR"
		nsels_x: 768
		nsels_y: 493
		sel_size_x: 0.011
		sel_size_y: 0.013
	}
}
^L
.ft
.fi
.RE
.PP
The number of rows and columns of frame grabber pixels for the camera model
\fIc\fP may be set to 480 and 512, respectively, by:
.PP
.RS
.nf
.B "VCameraNPixels(c)[0] = 512;"
.B "VCameraNPixels(c)[1] = 480;"
.fi
.RE
.PP
Its focal length may be set by:
.PP
.RS
.nf
.B "VCameraFocalLength(c) = 30.0;
.B "VCameraFocalLengthDefd(c) = TRUE;
.RE
.SH "SEE ALSO"
.na
.nh
.BR vcalsyn (1Vi),
.BR vcamcal (1Vi),
.br
.BR Vattribute (3Vi),
.BR Vfile (5Vi),
.BR Vlib (7Vi)
.ad
.hy
.SH AUTHORS
Calibration procedure by Roger Tsai [1].
Original implementation by Reg Willson <rgw@cs.cmu.edu>.
Vista implementation by Art Pope <pope@cs.ubc.ca>.
.SH REFERENCES
.IP [1] 4n
R. Tsai, ``A versatile camera calibration technique for high-accuracy 3D
machine vision metrology using off-the-shelf TV cameras and lenses'',
\fBIEEE J. Robotics and Automation\fP (1987), pp.\ 323-344.
.IP [2]
J. Weng, P. Cohen, M. Herniou, ``Camera calibration with distortion models
and accuracy evaluation'', \fBIEEE Trans. Pattern Analysis and Machine
Intelligence\fP (1992), pp.\ 965-980.
.SH "LIST OF ROUTINES"
.if t \{
.ds IN 38n
.\}
The following table summarizes other Vista library routines that operate on
camera models. Each routine is documented elsewhere by a section 3Vi man
page named for the routine.
.PP
For converting among coordinate systems:
.RS 2n
.IP \fBVWorldToCameraCoords\fP
.IP \fBVCameraToWorldCoords\fP
.IP \fBVWorldToImageCoords\fP
.IP \fBVImageToWorldCoords\fP
.IP \fBVDistortedToUndistortedImageCoords\fP
.IP \fBVUndistortedToDistortedImageCoords\fP
.IP \fBVDistortedToUndistortedSensorCoords\fP
.IP \fBVUndistortedToDistortedSensorCoords\fP
.IP \fBVImageToUndistortedSensorCoords\fP
.RE
.PP
For evaluating camera model accuracy:
.RS 2n
.IP \fBVNormalizedCameraCalibrationError\fP \*(IN
computes statistics of Weng et al.'s normalized calibration error [2]
.IP \fBVObjectSpaceCameraCalibrationError\fP
computes statistics of error in world coordinates
.RE
.PP
Miscellaneous:
.RS 2n
.IP \fBVCalibrateCamera\fP \*(IN
for estimating camera model parameters from calibrated data points
.IP \fBVSetCameraRotation\fP
for setting Euler angles and rotation matrix consistently
.RE
