.ds Vn 2.1
.TH VCameraError 3Vi "1 June 1994" "Vista Version \*(Vn"
.SH NAME
VCameraError \- evaluate camera model accuracy
.SH SYNOPSIS
.B "#include <vista/VCamera.h>"
.HP 10n
.na
.nh
.ft B
void VNormalizedCameraCalibrationError (VCamera \fIc\fP,
int\ \fInpoints\fP, VDouble\ \fIpw\fP[][3], VDouble\ \fIpid\fP[][2],
VDouble\ *\fImean\fP, VDouble\ *\fIstddev\fP, VDouble\ *\fImax\fP);
.ft
.hy
.ad
.HP 10n
.na
.nh
.ft B
void VObjectSpaceCameraCalibrationError (VCamera \fIc\fP,
int\ \fInpoints\fP, VDouble\ \fIpw\fP[][3], VDouble\ \fIpid\fP[][2],
VDouble\ *\fImean\fP, VDouble\ *\fIstddev\fP, VDouble\ *\fImax\fP);
.ft
.hy
.ad
.SH ARGUMENTS
.IP \fIc\fP 10n
Specifies the camera model to be evaluated.
.IP \fInpoints\fP
Specifies the number of data points.
.IP \fIpw\fP
Specifies the world coordinates of \fInpoints\fP data points.
.IP \fIpid\fP
Specifies the distorted image coordinates of \fInpoints\fP data points
(i.e., the location at which the points are actually detected by the camera
and frame grabber, as subject to various distortions).
.IP \fImean\fP
Returns the mean error over all data points, or it may be
.SB NULL
if the statistic isn't desired.
.IP \fIstddev\fP
Returns the standard deviation of the error over all data points, or it may
be
.SB NULL
if the statistic isn't desired.
.IP \fImax\fP
Returns the maximum error over all data points, or it may be
.SB NULL
if the statistic isn't desired.
.SH DESCRIPTION
Both routines use corresponding world and image coordinates of a set of
data points to evaluate the accuracy of a camera model. They back-project
each point from the image according to the camera model, and then measure the
distance between the projected ray and the point's world location. In
different ways the two routines compute an error measure based on this
distance. Both routines then report the mean, standard deviation, and/or
maximum value of the error measure.
.PP
\fBVNormalizedCameraCalibrationError\fP uses the \fInormalized calibration
error\fP measure described by Weng et al. [1]. This measure's principle
advantage is that it is insensitive to factors such as pixel quantization,
field of view, and object-to-camera distance.
.PP
\fBVObjectSpaceCameraCalibrationError\fP uses, as its error measure, the
minimum distance between the back-projected ray and the point's world
location.
.PP
The data points are specified by \fInpoints\fP and by two arrays, \fIpw\fP
and \fIpid\fP, each with \fInpoints\fP elements. The statistics are
returned via \fImean\fP, \fIstddev\fP, and \fImax\fP.
.SH "SEE ALSO"
.na
.nh
.BR VCamera (3Vi),
.BR Vlib (7Vi)
.ad
.hy
.SH AUTHOR
Original implementation by Reg Willson <rgw@cs.cmu.edu>.
Vista implementation by Art Pope <pope@cs.ubc.ca>.
.SH REFERENCES
.IP [1] 4n
J. Weng, P. Cohen, M. Herniou, ``Camera calibration with distortion models
and accuracy evaluation'', \fBIEEE Trans. Pattern Analysis and Machine
Intelligence\fP (1992), pp.\ 965-980.
