.ds Vn 2.1
.TH vcamcal 1Vi "10 June 1994" "Vista Version \*(Vn"
.SH NAME
vcamcal \- calibrate and evaluate a camera model using data points
.SH SYNOPSIS
\fBvcamcal -data\fI datafile\fR [\fB-\fIoption\fR ...] [\fIinfile\fR] [\fIoutfile\fR]
.SH DESCRIPTION
\fBvcamcal\fP uses a set of data points with specified world and image
coordinates to estimate camera model parameters and/or evaluate their
accuracy. The estimation procedure used is essentially that described by
Tsai [1], with enhancements by Reg Willson.
.PP
The data points are read from \fIdatafile\fP as an attribute list whose
format is as described under ``Camera Calibration'' on the
\fBVCamera\fP(3Vi) man page. See that man page for restrictions on the data
points.
.PP
Each camera model found in the input file, \fIinfile\fP, is then calibrated
and/or evaluated, as specified by program options, before being copied to
the output file, \fIoutfile\fP. Other contents of \fIinfile\fP are copied
unchanged.
.SH "COMMAND LINE OPTIONS"
\fBvcamcal\fP accepts the following options:
.IP \fB-help\fP 15n
Prints a message describing options.
.IP "\fB-in\fP \fIinfile\fP ..."
Specifies a Vista data file containing the camera model(s) to be calibrated.
.IP "\fB-out\fP \fIoutfile\fP"
Specifies where to write the calibrated camera model(s).
.IP "\fB-data\fP \fIdatafile\fP"
Specifies a Vista data file containing the set of data points.
.IP "\fB-name\fP \fIname\fP"
Specifies the name of the object in \fIdatafile\fP whose value is an
attribute list of data point coordinates. Default: \fBpoints\fP.
.IP "\fB-coplanar true\fP | \fBfalse\fP"
Specifies whether the data points are coplanar. Default: \fBfalse\fP.
.IP "\fB-estimate all\fP | \fBextrinsics\fP | \fBnone\fP"
Specifies which camera model parameters are to be estimated:
.RS
.IP \fBall\fP 12n
Both intrinsic parameters (\fIC\fP, \fID\fP, \fIk1\fP, \fIsx\fP, and
\fIf\fP) and extrinsic ones (\fIR\fP and \fIT\fP) are estimated.
.IP \fBextrinsics\fP
Only extrinsic parameters are estimated.
.IP \fBnone\fP
No parameters are estimated or modified. This is useful when \fBvcamcal\fP
is being used only to evaluate camera model accuracy.
.RE
.IP "\fB-report true\fP | \fBfalse\fP"
Specifies whether error measures are to be reported. These are statistics
of the normalized calibration error and object space calibration error
described by \fBVCameraError\fP(3Vi). They are computed after any parameter
estimation, using the same data points as were used in the estimation.
Default: \fBtrue\fP.
.IP "\fB-iterations\fP \fIi\fP"
Specifies a limit on iterations performed by the calibration procedure
during its function-minimization steps. Failure to locate a minima in this
many iterations signals an ill-conditioned problem. Default: 200.
.IP "\fB-tolerance\fP \fIt\fP"
Specifies the desired accuracy for the function-minimization steps
performed by the calibration procedure. Failure to decrease by this
fractional amount during an iteration signals successful localization of a
minima. Default: 1e-10.
.PP
Input and output files can be specified on the command line or allowed to
default to the standard input and output streams.
.SH "SEE ALSO"
.BR VCamera (3Vi),
.BR VCameraError (3Vi),
.BR vcalsyn (1Vi),
.BR Vista (7Vi)
.SH NOTES
The \fB-coplanar\fP option must indicate whether the data points are
coplanar. If they are, they must all have a Z coordinate of zero.  The
horizontal scale factor, \fIsx\fP, can be estimated only if the data points
are not coplanar.
.PP
Parameters not estimated and other attributes of a camera model are copied
unchanged from input to output.
.PP
Even when \fBvcamal\fP is being used to estimate intrinsic parameters, the
input camera model should contain initial guesses for \fIC\fP and \fIsx\fP.
A reasonable guess for \fIC\fP is the image coordinates of the frame buffer
center, which is the default when no \fBimage_center_x\fP and
\fBimage_center_y\fP attributes are present. A reasonable guess for
\fIsx\fP is 1.0, which is its default.
.SH AUTHORS
Calibration procedure by Roger Tsai [1].
Original implementation by Reg Willson <rgw@cs.cmu.edu>.
Vista implementation by Art Pope <pope@cs.ubc.ca>.
.SH REFERENCES
.IP [1] 4n
R. Tsai, ``A versatile camera calibration technique for high-accuracy 3D
machine vision metrology using off-the-shelf TV cameras and lenses'',
\fBIEEE J. Robotics and Automation\fP (1987), pp.\ 323-344.
