.ds Vn 2.1
.ds ID /usr/include
.ds SD /usr/src/vista
.TH VX 3Vi "16 June 1994" "Vista Version \*(Vn"
.SH NAME
VX \- routines for building Vista applications under the X Window System
.SH SYNOPSIS
.nf
\fBcc\fP\0\0...\0\0\fB-I\*(ID\fP
.PP
.B #include <vista/VX.h>
.PP
.B ld\0\0\fR...\fP\0\0-lvx\0\0-lvista\0\0-lXm\0\0-lXt\0\0-lXmu\0\0-lXext\0\0-lX11\0\0-lm\0\0-lgcc\0\0\fR...\fP
.fi
.SH DESCRIPTION
.SS Introduction
The VX library provides a simple set of C functions for building Vista 
applications under the X Window System. It allows programmers with very 
little knowledge of X to efficiently build standard Vista applications for 
the X Window System. VX is built on top of the X Toolkit and the 
OSF/Motif widget set.
.PP
A tutorial introduction to VX is included in the document ``Introduction to
Vista Programming'', which is available on-line in the directory
\fB\*(SD/doc/intro\fP.
.PP
Every application built with VX has a standard main window, which 
contains a horizontal menu bar, a drawing area for displaying a Vista 
image, and an optional message area. Lines can be drawn on top of a Vista 
image in the drawing area. An application may also pop up dialog boxes to 
interact with the user. 
.PP
Applications built with VX are event-driven. A programmer constructs 
an application with VX by associating events (i.e. the actions of the 
user, such as pressing a key) with functions. These functions are called 
callback functions because they are first registered with VX and then 
called by VX whenever their associated events occur. 
.PP
VX provides the following routines for building Vista applications:
.RS 2n
.IP \(bu 2n
initialization routines,
.IP \(bu
menu specification and callback registration routines,
.IP \(bu
image display routines,
.IP \(bu
line drawing routines,
.IP \(bu
text drawing routines,
.IP \(bu
dialog manipulation routines, and
.IP \(bu
miscellaneous routines.
.PP
.RE
.SS Routines
The following table summarizes the VX routines.
.PP
Initialization routines:
.RS 2n
.IP \fBVXInit\fP 20n
Initialize VX, process command-line arguments, and set default 
resources. 
.IP \fBVXAppMainLoop\fP
Perform the application's main event loop. 
.RE
.PP
Menu specification and callback registration routines:
.RS 2n
.IP \fBVXAddMenu\fP 20n
Add a menu to the menu bar of the application, and associate callback 
functions with the menu entries. 
.IP \fBVXAddInputCallback\fP
Associate a callback function with a particular type of user input.
.RE
.PP
Image display routine:
.RS 2n
.IP \fBVXSetImage\fP 20n
Display a region of an image. 
.RE
.PP
Line drawing routines:
.RS 2n
.IP \fBVXSetLineColor\fP 20n
Specify the current line color.
.IP \fBVXSetLineWidth\fP
Specify the current line width.
.IP \fBVXDrawLine\fP
Draw a line on the image.
.IP \fBVXDrawEdges\fP
Draw a set of edges on the image as line segments.
.IP \fBVXClearLines\fP
Clear all existing lines from the image.
.RE
.PP
Text drawing routines:
.RS 2n
.IP \fBVXSetTextColor\fP 20n
Specify the current text color.
.IP \fBVXSetTextFont\fP
Specity the current text font.
.IP \fBVXDrawText\fP
Draw a text string on the image.
.IP \fBVXClearTexts\fP
Clear all existing text strings from the image.
.RE
.PP
Overlays manipulation routines:
.RS 2n
.IP \fBVXStoreOverlays\fP 20n
Store the current states of all existing overlay objects so that
these objects can later be restored
by \fBVXRestoreOverlays\fP.
.IP \fBVXRestoreOverlays\fP
Remove all existing overlay objects and restore the overlay objects
previously stored by \fBVXStoreOverlays\fP.
.RE
.PP
Dialog manipulation routines:
.RS 2n
.IP \fBVXPopupMessageBox\fP 20n
Pop up a modeless dialog box for displaying a brief message.
.IP \fBVXPopupTextBox\fP
Pop up a modeless dialog box for displaying a piece of text.
.IP \fBVXPopupInputBox\fP
Pop up a modal dialog box for inputting a string.
.IP \fBVXPopupYesNoBox\fP
Pop up a modal dialog box for obtaining a yes/no answer.
.IP \fBVXPopupFileBox\fP
Pop up a modal dialog box for file selection.
.RE
.PP
Miscellaneous routines:
.RS 2n
.IP \fBVXDisplayMessage\fP 20n
Display a message in the message area of the main window.
.IP \fBVXShowMessageArea\fP
Show the message area of the main window.
.IP \fBVXHideMessageArea\fP
Hide the message area of the main window.
.IP \fBVXGetImageViewWidget\fP
Return the widget ID of the VImageView widget.
.IP \fBVXGetApplicationShell\fP
Return the widget ID of the top level application shell.
.IP \fBVXIsColorDisplay\fP
Determine the color capability of the current display device.
.IP \fBVXReportValidOptions\fP
Print usage information for VX command line options.
.RE
.SS "Command Line Options"
VX programs recognize command line options. These include options
recognized by the X Toolkit (see \fBX\fP(1X11)), any options implemented by
the programmer, plus the following option recognized by VX:
.IP "\fB-palette\fP \fBany\fP | \fBdefault\fP | \fBbest\fP | \fBgray\fP | \fBgrey\fP"
Normally VX tries to use a palette of colors that gives it the best chance
of coexisting well with other X clients. This is called the default
palette. However, because it is designed to be shared, the default palette
often does not allow images to be displayed with the greatest possible
fidelity.  Specifying \fB-palette best\fP causes VX to instead use a
palette optimized for displaying the most complete range of common RGB
colors (mostly reds and greens). Specifying \fB-palette gray\fP (or
\fBgrey\fP) causes it to use a palette optimized for displaying grey
shades. (When you choose \fBbest\fP, \fBgray\fP, or \fBgrey\fP, you may
find that VX uses a different colormap than that used by other windows on
your screen, and you will then have to move the pointer into a window for
it to display correctly.) The default, \fBany\fP, means try for the default
palette, otherwise the best palette, otherwise the gray palette.
.SS "Error Handling"
VX installs its own handler for warning messages issued through
\fBVWarning\fP(3Vi) (see \fBVSetWarningHandler\fP(3Vi)). The handler is
installed when the application enters its main event loop with a call to
\fBVXMainEventLoop\fP(3Vi).  It displays warning messages in a modal dialog
box, which the user must dismiss before \fBVWarning\fP will return.
.SS Widgets
An application built with VX uses the Vista VImageView widget as well 
as various other widgets drawn from the OSF/Motif toolkit. Although 
VX can be used without any knowledge of the underlying widgets, it is 
helpful to know the hierarchy of widgets created in order to customize the 
appearance of the program by specifying resource values. In the notation 
below, indentation indicates hierarchical structure. {...} means zero or 
more occurrences of the enclosed elements. Each line represents a single 
widget, with widget class name given first and widget (instance) name given 
second. In some cases, these are followed by a parenthetical comment. In 
cases where the widget instance name cannot be guaranteed, it is omitted. 
.PP
.nf
ApplicationShell
    XmMainWindow  mainWindow
        XmRowColumn  menuBar  (as menu bar)
            { XmCascadeButton  (see below for name) }
            XmMenuShell  popup_pullDownMenu
                { XmRowColumn
                    { XmPushButton  (see below for name) } }
    XmFrame  imageViewFrame
        VImageView  imageView  (as drawing area)
    XmFrame  msgAreaFrame
        XmScrolledWindow  SWmsgArea
            XmText  msgArea  (as message area)
    XmDialogShell  messageBox_popup  (used by \fBVXPopupMessageBox\fP)
        XmMessageBox  messageBox 
    XmDialogShell  textBox  (used by \fBVXPopupTextBox\fP)
        XmForm  form
            XmScrolledWindow  textAreaSW
                XmText  textArea
            XmPushButton  closeButton
    XmDialogShell  inputBox_popup  (used by \fBVXPopupInputBox\fP)
        XmSelectionBox  inputBox
    XmDialogShell  yesNoBox_popup  (used by \fBVXPopupYesNoBox\fP)
        XmMessageBox  yesNoBox
    XmDialogShell  fileBox_popup  (used by \fBVXPopupFileBox\fP)
        XmFileSelectionBox  fileBox
    XmDialogShell  warningBox_popup  (used by the nonfatal error handler)
        XmMessageBox  warningBox
.fi
.PP
The widget instance name of a button (XmCascadeButton) in the menu bar is 
equivalent to the \fImenu_name\fP argument of the call to \fBVXAddMenu\fP 
that creates that menu. The widget instance name of a menu entry button 
(XmPushButton) in a menu is equivalent to the \fIentry_name\fP argument of 
the call to \fBVXAddMenu\fP that creates that menu entry. (Any underscore 
used to indicate a mnemonic in \fImenu_name\fP or \fIentry_name\fP is not 
included in the widget instance name.) 
.SS Resources
There is only one application resource for an application built with 
VX:
.IP "\fBmessageAreaNLines\fP (Class: \fBMessageAreaNLines\fP )"
specifies the number of lines visible in the message area of the main 
window. Default: 0, which indicates no message area. 
.SH EXAMPLES
Sample VX programs are located in the following directories:
.RS
.IP \(bu 2n
\*(SD/examples/drawcross
.IP \(bu 
\*(SD/examples/drawtext
.IP \(bu 
\*(SD/examples/rubberband
.RE
.SH SEE ALSO
.na
.nh
.I Introduction to Vista Programming
.br
.BR VXInit (3Vi),
.BR VXAppMainLoop (3Vi),
.br
.BR VXAddMenu (3Vi),
.BR VXAddInputCallback (3Vi),
.br
.BR VXSetImage (3Vi),
.br
.BR VXSetLineColor (3Vi), 
.BR VXSetLineWidth (3Vi),
.br
.BR VXDrawLine (3Vi), 
.BR VXDrawEdges (3Vi), 
.BR VXClearLines (3Vi),
.br
.BR VXSetTextColor (3Vi),
.BR VXSetTextFont (3Vi),
.br
.BR VXDrawText (3Vi),
.BR VXClearTexts (3Vi),
.br
.BR VXStoreOverlays (3Vi),
.BR VXRestoreOverlays (3Vi),
.br
.BR VXPopupMessageBox (3Vi),
.BR VXPopupTextBox (3Vi),
.BR VXPopupInputBox (3Vi),
.BR VXPopupYesNoBox (3Vi),
.BR VXPopupFileBox (3Vi),
.br
.BR VXDisplayMessage (3Vi),
.BR VXShowMessageArea (3Vi),
.BR VXHideMessageArea (3Vi),
.br
.BR VXGetImageViewWidget (3Vi),
.BR VXGetApplicationShell (3Vi),
.br
.BR VXIsColorDisplay (3Vi),
.br
.BR X (1),
.I OSF/Motif Programmer's Guide
.ad
.hy
.SH AUTHOR
Daniel Ko <ko@cs.ubc.ca>
