.ds Vn 2.1
.TH VXAddInputCallback 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VXAddInputCallback \- associate a callback function with user input
.SH SYNOPSIS
.nf
.ta 25n
.B #include <vista/VX.h>
.PP
.ft B
void VXAddInputCallback (\fIinput_type\fP, \fIcallback\fP, \fIclient_data\fP)
.RS
VXInputType \fIinput_type\fP;
VXInputCallback \fIcallback\fP;
VPointer \fIclient_data\fP;
.RE
.PP
.ft B
typedef enum {
.RS
VXIkeyPress,	/* a key was pressed */
VXIbuttonPress,	/* a button was pressed */
VXIbuttonRelease,	/* a button was released */
VXIpointerMotion	/* the pointer was moved */
.RE
} VXInputType;
.PP
.ft B
typedef enum {
.RS
VXMshift = (1<<0),	/* the Shift key */
VXMctrl = (1<<2),	/* the Ctrl key */
VXMbutton1 = (1<<8),	/* pointer button 1 */
VXMbutton2 = (1<<9),	/* pointer button 2 */
VXMbutton3 = (1<<10),	/* pointer button 3 */
VXMbutton4 = (1<<11),	/* pointer button 4 */
VXMbutton5 = (1<<12),	/* pointer button 5 */
VXMmod1 = (1<<3),	/* modifier 1 */
VXMmod2 = (1<<4),	/* modifier 2 */
VXMmod3 = (1<<5),	/* modifier 3 */
VXMmod4 = (1<<6),	/* modifier 4 */
VXMmod5 = (1<<7)	/* modifier 5 */
.RE
} VXModifierMask;
.PP
.ft B
typedef struct {
.RS
VXInputType input_type;	/* type of user input */
int value;	/* ASCII code or button number */
VXModifierMask modifiers;	/* combination of modifiers */
int row;	/* row coordinate of user input */
int column;	/* column coordinate of user input */
.RE
} VXInputDataRec, *VXInputData;
.PP
.ft B
typedef void (*) (VXInputData, VPointer) VXInputCallback;
.DT
.fi
.SH ARGUMENTS
.IP \fIinput_type\fP 10n
Specifies the type of user input to be associated with the callback 
function. 
.IP \fIcallback\fP
Specifies the callback function to be associated with \fIinput_type\fP. 
.IP \fIclient_data\fP
Specifies the value to be passed to \fIcallback\fP whenever it is invoked 
by VX. 
.SH DESCRIPTION
\fBVXAddInputCallback\fP is a routine in Vista's VX library for associating a
callback function with a particular type of user input.  The callback
function will be invoked whenever user input of the type \fIinput_type\fP
occurs while the pointer is within the main window's drawing area.
.PP
The prototype for the callback function is:
.PP
.RS
.nf
.B void MyCallback (VXInputData \fIinput_data\fP, VPointer \fIclient_data\fP)
.fi
.RE
.PP
The \fIinput_data\fP argument describes the user input with a structure 
whose fields have the following meanings:
.RS 2n
.IP \fBinput_type\fP 10n
Specifies the type of user input. This field is useful for a callback 
function that is associated with more than one type of user input. 
.IP \fBvalue\fP
Specifies the 
.SB ASCII
code of the key that was pressed, or the number of the mouse button that was 
pressed or released. The buttons are numbered 1 through 5, with the
leftmost being 1.
.IP \fBmodifiers\fP
Specifies the state of the buttons and modifier keys when the user input 
occurred. It can be any combination of \fBVXModifierMask\fP bits combined 
by logical ``or''.
.IP "\fBrow\fP, \fBcolumn\fP"
Specify the row and column coordinates of the pointer when the user input 
occurred. The coordinate system of the currently displayed image is used. 
.RE
.SH EXAMPLE
The following code fragment associates \fBPrintRowColumn\fP with mouse 
button presses. \fBPrintRowColumn\fP prints the pointer location provided 
it was button 1 that was pressed.
.PP
.nf
.ft B
.RS
void PrintRowColumn (VXInputData input_data, VPointer client_data)
{
	if (input_data->value == 1)
		printf ("(%3d,%3d)", input_data->row, input_data->column);
}

	\fR...\fP
	VXAddInputCallback (VXIbuttonPress, PrintRowColumn, NULL);
	\fR...\fP
.RE
.fi
.SH SEE ALSO
.BR VX (3Vi)
.SH NOTES
Multiple callback functions can be associated with a single input type, 
but they're invoked in arbitrary order.
.SH DIAGNOSTICS
.IP "``Callback function cannot be NULL.''"
The callback function was specified as
.SB NULL.
A non-\c
.SB NULL
function is expected.
.IP "``Invalid input type.''"
The \fIinput_type\fP argument does not specify a valid input type.
.IP "``Main event loop already started.''"
The main event loop has already been started with a prior call to 
\fBVXAppMainLoop\fP(3Vi). \fBVXAddInputCallback\fP must be called before 
the start of the main event loop. 
.IP "``VX not initialized.''"
The VX library has not been initialized with a prior call to 
\fBVXInit\fP(3Vi). 
.SH AUTHOR
Daniel Ko <ko@cs.ubc.ca>
