.ds Vn 2.1
.TH VXAddMenu 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VXAddMenu \- add a pulldown menu to the menu bar of a VX application
.SH SYNOPSIS
.nf
.B #include <vista/VX.h>
.PP
\fBvoid VXAddMenu (\fImenu_name\fR 
.RS 17n
[\fB, \fIentry_name\fB, \fIaccelerator\fB, \fIcallback\fB, \
\fIclient_data\fR ...]\fB,
NULL)
.RE
.RS
.ft B
VStringConst \fImenu_name\fP, \fIentry_name\fP, \fIaccelerator\fP;
VXCallback \fIcallback\fP;
VPointer \fIclient_data\fP;
.RE
.SH ARGUMENTS
.IP \fImenu_name\fP 10n
Specifies the menu name to appear on the menu bar. This name may contain a 
mnemonic character, which is specified by preceding it with an underscore. 
.IP \fIentry_name\fP
Specifies the name of a menu entry. This name may contain a mnemonic
character, which is specified by preceding it with an underscore.
.IP \fIaccelerator\fP
Specifies the accelerator for the menu entry. An accelerator is a string of 
the form \fB"\fImodifier\fB<Key>\fIchar\fB"\fR. The \fImodifier\fP 
component may be omitted; if present, it is one of \fBCtrl\fP, \fBShift\fP, 
\fBLock\fP, \fBMeta\fP, \fBHyper\fP, \fBSuper\fP, \fBAlt\fP, \fBMod1\fP, 
\&..., \fBMod5\fP, \fBButton1\fP, ..., or \fBButton5\fP. The \fIchar\fP 
component is any member of the set {\ \fBA\fP, ..., \fBZ\fP, \fBa\fP, ..., 
\fBz\fP, \fB0\fP, ..., \fB9\fP\ }. The \fIaccelerator\fP argument may be 
.SB NULL 
to indicate that there is no accelerator for the menu entry. 
.IP \fIcallback\fP
Specifies a callback function to be invoked by whenever the menu entry is 
selected by the user. The \fIcallback\fP argument may be 
.SB NULL
to indicate that no callback is associated with the menu entry. 
.IP \fIclient_data\fP
Specifies the value to be passed to \fIcallback\fP whenever it is
invoked by VX.
.SH DESCRIPTION
\fBVXAddMenu\fP adds a pulldown menu to the menu bar of an application 
managed by the VX library. New menus are added to the right of existing 
menus, except that a Help menu (i.e., one with \fImenu_name\fP equal to 
\fBHelp\fP) is always placed at the extreme right of the menu bar. The 
order in which the menu entries appear in the pulldown menu is the same as 
the order in which they are specified in the argument list. 
.PP
The prototype for a callback function is:
.RS
.nf
.B void MyCallback (VPointer \fIclient_data\fP)
.fi
.RE
.SH EXAMPLE
The following code fragment creates a File menu with two
entries, Open and Quit.
.PP
.nf
.ft B
.RS
void Open (VPointer client_data);
void Quit (VPointer client_data);

	\fR...\fP
	VXAddMenu ("_File",
		"Open", "Ctrl<Key>O", Open, NULL,
		"Quit", "Ctrl<Key>Q", Quit, NULL,
		NULL);
	\fR...\fP
.RE
.fi
.PP
In the above example, the mnemonic for the File menu is the character
``F'', so the user can press Alt-F to invoke the File menu. In addition,
the accelerator for the menu entry Open is Ctrl-O, so the user can press
Ctrl-O or Ctrl-o to select the menu entry Open directly without first
invoking the File menu.
.SH SEE ALSO
.BR VX (3Vi)
.SH DIAGNOSTICS
.IP "``Accelerator \fIaccelerator\fP is too long.''"
The accelerator description string \fIaccelerator\fP is too long.
.IP "``Accelerator \fIaccelerator\fP is invalid.''"
The format of the accelerator description string \fIaccelerator\fP is invalid.
.IP "``Error in converting mnemonic to KeySym.''"
The mnemonic specified cannot be understood. 
.IP "``Main event loop already started.''"
The main event loop has already been started with a prior call to 
\fBVXAppMainLoop\fP(3Vi). \fBVXAddMenu\fP must be called before the start 
of the main event loop. 
.IP "``Menu entry label \fIentry_name\fP is too long.''"
The menu entry label \fIentry_name\fP is too long to fit inside a menu. 
.IP "``VX not initialized.''"
The VX library has not been initialized with a prior call to 
\fBVXInit\fP(3Vi). 
.SH AUTHOR
Daniel Ko <ko@cs.ubc.ca>
