.ds Vn 2.1
.TH VXInit 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VXInit, VXAppMainLoop \- initialize and run a VX application
.SH SYNOPSIS
.nf
.B #include <vista/VX.h>
.PP
.ft B
void VXInit (\fIclass\fP, \fIresources\fP, \fIargc\fP, \fIargv\fP)
.RS
VStringConst \fIclass\fP;
VStringConst *\fIresources\fP;
int *\fIargc\fP;
char **\fIargv\fP;
.RE
.PP
.B void VXAppMainLoop (void)
.fi
.SH ARGUMENTS
.IP \fIclass\fP 10n
Specifies the application's class name, which is used for fetching X 
resource values. 
.IP \fIresources\fP
Specifies default resource settings. For an example of initializing 
\fIresources\fP, see the \fBEXAMPLE\fP section. 
.IP \fIargc\fP
Specifies the number of command line arguments to be parsed, and returns 
the number of arguments that were not recognized as valid options.
.IP \fIargv\fP
Specifies a vector of command line arguments to be parsed, and returns a 
vector of the arguments that were not recognized as valid options.
.SH DESCRIPTION
\fBVXInit\fP initializes the VX library, loads the values of X application 
resources, and parses command line arguments to extract standard X Toolkit 
options (see \fBX\fP(1)). \fBVXInit\fP must be called before any other 
VX routines are invoked. 
.PP
\fBVXAppMainLoop\fP runs the application's main event loop. It repeatedly 
accepts events from X Windows, such as those related to user input, and 
calls the appropriate callback routines registered with 
\fBVXAddInputCallback\fP(3Vi) and \fBVXAddMenu\fP(3Vi). \fBVXAppMainLoop\fP 
never returns to its caller.
.SH "RETURN VALUES"
\fBVXInit\fP returns a revised \fIargv\fP containing only the command line 
arguments that it did not recognize as valid options. In *\fIargc\fP it 
returns the number of such arguments. \fBVXInit\fP moves elements of the 
\fIargv\fP vector of strings so that all arguments that are not part of 
valid options are in the first *\fIargc\fP elements. At minimum, there will 
be one: the program's name. Others might be options that, for some reason, 
the programmer chooses not to handle with \fBVXInit\fP. 
.SH EXAMPLE
The following code fragment demonstrates the use of default resources in 
setting the widget foreground color to yellow, and the number of lines in 
the message area to 2.
.PP
.nf
.ft B
.RS
main (int argc, char **argv) 
{
	static VStringConst resources[] = {
		"*foreground: yellow",
		"MyApp.messageAreaNLines: 2",
		NULL
	};

	VXInit ("MyApp", resources, & argc, argv);

	\fRCreate menus and register input callbacks\fP

	VXAppMainLoop ();
}
.RE
.fi
.SH SEE ALSO
.BR X (1),
.BR VX (3Vi)
.SH DIAGNOSTICS
.IP "``VX already initialized.''"
Duplicate calls have been made to \fBVXInit\fP.
.IP "``VX not initialized.''"
\fBVXAppMainLoop\fP has been called without a prior call to \fBVXInit\fP. 
.SH AUTHOR
Daniel Ko <ko@cs.ubc.ca>
