.ds Vn 2.1
.TH VXPopupModal 3Vi "24 April 1993" "Vista Version \*(Vn"
.SH NAME
VXPopupFileBox, VXPopupInputBox, VXPopupYesNoBox \- pop up a dialog box for input
.SH SYNOPSIS
.nf
.B #include <vista/VX.h>
.PP
.ft B
VString VXPopupFileBox (\fItitle\fP)
.RS
VStringConst \fItitle\fP;
.RE
.PP
.ft B
VString VXPopupInputBox (\fItitle\fP, \fIprompt\fP, \fIvalue\fP)
.RS
VStringConst \fItitle\fP, \fIprompt\fP, \fIvalue\fP;
.RE
.PP
.ft B
VXAnswer VXPopupYesNoBox (\fItitle\fP, \fIprompt\fP)
.RS
VStringConst \fItitle\fP, \fIprompt\fP;
.RE
.PP
.ft B
typedef enum {
.RS
VXAyes,    	/* ``Yes'' button was pressed */
VXAno,     	/* ``No'' button was pressed */
VXAcancel 	/* ``Cancel'' button was pressed */
.RE
} VXAnswer;
.fi
.SH ARGUMENTS
.IP \fItitle\fP 10n
Specifies the title of the dialog box.
.IP \fIprompt\fP
Specifies the prompting question for an input box or yes/no box.
.IP \fIvalue\fP
Specifies the initial value of an input box's editable text field.
.SH DESCRIPTION
These routines, part of the VX library, obtain user input by popping up
modal dialog boxes. A modal dialog box forces the user to respond to the
box before he or she can interact with the main window of the application.
The routines don't return until the user has responded.
.PP
\fBVXPopupFileBox\fP pops up a file selection dialog box, which allows the 
user to browse through the file system to select a file. Clicking either 
the ``OK'' button or the ``Cancel'' button in the dialog box will close the 
box. \fBVXPopupFileBox\fP indicates which button was pressed by its return 
value. 
.PP
\fBVXPopupInputBox\fP pops up an input dialog box, which contains the 
prompt specified by \fIprompt\fP, an editable text field, and two buttons. 
The prompt is used for specifying what kind of information is expected from 
the user. The editable text field initially contains the string 
\fIvalue\fP, but it may be changed by the user. Clicking either of the two 
buttons, ``OK'' or ``Cancel'', closes the box. \fBVXPopupInputBox\fP 
returns an indication of which button was pressed, along with the edited 
contents of the text field.
.PP
\fBVXPopupYesNoBox\fP pops up a yes/no dialog box, which contains the 
prompt specified by \fIprompt\fP plus buttons labelled ``Yes'', ``No'', and 
``Cancel''. Clicking a button closes the box and causes 
\fBVXPopupYesNoBox\fP to return an indication of which button was pressed. 
.SH "RETURN VALUES"
\fBVXPopupFileBox\fP returns the fully qualified file name selected by the 
user if the ``OK'' button was clicked, and 
.SB NULL
otherwise.
.PP
\fBVXPopupInputBox\fP returns the final contents of the editable text field 
if the ``OK'' button was clicked, and 
.SB NULL
otherwise.
.PP
The strings returned by \fBVXPopupFileBox\fP and \fBVXPopupInputBox\fP have 
to be released by the caller using \fBVFree\fP(3Vi). 
.PP
\fBVXPopupYesNoBox\fP returns \fBVXAyes\fP, \fBVXAno\fP, or \fBVXAcancel\fP 
according to which button was clicked. 
.SH SEE ALSO
.na
.nh
.BR VXPopupMessageBox (3Vi),
.BR VXPopupTextBox (3Vi),
.BR VX (3Vi)
.ad
.hy
.SH DIAGNOSTICS
.IP "``Main event loop not started.''" 
The main event loop has not been started by a prior call to 
\fBVXAppMainLoop\fP(3Vi). This function must be called inside the main 
event loop by a callback function. 
.IP "``VX not initialized.''"
The VX library has not been initialized with a prior call to 
\fBVXInit\fP(3Vi). 
.SH AUTHOR
Daniel Ko <ko@cs.ubc.ca>
